/* @(#) mpVideoDecode.c 96/08/29 1.16 */
/*	file: mpVideoDecode.c
	M2 MPEG Video Decoder entry point
	3/23/95 George Mitsuoka
	The 3DO Company Copyright 1995 */

#include <string.h>
#include <kernel/types.h>
#include <kernel/debug.h>
#include "mpVideoDecode.h"
#include "mpVideoDriver.h"
#include "MPEGVideoParse.h"


void mpVideoDecode(tVideoDeviceContext *theUnit)
{
	int32 parseResult, flags, pts, userData, result;
	MPEGStreamInfo streamInfo;

	flags = 0L; pts = 0L; userData = 0L;

	/* initialize streamInfo */
	memset( &streamInfo, 0, sizeof( MPEGStreamInfo ) );
	result = MPOpen(theUnit, &(streamInfo.streamID));

	if( result )
	{
		PRNT(("mpVideoDecode: MPOpen returns %ld! ack!\n",result));
		goto abort;
	}
	while( 1 )
	{
		parseResult = MPVideoParse(theUnit, &streamInfo);

		/* check return value */
		switch( parseResult )
		{
			case MPVFLUSHWRITE:			/* complete the current write buffer */
				PRNT(("mpVideoDecode: MPVideoParse returns MPVFLUSHWRITE\n"));
				MPVCompleteWrite(theUnit, flags, pts, userData);

				do
				{
					result = MPOpen(theUnit, &(streamInfo.streamID));
					if( result == MPVFLUSHWRITE )
					{
						PRNT(("mpVideoDecode: MPOpen returns MPVFLUSHWRITE\n"));
						MPVCompleteWrite(theUnit, flags, pts, userData);
					}
					else if (result == MPVFLUSHDRIVER)
					{
						PRNT(("mpvVideoDecode: MPOpen returns MPVFLUSHDRIVER\n"));
						theUnit->branchState = JUST_BRANCHED;
						ResetReferenceFrames(theUnit);
				    }
					else if (result == ABORTED)
					{
						PRNT(("mpVideoDecode: MPOpen returns ABORTED\n"));
						goto abort;
					}
					else if( result )
						PERR(("mpVideoDecode: MPOpen returns %lx!\n",result));
				}
				while(result);
				break;
			case MPVFLUSHDRIVER:
				PRNT(("mpVideoDecode: GOT MPVFLUSHDRIVER.\n"));
				theUnit->branchState = JUST_BRANCHED;
				ResetReferenceFrames(theUnit);

			case MPVFLUSHREAD:			/* complete current read buffer */
				PRNT(("mpVideoDecode: MPVideoParse returns MPVFLUSHREAD\n"));

				do {
					result = MPOpen(theUnit, &(streamInfo.streamID));
					if( result == MPVFLUSHWRITE  )
					{
						PRNT(("mpVideoDecode: MPOpen returns MPVFLUSHWRITE\n"));
						MPVCompleteWrite(theUnit, flags, pts, userData);
					}
					else if (result == MPVFLUSHDRIVER) {
					   PRNT(("mpvVideoDecode: MPOpen returns MPVFLUSHDRIVER\n"));
					   theUnit->branchState = JUST_BRANCHED;
					   ResetReferenceFrames(theUnit);
					}
					else if (result == ABORTED) 
					{
						PRNT(("mpVideoDecode: MPOpen returns ABORTED\n"));
						goto abort;
					}
					else if( result )
						PERR(("mpVideoDecode: MPOpen returns %lx!\n",result));
				} while (result);
				break;
			case ABORTED:
				PRNT(("mpVideoDecode: MPVideoParse returns ABORTED\n"));
				goto abort;
				break;
			default:					/* something weird happened */
				PRNT(("mpVideoDecode: unexpected %ld result MPVideoParse\n",
					   parseResult));
				break;
		}
	}
abort:
	DestroyVCDOptimize(streamInfo.vcdInfo);
	return;
}

void
ResetReferenceFrames(tVideoDeviceContext *theUnit)
{
	theUnit->refPicFlags[0] = REFPICEMPTY;
	theUnit->refPicFlags[1] = REFPICEMPTY;
	theUnit->nextRef = 0;
}

