/* @(#) mpVideoDriver.c 96/08/09 1.30 */
/*	file: mpVideoDriver.c
	M2 MPEG Video Driver
	2/28/95 George Mitsuoka
	The 3DO Company Copyright (C) 1995 */

#define MPVIDEODRIVER_C

#include <string.h>
#include <kernel/debug.h>
#include <kernel/io.h>
#include <kernel/mem.h>
#include <kernel/super.h>
#include <graphics/bitmap.h>
#include <graphics/graphics.h>
#include <loader/loader3do.h>
#include "M2MPEGUnit.h"
#include "mpVideoDriver.h"


#define DEBUG(a)	{ PRNT(("MPEG Video Driver S: ")); PRNT(a); }
#define UDEBUG(a)	{ PRNT(("MPEG Video Driver U: ")); PRNT(a); }
#define INTERNAL_MPEG_DRIVERNAME		"mpegvideo"
#define INTERNAL_MPEG_DEVICENAME		"Portfolio Device"
#define INTERNAL_MPEG_DRIVERIDENTITY	DI_OTHER
#define INTERNAL_MPEG_MAXSTATUSSIZE	    sizeof( DeviceStatus )
#define INTERNAL_MPEG_DEVICEFLAGWORD	DS_DEVTYPE_OTHER
#define INTERNAL_MPEG_DRIVERPRI			150
#define INTERNAL_MPEG_DEVICEPRI			150
#define INTERNAL_MPEG_COMMAND1NAME		"control command"
#define INTERNAL_MPEG_COMMAND2NAME		"write command"
#define INTERNAL_MPEG_COMMAND3NAME		"read command"

static void	mpvDriverAbortIO( IOReq *ior );
static Item	mpvDriverInit( Driver *drv );
static Err mpvDriverDelete(Driver* drv);
static Item mpvDriverClose(Driver* drv, Task* task);
static Item	mpvDeviceInit( Device *dev );
static Err mpvDeviceDelete( Device *dev );
static Item	mpvDeviceOpen(Device *dev, Task* task);
static Err mpvDeviceClose(Device *dev, Task* task);
static int32 mpvCmdWrite( IOReq *ior );
static int32 mpvCmdRead( IOReq *ior );
static int32 mpvOldCmdRead(IOReq* ior);
static int32 mpvCmdStatus( IOReq *ior );
static int32 mpvCmdControl( IOReq *ior );
static void StartWrite(tVideoDeviceContext *theUnit);
static void StartRead(tVideoDeviceContext *theUnit);
static void QueueReq(IOReq *ior);
static int32 FreeDeviceResources(Device* dev);

static DriverCmdTable CmdTable[] =
{
	CMD_STATUS,	          mpvCmdStatus,
	MPEGVIDEOCMD_CONTROL, mpvCmdControl,
	MPEGVIDEOCMD_WRITE,	  mpvCmdWrite,
	MPEGVIDEOCMD_READ,    mpvCmdRead,
	CMD_READ,	          mpvOldCmdRead,  /* used by MPEGVideoSubscriber. */
};

/*	there is one driver arguments table per driver */
static void QueueReq( IOReq *ior )
{
	int32 interrupts;
	tVideoDeviceContext *theUnit;

	theUnit = MPEGSTATE(ior->io_Dev);

	/* add request to the appropriate queue */
	/* support obsolete CMD_WRITE for now */
	if( (ior->io_Info.ioi_Command == MPEGVIDEOCMD_WRITE) ||
		(ior->io_Info.ioi_Command == CMD_WRITE) )
	{
		/* disable interrupts while queueing the request */
		interrupts = Disable();
		AddTail( &(theUnit->writeQueue), (Node *) ior );
		PRNT(("Add: Write Q=%d\n", GetNodeCount(&(theUnit->writeQueue))));
		Enable( interrupts );
	}
	/* support obsolete CMD_READ for now */
	else if( (ior->io_Info.ioi_Command == MPEGVIDEOCMD_READ) ||
			 (ior->io_Info.ioi_Command == CMD_READ) )
	{
		/* disable interrupts while queueing the request */
		interrupts = Disable();
		AddTail( &(theUnit->readQueue), (Node *) ior );
		PRNT(("Add: Read Q=%d\n", GetNodeCount(&(theUnit->readQueue))));
		Enable( interrupts );
	}
}

IOReq *NextWriteReq( tVideoDeviceContext *theUnit )
{
	PRNT(("Rem: Write Q=%d\n", GetNodeCount(&(theUnit->writeQueue))-1));
	return( (IOReq *) RemHead( &(theUnit->writeQueue) ) );
}

IOReq *NextReadReq( tVideoDeviceContext *theUnit )
{
	PRNT(("Rem: Read Q=%d\n", GetNodeCount(&(theUnit->readQueue))-1));
	return( (IOReq *) RemHead( &(theUnit->readQueue) ) );
}

static void StartWrite(tVideoDeviceContext *theUnit)
{
	/* signal the decoder task that a write buffer is available */
    SuperInternalSignal(theUnit->task, theUnit->signalWriteAvail);
}

static void StartRead(tVideoDeviceContext *theUnit)
{
	/* signal the decoder task that a read buffer is available */
	SuperInternalSignal( theUnit->task, theUnit->signalReadAvail );
}

/*	int32 MPVRead(tVideoDeviceContext* theUnit, uint8 **buf, int32 *len,
				   uint32 *pts, uint32 userData)

		MPVRead returns in buf a pointer to a buffer containing
		raw MPEG video data. The byte length of the buffer is returned
		in len. The required unit is passed to the decode
		routine when invoked. The return value from MPVRead can be
		a system error code (< 0), zero for normal completion,
		MPVPTSVALID, indicating that the pts and userData values have been set,
		or MPVFLUSHWRITE, indicating that the decode routine should
		complete its current write buffer */

int32 MPVRead(tVideoDeviceContext *theUnit, uint8 **buf, int32 *len,
			  uint32 *pts, uint32 *userData)
{
	int32 waitResult;
	IOBuf *theWriteBuf;
	FMVIOReqOptions *optionPtr;

	UDEBUG(("MPVRead ... "));

	/* is the queue empty? */
	while( IsEmptyList( &VDCWRITEQ( theUnit ) ) )
	{
		/* wait for a write request to be queued */
		waitResult = WaitSignal(VDCSIGWRITE(theUnit) |
								VDCSIGABORTREAD(theUnit) |
								VDCSIGQUIT(theUnit));
		if ((waitResult & SIGF_ABORT) || (waitResult == ABORTED) ||
			(waitResult & VDCSIGQUIT(theUnit)))
		{
			/* ack! this should never happen! */
			DEBUG(("MPVRead: Got Aborted.\n"));
			*len = 0L;
			return( ABORTED );
		}
		if( waitResult & VDCSIGABORTREAD( theUnit ) )
		{
			/* the current read IO req was aborted */
			*len = 0L;
			return( MPVFLUSHWRITE );	/* a read req is a decoder write */
		}
	}
	/* get next IOReq from the queue */
	VDCCURWRITE( theUnit ) = (IOReq *) RemHead( &VDCWRITEQ( theUnit ) );
	PRNT(("Rem: Write Q=%d\n", GetNodeCount(&VDCWRITEQ(theUnit))));

	/* set the buf and len return values */
	theWriteBuf = &(VDCCURWRITE( theUnit )->io_Info.ioi_Send);
	*buf = (uint8 *) theWriteBuf->iob_Buffer;
	*len = theWriteBuf->iob_Len;

	optionPtr = (FMVIOReqOptions *) VDCCURWRITE(theUnit)->io_Info.ioi_CmdOptions;
	if (optionPtr)
	{
		/* check for valid pts */
		if (optionPtr->FMVOpt_Flags & FMVValidPTS)	{
			*pts = optionPtr->FMVOpt_PTS;
			*userData = optionPtr->FMVOpt_UserData;
			return( MPVPTSVALID );

		} else if (optionPtr->FMVOpt_Flags & FMV_FLUSH_FLAG) {
			/* Flush flag is set. Flush the driver. */
			PRNT(("--->MPVREAD: GOT FMV_FLUSH_FLAG.\n"));
			MPVCompleteRead(theUnit);
			return MPVFLUSHDRIVER;
		} else if (optionPtr->FMVOpt_Flags & FMV_END_OF_STREAM_FLAG) {
			/* Driver has received a complete picture. Don't wait for
			 * another picture. Complete the IOs.
			 */
			/*******************************************************/
			/* 911. WARNNING: This is a hack! NOT TESTED COMPLETELY.
			/* ONLY TESTED FOR STILL IMAGES. If the last picture is an
			/* I frame...!! What happens to the PTS.
            /********************************************************/
			PRNT(("--->MPVREAD: GOT FMV_END_OF_STREAM_FLAG.\n"));
			MPVCompleteRead(theUnit);
			MPVCompleteWrite(theUnit, 0, 0, 0);
			return MPVFLUSHDRIVER;
		}
	}

	UDEBUG(("buf: %08lx, len: %08lx\n",*buf,*len));

	return 0L;
}

int32 MPVCompleteRead(tVideoDeviceContext *theUnit)
{
	IOReq *current;

	UDEBUG(("MPVCompleteRead\n"));

	/* is there a current write request? */
	if( (current = VDCCURWRITE( theUnit )) != (IOReq *) 0L )
	{
		/* was it aborted? */
		if( VDCFLAGS( theUnit ) & VDCFLAGABORTWRITE )
		{
			current->io_Actual = 0L;
			VDCFLAGS( theUnit ) ^= VDCFLAGABORTWRITE;
		}
		else
		{
			current->io_Actual = current->io_Info.ioi_Send.iob_Len;
		}
		/* we're done with this IOReq */
		VDCCURWRITE( theUnit ) = (IOReq *) 0L;
		CompleteIO( current );
	}
	return( 0L );
}

static void CompleteReadIO( IOReq *ior )
{
	IOBuf *theReadBuf = &(ior->io_Info.ioi_Recv);

	/* complete IOReq */
	ior->io_Actual = theReadBuf->iob_Len;
	CompleteIO( ior );
}

static int32 GetNextRead( tVideoDeviceContext *theUnit )
{
	int32 waitResult;

	/* is the queue empty? */
	while( IsEmptyList( &VDCREADQ( theUnit ) ) )
	{
		/* queue empty, wait for next read request to be queued */
		waitResult = WaitSignal(VDCSIGREAD(theUnit) |
								VDCSIGABORTWRITE(theUnit) |
								VDCSIGQUIT(theUnit));
		if ((waitResult & SIGF_ABORT) || waitResult == ABORTED ||
			(waitResult & VDCSIGQUIT(theUnit)))
		{
			/* ack! this should never happen! */
			DEBUG(("GetNextRead: Got Aborted.\n"));
			return( ABORTED );
		}
		if( waitResult & VDCSIGABORTWRITE( theUnit ) )
		{
			/* the current write IO req was aborted */
			return( MPVFLUSHREAD );		/* our write req is a decoder read */
		}
	}
	/* get next IOReq from the queue */
	VDCCURREAD( theUnit ) = (IOReq *) RemHead( &VDCREADQ( theUnit ) );
	PRNT(("Rem: Read Q=%d\n", GetNodeCount(&VDCREADQ(theUnit))));
	VDCREADOFFSET( theUnit ) = 0L;

	return( 0L );
}

/*	note that a 'write' from the decoder goes out to a 'read' IOReq */

int32 MPVNextWriteBuffer(tVideoDeviceContext *theUnit, uint32 **buf,
						 uint32 *len)
{
	int32 result;
	IOBuf *theReadBuf;

	UDEBUG(("MPVNextWriteBuffer ..."));

	while( 1 )
	{
		/* no current read request? */
		if( VDCCURREAD( theUnit ) == (IOReq *) 0L )
		{
			/* get the next read buffer */
			result = GetNextRead( theUnit );
			if( result )
				return( result );
		}
		/* support obsolete CMD_READ for now */
		if( VDCCURREAD( theUnit )->io_Info.ioi_Command == CMD_READ )
		{
			theReadBuf = &(VDCCURREAD( theUnit )->io_Info.ioi_Recv);

			*buf = (uint32 *) theReadBuf->iob_Buffer;
			*len = theReadBuf->iob_Len;
			break;
		}
		else
		{
			IOReq *ior;
			Item bmItem;
			Bitmap *bm;

			/* rather than a buffer, we should have been passed a Bitmap Item */
			ior = VDCCURREAD( theUnit );
			if( (uint32) ior->io_Info.ioi_Send.iob_Buffer & 0x3L )
				goto failRetry;
			if( ior->io_Info.ioi_Send.iob_Len != sizeof( Item ) )
				goto failRetry;

			bmItem = *((Item *) (ior->io_Info.ioi_Send.iob_Buffer));

			if( (bm = CheckItem( bmItem,NST_GRAPHICS,GFX_BITMAP_NODE )) == NULL)
				goto failRetry;
			if( (theUnit->depth == 16) &&
				( (bm->bm_Type != BMTYPE_16) &&
				  (bm->bm_Type != BMTYPE_16_ZBUFFER) ) )
				goto failRetry;
			if( (theUnit->depth == 32) && (bm->bm_Type != BMTYPE_32) )
				goto failRetry;
			if( (theUnit->xSize != bm->bm_Width) ||
				(theUnit->ySize != bm->bm_Height) )
				goto failRetry;
			*buf = bm->bm_Buffer;
			*len = bm->bm_BufferSize;
			break;
		failRetry:
			PERR(("Bad Bitmap Item in read IOReq\n"));
			ior->io_Error = BADIOARG;
			CompleteIO( ior );
		}
	}

	UDEBUG(("buf: %08lx len: %08lx\n",*buf,*len));

	return( 0L );
}

int32 MPVCompleteWrite(tVideoDeviceContext* theUnit, int32 ptsValid,
					   uint32 pts, uint32 userData)
{
	IOReq *current;

	/* is there a current read request? */
	if( (current = VDCCURREAD( theUnit )) != (IOReq *) 0L )
	{
		/* was it aborted? */
		if( VDCFLAGS( theUnit ) & VDCFLAGABORTREAD )
		{
			/* force completion */
			current->io_Actual = 0L;
			VDCFLAGS( theUnit ) ^= VDCFLAGABORTREAD;
		}
		/* we're done with this IOReq */
		VDCCURREAD( theUnit ) = (IOReq *) 0L;
		if( ptsValid )
		{
			current->io_Extension[ 0 ] = pts;
			current->io_Extension[ 1 ] = 0L;
			current->io_Flags |= FMVValidPTS;
			current->io_Info.ioi_CmdOptions = userData;
		}
		else
		{
			current->io_Flags &= ~FMVValidPTS;
		}
		CompleteReadIO( current );
	}
	return( 0L );
}

static void
mpvDriverAbortIO(IOReq *ior)
{
	tVideoDeviceContext *theUnit;

	/* interrupts are disabled by the system prior to entry */
	PRNT(("%s: DrvrAbortIO\n",INTERNAL_MPEG_DEVICENAME));
	DEBUG(("Abort IO\n"));

    /* verify valid unit number */
    /* change this to support multiple devices */
	ior->io_Error = ABORTED;

	theUnit = MPEGSTATE(ior->io_Dev);

	if( ior == theUnit->currentWriteReq )
	{
		DEBUG((" current write\n"));
		VDCFLAGS( theUnit ) |= VDCFLAGABORTWRITE;
		SuperInternalSignal( theUnit->task, theUnit->signalAbortWrite );
	}
	else if( ior == theUnit->currentReadReq )
	{
		DEBUG((" current read\n"));
		VDCFLAGS( theUnit ) |= VDCFLAGABORTREAD;
		SuperInternalSignal( theUnit->task, theUnit->signalAbortRead );
	}
	else
	{
		DEBUG((" node\n"));
		RemNode( (Node *) ior );
		ior->io_Actual = 0L;
		SuperCompleteIO( ior );
	}
}

/*	DriverInit gets called once when the driver is created
 *	it returns the driver item or a negative value on error
 */
static Item
mpvDriverInit(Driver *drv)
{
Err result;

	PRNT(("%s: DriverInit\n",INTERNAL_MPEG_DRIVERNAME));

	result = M2MPEGInit();
	if (result < 0)
	    return result;

	result = AllocStripBuffer(NULL);
	if (result < 0)
	{
	    M2MPEGDelete();
            return result;
        }

	return( drv->drv.n_Item );
}

static Err
mpvDriverChangeOwner(Driver *drv, Item newOwner)
{
	TOUCH( drv );
	return M2MPEGChangeOwner( newOwner );
}

static Err
mpvDriverDelete(Driver* drv)
{
	TOUCH(drv);
	FreeStripBuffer(NULL);
	return M2MPEGDelete();
}

static Item
mpvDriverClose(Driver* drv, Task* task)
{
	TOUCH(task);
	return drv->drv.n_Item;
}

Item
mpvCreateDevice(void)
{
	Item drvrItem;

	/*	create the driver */
	drvrItem = CreateItemVA(MKNODEID(KERNELNODE,DRIVERNODE),
		TAG_ITEM_NAME,			INTERNAL_MPEG_DRIVERNAME,
		TAG_ITEM_PRI,			INTERNAL_MPEG_DRIVERPRI,
		CREATEDRIVER_TAG_CMDTABLE,	CmdTable,
		CREATEDRIVER_TAG_NUMCMDS,	(sizeof(CmdTable)/sizeof(CmdTable[0])),
		CREATEDRIVER_TAG_CREATEDRV,	mpvDriverInit,
		CREATEDRIVER_TAG_CHOWN_DRV,	mpvDriverChangeOwner,
		CREATEDRIVER_TAG_CREATEDEV,	mpvDeviceInit,
		CREATEDRIVER_TAG_DELETEDEV,	mpvDeviceDelete,
		CREATEDRIVER_TAG_OPENDEV,	mpvDeviceOpen,
		CREATEDRIVER_TAG_CLOSEDEV,	mpvDeviceClose,
		CREATEDRIVER_TAG_ABORTIO,	mpvDriverAbortIO,
		CREATEDRIVER_TAG_DEVICEDATASIZE,sizeof(tVideoDeviceContext),
		CREATEDRIVER_TAG_CLOSEDRV,	mpvDriverClose,
		CREATEDRIVER_TAG_DELETEDRV,	mpvDriverDelete,
		CREATEDRIVER_TAG_MODULE,	FindCurrentModule(),
		TAG_END);

	PRNT(("%s: Creating driver returns drvrItem=%d\n",
		  INTERNAL_MPEG_DRIVERNAME,drvrItem));

	return drvrItem;
}

/*	Each DevInit gets called once when the device is created
 *	and returns the device item or a negative value on error
 */
static Item
mpvDeviceInit( Device *dev )
{
	PRNT(("%s: DevInit\n",INTERNAL_MPEG_DEVICENAME));
	if (!dev->dev_DriverData)
	{
		PERR(("mpvDeviceInit: What happened to my dev_DriverData.\n"));
		return(MakeKErr(ER_SEVERE,ER_C_STND,ER_DeviceError));
	}
	if (mpNewThread(MPEGSTATE(dev)))
		return(MakeKErr(ER_SEVERE,ER_C_STND,ER_DeviceError));

	return( dev->dev.n_Item );
}

/*    Each DevDelete gets called once when the device is deleted */
static Err
mpvDeviceDelete( Device *dev )
{
	PRNT(("%s: DevDelete\n",INTERNAL_MPEG_DEVICENAME));
	return FreeDeviceResources(dev);
}

/*	each DevOpen gets called once each time the device is opened
 *	and returns the device item or a negative value on error
 */
static Item
mpvDeviceOpen(Device *dev, Task* task)
{
	PRNT(("%s: DevOpen %ld\n",INTERNAL_MPEG_DEVICENAME,dev->dev.n_OpenCount));
	TOUCH(task);
	return( dev->dev.n_Item );
}

/* each DevClose gets called once each time the device is closed */
static Err
mpvDeviceClose(Device *dev, Task* task)
{
	TOUCH(task);
	PRNT(("%s: DevClose %ld\n",INTERNAL_MPEG_DEVICENAME,dev->dev.n_OpenCount));
	return dev->dev.n_Item;
}

static int32
CheckOptions( IOReq *ior )
{
	FMVIOReqOptions *optionsPtr;

	optionsPtr = (FMVIOReqOptions *) ior->io_Info.ioi_CmdOptions;

	/* is there a FMVIOReqOptions struct? */
	if( !optionsPtr )
		return( 0L );

	/* verify alignment */
	if( (uint32) optionsPtr & 0x3L )
	{
		PERR(("CmdOptions not long aligned\n"));
		return( BADIOARG );
	}
	/* verify addresses */
	if( !IsMemReadable( optionsPtr, sizeof(FMVIOReqOptions) ) )
	{
		PERR(("CmdOptions doesn't point to FMVIOReqOptions\n"));
		return( BADIOARG );
	}
	/* verify reserved fields */
	if( optionsPtr->Reserved2 | optionsPtr->Reserved3 )
	{
		PERR(("FMVIOReqOptions reserved fields non-zero\n"));
		return( BADIOARG );
	}
	return( 0L );
}

static int32
mpvCmdWrite( IOReq *ior )
{
	PRNT(("%s: %s\n",INTERNAL_MPEG_DEVICENAME, INTERNAL_MPEG_COMMAND2NAME));

	if( CheckOptions( ior ) )
	{
		ior->io_Error = BADIOARG;
		return( 1L );				/* synchronous, don't call completeio */
	}
	ior->io_Flags &= ~IO_QUICK;		/* asynchronous, clear IO_QUICK */
	QueueReq( ior );
	StartWrite(MPEGSTATE(ior->io_Dev));

	return( 0L );
}

static int32
mpvCmdRead( IOReq *ior )
{
	PRNT(("%s: %s\n",INTERNAL_MPEG_DEVICENAME,INTERNAL_MPEG_COMMAND3NAME));

	if( CheckOptions( ior ) )
	{
		ior->io_Error = BADIOARG;
		return( 1L );				/* synchronous, don't call completeio */
	}
	ior->io_Flags &= ~IO_QUICK;		/* asynchronous, clear IO_QUICK */
	QueueReq( ior );
	StartRead(MPEGSTATE(ior->io_Dev));

	return( 0L );
}

/*	driver specific commands begin here */
static int32
mpvCmdControl( IOReq *ior )
{
	tVideoDeviceContext savedContext, *theUnit;
	CODECDeviceStatus *cmdList;
	TagArgP commands;
	int     i;

	PRNT(("%s: %s\n",INTERNAL_MPEG_DEVICENAME, INTERNAL_MPEG_COMMAND1NAME));

	theUnit = MPEGSTATE(ior->io_Dev);

	cmdList = (CODECDeviceStatus *) ior->io_Info.ioi_Send.iob_Buffer;

	/* null pointer? */
	if( !cmdList )
	{
		ior->io_Error = BADIOARG;
		goto abort;
	}
	/* verify alignment */
	if( (uint32) cmdList & 0x3L )
	{
		PERR(("CmdControl buffer not long aligned\n"));
		ior->io_Error = BADIOARG;
		goto abort;
	}
	/* verify addresses */
	if( !IsMemReadable( cmdList, sizeof( CODECDeviceStatus ) ) )
	{
		PERR(("CmdControl buffer doesn't point to CODECDeviceStatus\n"));
		ior->io_Error = BADIOARG;
		goto abort;
	}
	commands = cmdList->codec_TagArg;

	/* save our context in case one of the commands is bad */
	savedContext = *theUnit;

	/* loop through commands */
	for( i = 0; i < kMAX_CODEC_ARGS; i++ )
	{
		if( commands[i].ta_Tag == TAG_END )
			break;

		switch ( commands[i].ta_Tag )
		{
			case VID_CODEC_TAG_HSIZE:
				savedContext.xSize = (uint32) commands[i].ta_Arg;
				break;
			case VID_CODEC_TAG_VSIZE:
				savedContext.ySize = (uint32) commands[i].ta_Arg;
				break;
			case VID_CODEC_TAG_DEPTH:
				savedContext.depth = (int32) commands[i].ta_Arg;
				if( savedContext.depth < 0 )
				{
					PERR(("Obsolete depth specification (%ld)\n",
							savedContext.depth));
					PERR(("    use VID_CODEC_TAG_M2MODE\n"));
					savedContext.depth = -savedContext.depth;
					savedContext.operaMode = 0;
				}
				break;
			case VID_CODEC_TAG_M2MODE:
				savedContext.operaMode = 0;
				break;
			case VID_CODEC_TAG_YUVMODE:
				savedContext.outputRGB = 0;
				break;
			case VID_CODEC_TAG_RGBMODE:
				savedContext.outputRGB = 1;
				break;
			case VID_CODEC_TAG_DITHER:
				PERR(("WARNING: VID_CODEC_TAG_DITHER ignored\n"));
				break;
			case VID_CODEC_TAG_STANDARD:
				savedContext.resampleMode = (unsigned) commands[i].ta_Arg;
				break;
			case VID_CODEC_TAG_PLAY:
				savedContext.playMode = (unsigned) MPEGMODEPLAY;
				break;
			case VID_CODEC_TAG_SKIPFRAMES:
				savedContext.skipCount = (int32) commands[i].ta_Arg;
				break;
			case VID_CODEC_TAG_KEYFRAMES:
				savedContext.playMode = (unsigned) MPEGMODEIFRAMESEARCH;
				break;
			default:
				PERR(("CmdControl: unrecognized command %ld\n",
					  commands[i].ta_Tag));
				ior->io_Error = BADIOARG;
				goto abort;
		}
	}
	/* finished reading through command list, validate args */
	if( M2MPGValidateParameters( &savedContext ) < 0L )
	{
		ior->io_Error = BADIOARG;
		goto abort;
	}
	/* requested settings are okay, copy them in */
	*theUnit = savedContext;
abort:
	return( 1L );					/* synchronous, don't call completeio */
}

static int32
mpvOldCmdRead( IOReq *ior )
{
#if 0
	uint32 *dst = (uint32 *)ior->io_Info.ioi_Recv.iob_Buffer;
	int32 len = ior->io_Info.ioi_Recv.iob_Len;

	PRNT(("CmdRead: dst = 0x%08lx len = %ld\n",dst,len));
#endif
	return mpvCmdRead(ior);
}

/* CmdStatus is boilerplate */
static int32
mpvCmdStatus( IOReq *ior )
{
	DeviceStatus *dst = (DeviceStatus *) ior->io_Info.ioi_Recv.iob_Buffer;
	DeviceStatus mystat;
	int32 len = ior->io_Info.ioi_Recv.iob_Len;

	PRNT(("%s: CmdStatus dst=%08lx len=%d\n",INTERNAL_MPEG_DEVICENAME,dst,len));
	if( len < 8 )
		goto abort;

	memset( &mystat,0,sizeof( DeviceStatus ) );

	/*	check driver.h to see if there are more appropriate
		values for your driver */
	mystat.ds_DriverIdentity = INTERNAL_MPEG_DRIVERIDENTITY;
	mystat.ds_MaximumStatusSize = INTERNAL_MPEG_MAXSTATUSSIZE;
	mystat.ds_DeviceFlagWord = INTERNAL_MPEG_DEVICEFLAGWORD;

	if( len > sizeof( DeviceStatus ) )
		len = sizeof( DeviceStatus );
	memcpy( dst,&mystat,len );
	ior->io_Actual = len;

	return( 1 );

abort:
	ior->io_Error = BADIOARG;
	return( 1 );
}

static int32
FreeDeviceResources(Device* dev)
{
	Err err = 0;

	tVideoDeviceContext* theUnit = MPEGSTATE(dev);
	if (!theUnit)
		return 0;

	if (theUnit->refPic[0])
		FreeMem(theUnit->refPic[0], theUnit->refPicSize);
	if (theUnit->refPic[1])
		FreeMem(theUnit->refPic[1], theUnit->refPicSize);
	theUnit->refPic[0] = NULL;
	theUnit->refPic[1] = NULL;
	theUnit->refPicSize = 0;

	/* Check if the driver's thread is till around. If yes kill it. */
	if (CheckItem(theUnit->taskItem, KERNELNODE, TASKNODE))
		SuperInternalSignal(theUnit->task, theUnit->signalQuit);

	return err;
}
