/* @(#) videoDeviceContext.h 96/04/08 1.7 */
/*	file: videoDeviceContext.h
	tVideoDeviceContext definition
	4/12/94 George Mitsuoka
	The 3DO Company Copyright (C) 1995 */

#ifndef VIDEO_DEVICECONTEXT_HEADER
#define VIDEO_DEVICECONTEXT_HEADER

#include <kernel/types.h>
#include <kernel/io.h>
#include <kernel/task.h>

typedef
    struct                          /* unit structure */
    {
		struct Task *task;			/* task to signal */
		Item taskItem;				/* task's item number */
        List writeQueue;            /* queue of write IOReqs */
        List readQueue;             /* queue of read IOReqs */
        IOReq *currentWriteReq;     /* current write request */
        IOReq *currentReadReq;      /* current read request */
		int32 readReqOffset;		/* offset into current read req buffer */
		int32 signalWriteAvail;		/* new write IOReq available */
		int32 signalReadAvail;		/* new read IOReq available */
		int32 signalAbortWrite;		/* abort the current write req */
		int32 signalAbortRead;		/* abort the current read req */
		int32 signalQuit;           /* device closed, exit thread */
		uint32 flags;				/* device flags */
		uint32 branchState;         /* unit's branch state */

		/* M2 MPEG video specific stuff */
		int32 signalStripBufferError;	/* strip buffer error */
		int32 signalEverythingDone;		/* output formatter, parser done */
		int32 signalOutputFormatter;	/* output formatter done */
		int32 signalOutputDMA;			/* output DMA completed */
		int32 signalBitstreamError;		/* parser error */
		int32 signalEndOfPicture;		/* parser completed */
		int32 signalVideoBitstreamDMA;	/* bitstream buffer exhausted */
		int32 signalTimeout;			/* hardware timeout signal */
		int32 signalMask;				/* bitwise or of interrupt signals */
		Item timerIOReqItem;			/* for timeout handling */

		uint32 *refPic[ 2 ];			/* pointers to reference pictures */
		uint32 refPicSize;				/* size in bytes */
		uint32 refPicFlags[ 2 ];		/* state of reference pictures */
		uint32 *destBuffer;				/* output buffer provided by app */
		uint32 destBufferSize;			/* size of the output buffer */

		/* decode parameters */
		unsigned	unused0:8,			/* align fields to nibble boundaries */
					sbSizeCode:4,		/* strip buffer size code */
					playMode:4,			/* play or I frame search */
					operaMode:4,		/* output in opera format */
					resampleMode:4,		/* turn on resampling if available */
					outputRGB:4,		/* output in RGB */
					nextRef:4;			/* next destination reference picture */
		int32 depth;					/* output pixel depth */
		int32 pixelSize;				/* calculated from depth */
		int32 skipCount;				/* for skipping B frames */
		uint32 xSize, ySize;			/* image size */
		uint32 hStart,vStart;			/* crop rectangle upper left */
		uint32 hStop,vStop;				/* lower right, all macroblock units */
		uint32 dither[ 2 ];				/* dither matrix upper, lower */
		unsigned	unused1:20,			/* align field to nibble boundaries */
					DSB:4,				/* DSB bit */
					alphaFill:8;		/* alpha fill value */
    }
    tVideoDeviceContext;

typedef struct {
	uint32 *address;		/* aligned stripBuffer address */
	uint32 size;			/* size of aligned stripBuffer */
} StripBuffer;

extern StripBuffer gStripBuffer;

/* Units branching states */
enum {
	NO_BRANCH = 0,           /* Playing normally. No branching yet. */
	JUST_BRANCHED,           /* Just recieved the branch notification */
	FIRST_REF_AFTER_BRANCH,  /* Recieved one I frame after branch notification.*/
};

#define VDCTASK( p )			((p)->task)
#define VDCTASKITEM( p )		((p)->taskItem)
#define VDCWRITEQ( p )			((p)->writeQueue)
#define VDCREADQ( p )			((p)->readQueue)
#define VDCCURWRITE( p )		((p)->currentWriteReq)
#define VDCCURREAD( p )			((p)->currentReadReq)
#define VDCREADOFFSET( p )		((p)->readReqOffset)
#define VDCSIGQUIT( p )	    	((p)->signalQuit)
#define VDCSIGWRITE( p )		((p)->signalWriteAvail)
#define VDCSIGREAD( p )			((p)->signalReadAvail)
#define VDCSIGABORTWRITE( p )	((p)->signalAbortWrite)
#define VDCSIGABORTREAD( p )	((p)->signalAbortRead)
#define VDCDECODEBUF( p )		((p)->decodeBuf)
#define VDCBUFSIZE( p )			((p)->bufSize)
#define VDCWRITESIZE( p )		((p)->lastWriteSize)
#define VDCFLAGS( p )			((p)->flags)

#define VDCFLAGABORTWRITE		(1 << 0)
#define VDCFLAGABORTREAD		(1 << 1)

#define REFPICEMPTY				0		/* reference picture is empty */
#define REFPICDECODED			1		/* picture has been decoded */

#define REFBYTESPERPIXEL		3 / 2

#endif

