/* @(#) autodocs.c 96/09/17 1.7 */

/**
|||	AUTODOC -class CLT -name CLT_CreatePipCommandList
|||	Creates a command list snippet to load a PIP.
|||
|||	  Synopsis
|||
|||	    Err CLT_CreatePipCommandList(CltPipControlBlock* txPipCB);
|||
|||	  Description
|||
|||	    Creates a command list snippet to load the specified block of memory
|||	    as a PIP, or Pen Index Palette, for indexed textures to use.  The
|||	    fields of the CltPipControlBlock should be filled in as follows:
|||
|||	    pipData
|||	        Pointer to the PIP data in RAM
|||
|||	    pipIndex
|||	        Starting index of the data to load
|||
|||	    pipNumEntries
|||	        Number of entries in the data to be loaded
|||
|||	  Arguments
|||
|||	    txPipCB
|||	        Pointer to a CltPipControlBlock structure.  On entry, txPipCB
|||	        should be filled in as described above.  On exit, the field
|||	        pipCommandList will be filled in with a command list snippet that
|||	        can be copied into a command list buffer when the PIP load needs
|||	        to occur.  See CLT_CopySnippetData() for more details.
|||
|||	  Return Value
|||
|||	    Returns 0 for success or a negative error code for failure.
|||
|||	  Implementation
|||
|||	    Library routine in libclt.a
|||
|||	  See Also
|||
|||	    CLT_CopySnippetData(), CLT_ComputePipLoadCmdListSize()
|||
**/

/**
|||	AUTODOC -class CLT -name CLT_ComputePipLoadCmdListSize
|||	Computes the size of a PIP load command list.
|||
|||	  Synopsis
|||
|||	    int32 CLT_ComputePipLoadCmdListSize(CltPipControlBLock* txPipCB);
|||
|||	  Description
|||
|||	    Computes the size of a PIP load command list snippet, without actually
|||	    creating the snippet.  This routine is most useful when users want to
|||	    have a PIP load command list be created directly into the command list
|||	    buffer, and need to know how much space will be needed ahead of time
|||	    to prevent overflowing the command list buffer.
|||
|||	  Arguments
|||
|||	    txPipCB
|||	        Pointer to a CltPipControlBlock structure.  See
|||	        CLT_CreatePipCommandList() for information on how to correctly
|||	        fill in this structure.
|||
|||	  Return Value
|||
|||	    Returns the necessary size, in words, if successful, or a negative
|||	    error code if the call fails.
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    CLT_CreatePipCommandList(), GS_Reserve()
|||
**/

/**
|||	AUTODOC -class CLT -name CLT_CreateTxLoadCommandList
|||	Creates a command list snippet to load a texture.
|||
|||	  Synopsis
|||
|||	    Err CLT_CreateTxLoadCommandList(CltTxLoadControlBlock* txLoadCB);
|||
|||	  Description
|||
|||	    Creates a command list snippet to load the specified texture into
|||	    the Triangle Engine's TRAM, or Texture RAM.  For uncompressed textures
|||	    which do not require runtime carving, the data for the one or more
|||	    levels of detail is transferred as one large block of contiguous RAM,
|||	    for optimal performance.  For indexed textures, the PIP must be loaded
|||	    separately, by calling CLT_CreatePipCommandList().  The fields of
|||	    the txLoadCB field should be filled in as follows:
|||
|||	    textureBlock
|||	        Pointer to a CltTxData block.  This block contains data about the
|||	        actual texels, such as num LODs, texture format, etc.
|||
|||	    numLOD
|||	        Number of LODs (Levels of Detail) in texture
|||
|||	    XWrap
|||	        Wrap mode in X-direction (0=Clamp, 1=Tile)
|||
|||	    YWrap
|||	        Wrap mode in Y-direction (0=Clamp, 1=Tile)
|||
|||	    XSize
|||	        Width of area to load.  Normally equal to the minX field of the
|||	        textureBlock structure
|||
|||	    YSize
|||	        Height of area to load.  Normally equal to the minY field of the
|||	        textureBlock structure
|||
|||	    XOffset
|||	        Left edge of sub-texture to load.  For non-carved
|||	        textures, this will normally be 0.
|||
|||	    YOffset
|||	        Top edge of sub-texture to load.  For non-carved textures, this
|||	        will normally be 0.
|||
|||	    tramOffset
|||	        Offset into the 16k TRAM where the texture should be placed.
|||	        For uncompressed, uncarved textures, this value must be
|||	        32-bit aligned.
|||
|||	    tramSize
|||	        Size texture will require in TRAM.
|||
|||	  Arguments
|||
|||	    txLoadCB
|||	        Pointer to a CltTxLoadControlBlock structure.  On entry, txLoadCB
|||	        must be filled in as described above.  On exit, the field
|||	        lcbCommandList will contain a command list snippet to load and
|||	        use the specified texture.  Additionally, the useCommandList can
|||	        be used to just re-use the texture, assuming that it is already
|||	        loaded.  The useCommandList is most useful when multiple textures
|||	        reside in the TRAM at once, and the user data needs to alternate
|||	         between these different textures.
|||
|||	  Return Value
|||
|||	    Returns 0 for success or a negative error code for failure.
|||
|||	  Implementation
|||
|||	    Library routine in libclt.a
|||
|||	  See Also
|||
|||	    CLT_CopySnippetData(), CLT_ComputeTxLoadCmdListSize(),
|||	    CLT_CreatePipCommandList()
|||
**/

/**
|||	AUTODOC -class CLT -name CLT_ComputeTxLoadCmdListSize
|||	Computes the size of a texture load command list.
|||
|||	  Synopsis
|||
|||	    int32 CLT_ComputeTxLoadCmdListSize(CltTxLoadControlBlock* txLoadCB);
|||
|||	  Description
|||
|||	    Computes the size of a texture load command list snippet, without
|||	    actually creating the snippet.  This routine is most useful when users
|||	    want to have a texture load command list be created directly into the
|||	    command list buffer, and need to know how much space will be needed
|||	    ahead of time to prevent overflowing the command list buffer.
|||
|||	  Arguments
|||
|||	    txLoadCB
|||	        Pointer to a CltTxLoadControlBlock structure.  See
|||	        CLT_CreateTxLoadCommandList() for information on how to correctly
|||	        fill in this structure.
|||
|||	  Return Value
|||
|||	    Returns the necessary size, in words, if successful, or a negative
|||	    error code if the call fails.
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    CLT_CreateTxLoadCommandList(), GS_Reserve()
**/

/**
|||	AUTODOC -class CLT -name CLT_InitSnippet
|||	Initializes a CltSnippet structure.
|||
|||	  Synopsis
|||
|||	    void CLT_InitSnippet(CltSnippet* s);
|||
|||	  Description
|||
|||	    Initializes a CltSnippet structure.  The structure must be created
|||	    before calling this routine, either by calling AllocMem() or by just
|||	    declaring a variable of type CltSnippet.  Once initialized, a
|||	    CltSnippet can be used in many other CLT calls.
|||
|||	    Note that initializing a CltSnippet does not allocate any memory for
|||	    the snippet's data space.  To allocate that memory, call
|||	    CLT_AllocSnippet() after calling CLT_InitSnippet().
|||
|||	  Arguments
|||
|||	    s
|||	        Pointer to a valid CltSnippet
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    CLT_AllocSnippet(), CLT_CopySnippet(), CLT_CopySnippetData(),
|||	    CLT_FreeSnippet(), AllocMem()
**/

/**
|||	AUTODOC -class CLT -name CLT_AllocSnippet
|||	Allocates space for the data for a CltSnippet.
|||
|||	  Synopsis
|||
|||	    Err CLT_AllocSnippet(CltSnippet* s, uint32 nWords);
|||
|||	  Description
|||
|||	    Allocate space for the specified number of words of command list
|||	    data for a given CltSnippet.  The CltSnippet should already have been
|||	    initialized by calling CLT_InitSnippet().
|||
|||	  Arguments
|||
|||	    s
|||	        Pointer to a valid, initialized CltSnippet
|||
|||	    nWords
|||	        Number of words of memory to be allocated for the CltSnippet
|||
|||	  Return Value
|||
|||	    Returns 0 for success or a negative error code for failure.
|||
|||	  Caveats
|||
|||	    If the CltSnippet already has memory allocated for its data, this
|||	    memory will be left dangling in memory.  Use CLT_FreeSnippet() to
|||	    free the memory first.
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    CLT_FreeSnippet(), CLT_InitSnippet()
**/

/**
|||	AUTODOC -class CLT -name CLT_FreeSnippet
|||	Frees memory allocated for a CltSnippet's data.
|||
|||	  Synopsis
|||
|||	    void CLT_FreeSnippet(CltSnippet* s);
|||
|||	  Description
|||
|||	    Frees the memory allocated to a CltSnippet's data area.  The memory
|||	    is only freed if it was allocated for the snippet.  That is, there
|||	    are times when several snippets may all share the same data area, such
|||	    as when one is only supposed to represent a smaller portion of another.
|||	    In that case, calling CLT_FreeSnippet() on the smaller sub-snippet will
|||	    not cause the smaller portion of memory to be freed from the larger
|||	    memory block.
|||
|||	  Arguments
|||
|||	    s
|||	        Pointer to a valid, initialized CltSnippet
|||
|||	  Caveats
|||
|||	    In the case where a snippet is using memory from a larger snippet,
|||	    and CLT_FreeSnippet() is called to free the memory used by the larger
|||	    snippet, there is no way to mark the smaller snippet that is sharing
|||	    the memory as freed.  This could result in a CltSnippet pointing to
|||	    garbage memory.
|||
|||	  Implementation
|||
|||	    Library call implemented in libclt.a
|||
|||	  See Also
|||
|||	    CLT_AllocSnippet()
**/

/**
|||	AUTODOC -class CLT -name CLT_CopySnippetData
|||	Copies the data portion of a command list snippet to a specified
|||	location.
|||
|||	  Synopsis
|||
|||	    void CLT_CopySnippetData(uint32** dest, CltSnippet* src)
|||
|||	  Description
|||
|||	    Copies data from a CltSnippet to another memory location, updating
|||	    the dest ptr as it goes.  Most often, this routine is used to copy
|||	    command list snippets into a command list buffer within the GState.
|||
|||	  Arguments
|||
|||	    dest
|||	        Pointer to a pointer to a buffer of 32-bit quantities.  On
|||	        exit from this routine, *dest will be updated to point to just
|||	        beyond the copied data area, so that a subsequent call to
|||	        CLT_CopySnippetData() can be made without updating the pointer
|||	        manually.
|||
|||	    src
|||	        Pointer to a valid, initialized CltSnippet
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    GS_Reserve(), GS_SendList(), CLT_InitSnippet()
**/
/**
|||	AUTODOC -class CLT -name CLT_CopySnippet
|||	Copies the data area of a command list snippet into the data area
|||	of another snippet.
|||
|||	  Synopsis
|||
|||	    void CLT_CopySnippet(CltSnippet *dest, CltSnippet* src)
|||
|||	  Description
|||
|||	    Copies data from one CltSnippet to another CltSnippet. No checks are made for the
|||	    case where the destination CltSnippet cannot hold the source data.
|||
|||	  Arguments
|||
|||	    dest
|||	        Pointer to a valid, initialized CltSnippet which will hold the destination
|||	        data. 
|||
|||	    src
|||	        Pointer to a valid, initialized CltSnippet holding the source data.
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    CLT_CopySnippetData()
**/

/**
|||	AUTODOC -class CLT -name CLT_SetSrcToCurrentDest
|||	Sets the destination blender so that source blends will occur with
|||	the current frame buffer.
|||
|||	  Synopsis
|||
|||	    void CLT_SetSrcToCurrentDest(GState* g);
|||
|||	  Description
|||
|||	    This routine adds some commands to the current command list buffer
|||	    of a GState which set the destination blend source buffer equal to
|||	    the current destination frame buffer.  This is usually most useful
|||	    when trying to make objects appear translucent.  To achieve a
|||	    translucent effect, an object gets blended with the current frame
|||	    buffer in the destination blender of the Triangle Engine.
|||
|||	    Note that this routine does not actually enable source blending, nor
|||	    does it set up the blend operation.  It merely sets the following
|||	    CLT attributes: DBSRCBASEADDR, DBSRCXSTRIDE, DBSRCOFFSET, DBSRCCNTL.
|||
|||	  Arguments
|||
|||	    g
|||	        Pointer to a GState object
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    GS_Create(), GS_GetDestBuffer()
**/

/**
|||	AUTODOC -class CLT -name CLT_ClearFrameBuffer
|||	Clears the frame buffer and/or Z-buffer via CLT commands.
|||
|||	  Synopsis
|||
|||	    void CLT_ClearFrameBuffer(GState* gs, float red, float green,
|||	             float blue, float alpha, bool clearScreen, bool clearZ);
|||
|||	  Description
|||
|||	    Clear the frame buffer and/or Z-buffer via the Triangle Engine.
|||	    This is done by drawing two large triangles of the specified
|||	    color.  If clearing of the Z-buffer is desired, 0 is written to the
|||	    whole Z-buffer as well.  Note that because the screen clear is done
|||	    via Triangle Engine commands, the clear doesn't actually occur until
|||	    the command list is sent to the Triangle Engine.
|||
|||
|||	  Arguments
|||
|||	    gs
|||	        Pointer to a GState object
|||
|||	    red, green, blue, alpha
|||	        Floating point numbers between 0.0 and 1.0, inclusive, which
|||	        specify the intensities of the R, G, B, and alpha channels,
|||	        respectively.
|||
|||	    clearScreen
|||	        Boolean specifying whether the screen should be cleared. This
|||	        would be FALSE when the user only wants to clear the Z-buffer.
|||
|||	    clearZ
|||	        Boolean specifying whether the Z-buffer should be cleared.
|||	        Note that this value is ignored when a Z-buffer is not being
|||	        used.  Also, when a Z-buffer is being used, this routine will
|||	        NOT enable Z-buffering if clearZ is set to TRUE.  It is the
|||	        responsibility of the user's code to enable Z-buffering before
|||	        calling this routine.  This is done because normally, the
|||	        Z-buffer would just be enabled once when the application
|||	        first starts up, whereas clearing the Z-buffer would be done
|||	        once per frame.
|||
|||	  Implementation
|||
|||	    Library call in libclt.a
|||
|||	  See Also
|||
|||	    GS_Create(), GS_SendList()
**/

/**
|||	AUTODOC -class CLT -group Globals -name CltNoTextureSnippet
|||	Global variable used to disable texturing.
|||
|||	  Description
|||
|||	    The CltSnippet CltNoTextureSnippet(@) can be inserted by the user into
|||	    the current command list buffer when texturing needs to be disabled.
|||	    This CltSnippet contains the following commands:
|||	        1.  Sync the Triangle Engine
|||	        2.  Disable the TEXTUREENABLE bit of the TXTADDRCNTL register
|||	        3.  Set the TXTTABCNTL COLOROUT to PRIMCOLOR, and the ALPHAOUT
|||	            to PRIMALPHA
|||
|||	    To actually use this CltSnippet, call CLT_CopySnippetData().
|||
|||	  Implementation
|||
|||	    Global variable exported in library libclt.a
|||
|||	  See Also
|||
|||	    CltEnableTextureSnippet(@), CLT_CopySnippetData()
**/

/**
|||	AUTODOC -class CLT -group Globals -name CltEnableTextureSnippet
|||	Global variable used to enable texturing.
|||
|||	  Description
|||
|||	    The CltSnippet CltEnableTextureSnippet(@) can be inserted by the user
|||	    into the current command list buffer when texturing needs to be
|||	    enabled.  This CltSnippet contains the following commands:
|||	        1.  Enable the TEXTUREENABLE bit of the TXTADDRCNTL register.
|||
|||	    To actually use this CltSnippet, call CLT_CopySnippetData().
|||
|||	    Note that if the CltSnippet CltNoTextureSnippet(@) is used to disable
|||	    texturing, using CltEnableTextureSnippet(@) will not completely restore
|||	    state.  It will also be necessary for the user's code to set the
|||	    COLOROUT and ALPHAOUT fields of the TXTTABCNTL register, to either
|||	    output the texture's colors, or to a blend operation between the
|||	    primitive's color and the texture's color.
|||
|||	  Implementation
|||
|||	    Global variable exported in library libclt.a
|||
|||	  See Also
|||
|||	    CltNoTextureSnippet(@), CLT_CopySnippetData()
**/

/* Keep the compiler happy */
extern int foo;
