/* @(#) 95 10-09-95 1.0 m2_cdtest.c */

/* This is the attempt to make a CD test that will exercise everything
   and work using the user interface in use for the opera CD test (adapted).
   It contains nice menus and works from the control pad to do the things
   we want to.
   
   Nice updates:  Jun 05,'96 tsw:  Added better tests when trying to kill M2 CD daemon.
   We now print out the error code.
   
   Previous additions to code are:  Method to go to other CD test routine.
*/


/* Include things */
#include <kernel/driver.h>
#include <kernel/mem.h>
#include <kernel/task.h>
#include <kernel/io.h>
#include <kernel/debug.h>
#include <device/cdrom.h>
#include <misc/event.h>
#include <misc/script.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "seek.h"			/* Interface to original seek routines */
#include "su.h"				/* Has 'time_seek' among others */

#include "miniui.h"			/* It has the user interface things */

#define DBUG(x)	{ printf x ; }
							
							
#define READMINBLOCK  150		/* In original 'seek.c' */

struct	cur_hi
		{
		int	cur;
		int hi;
		};
struct lo_hi
		{
		int lo;
		int hi;
		};

typedef struct
		{
		struct cur_hi	minutes;
		struct cur_hi	seconds;
		struct cur_hi	frame;
		int				num;
		} BLOCK;
		


/* Prototypes as necessary... */
void do_kill (ustring *, Selector *);
void do_op_cl (ustring *, Selector *);
void do_rsubq (ustring *, Selector *);
void do_status (ustring *, Selector *);
void do_cdtest (ustring *, Selector *);
void do_read (ustring *, Selector *);
void do_defaults (ustring *, Selector *);
void do_gettoc (ustring *, Selector *);
void do_seek_test (ustring *, Selector *);
void do_real_seek_test (ustring *, Selector *);
void do_display (ustring *, Selector *);
char *bin_string (unsigned int bin, int nbits);
int do_read_block (void *buffer, long block, int num_sec);
void inc_dec_block (int inc_dec, BLOCK *block);
void execute_read (int the_read_type);


/* General pointers to details */
int daemon_alive = 1;			/* We assume so */
Err send_error = 0;
Err wait_error = 0;
int bytes_returned = 0;
const int zero = 0;
const int one = 1;
const int three = 3;
int read_type = 1;					/* Tells the type of read to do */

BLOCK block = { {0, 63}, {2, 59}, {0, 74}, 2};

char *text_line = "No Data Presently";

char *d_or_a[] = {"Dead", "Alive", "Unknown"};
char *read_type_text[] = {"Scan read", "CD Rom read", "Read", "Block Read"};

Item devItem;					/* Device item from open of CD Rom */

static uint8 *buf = NULL;		/* General buffer pointer (global??) */
static int len;					/* Length of buffer */

long num_blocks;				/* Number of blocks */
int block_size;


ustring main_menu[] =
{
PRINT1 (20, 28, devItem, "Main Menu - CD Rom test (I: %d)"),
MPROC (20, 40, do_kill, "Kill M2 CD Daemon"),
MIXTEXT (210, 40, daemon_alive, d_or_a),
MPROC (20, 52, do_op_cl, "Open [A] Close [B] Drawer"),
MPROC (20, 64, do_rsubq, "Read SubQ packet"),
MPROC (20, 76, do_status, "Obtain status word"),
MPROC (170, 76, do_cdtest, "Execute CDTEST prog."),
VARIABLE (20, 100, block.minutes.cur, zero, block.minutes.hi, 2, "Min: "),
VARIABLE (80, 100, block.seconds.cur, zero, block.seconds.hi, 2, "Sec: "),
VARIABLE (140, 100, block.frame.cur, zero, block.frame.hi, 2, "Frm: "),
MTEXT (200, 100, "<--Block"),
MCHOICEC (20, 112, read_type, zero, three, read_type_text),
MPROC (120, 112, do_read, "Execute read routine"),
MPROC (20, 132, do_defaults, "Get/display defaults"),
MPROC (20, 144, do_gettoc, "Obtain TOC Info"),
MPROC (20, 156, do_seek_test, "Seek test(s)"),

PRINT3 (20, 168, send_error, wait_error, bytes_returned, 
			"ESend=0x%X, EWait=0x%X, bytes=%d"),
	
PRINT1 (20, 180, text_line, "%s"),

MLAST
};


/* Information:  There are 7 dots/character in the font I'm using */
#define BUFFER_LINES	16				/* Number of lines I can display */
#define BYTES_ACROSS	8
										/* Display of buffer lines */
static char buffer_text[BUFFER_LINES][BYTES_ACROSS * 6];		
ustring display_menu[] =
{
PRINT3 (20, 20, block.minutes.cur, block.seconds.cur, block.frame.cur,
		"Block: %02d:%02d:%02d"),
PRINT1 (132, 20, block.num, "(0x%X)"),
MPROC (209, 20, do_display, "(<- ->)"),


MTEXT (20, 32, buffer_text[0]),
MTEXT (20, 44, buffer_text[1]),
MTEXT (20, 56, buffer_text[2]),
MTEXT (20, 68, buffer_text[3]),
MTEXT (20, 80, buffer_text[4]),
MTEXT (20, 92, buffer_text[5]),
MTEXT (20, 104, buffer_text[6]),
MTEXT (20, 116, buffer_text[7]),
MTEXT (20, 128, buffer_text[8]),
MTEXT (20, 140, buffer_text[9]),
MTEXT (20, 152, buffer_text[10]),
MTEXT (20, 164, buffer_text[11]),
MTEXT (20, 176, buffer_text[12]),
MTEXT (20, 188, buffer_text[13]),
MTEXT (20, 200, buffer_text[14]),
MTEXT (20, 212, buffer_text[15]),

MLAST
};




/* Menu structure for the seek tests. */
char *seek_routine_text[] =
	{
	"Logarithmic seek test",			/* 0 */
	"Repeat seek test",					/* 1 */
	"Linear seek test",					/* 2 */
	"Random seek test",					/* 3 */
	"Single seek test",					/* 4 */
	"Histogram seek test",				/* 5 */
	"Long seek test",					/* 6 */
	"Short seek test",					/* 7 */
	"Play CD test (repeatability)"		/* 8 */
	};

struct
	{
	int min;
	int sec;
	int frm;
	} from_seek, to_seek;
	
int max_seek_test = sizeof(seek_routine_text) / sizeof(seek_routine_text[0]) - 1;
int seek_test_number = 0;
int number_of_passes = 1;
int deltas = 32;
int nine999 = 9999;
char *seek_result = "No information presently";
	
ustring seek_menu[] = 
{
MCHOICECX (20, 20, seek_test_number, zero, max_seek_test, seek_routine_text,
				do_real_seek_test),
MTEXT (20, 32, "From:   :  :"),
VARIABLE (62, 32, from_seek.min, zero, block.minutes.hi, 2, NULL),
VARIABLE (83, 32, from_seek.sec, zero, block.seconds.hi, 2, NULL),
VARIABLE (104, 32, from_seek.frm, zero, block.frame.hi, 2, NULL),
MTEXT (146, 32, "To:     :  :"),
VARIABLE (188, 32, to_seek.min, zero, block.minutes.hi, 2, NULL),
VARIABLE (209, 32, to_seek.sec, zero, block.seconds.hi, 2, NULL),
VARIABLE (230, 32, to_seek.frm, zero, block.frame.hi, 2, NULL),
VARIABLE (20, 44, number_of_passes, one, nine999, 4, "Passes: "),
VARIABLE (146, 44, deltas, one, nine999, 4, "Deltas: "),
PRINT1 (20, 56, seek_result, "Results:\n%s"),

MLAST
};


char *bin_string (unsigned int bin, int nbits)
{
/* Form up a bunch of bits in a static string */
static char bits[50];

char *store = bits;				/* Point to the beginning of the array */
unsigned examine = 1 << (nbits - 1);

do	{
	*store++ = (bin & examine) ? '1' : '.';
	if (examine & 0x11111110)
		*store++ = ' ';			/* Space every 4th one (except last) */
	} while (examine >>= 1);

*store = '\0';					/* Make sure there is a terminator */
return bits;
}


void do_kill (ustring *menu, Selector *select)
{
/* Execute the kill of the CDRom Daemon (or not) and display the results */
Item daemon;
Err err;
static char result[100];		/* If we get an error */

if (daemon_alive)
	{
	/* It is 'alive' try to kill it */
	daemon = FindTask ("M2CD Daemon");
	if (daemon >= 0)
		{
		err = SendSignal (daemon, SIGF_ABORT);
		if (err < 0)
			{
			send_error = err;
			GetSysErr (result, 100, err);
			text_line = result;
			strcat (result, "\nCan't kill Daemon!!");
			return;
			}
		/* Now wait for it to die.  I don't know if this works?? */
		while (FindTask ("M2CD Daemon") == daemon)
			;
		daemon_alive = 0;		/* "He's dead, Jim." */
		}
	else
		daemon_alive = 2;		/* Zombie, or something else!! */
	}
/* Can't kill something that isn't there!! */
return;
}




void do_op_cl (ustring *menu, Selector *select)
{
/* Do an 'open' or 'close' of the CD Rom drawer */

IOReq *ior;				/* The I/O request */
Item iorItem;			/* So we can get a goodie */
IOInfo	ioInfo;			/* Where the I/O request is put */

printf ("Open/close of drawer.  Joyedge is: %08X\n", select->joyedge);

/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

if (select->joyedge & ControlB)
	{
	/* Close the drawer */
	ioInfo.ioi_Command = CDROMCMD_CLOSE_DRAWER;
	send_error = SendIO (iorItem, &ioInfo);
	
	wait_error = WaitIO (iorItem);
	text_line = "Door should be closed (die)";
	}
if (select->joyedge & ControlA)
	{
	/* Open the drawer */
	ioInfo.ioi_Command = CDROMCMD_OPEN_DRAWER;
	send_error = SendIO (iorItem, &ioInfo);
	
	wait_error = WaitIO (iorItem);
	text_line = "Door should be open";
	}
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */
return;
}



void do_rsubq (ustring *menu, Selector *select)
{
/* Read the subq data an show what it is */

IOReq *ior;					/* The I/O request */
Item iorItem;				/* So we can get a goodie */
IOInfo	ioInfo;				/* Where the I/O request is put */
SubQInfo qCode;				/* Where we get the sub q info */
CDROMCommandOptions cmdOpts;/* Where we put the options in forming */
static char results[80];	/* Result text */
int i;						/* General index */


execute_read (read_type);	/* This is the normal read command */

/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

ioInfo.ioi_Command = CDROMCMD_READ_SUBQ;
ioInfo.ioi_Recv.iob_Buffer = &qCode;
ioInfo.ioi_Recv.iob_Len = sizeof (qCode);


send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);
strcpy (results, "SubQ: ");
for (i = 0; i < sizeof(qCode); i++)
	{
	sprintf (&results[6 + i * 3], "%02X ", ((uint8 *)&qCode)[i]);
	/* 				  ^ length of leader text (see strcpy) */
	}
strcat (results, "\n      vB aC tN iX MN:SC:FR    aM:aS:aF");
text_line = results;
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */

/* May want to increment if that's how we got here */
if (select->joyedge & ControlRight)
	{
	/* Increment the address for next time */
	inc_dec_block (+1, &block);
	}
if (select->joyedge & ControlLeft)
	{
	/* Decrement the address for next time */
	inc_dec_block (-1, &block);
	}

return;
}



void do_cdtest (ustring *menu, Selector *select)
{
	/* This executes the 'other' CD-Rom test so we can say we have one test */
	int32	pStatus;
	static char line[100];


	strcpy (line, "cdtest");
	ExecuteCmdLineVA (line, &pStatus, TAG_END);
	sprintf (line, "Preformed CD stress test\nStatus = 0x%08X\n", pStatus);
	GetSysErr (strchr (line, '\0'), 100, pStatus);
	text_line = line;
}

void do_status (ustring *menu, Selector *select)
{
IOReq *ior;					/* The I/O request */
Item iorItem;				/* So we can get a goodie */
IOInfo	ioInfo;				/* Where the I/O request is put */
DeviceStatus devStat;		/* Where we get the status */
static char results[100];	/* Result text */
int i;						/* General index */


/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

ioInfo.ioi_Command = CMD_STATUS;
ioInfo.ioi_Recv.iob_Buffer = &devStat;
ioInfo.ioi_Recv.iob_Len = sizeof (devStat);

/* Get the status */

send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);
strcpy (results, "Stat: ");
strcat (results, bin_string (devStat.ds_DeviceFlagWord, 16));
strcat (results, "\nStat: ");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_DOOR)
	strcat (results, "Door ");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_DISC_IN)
	strcat (results, "Disc ");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_SPIN_UP)
	strcat (results, "Spn ");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_8X_SPEED)
	strcat (results, "8x");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_6X_SPEED)
	strcat (results, "6x");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_4X_SPEED)
	strcat (results, "4x");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_DOUBLE_SPEED)
	strcat (results, "2x");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_ERROR)
	strcat (results, " Err");
if (devStat.ds_DeviceFlagWord & CDROM_STATUS_READY)
	strcat (results, " Rdy");
/* And anything else I can think of */
block.minutes.hi = devStat.ds_DeviceBlockCount / 60 / 75;
sprintf (strrchr (results, '\0'), " Minutes=%d", block.minutes.hi);
sprintf (strrchr (results, '\0'), "\nStat: Bksz=%d,", devStat.ds_DeviceBlockSize);
sprintf (strrchr (results, '\0'), " Blocks=%d", devStat.ds_DeviceBlockCount);
text_line = results;
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */
return;
}


void do_read (ustring *menu, Selector *select)
{
/* From the menu, do the thing... */
static Selector *select2 = NULL;/* Selection goodie for data display */


execute_read (read_type);					/* Do the read!! */

/* May want to go another screen to examine the data */

/* Init the display things */
do_display (NULL, NULL);
/* Calculate numerical block number, taking into account offset. */
block.num = ((block.minutes.cur * 60) + block.seconds.cur - 2) * 75 + block.frame.cur;
DOUI (display_menu, &select2);

text_line = "Block read.";

if (select2->joynew & ControlStart)
	{
	inc_dec_block (+1, &block);
	text_line = "Block read.  Location Incremented.";
	}

}





void execute_read (int the_read_type)
{
/* This just reads the block, does nothing more */

IOReq *ior;						/* The I/O request */
Item iorItem;					/* So we can get a goodie */
IOInfo	ioInfo;					/* Where the I/O request is put */
DeviceStatus devStat;			/* Where we get the status */
CDROMCommandOptions cmdOpts;	/* Where we put the options in forming */
long addLen;					/* "used in calculations" */
int i;							/* General index */


/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

ioInfo.ioi_Command = CMD_STATUS;
ioInfo.ioi_Recv.iob_Buffer = &devStat;
ioInfo.ioi_Recv.iob_Len = sizeof (devStat);

/* Get the status */
DoIO (iorItem, &ioInfo);		/* Get status (to find out what type of CD) */

if (devStat.ds_DeviceBlockSize == CDROM_MODE1)
	{
	/* We have CD Rom data blocks */
	cmdOpts.asFields.densityCode = CDROM_DEFAULT_DENSITY;
	addLen = 0;					/* Additional block length */
	cmdOpts.asFields.blockLength = CDROM_MODE1 + addLen;
	cmdOpts.asFields.speed = 0;	/* Use defaults as set! */
	}
else
	{
	/* We have audio blocks */
	cmdOpts.asFields.densityCode = CDROM_DIGITAL_AUDIO;
	cmdOpts.asFields.blockLength = CDROM_AUDIO;
	cmdOpts.asFields.speed = 0;		/* Use defaults!! */
	}
/* Set the location to read */
/* We specify location based on mm:ss:ff */
ioInfo.ioi_Offset = block.minutes.cur << 16 |
					block.seconds.cur << 8  |
					block.frame.cur;
cmdOpts.asFields.addressFormat = CDROM_Address_Abs_MSF;
cmdOpts.asFields.pitch = CDROM_PITCH_NORMAL;

ioInfo.ioi_CmdOptions = cmdOpts.asLongword;
/* For the moment we read ONE block */
len = 1 /* Block length */ * cmdOpts.asFields.blockLength;
if (buf != NULL)
	FreeMem (buf, len);				/* Free up the memory of the block allocated??
									   previously (if it was).  This allows the
									   block to stay around until we want to re-use
									   it for some reason. */
buf = AllocMem (len, MEMTYPE_NORMAL);
if (buf == NULL)
	{
	printf ("Could not allocate buffer of length %d\n", len);
	return;
	}
ioInfo.ioi_Recv.iob_Buffer = buf;
ioInfo.ioi_Recv.iob_Len = len;

switch (the_read_type)
	{
	case 0:		/* CDROMCMD_SCAN_READ */
	ioInfo.ioi_Command = CDROMCMD_SCAN_READ;
	break;
	
	case 1:		/* CDROMCMD_READ */
	ioInfo.ioi_Command = CDROMCMD_READ;
	break;
	
	case 2:		/* CMD_READ */
	ioInfo.ioi_Command = CMD_READ;
	break;
	
	case 3:		/* CMD_BLOCKREAD */
	ioInfo.ioi_Command = CMD_BLOCKREAD;
	break;
	
	default:
	break;
	}

/* Now do the REAL read I/O Operation */

send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);
/* Data is in block.[minutes|seconds|frame].cur */

	
/* The data goes here... */
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */
return;
}



void do_defaults (ustring *menu, Selector *select)
{
/* Display & set defaults.  Possibly a sub screen */
IOReq *ior;						/* The I/O request */
Item iorItem;					/* So we can get a goodie */
IOInfo	ioInfo;					/* Where the I/O request is put */
CDROMCommandOptions cmdOpts;	/* Where we get the option stuff */
static Selector *select2 = NULL;/* Starter for selection thing */
static char results[200];		/* Result text */
int i;							/* General index */

/* Menu variables and limits w/ text */
static int the_density;			/* Density code as integer */
static struct lo_hi den_lohi = {1, 4};
static char *den_text[] = 
	{
	"Density: Default",
	"Density: Data",
	"Density: Mode 2 XA",
	"Density: Digital Audio"
	};
static int the_error_rec;		/* Error recovery code as integer */
static struct lo_hi err_rec_lohi = {1,3};
static char *err_rec_text[] =
	{
	"Default recovery",
	"CIRC Retries only",
	"Best attempt recovery"
	};
static int the_addr_fmt;		/* Address format as integer */
static struct lo_hi addr_fmt_lohi = {1,2};
static char *addr_fmt_text[] =
	{
	"Address Blocks",
	"Address Abs MSF"
	};
static int the_retry_shift;		/* Retry shift as integer */
static struct lo_hi retry_lohi = {0, 7};
static char *retry_text[] =
	{
	"Retry: Zero",		/* 2**0 - 1 */
	"Retry: 1",			/* 2**1 - 1 */
	"Retry: 3",			/* 2**2 - 1 */
	"Retry: 7",			/* 2**3 - 1 */
	"Retry: 15",		/* 2**4 - 1 */
	"Retry: 31",		/* 2**5 - 1 */
	"Retry: 63",		/* 2**6 - 1 */
	"Retry: 127"		/* 2**7 - 1 */
	};
static int the_speed;			/* Speed code as integer */
static struct lo_hi speed_lohi = {1, 5};
static char *speed_text[] =
	{
	"Single (1x) speed",
	"Double (2x) speed",
	"Quad (4x) speed",
	"Six times speed",
	"Eight times speed"
	};
static int the_pitch;			/* Pitch as integer */
static struct lo_hi pitch_lohi = {1, 3};
static char *pitch_text[] =
	{
	"Pitch slow",
	"Pitch normal",
	"Pitch fast"
	};
static int block_length;		/* Block length as integer */
static int block_code;			/* As coded integer from list */
static struct lo_hi block_lohi = {0, 6};	/* Translated later */
static char *block_text[] =
	{
	"Block size: Mode 1 (2048)",			/* Standard CD Rom block size */
	"Block size: Audio (2352)",
	"Block size: Audio subcode (2448)",
	"Block size: Mode 2/Form 1 (2048)",
	"Block size: Mode 2/Form 1\n  w/ subheader (2056)",
	"Block size: Mode 2/Form 2 (2324)",
	"Block size: Mode 2/Form 2\n  w/ subheader (2332)"
	};
static int block_nums[] =	/* These are the lengths for the above items */
	{
	CDROM_MODE1,						/* This is default (first position) */
	CDROM_AUDIO,
	CDROM_AUDIO_SUBCODE,
	CDROM_MODE2FORM1,
	CDROM_MODE2FORM1_SUBHEADER,
	CDROM_MODE2FORM2,
	CDROM_MODE2FORM2_SUBHEADER
	};


/* Menu structure for the defaults test */

static char lines_of_display [2000];	/* Has new lines in it!! */
static int track_number, trk_lo, trk_hi;

static ustring def_display[] =
{
MTEXT (20, 20, "CD Rom defaults:"),
MEXIT (160, 20, "EXIT"),
MEXIT (200, 20, "SAVE"),
#define SAVE_ITEM 2			/* Item number of the 'SAVE' */
MCHOICEC (20, 32, the_density, den_lohi.lo, den_lohi.hi, den_text),
MCHOICEC (20, 44, the_error_rec, err_rec_lohi.lo, err_rec_lohi.hi, err_rec_text),
MCHOICEC (20, 56, the_addr_fmt, addr_fmt_lohi.lo, addr_fmt_lohi.hi, addr_fmt_text),
MCHOICEC (20, 68, the_retry_shift, retry_lohi.lo, retry_lohi.hi, retry_text),
MCHOICEC (20, 80, the_speed, speed_lohi.lo, speed_lohi.hi, speed_text),
MCHOICEC (20, 92, the_pitch, pitch_lohi.lo, pitch_lohi.hi, pitch_text),
MCHOICEC (20, 104, block_code, block_lohi.lo, block_lohi.hi, block_text),

MLAST
};



/* Need a new menu item to allow setting of the values once they are obtained!! */

/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

ioInfo.ioi_Command = CDROMCMD_SETDEFAULTS;
ioInfo.ioi_Recv.iob_Buffer = &cmdOpts;
ioInfo.ioi_Recv.iob_Len = sizeof (cmdOpts);
ioInfo.ioi_CmdOptions = 0;		/* Input of options??!! (keep defaults) */

send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);

/* User interface to change might go here */
sprintf (results, "Den: %X, ErrRec: %X, AddFmt: %X, Retry: %d\n",
				cmdOpts.asFields.densityCode,
				cmdOpts.asFields.errorRecovery,
				cmdOpts.asFields.addressFormat,
				(1 << cmdOpts.asFields.retryShift) - 1);

sprintf (strrchr (results, '\0'), "Spd: %X, Ptch: %X, BlockLen: %d",
				cmdOpts.asFields.speed,
				cmdOpts.asFields.pitch,
				cmdOpts.asFields.blockLength);


/* Somewhere here we set ioInfo.ioi_CmdOptions to cmdOpts.asLongword, 
   then do the command again */
/* Set the variables from the obtained things */
the_density = cmdOpts.asFields.densityCode;
the_error_rec = cmdOpts.asFields.errorRecovery;
the_addr_fmt = cmdOpts.asFields.addressFormat;
the_retry_shift = cmdOpts.asFields.retryShift;
the_speed = cmdOpts.asFields.speed;
the_pitch = cmdOpts.asFields.pitch;
for (block_code = 0; 
     block_code < (sizeof(block_nums)/sizeof(block_nums[0])); 
	 block_code++)
	{
	if (block_nums[block_code] == cmdOpts.asFields.blockLength)
		break;
	}
if (block_code >= (sizeof(block_nums)/sizeof(block_nums[0])))
	block_code = 0;				/* Set to default if it doesn't match!! */
	
DOUI (def_display, &select2);

/* Now set back and store (when we do it!!) */
cmdOpts.asFields.densityCode = the_density;
cmdOpts.asFields.errorRecovery = the_error_rec;
cmdOpts.asFields.addressFormat = the_addr_fmt;
cmdOpts.asFields.retryShift = the_retry_shift;
cmdOpts.asFields.speed = the_speed;
cmdOpts.asFields.pitch = the_pitch;
cmdOpts.asFields.blockLength = block_nums[block_code];
ioInfo.ioi_CmdOptions = cmdOpts.asLongword;


if (select2->item == SAVE_ITEM)
	{
	send_error = SendIO (iorItem, &ioInfo);
	
	wait_error = WaitIO (iorItem);
	strcat (results, "\nSet to new values!");
	}

/* The info goes here... */
text_line = results;
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */

return;
}




void do_gettoc (ustring *menu, Selector *select)
{
/* Table of contents */
IOReq *ior;					/* The I/O request */
Item iorItem;				/* So we can get a goodie */
IOInfo	ioInfo;				/* Where the I/O request is put */
CDROM_Disc_Data discInfo;	/* Where we get the toc stuff */
static Selector *select2 = NULL;	/* Starter for selection thing */
int i;						/* General index */

/* Menu structure for the table of contents test */

static char lines_of_display [2000];	/* Has new lines in it!! */
static int track_number, trk_lo, trk_hi;

static ustring toc_display[] =
{
MTEXT (20, 20, "Table of contents:"),
MEXIT (160, 20, "EXIT"),
VARIABLE (209, 20, track_number, trk_lo, trk_hi, 2, "Track: "),
MTEXT (20, 32, lines_of_display),

MLAST
};


/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

ioInfo.ioi_Command = CDROMCMD_DISCDATA;
ioInfo.ioi_Recv.iob_Buffer = &discInfo;
ioInfo.ioi_Recv.iob_Len = sizeof (discInfo);

/* Get the TOC info */

send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);
sprintf (lines_of_display, "Disc ID: %02X, F:%d/L:%d Trk, End: %02d:%02d:%02d\n",
			discInfo.info.discID,
			discInfo.info.firstTrackNumber,
			discInfo.info.lastTrackNumber,
			discInfo.info.minutes,
			discInfo.info.seconds,
			discInfo.info.frames);

sprintf (strrchr (lines_of_display, '\0'), "Ses Info: %02X, Ses MSF: %02d:%02d:%02d\n",
			discInfo.session.valid,
			discInfo.session.minutes,
			discInfo.session.seconds,
			discInfo.session.frames);
			
/* We don't have much room for these things.  Hopefully they won't go over... */
trk_lo = discInfo.info.firstTrackNumber;
trk_hi = discInfo.info.lastTrackNumber;
track_number = trk_lo;
for (i = discInfo.info.firstTrackNumber;
	 i <= discInfo.info.lastTrackNumber;
	 i++)
	{
	sprintf (strrchr (lines_of_display, '\0'), "%2d: %02X-%02d:%02d:%02d",
			i,
			discInfo.TOC[i].addressAndControl,
			discInfo.TOC[i].minutes,
			discInfo.TOC[i].seconds,
			discInfo.TOC[i].frames);
			
	if (i - discInfo.info.firstTrackNumber & 1)
		{
		/* Odd things */
		sprintf (strrchr (lines_of_display, '\0'), "\n");
		}
	else
		{
		/* Even things */
		sprintf (strrchr (lines_of_display, '\0'), " - ");
		}
	}
/* May want to add new line and some more things */

#if 0
/* Test for me to see!! */
for (i = 0; i < 100; i++)
	{
	printf ("Track number: %2d, A&C: %02X Time: %02d:%02d:%02d\n", 
		i,
		discInfo.TOC[i].addressAndControl,
		discInfo.TOC[i].minutes,
		discInfo.TOC[i].seconds,
		discInfo.TOC[i].frames);
	}
#endif


/* Go to seperate menu to display the table of contents!! */
DOUI (toc_display, &select2);

/* Set the disk address to the beginning of the track */
block.minutes.cur = discInfo.TOC[track_number].minutes;
block.seconds.cur = discInfo.TOC[track_number].seconds;
block.frame.cur = discInfo.TOC[track_number].frames;


text_line = "Table of contents displayed";
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */
return;
}




int main (int argc, char *argv[])
{
/* Main routine, puts the main display up and does the rest... */
Selector *the_select = NULL;		/* Allocation for selection thing */
int32 err;
List *list;

printf ("M2 CD Rom test compiled on %s at %s (tsw)\n", __DATE__, __TIME__);

#ifdef ONE_DOT_ONE
	devItem = OpenDevice ("CD-ROM");	/* Was 'OpenNamedDevice' */
#else
	err = CreateDeviceStackListVA (&list, "cmds", 
				DDF_EQ, DDF_INT, 1, CDROMCMD_READ, NULL);
	if (err < 0)
		{
		DBUG (("Device stack list: Obtaining devices (0x%08X)\n", err));
		PrintfSysErr (err);
		return -1;		/* Need more error */
		}
	if (GetNodeCount (list) != 1)
		{
		DBUG (("Wrong number of nodes obtained (%d)\n", GetNodeCount (list)));
		return -1;
		}
	devItem = OpenDeviceStack ((DeviceStack *)FirstNode (list));
	/* Remove list stuff */
	DeleteDeviceStackList (list);
#endif	/* 1.1 / 1.2 code */
if (devItem < 0)
	{
	printf ("Cannot open up CD Rom Device, abort...\n");
	PrintfSysErr(devItem);
	exit (1);
	}
printf ("Dev Item is: %d\n", devItem);

do
	{
	DOUI (main_menu, &the_select);		/* Goes into the menu, and does things */
	}
	while ((the_select->joynew & ControlX) == 0);	/* Wait for stop! */
exit (0);
}




/* The routine that will display the I/O buffer for us to see */

void do_display (ustring *menu, Selector *select)
{
int i;
int j;
int db;
int line;			/* Line number */
static int start_loc;

if (select == NULL)
	{
	/* Initial fill */
	start_loc = 0;
	}

else if (select->joyedge & ControlLeft)
	{
	/* Decrement the address, re-display */
	start_loc -= BYTES_ACROSS;
	}

else if (select->joyedge & ControlRight)
	{
	/* Increment the address, re-display */
	start_loc += BYTES_ACROSS;
	}

else if (select->joyedge & (ControlA | ControlUp))
	{
	/* Decrement the address by a full display, re-display */
	start_loc -= BYTES_ACROSS * BUFFER_LINES;
	}

else if (select->joyedge & (ControlC | ControlDown))
	{
	/* Increment the address by a full display, re-display */
	start_loc += BYTES_ACROSS * BUFFER_LINES;
	}

else
	{
	/* Nothing to do !! */
	return;
	}


/* Adjust the buffer limits, if required.  */
if (start_loc < 0)
	start_loc = 0;
if (start_loc + BUFFER_LINES * BYTES_ACROSS > len)
	start_loc = len - BUFFER_LINES * BYTES_ACROSS;

/* Display the contents */
line = 0;
for (i = start_loc; 
	 i < (start_loc + BUFFER_LINES * BYTES_ACROSS); 
	 i += BYTES_ACROSS)
	{
	sprintf (buffer_text[line], "%03X: ", i);
	for (j = 0; j < BYTES_ACROSS; j++)
		{
		db = ((char *)buf)[i + j];
		sprintf (strrchr (buffer_text[line], '\0'), "%02X ", db);
		}
	sprintf (strrchr (buffer_text[line], '\0'), " |");
	for (j = 0; j < BYTES_ACROSS; j++)
		{
		db = ((char *)buf)[i + j] & 0x7f;
		if (db < ' ' || db > '~')
			db = '.';
		sprintf (strrchr (buffer_text[line], '\0'), "%c", db);
		}
	sprintf (strrchr (buffer_text[line], '\0'), "|");
	line++;
	}
}


void do_seek_test (ustring *menu, Selector *select)
{
/* Generalized seek test(s) */
IOReq *ior;							/* The I/O request */
Item iorItem;						/* So we can get a goodie */
IOInfo	ioInfo;						/* Where the I/O request is put */
DeviceStatus devStat;				/* Where we get the status */
CDROM_Disc_Data discInfo;			/* Where we get the toc stuff */
int i;								/* General index */
static Selector *select2 = NULL;	/* Selection goodie for data display */

/* Here we initialize things for the seek test.  */
/* At the mement, we just look at the status to get the size of the CD, and its block
   size.  */

/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

ioInfo.ioi_Command = CMD_STATUS;
ioInfo.ioi_Recv.iob_Buffer = &devStat;
ioInfo.ioi_Recv.iob_Len = sizeof (devStat);

/* Get the status info */

send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);

if (ior->io_Actual != sizeof (devStat))
	{
	/* Didn't obtain the status correctly */
	text_line = "Seek:  Incorrect status obtained";
	return;
	}
	
num_blocks = devStat.ds_DeviceBlockCount - READMINBLOCK;
block_size = devStat.ds_DeviceBlockSize;
if (select2 == NULL)
	{
	/* We haven't been here before!! */
	to_seek.frm = num_blocks % 75;
	to_seek.sec = num_blocks / 75 % 60;
	to_seek.min = num_blocks / (60 * 75);
	number_of_passes = 10;
	}

/* Clear out the I/O request */
DeleteIOReq (iorItem);			/* We don't want it any more */

/* Now pop up the seek test menu things, and do that... */

DOUI (seek_menu, &select2);			/* Enter the sub-menu */
text_line = "Seek:  test complete";	/* Message on return */
return;
}


/* General read routine, used for the seek group of tests, called from there */
int do_read_block (void *buffer, long block, int num_sec)
{

IOReq *ior;						/* The I/O request */
Item iorItem;					/* So we can get a goodie */
IOInfo	ioInfo;					/* Where the I/O request is put */
CDROMCommandOptions cmdOpts;	/* Where we put the options in forming */
long addLen;					/* "used in calculations" */
int32 err;						/* Error goodie */
int bytes_returned;				/* Number of bytes read from disk */

if (block > num_blocks)
	return -1;					/* Out of range, just fail!! */

/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, devItem, 0);
ior = (IOReq *)LookupItem (iorItem);

if (block_size == CDROM_MODE1)
	{
	/* We have CD Rom data blocks */
	cmdOpts.asFields.densityCode = CDROM_DEFAULT_DENSITY;
	addLen = 0;					/* Additional block length */
	cmdOpts.asFields.blockLength = CDROM_MODE1 + addLen;
	cmdOpts.asFields.speed = 0;	/* Default speed (whatever it is!)!! */
	}
else
	{
	/* We have audio blocks */
	cmdOpts.asFields.densityCode = CDROM_DIGITAL_AUDIO;
	cmdOpts.asFields.blockLength = CDROM_AUDIO;
	cmdOpts.asFields.speed = CDROM_SINGLE_SPEED;
	}
/* Set the location to read */
/* We specify location based on the argument passed to us. */
ioInfo.ioi_Offset = block + READMINBLOCK;	/* Skip first 2 secs */
cmdOpts.asFields.addressFormat = CDROM_Address_Blocks;
cmdOpts.asFields.pitch = CDROM_PITCH_NORMAL;

ioInfo.ioi_CmdOptions = cmdOpts.asLongword;

ioInfo.ioi_Recv.iob_Buffer = buffer;
ioInfo.ioi_Recv.iob_Len = num_sec * cmdOpts.asFields.blockLength;
ioInfo.ioi_Command = CMD_READ;

err = DoIO (iorItem, &ioInfo);
if (err < 0)
	return err;

bytes_returned = ior->io_Actual;
if (bytes_returned != num_sec * cmdOpts.asFields.blockLength)
	return -1;								/* Non-match!! */

/* Clear out the I/O request */
DeleteIOReq (iorItem);						/* We don't want it any more */

return bytes_returned;						/* No error */
}
	



void do_real_seek_test (ustring *menu, Selector *select)
{
long blocks;
long to;
long from;
long time;
int i;										/* General counter */
static char result[15*40];					/* Number of lines we reserve */


/* We have a seek test selected, do it!! */

blocks = num_blocks;						/* On the CD */
											/* This is corrected for 2 second things */
if (blocks < 0)
	{
	text_line = "Seek: Error in block count.";
	return;
	}
	
from = ((from_seek.min * 60) + from_seek.sec) * 75 + from_seek.frm;
to = ((to_seek.min * 60) + to_seek.sec) * 75 + to_seek.frm;


switch (*menu[0].varptr)
	{
	case /* "Logarithmic seek test",			*/ 0:
		seek_result = log_seeks(blocks);
		break;
		
	case /* "Repeat seek test",					*/ 1:
		seek_result = repeat_test(from, to, number_of_passes);
		break;
		
	case /* "Linear seek test",					*/ 2: 
		seek_result = linear_rate_test();
		break;
		
	case /* "Random seek test",					*/ 3: 
		seek_result = rand_seeks(blocks, number_of_passes);
		break;
		
	case /* "Single seek test",					*/ 4: 
		if (from == -1)
			from = 0;
		if (to == -1 ) 
			to = blocks;
		if (number_of_passes == -1)
			number_of_passes = 1;
		if (number_of_passes > 10)
			number_of_passes = 10;
			
		result[0] = '\0';						/* Init result string */
		
		for (i = 0; i < number_of_passes; i++)
			{
			time = time_seek(from, to);
			sprintf (strrchr (result, '\0'), "Fm %ld To %ld, ET: %ld\n",
					from, to, time);
			}
		*strrchr (result, '\n') = '\0';		/* Delete last new line */
		seek_result = result;
		break;
		
	case /* "Histogram seek test",				*/ 5: 
		if (from < 0)
			from = blocks/2 - 55500 ;			/* 1/3 */
		if (to < 0)
			to = blocks/2 + 55500;				/* 2/3 */
		if (number_of_passes <= 0)
			number_of_passes = 100;
		seek_result = histogram_seeks (from, to, number_of_passes, blocks);
		break;
		
	case /* "Long seek test",					*/ 6: 
		if (from < 0)
			from = 5;
		seek_result = long_seeks (from, blocks);
		break;
		
	case /* "Short seek test",					*/ 7: 
		if (from < 0)
			from = 5 ;
		if (deltas < 0) 
			deltas = 32 ;
		seek_result = short_seeks (blocks, from, deltas);
		break;
		
	case /* "Play CD test (repeatability)"		*/ 8: 
		/* We can start from zero because the read function adds the offset! */
		if (from == -1)
			from = 0;
		if (to == -1) 	
			to = blocks;
		if (to < from) 
			to = from + 1;
		
		seek_result = playfunction (from, to);
		break;
		
	default:
		/* Its broke!! */
		;
	}
return;
}



/* This routine will increment and decrement the block value as needed.  The only
   restriction is that the value can only be under 75 (so as to produce ONE carry
   or borrow).  Sorry that the routine is not more general than that. */
   
void inc_dec_block (int inc_dec, BLOCK *block)
	{
	/* Increment/decrement the location for ease of use!!! */
	block->frame.cur += inc_dec;		/* Any direction... */
	if (block->frame.cur >= 75)
		{
		/* Carry from frame to seconds */
		block->seconds.cur++;
		block->frame.cur -= 75;
		if (block->seconds.cur >= 60)
			{
			/* Carry from seconds to minutes */
			block->minutes.cur++;
			block->seconds.cur -= 60;
			}
		}
	if (block->frame.cur < 0)
		{
		block->seconds.cur -= 1;
		block->frame.cur += 75;
		/* Borrow from seconds to frame */
		if (block->seconds.cur < 0)
			{
			/* Borrow from minutes to seconds */
			block->minutes.cur -= 1;
			block->seconds.cur += 60;
			}
		}
	}
