/* MiniUi.h   */
/* This is needed to get "int32's".  They are defined as 'long', not
   'int'.  This causes all sorts of junk if we try funny things! */
/* For these routines, I changed it back to 'ints'.  The compiler complains
   if things differ too much.  */
#include <kernel/types.h>


typedef enum UITYPE
	{
	/* One can disable the menu item by making it negative, or some
	   number that is not in the table.  Making it negative may be
	   easier to allow it to be positive later.  The value 'disabled'
	   is included here to specify things if needed. */
	text = 1,
	indextext,
	indrtext,
	truefalse,
	mchoice,
	mchoicec,		/* Circular choice */
	variable,
	printone,
	printtwo,
	printthree,
	menuexit,
	menuproc,
	disabled,		/* To turn off a menu item */
	last
	}  UITYPE;		/* Type of menu items */

struct ustring;
struct selector;
typedef void (*callProc)(struct ustring *, struct selector *);

typedef struct ustring
	{
	UITYPE		type;
	int		x;
	int		y;
	int		*varptr;		/* Variable pointer */
	union
		{
		int32		varwidth;	/* Width of printed variable */
		callProc	varproc;	/* Procedure to execute */
		}	q;
	int		*varmin;		/* Min value */
	int		*varmax;		/* Max value */
	char  	 	*text;			/* Text to display */
	char 		**mtext;
	} ustring;		/* Array of items for the menu displayed */

typedef  struct selector
	{
	int	nitems;
	int	item;
	unsigned int	joyold;
	unsigned int	joynew;
	unsigned int	joyedge;
	} Selector;		/* Define a selector group */




/* Prototypes, this file */
void DOUI (ustring menu[], Selector **the_select);

/* Other screen int32erface things */
void DrawScreen (void);
void PaintText (char *s, int32 x, int32 y, uint32 color);
void PaintLine (char *s, int32 y, uint32 color);

/* The following are used for the PaintLine routine */
#define LINEHEIGHT		12
#define LEFTMARGIN		20


/* Menu definition for text (unselectable) line */
#define MTEXT(x, y, xtext)  \
	{text, (x), (y), NULL, 0, NULL, NULL, (xtext), NULL}
/* Menu definition for text indirect line */
#define MPTEXT(x, y, xtext)  \
	{indrtext, (x), (y), NULL, 0, NULL, NULL, NULL, (xtext)}
/* Menu definition for non-selectable text array */
#define MIXTEXT(x, y, ix, arr) \
	{indextext, (x), (y), &(ix), 0, NULL, NULL, NULL, (arr)}
/* Menu definition for true/false item (settable) */
#define MBOOL(x, y, var, string) \
	{truefalse, (x), (y), &(var), 0, NULL, NULL, (string), NULL}
/* Menu definition for menu choice (non circular) */
#define MCHOICE(x, y, var, low, hi, arr) \
	{mchoice, (x), (y), &(var), 0, &(low), &(hi), NULL, (arr)}
/* Menu definition for circular menu choice */
#define MCHOICEC(x, y, var, low, hi, arr) \
	{mchoicec, (x), (y), &(var), 0, &(low), &(hi), NULL, (arr)}
/* Menu definition for menu choice (non circular) that has a procedure */
#define MCHOICEX(x, y, var, low, hi, arr, proc) \
	{mchoice, (x), (y), &(var), (int32)&(proc), &(low), &(hi), NULL, (arr)}
/* Menu definition for circular menu choice that has a procedure */
#define MCHOICECX(x, y, var, low, hi, arr, proc) \
	{mchoicec, (x), (y), &(var), (int32)&(proc), &(low), &(hi), NULL, (arr)}
/* Menu definition for a variable (with limits) */
#define VARIABLE(x, y, var, low, hi, digits, xtext) \
	{variable, (x), (y), &(var), digits, &(low), &(hi), (xtext), NULL}
/* menu definition for print 1 formatted */
#define PRINT1(x, y, var, string) \
	{printone, (x), (y), &((int)var), 0, NULL, NULL, string, NULL}
/* Menu definition for print 2 formatted */
#define PRINT2(x, y, var1, var2, string) \
	{printtwo, (x), (y), &((int)var1), 0, &((int)var2), NULL, string, NULL}
/* Menu definition for print 3 formatted */
#define PRINT3(x, y, var1, var2, var3, string) \
	{printthree, (x), (y), &((int)var1), 0, &((int)var2), &((int)var3), string, NULL}
/* Meny definition for 'exit' function */
#define MEXIT(x, y, string) \
	{menuexit, (x), (y), NULL, 0, NULL, NULL, (string), NULL}
/* Menu definition for 'procedure' definition */
#define MPROC(x, y, proc, string) \
	{menuproc, (x), (y), NULL, (int32)&(proc), NULL, NULL, (string), NULL}
/* The last definition line */
#define MLAST \
	{last, 0, 0, NULL, 0, NULL, NULL, NULL, NULL}
/* These should make things easier!! */
