/* @(#) seek.c 95/10/23 1.2 */


/* Copyright 1995, The 3DO Company.  All rights reserved. */

/* Now this functions from the user interface from the general CD test routine. */


/*
 *  seek.c - test program for driving the CD-ROM.
 * 
 *
 * This is the main software for the seek tests. I always just use
 * seek with no arguements.
 * 
 * This prints seven columns of tab separated spaces which can then be pasted 
 * into an Excel spreadsheet for graphing.
 * 
 * The first is the number of tracks of the seek.
 * The second is the time for a forward seek from 0 to the number of tracks.
 * The third is the time for a reverse seek from the number of tracks to zero.
 * The fouth is a forward seek centered around the center of the disc.
 * The fifth is a reverse seek centered around the center of the disc.
 * The sixth is a forward seek of ending at the end of the disc.
 * The seventh is reverse seek starting at the end of the disc.
 * 
 * The number of tracks increases in a logarithmic fashion from 1 to the
 * total number of tracks on the disc.
 */

#include <kernel/types.h>
#include <kernel/item.h>
#include <kernel/mem.h>
#include <kernel/io.h>
#include <kernel/device.h>
#include <device/cdrom.h>
#include <misc/event.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "su.h"
#include "mytime.h"
#include "seek.h"					/* For this file, just to check!! */

#define READMINBLOCK 150
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))
#define ABS(a)   (((a)< 0)?(-(a)):(a))
#define streq(a,b) (!strcmp(a, b))
/* The following is poor programming practice, but it works most of the time */
#define DBG if (1 == 1) printf


static int verbose = FALSE ;

/* Prototypes referenced from other routines. */
int do_read_block (void *buffer, long block, int num_sec);
/* The above is in the routine 'm2_cdtest.c' */



/* Attempt to measure the linear velocity of the disc */
char * linear_rate_test(void)
{
	uint32	time;
	uint32	rate; 
	uint32	total = 0;
	int	i;
	static char result[15*40];
	
	/* result[0] = '\0'; */
	strcpy (result, "Linear velocity test:\n");
	for(i = 0; i < 10; i++)
	{
		(void) time_seek(0, 12);
		time = time_seek(20, 20);
		rate = (50 * 3141592) / time;
		printf("Time = %8d, rate = %8d 0.5 rate = %8d\n",time,rate, rate/2);
		sprintf (strchr (result, '\0'), "Ti = %8d, rate = %8d.\n", time, rate);
		
		total += time;
	}
	time = total/10;
	rate = (50 * 3141592) / time;
	printf("AVG  = %8d, rate = %8d 0.5 rate = %8d\n", time, rate, rate/2);
	sprintf (strchr (result, '\0'), "Av = %8d, rate = %8d", time, rate);
	return result;
}



char * playfunction(int from, int to)
{
	int i;
	static unsigned char *buf = NULL;
	static char result[80];
	uint32 elapsed_time;
	uint32 starting_time;
	uint32 elapsed_min;
	uint32 elapsed_sec;

	if (buf == NULL)
		buf = AllocOrDie(2532 + 98 + 10);
	
	printf("Buffer is located at %08X\n",buf);
	
	printf("Reading ...\n");
	starting_time = MyTime();					/* Starting point of the timer */
	for(i = from; i <= to; i++)
		do_read_block( buf,i,1 );
#if 0
	printf("Complete.\n");
#endif
	
	elapsed_time = MyTime() - starting_time;
	elapsed_min = elapsed_time / (60 * 1000 * 1000);
	elapsed_sec = (elapsed_time -= (elapsed_min * 60 * 1000 * 1000)) / (1000 * 1000);
	elapsed_time -= elapsed_sec * 1000 * 1000;
	sprintf (result, "Read  %ld blocks\nFrom: %ld\nTo:   %ld\n",
		to - from, from, to);
		
	sprintf (result + strlen(result), "Elapsed time: %d:%02d.%06d mm:ss.fract.", 
		elapsed_min, elapsed_sec, elapsed_time);
	FreeAllocMem (buf);
	buf = NULL;						/* Its not allocated anymore!! */
	return result;
}




char * rand_seeks(int blocks, int npasses)
{
	int		i;
	uint32	b;
	uint32 time0, time1, time;
	static uint32 *array = NULL;			/* Allocated to 'npasses' elements */
	static unsigned char *buf = NULL;
	static uint32 timer_overhead = 0;
	static char result[80];
	int min_seek = 10000;
	int max_seek = -1;						/* To display these... */
	int time_before;
	int time_after;

	b = (uint32)blocks;		/* We can go from 0 to 'blocks' now */
	
	if (npasses < 0)
		npasses = 100;

	npasses++ ;
	
#ifdef PRINT_ON_SCREEN	
	printf("creating random table of %d seeks\n",npasses ); 
#endif /* PRINT_ON_SCREEN */

#ifdef PRINT_ON_SCREEN	
	if (verbose)
		printf("This test does not report propertimes when verbose is true\n");
#endif /* PRINT_ON_SCREEN */

	if (buf == NULL)
		buf = AllocOrDie(2532 + 98 + 10);
	timer_overhead = TimerOverhead();

	if (array == NULL)
		array = (uint32 *)AllocOrDie(npasses*sizeof(uint32));
		
	for(i = 0; i < npasses; i++)
		array[i] = rand() % b; 

#ifdef PRINT_ON_SCREEN	
	printf("Seeking\n");
#endif /* PRINT_ON_SCREEN */
	
	do_read_block(buf, array[0], 1);
	time0 = MyTime();
	time_before = time0;

	for (i = 1; i < npasses; i++)
		{
#ifdef PRINT_ON_SCREEN	
		if (verbose)
			printf("%d\n",array[i] );
#endif /* PRINT_ON_SCREEN */
		do_read_block(buf, array[i], 1);
		time_after = MyTime();
		/* Delta time calculation.  Reuse variable.  */
		time_before = (time_after - time_before + 500) / 1000;		
		if (min_seek > time_before)
			min_seek = time_before;
		if (max_seek < time_before)
			max_seek = time_before;
		time_before = time_after;					/* For next time */
		}

	time1 = MyTime();
				
	time = time1 - time0 - timer_overhead;
	time /= 1000;
	
#ifdef PRINT_ON_SCREEN	
	printf("%d random seeks took %d for %d avg seek\n",npasses-1,time,time/(npasses-1));
#endif /* PRINT_ON_SCREEN */

	sprintf (result, "%d random seeks took %d\nAverage seek: %d\n",
		npasses - 1, time, time/(npasses - 1));
	sprintf (strrchr (result, '\0'), "Minimum time: %d, Maximum time: %d",
			min_seek, max_seek);
	FreeAllocMem (array);
	array = NULL;
	FreeAllocMem (buf);							/* Return */
	buf = NULL;									/* Tell future callers */
	return result;
}

/* measure the repeatability of the testing */
char * repeat_test(int start, int end, int npasses)
{
int		n;
int		s;
uint32	time;
int		maxs;
static char result[15*40];				/* Room to display things */

if (start <= 0)							/* If default start (at beginning) */
	start = 17*60*75;					/* Minute 17. */
if (end > 60 * 60 * 75)					/* At the hour (end) */
	end = 42*60*75;						/* Minute 42. */
if (npasses < 0 || npasses > 7)
	npasses = 7;
result[0] = '\0';						/* Start out the result buffer */

maxs = 5 + sectors_per_track (sector_to_track (end));

#ifdef PRINT_ON_SCREEN	
printf("from %d to %d through %d to %d\n",
		start, end, start + maxs - 1, end);
printf("from t%d to t%d through t%d to t%d\n",
		int2time (start), int2time (end), 
		int2time (start + maxs - 1), int2time (end));
#endif /* PRINT_ON_SCREEN */

for(n = 0; n < npasses; n++)
	{
	for(s = 0; s < maxs; s++)
		{
		time = time_seek(start-s, end) / 1000;
		sprintf (strrchr (result, '\0'), "%4d", time);
		if (s % 10 == 9)				/* Number of goodies per line (10) */
			strcat (result, "\n");
#ifdef PRINT_ON_SCREEN	
		printf("\t%d", time);
#endif /* PRINT_ON_SCREEN */
		}
#ifdef PRINT_ON_SCREEN	
	printf("\n");
#endif /* PRINT_ON_SCREEN */
	strcat (result, "\n");
	}
return result;
}


/* Find worst seeks for tracks based on sectors */
#define MAX_TRACK_ARRAY (50) 

char * log_seeks( int blocks)
{
	int tracks ;	
	int ftime, rtime;
	int max_tracks;
	int track_lengths[MAX_TRACK_ARRAY];
	int mode; /* zero = from zero to track, 1 = around middle, 2 = from outer to track */
	int mti;
	int start;
	int	end;
	static char result[40];

/* build list of max tracks */

	max_tracks = sector_to_track(blocks);
	tracks = max_tracks;
	mti = 0;
	while (tracks > 0)
	{
		track_lengths[mti] = tracks;
		tracks = (tracks*1000)/1300;	/* tracks /= 1.3 */
		
		mti++ ;
		if (mti == MAX_TRACK_ARRAY)
		{
			printf("Error: Array to small\n");
			exit(9);
		}
	}

	while (mti--)
	{
		/* Start at end of 'track_lengths' list and go back.  This will start at
		   a difference of 1 track */
		tracks = track_lengths[mti];
		printf("%5d ",tracks);
		
		for (mode = 0; mode < 3; mode++)
		{
			switch (mode)
			{
				case 0:		/* from zero to track */
					start = 0;
					end = tracks;
					break;
				case 1:		/* around middle */
					start = max_tracks/2-tracks/2;
					end   = max_tracks/2+tracks/2;
					break;
				case 2:		/* from outer to track */
					start = max_tracks-tracks;
					end = max_tracks;
					break;
				default:
					printf("Log_seeks internal puke\n");
					exit(9);
			}
#if 1
			get_minmin(track_to_sector(start), track_to_sector(end), &ftime, &rtime); 
#else
			ftime = start *1000;
			rtime = end*1000;
#endif
			printf(" \t%4d \t%4d", ftime/1000, rtime/1000);
		}
		printf("\n");
	}
	strcpy (result, "Log seek test done");
	/* I Need to cobble up a graph of the data to be useful!! */
	return result;
	
}

 
char * histogram_seeks(int from, int to, int npass, int blocks)
{
/* This accumulates a histogram of seek times until we get one
   time slot with 'npass' hits.  Then we try to display the
   results on the screen */
   
	/* number of histogram slots */
#define HIST 1000
	/* worst case max seek time in ms */
#define MAXTIME 1000
	/* Screen size for display things */
#define SCREEN_WIDTH 40
	/* Number of lines of dots to display */
#define NDOTS 10

	int i;
	int j;
	int time;
	int count_of_seeks = 0;
	int displayed_dots = 0;
	int max_bin;
	int top_bin;
	int index;
	int stars;
	int max_time = -1;
	int min_time = MAXTIME + 1;
	int results[HIST] ;
	static char cresult[41 * 13];	/* Enough for 13 lines */
	
	for (i = 0; i < HIST; i++)
		results[i] = 0;
	
	while ((to > blocks) || (from > blocks))
		{
			{
			static int flag = 0;

			if (!flag)
				{
				flag++;
				printf("Adjusting for disc \n");
				}
			}
		from--;
		to--;
		}
	if ((to < 0) || (from < 0))
		{
		printf("disc too small\n");
		from = 0;
		to = blocks;
		}
	
	while (TRUE)
		{
		time = (int)time_seek(from, to);
		time = (time+500)/1000;			/* convert to ms */
		if (time > MAXTIME)
			{
			time = MAXTIME;
			printf("Clipping extra long time \n");
			}
		i = (time * HIST) / MAXTIME;		/* calculate slot */
		if (i < min_time)
			min_time = i;
		if (i > max_time)
			max_time = i;
		results[i] ++;
		count_of_seeks++;
		/*
		printf("TIME:HITS:\t%d\t%d\n",(i*MAXTIME)/HIST,results[i]);
		*/

		if (results[i] >= npass)
			break;
		}
	top_bin = max_bin = i;						/* Time of max frequency */

	if (max_bin + (SCREEN_WIDTH / 2) > HIST)
		max_bin = HIST - (SCREEN_WIDTH / 2);	/* Upper edge */
	if (max_bin - (SCREEN_WIDTH / 2) <= 0)
		max_bin = (SCREEN_WIDTH / 2);			/* Lower edge */

#ifdef PRINT_ON_SCREEN	
	for (i = 0; i < SCREEN_WIDTH; i++)
		{
		index = i + max_bin - (SCREEN_WIDTH / 2);
		printf ("%4d ", index);
		for (stars = 0; stars < results[index] * 10 / npass; stars++)
			{
			printf ("*");
			}
		printf ("\n");
		}
#endif /* PRINT_ON_SCREEN */		

	stars = 10 * npass / NDOTS;

	sprintf (cresult, "Timing from %d to %d, dots=%d.%d\n",
				max_bin - (SCREEN_WIDTH / 2),
				max_bin + (SCREEN_WIDTH / 2) - 1,
				stars / 10, stars % 10);
	sprintf (strrchr (cresult, '\0'), "Min: %d, Peak: %d, Max: %d\n",
				min_time, top_bin, max_time);
	
#ifdef PRINT_ON_SCREEN
		printf ("Timing from %d to %d, dots=%d.%d\n",
				max_bin - (SCREEN_WIDTH / 2),
				max_bin + (SCREEN_WIDTH / 2) - 1,
				stars / 10, stars % 10);
#endif /* PRINT_ON_SCREEN */				

	for (j = 0; j < NDOTS; j++)
		{
		stars = npass * (NDOTS - j) / NDOTS;
		for (i = 0; i < SCREEN_WIDTH; i++)
			{
			index = i + max_bin - (SCREEN_WIDTH / 2);
			if (j == 0)
				displayed_dots += results[index];
			if (results[index] >= stars)
				{
				strcat (cresult, "*");
#ifdef PRINT_ON_SCREEN				
				printf ("*");
#endif /* PRINT_ON_SCREEN */				
				}
			else
				{
				strcat (cresult, " ");
#ifdef PRINT_ON_SCREEN				
				printf (".");
#endif /* PRINT_ON_SCREEN */				
				}
			}
			strrchr (cresult, '*')[1] = '\0';
			strcat (cresult, "\n");
#ifdef PRINT_ON_SCREEN			
			printf ("\n");
#endif /* PRINT_ON_SCREEN */			
		}
		
	/* Note:  the max number of hits in any 'bin' is 'npass' */
	/* I really want to put up a histogram here!! */
#ifdef PRINT_ON_SCREEN	
	printf("\n\n");
	
	for (i = 0; i < HIST; i++)
		{
		if (results[i])
			{
			/* If there is anything in the accumulator 'bin' */
			printf("Time:Hits:%%Max\t%d\t%d\t%d\n",
				i * MAXTIME / HIST, results[i], 
				100 * results[i] / count_of_seeks);
			}
		}
#endif /* PRINT_ON_SCREEN */		
	sprintf (strrchr (cresult, '\0'), 
			"Seeks: %d, Displayed: %d", count_of_seeks, displayed_dots);
	return cresult;
	
	
}

char * long_seeks(int starts, int blocks)
{
	int s, d;
	int min, max;
	int dist;
	int start, end;
	static char result[40];
	
	for (d = -starts; d <= starts; d++)
		{
		dist = (blocks * d) / starts;
		
		printf("%08d",dist);
	
		for(s = 0; s <= starts; s++)
			{
			/* calc start location on the disc */
			start = (blocks * s) / starts;  
			end = start + dist;
			
			min = 0;
			max = 0;
			
			if ((end >= 0) && (end <= blocks))
				{
				get_minmax(start, end, &min, &max);
				printf("\t%d\t%d",min/1000,max/1000); 
				}
			else
				printf("\txxx\txxx");
			}
		printf("\n");
	}
	strcpy (result, "long seek test");
	return result;
}

char * short_seeks(int blocks, int starts, int dels)
{
	int s;
	int min, max;
	int dist;
	int start, end;
	int vs, ve, vl;
	static char result[40];
	
	vs = dels;
	ve = blocks - dels;
	vl = ve-vs;

	printf("%8d",0);
	for(s = 0; s <= starts; s++)
	{
		start = (100*s)/starts;
		printf("\tv%03d\t^%03d",start,start);
	}	
	printf("\n");
	
	for(dist = -dels/4; dist <= dels; dist++)
	{
		printf("%8d", dist);
		for(s = 0; s <= starts; s++)
		{
			start = vs+(vl * s) / starts;  /* calc start location on the disc */
			end = start+dist;
			get_minmax(start, end, &min, &max);
			printf("\t%4d\t%4d",min/1000,max/1000); 
		}
		printf("\n");
	}
	strcpy (result, "Short seek test\nSee Portfolio Terminal.");
	return result;
}

