/* @(#) serial.c 95/12/22 1.5 */

/* generic serial driver test */

/* Modifications by tsw Feb 07, 1996 */

/* Reference: <http://rhett/Portfolio_HTML/includes/device/serial.h> */

/* I don't know how this is to behave when there are two serial ports around
   like the one on the dev-card, and the one on the PCMCIA card.  It may need
   to be altered somewhat... */

#include <kernel/types.h>
#include <kernel/io.h>
#include <kernel/devicecmd.h>
#include <device/serial.h>

#include <string.h>			/* For the str... functions */
#include <stdio.h>			/* For printf and friends */
#include <stdlib.h>			/* For 'exit' */



/* Prototypes: */
static void write_serial (char *string_data);
static Item CreateSerialIOReq (void);
static Err DeleteSerialIOReq (Item ioreq);
static void serial_error (Err result, char *text);
static void read_serial (char *string_data, int buf_len);


/*****************************************************************************/


static Item CreateSerialIOReq (void)
{
Item  device;
Item  ioreq;
List *list;
Err   result;

    result = CreateDeviceStackListVA(&list,
                "cmds", DDF_EQ, DDF_INT, 10,
		        CMD_STREAMREAD,
                        CMD_STREAMWRITE,
                        SER_CMD_STATUS,
                        SER_CMD_SETCONFIG,
                        SER_CMD_GETCONFIG,
                        SER_CMD_WAITEVENT,
                        SER_CMD_SETRTS,
                        SER_CMD_SETDTR,
                        SER_CMD_SETLOOPBACK,
                        SER_CMD_BREAK,
		NULL);
    printf ("We have %d nodes that are like serial I/O.\n", GetNodeCount (list));
    if (result >= 0)
    {
        if (!IsEmptyList(list))
        {
            device = result = OpenDeviceStack((DeviceStack *) FirstNode(list));
            if (device >= 0)
            {
                ioreq = result = CreateIOReq(NULL,0,device,0);
                if (ioreq < 0)
                    CloseDeviceStack(device);
            }
        }
        else
        {
            result = MAKEKERR(ER_SEVER,ER_C_STND,ER_NoHardware);
        }
        DeleteDeviceStackList(list);
    }

    return result;
}


/*****************************************************************************/


static Err DeleteSerialIOReq (Item ioreq)
{
IOReq  *io;
Device *dev;
Err     result;

#if 0
	printf ("The IO request item is: 0X%08X\n", ioreq);
    io = (IOReq *)CheckItem(ioreq,KERNELNODE,IOREQNODE);
    if (!io)
        return BADITEM;

    dev    = io->io_Dev;
#endif    
    result = DeleteIOReq(ioreq);
#if 0
    	printf ("Deleted IO request, now close device stack\n");
    CloseDeviceStack(dev->dev.n_Item);	/* ??? */

	printf ("Device stack thing: 0X%08X\n", dev->dev.n_Item);
#endif	
    return result;
}


/*****************************************************************************/



/* ******************** */

static void serial_error (Err result, char *text)
{
/* Exit with printing the error code. */
printf ("%s", text);
PrintfSysErr (result);
exit (-1);			/* Error */
}


/* *********** */


static char inputBuffer[100];
static Item       ioreq;		/* Global now */


int main(void)
{
IOInfo		ioInfo;
Err		result;
SerConfig	sc;
uint32		i;
SerStatus	the_status;

ioreq = CreateSerialIOReq();
if (ioreq < 0)
	{
	serial_error (ioreq, "Couldn't create serial I/O:");
	}
	
printf("Serial device active!\n");

memset (&sc, 0, sizeof(sc));			/* Needed to init things */
sc.sc_BaudRate           = 9600;		/* Was 56700 */
sc.sc_Handshake          = SER_HANDSHAKE_NONE;
sc.sc_WordLength         = SER_WORDLENGTH_8;
sc.sc_Parity             = SER_PARITY_NONE;
sc.sc_StopBits           = SER_STOPBITS_1;

memset(&ioInfo, 0, sizeof(ioInfo));
ioInfo.ioi_Command         = SER_CMD_SETCONFIG;
ioInfo.ioi_Send.iob_Buffer = &sc;
ioInfo.ioi_Send.iob_Len    = sizeof(sc);
result = DoIO(ioreq, &ioInfo);

if (result < 0)
	{
	serial_error (result, "Setting config failed: ");
	}


memset(&ioInfo, 0, sizeof(ioInfo));
ioInfo.ioi_Command         = SER_CMD_SETRTS;
ioInfo.ioi_CmdOptions      = 1;		/* Turn on RTS */
result = DoIO(ioreq, &ioInfo);

if (result < 0)
	{
	serial_error (result, "Setting RTS failed: ");
	}

/* Get the status of things */

memset (&ioInfo, 0, sizeof(ioInfo));		/* Clear out request */
ioInfo.ioi_Command = SER_CMD_STATUS;
ioInfo.ioi_Recv.iob_Buffer = &the_status;
ioInfo.ioi_Recv.iob_Len = sizeof(the_status);
/* Now get it!! */
result = DoIO (ioreq, &ioInfo);
if (result < 0)
	{
	serial_error (result, "Couldn't get status: ");
	}
/* Now output the status info */
printf ("Flags:           0x%08X :", the_status.ss_Flags);
if (the_status.ss_Flags & SER_STATE_CTS)
	printf ("CTS ");
if (the_status.ss_Flags & SER_STATE_DSR)
	printf ("DSR ");
if (the_status.ss_Flags & SER_STATE_DCD)
	printf ("DCD ");
if (the_status.ss_Flags & SER_STATE_RING)
	printf ("RING ");
if (the_status.ss_Flags & SER_STATE_LOOPBACK)
	printf ("LOOP ");
if (the_status.ss_Flags & SER_STATE_RTS)
	printf ("RTS ");
if (the_status.ss_Flags & SER_STATE_DTR)
	printf ("DTR ");
printf ("\n");		/* Terminator */
printf ("Overflow errors: %d\n", the_status.ss_NumOverflowErrors);
printf ("Framing errors:  %d\n", the_status.ss_NumFramingErrors);
printf ("Parity errors:   %d\n", the_status.ss_NumParityErrors);
printf ("Breaks:          %d\n", the_status.ss_NumBreaks);
printf ("Dropped bytes:   %d\n", the_status.ss_NumDroppedBytes);
/* Done */

printf ("Serial device all setup\n");
write_serial ("Pack my box with five dozen liquor jugs.!\r\n");
/* !!!! The above completes, the next doesn't!! */
write_serial ("Please enter input.  Terminate with blank line\r\n");

/* What I need to do here is put in a 'timeout'.  First I need to understand
   exactly how to do this.  */

printf("Attempting to read serial data\n");

do
	{
	read_serial (inputBuffer, sizeof(inputBuffer) - 1);
	
#if 0
	printf("Read returned success\n");
	printf("io_Actual %d\n", strlen (inputBuffer));
#endif
	
#if 0
	for (i = 0; i < strlen (inputBuffer); i++)
	    printf("%c",inputBuffer[i]);
	printf ("\n");

#else
	printf ("%s\n", inputBuffer);
#endif
    
	} while (inputBuffer[0] != '\0');
write_serial ("That's all for now, serial test complete\r\n");
printf ("Completed serial test\n");

DeleteSerialIOReq(ioreq);

return 0;
}



static void write_serial (char *string_data)
{
IOInfo	ioInfo;
Err	result;


memset (&ioInfo, 0, sizeof(ioInfo));
ioInfo.ioi_Command         = CMD_STREAMWRITE;
ioInfo.ioi_Send.iob_Buffer = string_data;
ioInfo.ioi_Send.iob_Len    = strlen (ioInfo.ioi_Send.iob_Buffer);

result = DoIO(ioreq, &ioInfo);
if (result < 0)
	{
	serial_error (result, "Writing failed: ");
	}
return;
}



static void read_serial (char *string_data, int buf_len)
{
/* Read 1 charactrer at a time until a control character (usually a <cr>),
   or until the buffer is full.  */

IOInfo	ioInfoR;
IOInfo	ioInfoW;	/* For the echo!! */
Err	result;
char	input_char;
char 	*initial_string_data = string_data;
char	bs_echo [3] = "\b \b";		/* Exactly 3 characters */
int	i;		/* Dumb counter */

/* the variable 'buf_len' is for the charcters OTHER than the '\0' (null character)
   that gets stored at the end of the string */
/* Make it ONE less than the size of the character array */

/* Setup everything */
memset (&ioInfoR, 0, sizeof(ioInfoR));
ioInfoR.ioi_Command         = CMD_STREAMREAD;
ioInfoR.ioi_Recv.iob_Buffer = &input_char;
ioInfoR.ioi_Recv.iob_Len    = 1;

memset (&ioInfoW, 0, sizeof(ioInfoW));
ioInfoW.ioi_Command         = CMD_STREAMWRITE;
ioInfoW.ioi_Send.iob_Buffer = &input_char;
ioInfoW.ioi_Send.iob_Len    = 1;

while (1)
	{
	result = DoIO (ioreq, &ioInfoR);
	if (result < 0)
		{
		serial_error (result, "Reading failed: ");
		}
	if (input_char == '\b')		/* Backspace */
		{
		if (string_data <= initial_string_data)		/* Backspace from front */
			{
			input_char = '\a';			/* Alert */
			result = DoIO (ioreq, &ioInfoW);
			if (result < 0)
				{
				serial_error (result, "Beep bksp fail: ");
				}
			continue;
			}
		/* Otherwise we can do it!! */
		string_data--;					/* Backup */
		buf_len++;					/* Allow another */
		for (i = 0; i < sizeof(bs_echo); i++)
			{
			input_char = bs_echo[i];
			result = DoIO (ioreq, &ioInfoW);
			if (result < 0)
				{
				serial_error (result, "BKSP echo fail: ");
				}
			}
		continue;					/* Another input */
		}
	
	if (!(input_char & ~0x1f))
		break;			/* Control character */
	result = DoIO (ioreq, &ioInfoW);	/* Echo!! */
	if (result < 0)
		{
		serial_error (result, "Write echo failed: ");
		}
	if (--buf_len <= 0)
		break;
	*string_data++ = input_char;
	}
/* Echo a <cr> <lf> pair */
ioInfoW.ioi_Send.iob_Buffer = "\r\n";	/* Point to string */
ioInfoW.ioi_Send.iob_Len    = 2;
result = DoIO (ioreq, &ioInfoW);
if (result < 0)
	{
	serial_error (result, "<Cr> <lf> write failed: ");
	}
/* Terminate the string */
*string_data = '\0';
return;
}
