/* Information:  There are 7 dots/character in the font I'm using */
#define BUFFER_LINES	16				/* Number of lines I can display */
#define BYTES_ACROSS	8				/* Display of buffer lines */

#include <stdio.h>						/* Printf and friends. */
#include <misc/event.h>					/* Has events for control pad */
#include <stdlib.h>						/* Memory things */
#include <string.h>						/* str... */
#include <kernel/mem.h>					/* Memory allocation things */
#include <kernel/io.h>					/* More kernel stuff... */
#include "miniui.h"						/* User interface for the monitor */
#include "dump_record.h"


/* Externals to the other module */
static void execute_read (void);
static void do_display (ustring *menu, Selector *select);

static	char	*buf = NULL;		/* Point to buffer used by routine */
static	long	len;				/* Size of the I/O Operation */

static	char buffer_text[BUFFER_LINES][BYTES_ACROSS * 6];		
static	ustring display_menu[] =
	{
	PRINT2 (20, 20, block_num, block_num, "Block: %5d (0x%X)"),
	MPROC (209, 20, do_display, "(<- ->)"),
	
	/* The following is a display of 16 lines of things */
	/* Presently there are 8 bytes across (for a total of 128) */
	MTEXT (20, 32, buffer_text[0]),
	MTEXT (20, 44, buffer_text[1]),
	MTEXT (20, 56, buffer_text[2]),
	MTEXT (20, 68, buffer_text[3]),
	MTEXT (20, 80, buffer_text[4]),
	MTEXT (20, 92, buffer_text[5]),
	MTEXT (20, 104, buffer_text[6]),
	MTEXT (20, 116, buffer_text[7]),
	MTEXT (20, 128, buffer_text[8]),
	MTEXT (20, 140, buffer_text[9]),
	MTEXT (20, 152, buffer_text[10]),
	MTEXT (20, 164, buffer_text[11]),
	MTEXT (20, 176, buffer_text[12]),
	MTEXT (20, 188, buffer_text[13]),
	MTEXT (20, 200, buffer_text[14]),
	MTEXT (20, 212, buffer_text[15]),
	
	MLAST
	};




/* This routine is called from the main menu routine.  It is external */
void do_read (ustring *menu, Selector *select)
{
/* From the menu, do the thing... */
static Selector *select2 = NULL;/* Selection goodie for data display */


execute_read ();					/* Do the read!! */

/* May want to go another screen to examine the data */

/* Init the display things */
do_display (NULL, NULL);
DOUI (display_menu, &select2);

text_line = "Block read.";

if (select2->joynew & ControlStart)
	{
	block_num += 1;				/* Step the block number */
	text_line = "Block read.  Location Incremented.";
	}

}





static void execute_read (void)
{
/* This just reads the block, does nothing more */

IOReq *ior;						/* The I/O request */
Item iorItem;					/* So we can get a goodie */
IOInfo	ioInfo;					/* Where the I/O request is put */
DeviceStatus devStat;			/* Where we get the status */
long addLen;					/* "used in calculations" */
int i;							/* General index */


/* Clear out the I/O request */
memset (&ioInfo, 0, sizeof (ioInfo));
iorItem = CreateIOReq (0, 0, stcrdItem, 0);
ior = (IOReq *)LookupItem (iorItem);

memset (&ioInfo, 0, sizeof(ioInfo));
ioInfo.ioi_Command = CMD_STATUS;
ioInfo.ioi_Recv.iob_Buffer = &devStat;
ioInfo.ioi_Recv.iob_Len = sizeof (devStat);

/* Get the status */
DoIO (iorItem, &ioInfo);		/* Get status (to find out blocksize) */

/* Set the location to read */
/* We specify location based on given block size */
if (block_num < (int32)devStat.ds_DeviceBlockStart)
	{
	/* Troubles in river city!! */
	block_num = 0;
	}
if (block_num >= (int32)devStat.ds_DeviceBlockCount)
	{
	/* More troubles in river city!! */
	block_num = devStat.ds_DeviceBlockCount - 1;
	}

ioInfo.ioi_Offset = block_num;


/* For the moment we read ONE block */
len = 1 /* Block length */ * devStat.ds_DeviceBlockSize;
if (buf != NULL)
	FreeMem (buf, len);				/* Free up the memory of the block allocated??
									   previously (if it was).  This allows the
									   block to stay around until we want to re-use
									   it for some reason. */
buf = AllocMem (len, MEMTYPE_NORMAL);
if (buf == NULL)
	{
	printf ("Could not allocate buffer of length %d\n", len);
	return;
	}
ioInfo.ioi_Recv.iob_Buffer = buf;
ioInfo.ioi_Recv.iob_Len = len;
ioInfo.ioi_Command = CMD_BLOCKREAD;


/* Now do the REAL read I/O Operation */

send_error = SendIO (iorItem, &ioInfo);

wait_error = WaitIO (iorItem);
/* Data is in 'buf' */

	
/* The data goes here... */
bytes_returned = ior->io_Actual;
/* We ignore anything else!! */
DeleteIOReq (iorItem);			/* We don't want it any more */
return;
}



/* The routine that will display the I/O buffer for us to see */

static void do_display (ustring *menu, Selector *select)
{
int i;
int j;
int db;
int line;			/* Line number */
static int start_loc;

if (select == NULL)
	{
	/* Initial fill */
	start_loc = 0;
	}

else if (select->joyedge & ControlLeft)
	{
	/* Decrement the address, re-display */
	start_loc -= BYTES_ACROSS;
	}

else if (select->joyedge & ControlRight)
	{
	/* Increment the address, re-display */
	start_loc += BYTES_ACROSS;
	}

else if (select->joyedge & ControlA)
	{
	/* Decrement the address by a full display, re-display */
	start_loc -= BYTES_ACROSS * BUFFER_LINES;
	}

else if (select->joyedge & ControlC)
	{
	/* Increment the address by a full display, re-display */
	start_loc += BYTES_ACROSS * BUFFER_LINES;
	}

else if (select->joyedge & ControlLeftShift)
	{
	/* Increment the address by a full display, re-display */
	block_num -= 1;
	execute_read ();
	}

else if (select->joyedge & ControlRightShift)
	{
	/* Increment the address by a full display, re-display */
	block_num += 1;
	execute_read ();
	}

else
	{
	/* Nothing to do !! */
	return;
	}


/* Adjust the buffer limits, if required.  */
if (start_loc < 0)
	start_loc = 0;
if (start_loc + BUFFER_LINES * BYTES_ACROSS > len)
	start_loc = len - BUFFER_LINES * BYTES_ACROSS;

/* Display the contents */
line = 0;
for (i = start_loc; 
	 i < (start_loc + BUFFER_LINES * BYTES_ACROSS); 
	 i += BYTES_ACROSS)
	{
	sprintf (buffer_text[line], "%03X: ", i);
	for (j = 0; j < BYTES_ACROSS; j++)
		{
		db = ((char *)buf)[i + j];
		sprintf (strrchr (buffer_text[line], '\0'), "%02X ", db);
		}
	sprintf (strrchr (buffer_text[line], '\0'), " |");
	for (j = 0; j < BYTES_ACROSS; j++)
		{
		db = ((char *)buf)[i + j] & 0x7f;
		if (db < ' ' || db > '~')
			db = '.';
		sprintf (strrchr (buffer_text[line], '\0'), "%c", db);
		}
	sprintf (strrchr (buffer_text[line], '\0'), "|");
	line++;
	}
}

