/* The timer test routine   tsw Mar 25, 1996 */
/* Completed Mar 29, 1996 tsw */

/* Exercises the timer device as much as it can.  Uses the 'new' scaffold
   routines to print out success/failure */
   

#include <stdio.h>
#include <string.h>

#include <tests/3do_scaf.h>

#include <kernel/time.h>
#include <kernel/task.h>			/* For 'AllocSignal' */
#include <kernel/devicecmd.h>
#include <kernel/operror.h>
#include <misc/batt.h>				/* Had reference to battery backed up time */

/* Prototypes as needed */
static Item CreateTimerIOReq(void);
int32 delta_t (Item IOReqItem);



int main (int argc, char *argv[])

{
int 		ret_value = 0;			/* Returned from this routine (with default).  */
Item 		IOReqItem;
IOInfo		ioInfo;
Err			result;
TimeValVBL	the_time_v;			/* In VBL units (64 bits, a long time) */
TimeVal 	the_time_s;			/* Holds seconds and microseconds */
GregorianDate return_date;		/* From the 'ReadBattClock' call */
uint32		sec_of_day;			/* Combined seconds into the day */
uint32		diff_seconds;		/* Difference in seconds */

InitScaffold (PERIPHERALS, argc, argv);		/* Start up the scaffold process */

BeginTest (BAT, "CreateTimerIOReq");
IOReqItem = CreateTimerIOReq();
if (IOReqItem < 0)
	{
	LogError (IOReqItem);			/* Has error number when failure */
	LogResult (SETUP_FAILED, "Could not obtain device info for the timer device(s)");
	ret_value = 1;
	return ret_value;							/* Leave the scene if we haven't already */
	}
/* If this fails, we abort.  Otherwise, we call it a pass */
LogResult (PASSED, "We can now use the timer device");

BeginTest (BAT, "OpenBattFolio");
result = OpenBattFolio ();
if (result < 0)
	{
	LogError (result);
	LogResult (SETUP_FAILED, "Could not open 'batt' folio");
	ret_value = 1;
	return ret_value;
	}

/* Now we continue */
LogResult (PASSED, "We can access battery backed up clock");



BeginTest (BAT, "SETTIME_USEC");
	{
	TimeVal	new_time = {123, 456};			/* Set the time to this */
	
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_SETTIME_USEC;
	ioInfo.ioi_Send.iob_Buffer = &new_time;
	ioInfo.ioi_Send.iob_Len = sizeof(new_time);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "SETTIME_USEC failed");
		}
	else
		{
		LogResult (PASSED, "The time in seconds is: %lu.%06lu",
			new_time.tv_sec, new_time.tv_usec);
		}
	}
	/* Indented and blocked to get local variable */
	
BeginTest (BAT, "GETTIME_USEC");
	{
	
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
	ioInfo.ioi_Recv.iob_Buffer = &the_time_s;
	ioInfo.ioi_Recv.iob_Len = sizeof(the_time_s);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "GETTIME_USEC failed");
		}
	else
		{
		LogResult (PASSED, "The time in seconds is: %lu.%06lu",
			the_time_s.tv_sec, the_time_s.tv_usec);
		}
	}
	/* Indented and blocked to get local variable */
	
BeginTest (BAT, "SETTIME_VBL");
	{
	TimeValVBL new_time = 987654321LL;		/* A 64 bit thing */
	
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_SETTIME_VBL;
	ioInfo.ioi_Send.iob_Buffer = &new_time;
	ioInfo.ioi_Send.iob_Len = sizeof(new_time);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "SETTIME_VBL failed");
		}
	else
		{
		LogResult (PASSED, "The time in VBL units isnow set to: %Lu",
			new_time);				/* 'the_time' is a 64 bit thing. */
		}
	}
	/* Indented and blocked to get local variable */

BeginTest (BAT, "GETTIME_VBL");
	{
	
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_GETTIME_VBL;
	ioInfo.ioi_Recv.iob_Buffer = &the_time_v;
	ioInfo.ioi_Recv.iob_Len = sizeof(the_time_v);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "GETTIME_VBL failed");
		}
	else
		{
		LogResult (PASSED, "The time in VBL units is: %Lu",
			the_time_v);			/* 'the_time' is a 64 bit thing. */
		}
	}
	/* Indented and blocked to get local variable */

BeginTest (BAT, "DELAY_VBL");
	{
	int32 diff_time;			/* Holds microseconds */
	
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_DELAY_VBL;
	ioInfo.ioi_Offset = 1;					/* One VBL time */
	result = DoIO (IOReqItem, &ioInfo);
	delta_t (IOReqItem);					/* Init the timer */
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "DELAY_VBL failed");
		}
	else
		{
		/* LogResult (PASSED, "Syncronized with a VBL"); */
		
		/* Now delay another VBL to time it!! */
		result = DoIO (IOReqItem, &ioInfo);
		diff_time = delta_t (IOReqItem);	/* Get the difference */
		if (result < 0)
			{
			LogError (result);		/* Flaw */
			LogResult (FAILED, "DELAY_VBL failed");
			}
		else
			{
			LogResult (PASSED, "TIMERCMD_DELAY_VBL delays %ld microseconds",
					diff_time);
			}
			
		}
	}
	/* Indented and blocked to get local variable */

BeginTest (BAT, "DELAYUNTIL_VBL");
	{
	/* Wait for a couple of VBL's to pass */
	int32 diff_time;
	
	/* Get the time before the delay in VBL's */
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_GETTIME_VBL;
	ioInfo.ioi_Recv.iob_Buffer = &the_time_v;
	ioInfo.ioi_Recv.iob_Len = sizeof(the_time_v);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);
		LogResult (FAILED, "Fetch of time didn't happen\n");
		}
	else
		{
		delta_t (IOReqItem);				/* Init the timer */
		memset (&ioInfo, 0, sizeof(ioInfo));		/* Clear out the request */
		ioInfo.ioi_Command = TIMERCMD_DELAYUNTIL_VBL;
		ioInfo.ioi_Offset = /* This looks wrong!! */ the_time_v + 60;
		result = DoIO (IOReqItem, &ioInfo);			/* Do the wait!! */
		diff_time = delta_t (IOReqItem);			/* Get how long it takes */
		if (result < 0)
			{
			LogError (result);						/* Error condition */
			LogResult (FAILED, "DELAYUNTIL_VBL failed");
			}
		else
			{
			LogComment ("Next time: %lu vbl counts", ioInfo.ioi_Offset);
			LogResult (PASSED, "TIMERCMD_DELAYUNTIL_VBL delayed %ld microseconds (60 VBL's)",
						diff_time);
			}
		}
	}
	/* Block for local var's */
	
	
BeginTest (BAT, "DELAY_USEC");
	{
	int32 diff_time;						/* Holds microseconds */
	TimeVal delay_time = {0, 500000};		/* Half a sec */
	
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_DELAY_USEC;
	ioInfo.ioi_Send.iob_Buffer = &delay_time;
	ioInfo.ioi_Send.iob_Len = sizeof (delay_time);
	result = DoIO (IOReqItem, &ioInfo);
	delta_t (IOReqItem);					/* Init the timer */
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "DELAY_USEC failed\n");
		}
	else
		{
		/* LogResult (PASSED, "Syncronized with a 1/2 sec delay"); */
		
		/* Now delay another .5 sec to time it!! */
		result = DoIO (IOReqItem, &ioInfo);
		diff_time = delta_t (IOReqItem);	/* Get the difference */
		if (result < 0)
			{
			LogError (result);		/* Flaw */
			LogResult (FAILED, "DELAY_USEC failed");
			}
		else
			{
			LogResult (PASSED, "TIMERCMD_DELAY_USEC delays %ld microseconds",
					diff_time);
			}
			
		}
	}
	/* Indented and blocked to get local variable */


BeginTest (BAT, "DELAYUNTIL_USEC");
	{
	/* Wait for a couple of sec's to pass.  Uses the time obtained above. */
	int32 diff_time;
	TimeVal next_time;					/* Will this work?? */
	
	/* Get the time before the delay */
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
	ioInfo.ioi_Recv.iob_Buffer = &the_time_s;
	ioInfo.ioi_Recv.iob_Len = sizeof(the_time_s);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);
		LogResult (FAILED, "Fetch of time didn't happen\n");
		}
	else
		{
		delta_t (IOReqItem);				/* Init the timer */
		memset (&ioInfo, 0, sizeof(ioInfo));		/* Clear out the request */
		ioInfo.ioi_Command = TIMERCMD_DELAYUNTIL_USEC;
		ioInfo.ioi_Send.iob_Buffer = &next_time;
		ioInfo.ioi_Send.iob_Len = sizeof (next_time);
		next_time = the_time_s;				/* Structure assignment */
		next_time.tv_sec += 5;				/* Add 5 seconds to older time */
		result = DoIO (IOReqItem, &ioInfo);			/* Do the wait!! */
		diff_time = delta_t (IOReqItem);			/* Get how long it takes */
		if (result < 0)
			{
			LogError (result);						/* Error condition */
			LogResult (FAILED, "DELAYUNTIL_USEC failed\n");
			}
		else
			{
			LogResult (PASSED, "TIMERCMD_DELAYUNTIL_USEC delayed %ld microseconds",
						diff_time);
			LogComment ("The time we waited for in seconds is: %lu.%06lu",
				next_time.tv_sec, next_time.tv_usec);
				
			/* Get the current time and print it. */
			memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
			ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
			ioInfo.ioi_Recv.iob_Buffer = &the_time_s;
			ioInfo.ioi_Recv.iob_Len = sizeof(the_time_s);
			result = DoIO (IOReqItem, &ioInfo);
			if (result < 0)
				{
				LogError (result);
				LogResult (FAILED, "Fetch of time didn't happen (after the delay)\n");
				}
			else
				{
				LogComment ("The time now in seconds is: %lu.%06lu",
					the_time_s.tv_sec, the_time_s.tv_usec);
				}
			}
		}
	}
	/* Block for local var's */



BeginTest (BAT, "METRONOME_VBL");
	{
	/* Allow 10 steps of 60 VBL's each, print total time */
	int32	signal_number = AllocSignal(0);
	int		i;
	Err		status;
	TimeVal	new_time;
	
	/* Get initial time */
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
	ioInfo.ioi_Recv.iob_Buffer = &the_time_s;
	ioInfo.ioi_Recv.iob_Len = sizeof(the_time_s);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "GETTIME_USEC failed");
		}
	else
		{
		/* Now setup to click in 60 VBL increments */
		memset (&ioInfo, 0, sizeof(ioInfo));
		ioInfo.ioi_Command = TIMERCMD_METRONOME_VBL;
		ioInfo.ioi_Offset = 60;			/* About 1 second in NTSC VBL units */
		ioInfo.ioi_CmdOptions = signal_number;
		/* Now I assume that we can't 'DoIO', we must 'SendIO' */
		SendIO (IOReqItem, &ioInfo);
		/* Now wait for 5 signals */
		for (i = 0; i < 5; i++)
			{
			WaitSignal (signal_number);
			LogComment ("Wait interval is %d", i + 1);
			}
		/* Now Kill off the request */
		status = AbortIO (IOReqItem);
		if (status < 0)
			{
			LogError (status);
			LogResult (FAILED, "Couldn't abort the I/O operation");
			}
		status = WaitIO (IOReqItem);
		if (status < 0 && status != MakeErr (ER_FOLI, 
											 ER_KRNL, 
											 ER_SEVERE,
											 ER_E_SSTM,
											 ER_C_STND,
											 ER_Aborted) /* 0xD57B900A */)
			{
			LogError (status);
			LogResult (FAILED, "Couldn't do wait after abort function");
			}
		/* Now get the new time */
		memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
		ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
		ioInfo.ioi_Recv.iob_Buffer = &new_time;
		ioInfo.ioi_Recv.iob_Len = sizeof(new_time);
		result = DoIO (IOReqItem, &ioInfo);
		if (result < 0)
			{
			LogError (result);
			LogResult (FAILED, "Couldn't get second time value");
			}
		/* Now print out the two times */
		new_time.tv_usec -= the_time_s.tv_usec;
		new_time.tv_sec -= the_time_s.tv_sec;
		if (new_time.tv_usec < 0)
			{
			new_time.tv_usec += 1000000;	/* Plus 1 sec */
			new_time.tv_sec -= 1;			/* Minus 1 sec (a borrow) */
			}
		
		LogResult (PASSED, "METRONOME_VBL:  Difference in time is:  %lu.%06lu (about 5.0)",
			new_time.tv_sec, new_time.tv_usec);
	
		}
	}
	
BeginTest (BAT, "METRONOME_USEC");
	{
	/* Allow 10 steps of 60 VBL's each, print total time */
	int32	signal_number = AllocSignal(0);
	int		i;
	Err		status;
	TimeVal	new_time;
	TimeVal	interval_time = {1, 250000};	/* 1.25 seconds */
	
	/* Get initial time */
	memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
	ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
	ioInfo.ioi_Recv.iob_Buffer = &the_time_s;
	ioInfo.ioi_Recv.iob_Len = sizeof(the_time_s);
	result = DoIO (IOReqItem, &ioInfo);
	if (result < 0)
		{
		LogError (result);		/* Flaw */
		LogResult (FAILED, "GETTIME_USEC failed");
		}
	else
		{
		/* Now setup to click in 60 VBL increments */
		memset (&ioInfo, 0, sizeof(ioInfo));
		ioInfo.ioi_Command = TIMERCMD_METRONOME_USEC;
		ioInfo.ioi_Send.iob_Buffer = &interval_time;
		ioInfo.ioi_Send.iob_Len = sizeof (interval_time);	/* 1.25 seconds */
		ioInfo.ioi_CmdOptions = signal_number;
		/* Now I assume that we can't 'DoIO', we must 'SendIO' */
		SendIO (IOReqItem, &ioInfo);
		/* Now wait for 5 signals */
		for (i = 0; i < 5; i++)
			{
			WaitSignal (signal_number);
			LogComment ("Wait interval is %d", i + 1);
			}
		/* Now Kill off the request */
		status = AbortIO (IOReqItem);
		if (status < 0)
			{
			LogError (status);
			LogResult (FAILED, "Couldn't abort the I/O operation");
			}
		status = WaitIO (IOReqItem);
		if (status < 0 && status != MakeErr (ER_FOLI, 
											 ER_KRNL, 
											 ER_SEVERE,
											 ER_E_SSTM,
											 ER_C_STND,
											 ER_Aborted) /* 0xD57B900A */)
			{
			LogError (status);
			LogResult (FAILED, "Couldn't do wait after abort function");
			}
		/* Now get the new time */
		memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
		ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
		ioInfo.ioi_Recv.iob_Buffer = &new_time;
		ioInfo.ioi_Recv.iob_Len = sizeof(new_time);
		result = DoIO (IOReqItem, &ioInfo);
		if (result < 0)
			{
			LogError (result);
			LogResult (FAILED, "Couldn't get second time value");
			}
		/* Now print out the two times */
		new_time.tv_usec -= the_time_s.tv_usec;
		new_time.tv_sec -= the_time_s.tv_sec;
		if (new_time.tv_usec < 0)
			{
			new_time.tv_usec += 1000000;	/* Plus 1 sec */
			new_time.tv_sec -= 1;			/* Minus 1 sec (a borrow) */
			}
		
		LogResult (PASSED, "METRONOME_USEC:  Difference in time is:  %lu.%06lu (about 6.25)",
			new_time.tv_sec, new_time.tv_usec);
	
		}
	}


BeginTest (BAT, "Time correspondance/ReadBattClock");
	{
	int		test_second;
	TimeVal	first_time;
	TimeVal	second_time;
	int32	usec_diff;
	float	percent_diff;
	/* Wait until the second changed */
	ReadBattClock (&return_date);
	test_second = return_date.gd_Second;		/* Capture second */
	do
		{
		ReadBattClock (&return_date);
		} while (test_second == return_date.gd_Second);	/* Wait until it is different */
	/* The above could fail if the clock doesn't advance.  We assume it does! */
	sec_of_day = return_date.gd_Second + 
				((return_date.gd_Hour * 60) + return_date.gd_Minute) * 60;
	/* Test assumes that we continue in the same day.  It may fail if not! */
	
	delta_t (IOReqItem);				/* Setup for time difference */

#define SEC_INTV 60
	/* Now call the clock getting time until SEC_INTV seconds pass */
	do
		{
		ReadBattClock (&return_date);
		diff_seconds = return_date.gd_Second + 
				((return_date.gd_Hour * 60) + return_date.gd_Minute) * 60;
		diff_seconds -= sec_of_day;		/* Subtract original */
		} while (diff_seconds < SEC_INTV);	/* Wait for SEC_INTV secs!! */
		
	usec_diff = delta_t (IOReqItem);	/* Get difference */
	percent_diff = (float)(usec_diff - (SEC_INTV * 1000000)) / (float)(SEC_INTV * 10000);

	if (usec_diff < 990000 * SEC_INTV || 
		usec_diff > 1010000 * SEC_INTV)		/* between .99 and 1.01 of interval. */
		{
		LogResult (FAILED, "Time difference: %lu microseconds (should be %d secs)",
				usec_diff, SEC_INTV);
		}
	else
		{
		LogResult (PASSED, "Time difference: %lu microseconds (off by %.2f%%)", 
				usec_diff, percent_diff);
		}
	
	}



CloseBattFolio();				/* This shouldn't error anything!! */

TermScaffold();							/* Finish up the scaffold reporting */

return ret_value;				/* Exit from the main routine */
}


static Item CreateTimerIOReq(void)
{
Item  device;
Item  ioreq;
List *list;
Err   result;

result = CreateDeviceStackListVA (&list,
					"cmds", DDF_EQ, DDF_INT, 10,
					TIMERCMD_GETTIME_VBL,
					TIMERCMD_SETTIME_VBL,
					TIMERCMD_DELAY_VBL,
					TIMERCMD_DELAYUNTIL_VBL,
					TIMERCMD_METRONOME_VBL,
					TIMERCMD_GETTIME_USEC,
					TIMERCMD_SETTIME_USEC,
					TIMERCMD_DELAY_USEC,
					TIMERCMD_DELAYUNTIL_USEC,
					TIMERCMD_METRONOME_USEC,
					NULL);
LogComment ("We have %d nodes that are like timer I/O.", GetNodeCount (list));
if (result >= 0)
	{
	/* We have no errors */
	if (!IsEmptyList(list))
		{
		/* At least one device works for us! */
		device = result = OpenDeviceStack((DeviceStack *) FirstNode(list));
		if (device >= 0)
			{
			/* The first device can be opened */
			ioreq = result = CreateIOReq(NULL,0,device,0);
			if (ioreq < 0)
				CloseDeviceStack(device);
			}
		}
	else
		{
		/* We don't have what we need */
		result = MAKEKERR(ER_SEVER,ER_C_STND,ER_NoHardware);
		}
	/* Remove the remnants of the things */
	DeleteDeviceStackList(list);
	}

return result;
}



int32 delta_t (Item IOReqItem)
{
/* Calculate the time difference between this call and the last one. */
/* Note:  the difference can't be over 1.193+ hours, or it will be an incorrect
   answer.  This shouldn't be a problem!! */
   
static TimeVal the_time_start = {0, 0};			/* Holds seconds and microseconds */
static TimeVal the_time_finish = {0, 0};		/* Holds seconds and microseconds */
IOInfo		ioInfo;
uint32		diff_time;
Err			result;

memset (&ioInfo, 0, sizeof(ioInfo));	/* Clear it out */
ioInfo.ioi_Command = TIMERCMD_GETTIME_USEC;
ioInfo.ioi_Recv.iob_Buffer = &the_time_finish;
ioInfo.ioi_Recv.iob_Len = sizeof(the_time_finish);
result = DoIO (IOReqItem, &ioInfo);
if (result < 0)
	{
	LogError (result);		/* Flaw */
	LogComment ("GETTIME_USEC failed in 'delta_t'\n");
	}

/* This is not very optimized, but that's the compilers job!! */
diff_time = the_time_finish.tv_sec - the_time_start.tv_sec;
diff_time *= 1000000;			/* Make microseconds */
diff_time += the_time_finish.tv_usec - the_time_start.tv_usec;

the_time_start = the_time_finish;		/* Structure assignment! */

return diff_time;

}
