/* @(#) Cue_Signals_FT.c 96/07/26 1.2 */
/*	
 *	File:				Cue_Signals_FT.c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the functional tests for the Cue Signal Audio Calls.  This file contains five tests.
 *	Four samples are loaded and attached to sampler playing instruments.  A Cue is created for each instrument. 
 *	The signal provided by the Cue is placed in array as a reference in the tests.  Two cues are created 
 *	to keep an absolute measure of time.  Print out what the signals derived from the cues look like.
 *
 *	Test 1:	Play each sample until it is finished playing.  Check the current signals list to see if
 *			the signal which corresponds to the sample just finished playing matches the signal derived 
 *			from the cue.  Then clear  the current signals.
 *
 *	Test 2:	Play each sample until it is finished playing, but do not clear the current Signals.  Check 
 *			the current signals list to see if the signals are accumulating.
 *
 *	Test 3:	Play each sample until it is finished playing, but clear only the third signal from the current 
 *			signals list..  Check the current signals list to see if the cleared signal has been removed from
 *			current signals list.
 *
 *	Test 4:	Test SignalAtTime.  Call SignalAtTime with a particular signal.  After the specified time has
 *			passed, check to see if the signal is in the current signals list.
 *
 *	Test 5:	Test Trigger by setting the input, reset and set values to force the output to go from
 *			FALSE to TRUE and set the trigger.  Read current signals to see if the trigger signal is present
 *
 *	Audio Functions called:
 *
 *		CloseAudioFolio		CreateCue			CreateAttachment		LoadInstrument			LoadSample
 *		OpenAudioFolio		GetCueSignal		MonitorAttachment		CreateProbeVA			CreateProbeVA
 *		CreateInstrument	UnloadInstrument	UnloadSample			LoadPatchTemplate		WaitSignal
 *		GetCurrentSignals	SleepUntilTime		SignalAtTime			StartInstrument			StopInstrument
 *		SetKnob				DeleteCue			DeleteProbe				UnloadPatchTemplate		ConnectInstrumentParts
 *		DeleteAttachment
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <audio/parse_aiff.h>
#include <audio/patch.h>
#include <audio/patchfile.h>
#include <kernel/task.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>


#define	DURATION	480
/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	
	Err		theError;
	float32	theOutputValue;
	int32	theSignal[4], timeSignal[4], triggerSignal, i, j, theCurrentSignal, theCumulativeSignal, tempSignal, mask;
	Item 	theItem[4], theSample[4], theAttachment[4], theCue[4], sleepCue, timeCue[4], triggerCue,
			triggerItem, rLKnob, sLKnob, theTestProbe, theInputVoltage, constantVoltagePatch, theVoltageKnob;



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT, theError,"OpenAudioFolio",cleanup);
	

	/*	Create the Cues which will be tested		*/
	for ( i = 0; i < 4; i++ )
	{
		theCue[i]	= CreateCue( NULL );
		CHECKRESULT_SETUP(FT,theCue[i],"CreateCue",cleanup);

		theSignal[i]		= GetCueSignal( theCue[i] );

		timeCue[i]	= CreateCue( NULL );
		CHECKRESULT_SETUP(FT,theCue,"CreateCue",cleanup);
	
		timeSignal[i]		= GetCueSignal( timeCue[i] );

	}

	sleepCue	= CreateCue( NULL );
	CHECKRESULT_SETUP(FT,theCue,"CreateCue",cleanup);

	triggerCue	= CreateCue( NULL );
	CHECKRESULT_SETUP(FT,theCue,"CreateCue",cleanup);
	
	triggerSignal		= GetCueSignal( triggerCue );

	for ( i = 0; i < 4; i++ )
	{
		theItem[i]		= LoadInstrument("sampler_16_f1.dsp", 1, 100);
		CHECKRESULT_SETUP(FT,theItem[i],"LoadInstrument(sampler_16_f1.dsp)",cleanup);
	}
			
	theSample[0]		= LoadSample("Cowbell_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,theSample[0],"LoadSample(Cowbell_44K_16B_M.aiff)",cleanup);
	theSample[1]		= LoadSample("Marimba_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,theSample[1],"LoadSample(Marimba_44K_16B_M.aiff)",cleanup);
	theSample[2]		= LoadSample("PanFlute_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,theSample[2],"LoadSample(PanFlute_44K_16B_M.aiff)",cleanup);
	theSample[3]		= LoadSample("Trumpet_44K_16B_M.aiff");
	CHECKRESULT_SETUP(FT,theSample[3],"LoadSample(Trumpet_44K_16B_M.aiff)",cleanup);

	for ( i = 0; i < 4; i++ )
	{
		theAttachment[i]	= CreateAttachment( theItem[i], theSample[i], NULL );
		CHECKRESULT_SETUP(FT,theAttachment[i],"CreateAttachment",cleanup);
		MonitorAttachment( theAttachment[i], theCue[i], CUE_AT_END );
	}
		
	triggerItem			= LoadInstrument("schmidt_trigger.dsp", 1, 100);
	CHECKRESULT_SETUP(FT,triggerItem,"LoadInstrument(schmidt_trigger.dsp)",cleanup);
		
	sLKnob	= CreateKnob( triggerItem, "SetLevel", NULL );
	CHECKRESULT_SETUP(FT,sLKnob,"CreateKnob(schmidt_trigger.dsp: SetLevel)",cleanup);
	
	rLKnob	= CreateKnob( triggerItem, "ResetLevel", NULL );
	CHECKRESULT_SETUP(FT,rLKnob,"CreateKnob(schmidt_trigger.dsp: ResetLevel)",cleanup);


	theTestProbe	= CreateProbeVA( triggerItem, "Output", AF_TAG_TYPE, AF_SIGNAL_TYPE_WHOLE_NUMBER, TAG_END );
	CHECKRESULT_SETUP(FT,theTestProbe,"CreateProbeVA",cleanup);
	
	/*	Create an instrument patch which supplies a constant voltage. Set the voltage to .1	*/
	constantVoltagePatch	= LoadPatchTemplate( "ampout.patch" );
	theInputVoltage			= CreateInstrument (constantVoltagePatch, NULL);
	CHECKRESULT_SETUP(FT,theInputVoltage,"CreateInstrument(ampout.patch)",cleanup);
		
	theVoltageKnob			= CreateKnob( theInputVoltage, "Amp", NULL );
	CHECKRESULT_SETUP(FT, theVoltageKnob, "CreateKnob",cleanup);

	/*	Connect the constant voltage instrument to the schmidt trigger		*/		
	theError	= ConnectInstrumentParts(	theInputVoltage, "Output", 0, triggerItem, "Input", 0 	);
	CHECKRESULT_SETUP(FT, theError, "ConnectInstrumentParts(ampout.patch, Output, schmidt_trigger.dsp, Input)",cleanup);



	/*	Print out what the signals look like	*/
	for ( i = 0; i < 4; i++ )
		LogComment("Signal %d is (hex) %x", i, theSignal[i] );

	/****************************************************************************************************
	 *	Test 1. Play each sample. Wait until after each sample has finished playing.  Check to see what
	 *	the currently received signal is and then clear the currently received signal.
	 ****************************************************************************************************/
	
	for ( i = 0; i < 4; i++ )
	{
		BeginTest(FT, "Verify that a Signal is received after a sample is played");
		LogComment("%d. Playing sample %d", i, i );

		StartInstrument( theItem[i], NULL );							/*	Play sound	*/
		

		SleepUntilTime( sleepCue, GetAudioTime() + DURATION );			/*	Sleep until sample is finished until sample is finished	*/
		
		mask				= theSignal[i];
		theCurrentSignal	= GetCurrentSignals() & mask;
		
	
		if ( theCurrentSignal == theSignal[i] )							/*	Check the current signal	*/
			LogResult(PASSED, "" );
		else
			LogResult(FAILED, "Expected (hex) %x, but got (hex) %x as the current signal", theSignal[i], theCurrentSignal );

		
		theError	= ClearCurrentSignals( theSignal[i] );				/*	Clear received signal bits	*/
		CHECKRESULT_SETUP(FT,theError,"ClearCurrentSignals",cleanup);
		
		
		StopInstrument( theItem[i], NULL );								/*	Stop Playing the sound	*/


	}
		
	/****************************************************************************************************
	 *	Test 2. Play each sample. Wait until after each sample has finished playing.  Check to see what
	 *	the currently received signal is.  Do not clear the currently received signal.
	 ****************************************************************************************************/
	
	for ( i = 0; i < 4; i++ )
	{
		BeginTest(FT, "Verify that a Signals accumulate after the samples are played");
		LogComment("%d. Playing sample %d", i, i );

		StartInstrument( theItem[i], NULL );							/*	Play sound	*/
		

		SleepUntilTime( sleepCue, GetAudioTime() + DURATION );			/*	Sleep until sample is finished until sample is finished	*/
																			
		theCumulativeSignal	= 0;										/*	The signal mask should be the cumulative of previous signals	*/
		for ( j = i; j > -1; j-- )
			theCumulativeSignal = theCumulativeSignal | theSignal[j];
			mask				 = theCumulativeSignal;
			
			
		theCurrentSignal	= GetCurrentSignals() & mask;
		
		if ( theCurrentSignal == theCumulativeSignal )					/*	Check the current signal	*/
			LogResult(PASSED, "" );
		else
			LogResult(FAILED, "Expected (hex) %x, but got (hex) %x as the current signal", theCumulativeSignal, theCurrentSignal );

				
		StopInstrument( theItem[i], NULL );								/*	Stop Playing the sound	*/


	}
		
	/****************************************************************************************************
	 *	Test 3. Play each sample. Wait until after each sample has finished playing.  Check to see what
	 *	the currently received signal is. Clear the third currently received signal.
	 ****************************************************************************************************/
	
	for ( i = 0; i < 4; i++ )
	{
		theError	= ClearCurrentSignals( theSignal[i] );				/*	Clear received signal bits	*/
		CHECKRESULT_SETUP(FT,theError,"ClearCurrentSignals",cleanup);
	}
	
	
	for ( i = 0; i < 4; i++ )
	{
		BeginTest(FT, "Verify that ClearCurrentSignals target appropriate signals");
		LogComment("%d. Playing sample %d", i, i );

		StartInstrument( theItem[i], NULL );							/*	Play sound	*/
		

		SleepUntilTime( sleepCue, GetAudioTime() + DURATION );			/*	Sleep until sample is finished until sample is finished	*/
	
		if ( i == 3 )
		{
			tempSignal	= theSignal[0]|theSignal[1];
			tempSignal	= tempSignal|theSignal[3];
			mask		= tempSignal;
			
			theCurrentSignal	= GetCurrentSignals() & mask;
		
			if ( theCurrentSignal == tempSignal )							/*	Check the current signal	*/
				LogResult(PASSED, "" );
			else
				LogResult(FAILED, "Expected (hex) %x, but got (hex) %x as the current signal",  tempSignal, theCurrentSignal );
		}
		else
		{
			theCumulativeSignal	= 0;										/*	The signal mask should be the cumulative of previous signals	*/
			for ( j = i; j > -1; j-- )
				theCumulativeSignal = theCumulativeSignal | theSignal[j];
			mask				= theCumulativeSignal;
			theCurrentSignal	= GetCurrentSignals() & mask;
			
			
			if ( theCurrentSignal == theCumulativeSignal )							
				LogResult(PASSED, "" );
			else
				LogResult(FAILED, "Expected (hex) %x, but got (hex) %x as the current signal",  theCumulativeSignal, theCurrentSignal );
	
		}


		if ( i == 2 )
		{
			theError	= ClearCurrentSignals( theSignal[i] );				/*	Clear received signal bits	*/
			CHECKRESULT_SETUP(FT,theError,"ClearCurrentSignals",cleanup);
																		/*	Find out what the current signal is		*/
		}
			
		
		StopInstrument( theItem[i], NULL );								/*	Stop Playing the sound	*/


	}
		
	/****************************************************************************************************
	 *	Test 4. Test SignalAtTime and make sure that the signals are getting accumulated in GetCurrentSignals
	 ****************************************************************************************************/
	
	for ( i = 0; i < 4; i++ )
	{
		theError	= ClearCurrentSignals( theSignal[i] );				/*	Clear received signal bits	*/
		CHECKRESULT_SETUP(FT,theError,"ClearCurrentSignals",cleanup);
	}
	

	/*	Print out what the signals look like	*/
	for ( i = 0; i < 4; i++ )
		LogComment("Signal %d is (hex) %x", i, timeSignal[i] );
	

	for ( i = 0; i < 4; i++ )
	{
		SignalAtTime( timeCue[i], GetAudioTime() + DURATION/2 );
		SleepUntilTime( sleepCue, GetAudioTime() + DURATION );			/*	Sleep until sample is finished until sample is finished	*/

																			
		BeginTest(FT, "SignalAtTime");
		theCumulativeSignal	= 0;										/*	The signal mask should be the cumulative of previous signals	*/
		for ( j = i; j > -1; j-- )
			theCumulativeSignal = theCumulativeSignal | timeSignal[j];
		mask				= theCumulativeSignal;
		theCurrentSignal	= GetCurrentSignals() & mask;
		
			
		if ( theCurrentSignal == theCumulativeSignal )					/*	Check the current signal	*/
			LogResult(PASSED, "" );
		else
			LogResult(FAILED, "Expected (hex) %x, but got (hex) %x as the current signal", theCumulativeSignal, theCurrentSignal );
	}
	
	


		
	/****************************************************************************************************
	 *	Test 5. Test Trigger by setting the input, reset and set values to force the output to go from
	 *	FALSE to TRUE and set the trigger.  Read current signals to see if the trigger signal is present
	 ****************************************************************************************************/
	
	
	for ( i = 0; i < 4; i++ )
	{
		theError	= ClearCurrentSignals( timeSignal[i] );				/*	Clear received signal bits	*/
		CHECKRESULT_SETUP(FT,theError,"ClearCurrentSignals",cleanup);
	}
	

	theError = ArmTrigger( triggerItem, NULL, triggerCue, AF_F_TRIGGER_CONTINUOUS );
	CHECKRESULT_SETUP(FT,theError,"ArmTrigger",cleanup);
	

	BeginTest(FT, "Trigger #1");

	/*	Set the values for the input signal and the reset and set knobs	*/
	SetKnob( theVoltageKnob, .1 );
	SetKnob( sLKnob, .6 );
	SetKnob( rLKnob, .3 );

	StartInstrument( theInputVoltage, NULL );
	StartInstrument( triggerItem, NULL );

	SleepUntilTime( sleepCue, GetAudioTime() + 2 );
	ReadProbe( theTestProbe, &theOutputValue );
	
	if ( (Boolean)theOutputValue == 0 )
		LogResult(PASSED, "" );
	else
		LogResult(FAILED, "Output is %d (TRUE) when Input = .1;  SetLevel = .6; ResetLevel = .3." );


	StopInstrument( theInputVoltage, NULL );
	StopInstrument( triggerItem, NULL );

	BeginTest(FT, "Trigger #2");

	/*	Set the values for the input signal and the reset and set knobs	*/
	SetKnob( theVoltageKnob, .8 );
	SetKnob( sLKnob, .6 );
	SetKnob( rLKnob, .3 );

	StartInstrument( theInputVoltage, NULL );
	StartInstrument( triggerItem, NULL );

	SleepUntilTime( sleepCue, GetAudioTime() + 2 );
	ReadProbe( theTestProbe, &theOutputValue );
	
	if ( (Boolean)theOutputValue > 0 )
		LogResult(PASSED, "" );
	else
		LogResult(FAILED, "Output is %d (FALSE) when Input = .8;  SetLevel = .6; ResetLevel = .3." );

	SleepUntilTime( sleepCue, GetAudioTime() + 2 );

	BeginTest(FT, "Trigger #3");

	mask		= triggerSignal;
	/*	Find out what the current signal is		*/
	if ( (GetCurrentSignals()&mask) == triggerSignal )
		LogResult(PASSED, "" );
	else
		LogResult(FAILED, "Expected (hex) %x: The currently received signal is (hex) %x", triggerSignal, GetCurrentSignals() );
	

	StopInstrument( theInputVoltage, NULL );
	StopInstrument( triggerItem, NULL );



	for ( i = 0; i < 4; i++ )
	{
		DeleteAttachment( theAttachment[i] );
		UnloadSample( theSample[i] );
		UnloadInstrument( theItem[i] );
		DeleteCue( theCue[i] );
		DeleteCue( timeCue[i] );
	}
	
	DeleteCue( sleepCue );
	DeleteCue( triggerCue );
	UnloadInstrument( triggerItem );
	UnloadPatchTemplate( constantVoltagePatch );

	/****************************************************************************************************/
	
	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT,theError,"CloseAudioFolio");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

