/*	
 *	File:				DeleteEnvelopeEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the DeleteEnvelope error-checking test. An envelope is created. DepthTest_TopPortion.h is 
 *	then included.  This sets up the loops which allows the test program to call DeleteEnvelope for all possible
 *	combinations of the arguments found in the argument structures defined in DeleteEnvelopeEST_data.h. The test
 *	program indicates if the result returned by DeleteEnvelope with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to DeleteEnvelope
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include <TestMacros.h>
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/
int main(int argc, char ** argv)
{
	#include "DeleteEnvelopeEST_data.h"

	EnvelopeSegment 	theEnvelope[]	= {0, 0,    500,32000,    1000, 15000,    1400, 80000,    1900, 0 };
	int32				numOfPoints		= 5;	
	int32				p[10];	
	int32 				theError;
	Item 				Result;


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST, theError,"OpenAudioFolio()",cleanup);
	
	PARAM_01[0]   = CreateEnvelope( theEnvelope, numOfPoints, NULL );

	#define		expected   0
	#include <DepthTest_TopPortion.h>
	
	if( p[1] > 0 )
	{
		BeginTest(EST, "DeleteEnvelope()");
		Result = DeleteEnvelope(ARGUMENT_01);
		
		if (Result >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from DeleteEnvelope(%d)", 
							Result,ARGUMENT_01);
		else
			LogResult(PASSED, "");
	}
	else
	{
		BeginTest(EST, "DeleteEnvelope()");
		Result = DeleteEnvelope(ARGUMENT_01);
		
		if (Result < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from DeleteEnvelope(%d)", 
							Result,ARGUMENT_01);
			PrintfSysErr(Result);
		}
		else
			LogResult(PASSED, "");
	}

	#include <DepthTest_BottomPortion.h>

	cleanup:
	theError = CloseAudioFolio();
	CHECKRESULT_SETUP(EST, theError,"CloseAudioFolio()",cleanup);

	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

void fu (void){
	LoadInstrument("square.dsp", 1, 100);
}

