/* @(#) ConnectInstrumentsEST.c 96/07/16 1.1 */
/*	
 *	File:				ConnectInstrumentsEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the ConnectInstruments error-checking test.  This program loads two instruments.
 *	The Item numbers returned by the LoadInstrument call is set to the valid srcInstrument and dstInstrument
 *	Item numbers in the srcInstrument and dstInstrument argument structures.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call ConnectInstruments for all possible
 *	combinations of the arguments found in the argument structures defined in ConnectInstrumentsEST_data.h. The test
 *	program indicates if the result returned by ConnectInstruments with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to ConnectInstruments
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */



#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	#include "ConnectInstrumentsEST_data.h"
	
	Err		theError;
	int32	p[10];



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);

	PARAM_01[0]	= LoadInstrument("square.dsp", 1, 100);
	CHECKRESULT_SETUP(EST,PARAM_01[0],"LoadInstrument",cleanup);

	PARAM_02[0]	= "Output";

	PARAM_03[0]	= LoadInstrument("line_out.dsp", 1, 100);
	CHECKRESULT_SETUP(EST,PARAM_03[0],"LoadInstrument",cleanup);

	PARAM_04[0]	= "Input";
	
	#define expected 0
	#include "DepthTest_TopPortion.h"

	if( (p[1] > 0) || (p[2] > 0) || (p[3] > 0) || (p[4] > 0) )
	{
		BeginTest(EST, "ConnectInstruments");
		theError = ConnectInstruments(ARGUMENT_01,ARGUMENT_02,ARGUMENT_03, ARGUMENT_04);
		
		if (theError >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from ConnectInstruments(%d,%s,%d,%s)", 
							theError,ARGUMENT_01,ARGUMENT_02,ARGUMENT_03, ARGUMENT_04);
		else
			LogResult(PASSED, "");

		DisconnectInstrumentParts(ARGUMENT_03, ARGUMENT_04, 0);
	}
	else
	{
		BeginTest(EST, "ConnectInstruments");
		theError = ConnectInstruments(ARGUMENT_01,ARGUMENT_02,ARGUMENT_03, ARGUMENT_04);
		
		if (theError < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from ConnectInstruments(%d,%s,%d,%s)", 
							theError,ARGUMENT_01,ARGUMENT_02,ARGUMENT_03, ARGUMENT_04);
			PrintfSysErr(theError);
		}
		else
			LogResult(PASSED, "");

		DisconnectInstrumentParts(ARGUMENT_03, ARGUMENT_04, 0);
	}


	#include "DepthTest_BottomPortion.h"
	
	
	cleanup:
	UnloadInstrument(PARAM_01[0]);
	UnloadInstrument(PARAM_03[0]);

	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

