/* @(#) DeleteInstrumentEST.c 96/07/26 1.2 */
/*	
 *	File:				DeleteInstrumentEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the DeleteInstrument error-checking test.  This program loads an instrument template.
 *	The Item number returned by the LoadInsTemplate call is used by CreateInstrument to create an instrument
 *	The Item number returned by CreateInstrument is assigned to the valid instrumentItem argument in the 
 *	instrumentItem argument structures.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call DeleteInstrument for all possible
 *	combinations of the arguments found in the argument structures defined in DeleteInstrumentEST_data.h. The test
 *	program indicates if the result returned by DeleteInstrument with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to DeleteInstrument
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */


#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	#include "DeleteInstrumentEST_data.h"

	Err		theError;
	int32	p[10];
	Item	insTemplateItem;



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError	= OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);
	
	insTemplateItem = LoadInsTemplate("square.dsp",0);
	CHECKRESULT_SETUP(EST, insTemplateItem,"LoadInsTemplate",cleanup);

	PARAM_01[0] 	= CreateInstrument(insTemplateItem, NULL);
	CHECKRESULT_SETUP(EST, PARAM_01[0],"CreateInstrument",cleanup);

	#define expected 0
	#include "DepthTest_TopPortion.h"

	if ( p[1] > 0 )
	{
		BeginTest(EST, "DeleteInstrument");
		theError = DeleteInstrument(ARGUMENT_01);
		
		if (theError >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from DeleteInstrument(%d)", 
							theError,ARGUMENT_01);
		else
			LogResult(PASSED, "");
	}
	else
	{
		BeginTest(EST, "DeleteInstrument");
		theError = DeleteInstrument(ARGUMENT_01);
		
		if (theError < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from DeleteInstrument(%d)", 
							theError,ARGUMENT_01);
			PrintfSysErr(theError);
		}
		else
			LogResult(PASSED, "");
	}

	#include "DepthTest_BottomPortion.h"

	UnloadInsTemplate(insTemplateItem);

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

