/* @(#) LoadInsTemplateEST.c 96/07/16 1.1 */
/*	
 *	File:				LoadInsTemplateEST.c
 *	Author:				Eric Matsuno
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the LoadInsTemplate error-checking test.  This program loads an instrument template
 *	according to the arguments in the argument structure in LoadInsTemplateEST_data.h.   DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call LoadInsTemplate for all possible
 *	combinations of the arguments found in the argument structures defined in LoadInsTemplateEST_data.h. The test
 *	program indicates if the result returned by LoadInsTemplate with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to LoadInsTemplate
 *	should return no error.  All other calls should return an error.)  After the first test, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *	The second test tries to load templates for all of the dsp instruments in the Audio Folio. Every dsp instrument
 *	is listed in the ARGUMENT_01 argument structure. If  LoadInsTemplate fails, the test fails.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */


#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/

int main (int argc, char ** argv)
{
	#include "LoadInsTemplateEST_data.h"
	
	Err	theError;
	int32	p[10];
	Item Result;



	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);
	
	#define expected 0
	#include "DepthTest_TopPortion.h"

	if( (p[1] > 0) || (p[2] > 0) )
	{
		BeginTest(EST, "LoadInsTemplate( %s )", ARGUMENT_01);
		Result = LoadInsTemplate(ARGUMENT_01,ARGUMENT_02);
		
		if (Result >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from LoadInsTemplate(%s, NULL)", 
							Result,ARGUMENT_01);
		else
			LogResult(PASSED, "");
	}
	else
	{
		BeginTest(EST, "LoadInsTemplate( %s )", ARGUMENT_01);
		Result = LoadInsTemplate(ARGUMENT_01,ARGUMENT_02);
		
		if (Result < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from LoadInsTemplate(%s, NULL)", 
							Result,ARGUMENT_01);
			PrintfSysErr(Result);
		}
		else
			LogResult(PASSED, "");

	}

	UnloadInsTemplate(Result);
	
	#include "DepthTest_BottomPortion.h"
	
	for ( p[1] = 0, p[2] = 0; p[1] < 56; p[1]++ )
	{
		/*	Test all instruments with the proper arguments except for the one with bogus name
		*/
		if ( p[1] == 15 )		/*	line_in.dsp	*/
		{
			theError	= EnableAudioInput( 1, NULL );
			
			if ( theError >= 0 )
			{
				BeginTest(EST, "LoadInsTemplate( %s )", ARGUMENT_01);
				Result = LoadInsTemplate(ARGUMENT_01,ARGUMENT_02);
		
				if (Result < expected)
				{
					LogResult(FAILED, "Expected: Positive return, but got (hex)%x from LoadInsTemplate(%s, NULL)", 
									Result,ARGUMENT_01);
					PrintfSysErr(Result);
				}
				else
					LogResult(PASSED, "");
	
				UnloadInsTemplate(Result);
			}
			else
			{
				LogComment("FAILED: EnableAudioInput call returned (hex)%x", theError );
				PrintfSysErr(theError);
			}
		}
		else if ( p[1] != 1 )
		{
			BeginTest(EST, "LoadInsTemplate( %s )", ARGUMENT_01);
			Result = LoadInsTemplate(ARGUMENT_01,ARGUMENT_02);
		
			if (Result < expected)
			{
				LogResult(FAILED, "Expected: Positive return, but got (hex)%x from LoadInsTemplate(%s, NULL)", 
								Result,ARGUMENT_01);
				PrintfSysErr(Result);
			}
			else
				LogResult(PASSED, "");

			UnloadInsTemplate(Result);
		}
	}		

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

