/* @(#) ResumeInstrumentEST.c 96/07/26 1.2 */
/*	
 *	File:				ResumeInstrumentEST.c
 *	Author:				Eric Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains the ResumeInstrument error-checking test.  This program loads an instrument.
 *	The Item number returned by the LoadInstrument call is set to the valid Instrument
 *	Item number in the Instrument argument structure.  DepthTest_TopPortion.h is then
 *	included.  This sets up the loops which allows the test program to call ResumeInstrument for all possible
 *	combinations of the arguments found in the argument structures defined in ResumeInstrumentEST_data.h. The test
 *	program indicates if the result returned by ResumeInstrument with that certain configuration of arguments
 *	is correct.  (The first argument in each argument structure is valid.  The first call to ResumeInstrument
 *	should return no error.  All other calls should return an error.)  After the tests, DepthTest_BottomPortion.h
 *	is included to close all of the loops.
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include "TestMacros.h"
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/
/******************************************************************************/

int main (int argc, char ** argv)
{
	#include "ResumeInstrumentEST_data.h"
	
	Err		theError, Result;
	int32	p[10];

	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(EST,theError,"OpenAudioFolio",cleanup);
	
	PARAM_01[0] = LoadInstrument("square.dsp",0,100);

	#define expected 0
	#include "DepthTest_TopPortion.h"
	
	if( p[1] > 0 )
	{
		
		BeginTest(EST, "ResumeInstrument");
		Result = ResumeInstrument(ARGUMENT_01);
				
		if (Result >= expected)
			LogResult(FAILED, "Expected: Negative return, but got %d from ResumeInstrument(%d)", 
							Result,ARGUMENT_01);
		else
			LogResult(PASSED, "");
	}
	else
	{
			
		theError	= StartInstrument(ARGUMENT_01,NULL);
		CHECKRESULT(EST, theError,"StartInstrument");
		
		theError	= PauseInstrument(ARGUMENT_01);
		CHECKRESULT(EST, theError,"PauseInstrument");
			
		BeginTest(EST, "ResumeInstrument");
		Result = ResumeInstrument(ARGUMENT_01);
				
		if (Result < expected)
		{
			LogResult(FAILED, "Expected: Positive return, but got (hex)%x from ResumeInstrument(%d)", 
							Result,ARGUMENT_01);
			PrintfSysErr(Result);
		}
		else
			LogResult(PASSED, "");
	}

	#include "DepthTest_BottomPortion.h"
	
	cleanup:
	(void)UnloadInstrument(PARAM_01[0]);
	
	theError	= CloseAudioFolio();
	CHECKRESULT(EST,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

