/* @(#) InsPortInfoFT.c 96/07/17 1.1 */
/* 
 *	File:				InsPortInfoFT.c
 *	Author:				Eric Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This file contains calls to GetNumInstrumentPorts and GetInstrumentPortInfoByIndex.  This program loads every
 *	dsp instrument and calls GetNumInstrumentPorts to get the number of ports.  Then the number of ports is used
 *	as an index to GetInstrumentPortInfoByIndex, which returns specific information about each port.
 *
 *
 *	Uses the following calls:
 *
 *	OpenAudioFolio	 		CloseAudioFolio			LoadInstrument		GetNumInstrumentPorts
 *	GetInstrumentPortInfoByIndex					UnloadInstrument
 *
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */


#include <audio/audio.h>
#include <kernel/operror.h>
#include <stdio.h>

#include <TestMacros.h>
#include <tests/3do_scaf.h>

/**************************************Test Environment Globals****************/

int main (int argc, char ** argv)
{
	#include "InsPortInfoFT_data.h"
	
	Err						theError;
	int32					ivResult;
	int32					p[10], numOfPorts;
	Item 					Result;
	InstrumentPortInfo		info;
	uint32 					infoSize	= sizeof(InstrumentPortInfo), i;


	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	theError = OpenAudioFolio();
	CHECKRESULT_SETUP(FT,theError,"OpenAudioFolio",cleanup);
	
	#define expected 0

	BeginTest(FT, "GetNumInstrumentPorts/ GetInstrumentPortInfoByIndex");
	for ( p[1] = 0, p[2] = 0; p[1] < 56; p[1]++ )
	{
		/*	Test all instruments with the proper arguments except for the one with bogus name
		*/
		if ( p[1] == 15 )		/*	line_in.dsp	*/
		{
			theError	= EnableAudioInput( 1, NULL );
			
			if ( theError >= 0 )
			{
				Result = LoadInstrument(ARGUMENT_01, 1, 100);
				LogComment("LoadInstrument( %s ) returned %d (0x%x)", ARGUMENT_01, Result, Result);
		
				numOfPorts	= GetNumInstrumentPorts ( Result );
				LogComment("GetNumInstrumentPorts( %s ) returned %d (0x%x)", ARGUMENT_01, numOfPorts, numOfPorts );
				
				for ( i = 0; i < numOfPorts; i++ )
				{
					theError	= GetInstrumentPortInfoByIndex ( &info, infoSize, Result, i);
					LogComment("GetInstrumentPortInfoByIndex( %s ) returned %d (0x%x)",  ARGUMENT_01, theError, theError);
					
					LogComment("Name: %s\t\tPort Type: %d\t\tSignal Type: %d\t\tNumber of parts: %d", info.pinfo_Name, info.pinfo_Type, info.pinfo_SignalType, info.pinfo_NumParts );
				}
				printf("\n");
				
				UnloadInstrument(Result);
			}
			else
				LogComment("FAILED: EnableAudioInput call returned (hex)%x", theError );
		}
		else if ( p[1] != 1 )
		{
			Result = LoadInstrument(ARGUMENT_01, 1, 100);
			LogComment("LoadInstrument( %s ) returned %d (0x%x)", ARGUMENT_01, Result, Result);
		
			numOfPorts	= GetNumInstrumentPorts ( Result );
			LogComment("GetNumInstrumentPorts( %s ) returned %d (0x%x)", ARGUMENT_01, numOfPorts, numOfPorts );

			LogComment("GetNumInstrumentPorts( %s ) returned %d", ARGUMENT_01, numOfPorts );
				
			for ( i = 0; i < numOfPorts; i++ )
			{
				theError	= GetInstrumentPortInfoByIndex ( &info, infoSize, Result, i);
				LogComment("GetInstrumentPortInfoByIndex( %s ) returned %d (0x%x)",  ARGUMENT_01, theError, theError);
					
				LogComment("Name: %s\t\tPort Type: %d\t\tSignal Type: %d\t\tNumber of parts: %d", info.pinfo_Name, info.pinfo_Type, info.pinfo_SignalType, info.pinfo_NumParts );
			}
			printf("\n");

				
			UnloadInstrument(Result);
		}
	}		

	LogResult(InteractiveVerify("Port Information should match documented Audio specifications. Did Test Pass?"), "");

	cleanup:
	theError	= CloseAudioFolio();
	CHECKRESULT(FT,theError,"CloseAudioFolio()");


	/******************************************************************************/
	TermScaffold();
	
	return 1;
}

