/* @(#) BasicMachFT.c 96/07/26 1.2 */
/*	
 *	File:				BasicMachFT.c
 *	Author:				Eric Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	3/15/96
 *
 *	Version:			M2 2.0
 *
 *	This test checks the functionality of the Beep Folio.  It loads the basic_machine
 *	and assigns a sample to each of the channels.  The user can use the control pad
 *	to hear the samples on each channel and to change the voice parameters of each channel.
 *	Adjusting a parameter which does not belong to a voice
 *
 *	Control Pad Button						Function
 *
 *	Start									StartBeepChannel
 *	RightShift-Start						StopBeepChannel
 *	X										Exit Test Program
 *	A										Adjust Amplitude Mode
 *	B										Adjust Sample Rates Mode
 *	C										Adjust LFO Rates Mode
 *	RightShift-A							Adjust Amplitude Modulation Depth Mode
 *	RightShift-B							Adjust Frequency Modulation Depth Mode
 *	RightShift-C							Adjust Amplitude Rates Mode
 *	LefttShift-Start						Print out channel information
 *	LefttShift-A							Adjust stereo Left Amplitude							
 *	LefttShift-B							Adjust stereo Right Amplitude
 *	LefttShift-C							Adjust Global Amplitude
 *	LefttShift-RightShift-A					Adjust Filter Cutoff Mode (Voice 32 only)
 *	LefttShift-RightShift-B					Adjust Cutoff Modulation Depth (Voice 32 only)
 *	LefttShift-RightShift-C					Print out the Beep Time
 *	Up										Increase value of mode item
 *	Down									Decrease value of mode item
 *	Left									Goto Previous channel
 *	Right									Goto next channel
 *
 *
 *	Audio Functions called:
 *
 *		ConfigureBeepChannel		GetBeepTime				LoadBeepMachine		SetBeepChannelData
 *		SetBeepChannelDataNext		SetBeepVoiceParameter	StartBeepChannel	StopBeepChannel
 *		UnloadBeepMachine			GetAIFFSampleInfo
 *
 *  Copyright (C) 1996, an unpublished work by The 3DO Company.
 *  All rights reserved. This material contains confidential
 *  information that is the property of The 3DO Company. Any
 *  unauthorized duplication, disclosure or use is prohibited.
 *  
 */

#include <audio/parse_aiff.h>
#include <beep/beep.h>
#include <beep/basic_machine.h>
#include <loader/loader3do.h>
#include <misc/event.h>
#include <stdio.h>

#include <TestMacros.h>
#include <tests/3do_scaf.h>

#define	kAmp			0
#define	kSampRate		1
#define	kLFORate		2
#define	kAmpModDepth	3
#define	kFreqModDepth	4
#define	kAmpRate		5
#define kLeftAmp		6
#define kRightAmp		7
#define kGlobalAmp		8
#define kFilterCutoff	9
#define kCutoffModDepth	10

float32						gAmp[33];			/*	-1.0 to 1.0			*/
float32						gSampRate[33];		/*	0.0 to 88200.0		*/
float32						gLFORate[33];		/*	-2756.0 to 2756.0	*/
float32						gAmpModDepth[33];	/*	-1.0 to 1.0			*/
float32						gFreqModDepth[33];	/*	-44100 to 44100		*/
float32						gAmpRate[33];		/*	0 to 1.0			*/
float32						gLeftAmp[33];
float32						gRightAmp[33];
float32						gGlobalAmp;
float32						gFilterCutoff;		/*	-1.0 to 1.0			*/
float32						gCutoffModDepth;	/*	-1.0 to 1.0			*/

int32						gSignal1,
							gSignal2,
							gSignal3;

int							gSetting	= kAmp;
SampleInfo 					*sampleInfo1, *sampleInfo2, *sampleInfo3, *sampleInfo4, *sampleInfo5;
uint32						channelNum	= 0;

void DoThisButton( void );


int main(int argc, char ** argv)
{
	Err					theError;
	int					i;
	Item 				beepMachine;
	

	InitScaffold(AUDIO, argc, argv);					/* initializes this program */
	/******************************************************************************/

	gSignal1	= AllocSignal(0);
	gSignal2	= AllocSignal(0);
	gSignal3	= AllocSignal(0);

	/* Initialize the EventBroker. */
	theError = InitEventUtility(1, 0, LC_FocusListener);
	CHECKRESULT_SETUP(FT,theError,"InitEventUtility",cleanup);

	theError	= OpenBeepFolio();
	CHECKRESULT_SETUP(FT,theError,"OpenBeepFolio()",cleanup);

	beepMachine	= LoadBeepMachine( BEEP_MACHINE_NAME );
	CHECKRESULT_SETUP(FT,beepMachine,"LoadBeepMachine",cleanup);
		

	/* Get SampleInfo from AIFF file including the sample data */
	theError	= GetAIFFSampleInfo (&sampleInfo5, "LongFile_44K_16B_M.AIFF", 0);
	CHECKRESULT_SETUP(FT,theError,"GetAIFFSampleInfo(LongFile_44K_16B_M.AIFF)",cleanup);

	theError	= GetAIFFSampleInfo (&sampleInfo1, "44K_16B_M.AIFF", 0);
	CHECKRESULT_SETUP(FT,theError,"GetAIFFSampleInfo(44K_16B_M.AIFF)",cleanup);

	theError	= GetAIFFSampleInfo (&sampleInfo2, "Marimba_44K_16B_M.aiff", 0);
	CHECKRESULT_SETUP(FT,theError,"GetAIFFSampleInfo(Marimba_44K_16B_M.aiff)",cleanup);

	theError	= GetAIFFSampleInfo (&sampleInfo3, "PanFlute_44K_16B_M.aiff", 0);
	CHECKRESULT_SETUP(FT,theError,"GetAIFFSampleInfo(PanFlute_44K_16B_M.aiff)",cleanup);

	theError	= GetAIFFSampleInfo (&sampleInfo4, "Trumpet_44K_16B_M.aiff", 0);
	CHECKRESULT_SETUP(FT,theError,"GetAIFFSampleInfo(Trumpet_44K_16B_M.aiff)",cleanup);


	while ( channelNum < 32 )
	{
	
	/************************************************************************************************************/
	/*	Play 44K_16B_M.AIFF for channels 0, 4, 8, 12, 16, 20, 24, 28											*/
	/************************************************************************************************************/
		theError	= SetBeepChannelData( channelNum, sampleInfo1->smpi_Data, sampleInfo1->smpi_NumFrames );
		CHECKRESULT(FT,theError,"SetBeepChannelData(44K_16B_M.AIFF)");
		
		theError	= ConfigureBeepChannel( channelNum, 0);
		CHECKRESULT(FT,theError,"ConfigureBeepChannel()");
	
		SetBeepChannelData( channelNum, sampleInfo1->smpi_Data, sampleInfo1->smpi_NumFrames );
		StartBeepChannel( channelNum );
		StopBeepChannel( channelNum );
		
		theError	= SetBeepVoiceParameter( channelNum++, BMVP_AMPLITUDE,  1.0 );
		CHECKRESULT(FT,theError,"SetBeepVoiceParameter(channelNum, BMVP_AMPLITUDE,  1.0)");


	/************************************************************************************************************/
	/*	Play LongFile_44K_16B_M.AIFF for channels 1, 5, 9, 13, 17, 21, 25, 29									*/
	/************************************************************************************************************/
		theError	= SetBeepChannelData( channelNum, sampleInfo5->smpi_Data, sampleInfo5->smpi_NumFrames );
		CHECKRESULT(FT,theError,"SetBeepChannelData(LongFile_44K_16B_M.AIFF)");
		
		theError	= ConfigureBeepChannel( channelNum, 0);
		CHECKRESULT(FT,theError,"ConfigureBeepChannel()");

		SetBeepChannelData( channelNum, sampleInfo5->smpi_Data, sampleInfo5->smpi_NumFrames );
		StartBeepChannel( channelNum );
		StopBeepChannel( channelNum );
		
		theError	= SetBeepVoiceParameter( channelNum++, BMVP_AMPLITUDE,  1.0 );
		CHECKRESULT(FT,theError,"SetBeepVoiceParameter(channelNum, BMVP_AMPLITUDE,  1.0)");


	/************************************************************************************************************/
	/*	Loop three samples continuously for channels 2, 6, 10, 14, 18, 22, 26, 30								*/
	/************************************************************************************************************/
		theError	= SetBeepChannelData( channelNum, sampleInfo2->smpi_Data, sampleInfo2->smpi_NumFrames );
		CHECKRESULT(FT,theError,"SetBeepChannelData(Marimba_44K_16B_M.aiff)");
		theError	= SetBeepChannelDataNext( channelNum, sampleInfo3->smpi_Data, sampleInfo3->smpi_NumFrames, 0, gSignal1 );
		CHECKRESULT(FT,theError,"SetBeepChannelData(PanFlute_44K_16B_M.aiff)");
		
		theError	= ConfigureBeepChannel( channelNum, 0);
		CHECKRESULT(FT,theError,"ConfigureBeepChannel()");
	
		SetBeepChannelData( channelNum, sampleInfo2->smpi_Data, sampleInfo2->smpi_NumFrames );
		StartBeepChannel( channelNum );
		StopBeepChannel( channelNum );
		
		theError	= SetBeepVoiceParameter( channelNum++, BMVP_AMPLITUDE,  1.0 );
		CHECKRESULT(FT,theError,"SetBeepVoiceParameter(channelNum, BMVP_AMPLITUDE,  1.0)");

		
	/************************************************************************************************************/
	/*	Loop one samples continuously for channels 3, 7, 11, 15, 19, 23, 27, 31									*/
	/************************************************************************************************************/
		theError	= SetBeepChannelDataNext( channelNum, sampleInfo4->smpi_Data, sampleInfo4->smpi_NumFrames, BEEP_F_IF_GO_FOREVER, 0 );
		CHECKRESULT(FT,theError,"SetBeepChannelData(Trumpet_44K_16B_M.aiff, BEEP_F_IF_GO_FOREVER)");
		
		theError	= ConfigureBeepChannel( channelNum, 0);
		CHECKRESULT(FT,theError,"ConfigureBeepChannel()");
	
		SetBeepChannelData( channelNum, sampleInfo4->smpi_Data, sampleInfo4->smpi_NumFrames );
		StartBeepChannel( channelNum );
		StopBeepChannel( channelNum );
		
		theError	= SetBeepVoiceParameter( channelNum++, BMVP_AMPLITUDE,  1.0 );
		CHECKRESULT(FT,theError,"SetBeepVoiceParameter(channelNum, BMVP_AMPLITUDE,  1.0)");

		}

	/*	Set the amplitude to 0.0 for the filtered noise voice													*/
	
	theError	= SetBeepVoiceParameter( channelNum, BMVP_AMPLITUDE,  0.0 );
	CHECKRESULT(FT,theError,"SetBeepVoiceParameter(channelNum, BMVP_AMPLITUDE,  0.0)");

	theError	= StartBeepChannel( 0 );
	CHECKRESULT(FT,theError,"StartBeepChannel(0)");

	printf("If you do not hear anything at this point in the test, or if the voice sounds distorted or scratchy;\n");
	printf("then try doing the following with each channel:\n\n");
	printf("1.\tSelect the channel.\t\t\t\t\tPress Control Pad's Left or Right Button.\n");
	printf("2.\tStart the channel.\t\t\t\t\tPress Control Pad's Start Button.\n");
	printf("2.\tLower the channel's amplitude to 0.\tPress Control Pad's A Button followed by the Down Button.\n");
	printf("3.\tStop the channel.\t\t\t\t\tPress Control Pad's RightShift-Start Button.\n\n");
	printf("Do the previous three steps for all of the channels. When you are finished, select the channel\n");
	printf("to which you wish to listen, raise the volume to 1.0 and then start the channel.\n\n");
	printf("Perform the preceding steps only if you have not heard anything. See the M2 2.0 Release notes for more information.\n\n");
	channelNum		= 0;

	for ( i = 0; i < 33; i++ )
	{
		gAmp[i]				= 1.0;		/*	-1.0 to 1.0			*/
		gSampRate[i]		= 44100.0;	/*	0.0 to 88200.0		*/
		gLFORate[i]			= 1.0;		/*	-2756.0 to 2756.0	*/
		gAmpModDepth[i]		= 0.0;		/*	-1.0 to 1.0			*/
		gFreqModDepth[i]	= 0.0;		/*	-44100 to 44100		*/
		gAmpRate[i]			= .002;		/*	0 to 1.0			*/
		gLeftAmp[i]			= .125;
		gRightAmp[i]		= .125;
	}
	gGlobalAmp		= 1.0;
	gFilterCutoff	= .25;				/*	-1.0 to 1.0			*/
	gCutoffModDepth	= 0.0;				/*	-1.0 to 1.0			*/

	BeginTest(FT, "BeepFolio");
	DoThisButton();
	LogResult(InteractiveVerify("Did the Beep Machine perform according to specifications? Did Test Pass?"), "");

	KillEventUtility();

	
	DeleteSampleInfo( sampleInfo1 );
	DeleteSampleInfo( sampleInfo2 );
	DeleteSampleInfo( sampleInfo3 );
	DeleteSampleInfo( sampleInfo4 );
	DeleteSampleInfo( sampleInfo5 );
	
	
	theError	= UnloadBeepMachine( beepMachine );
	CHECKRESULT_SETUP(FT,theError,"UnloadBeepMachine",cleanup);

	cleanup:
	theError	= CloseBeepFolio();
	CHECKRESULT(FT,theError,"CloseBeepFolio()");
	

	/******************************************************************************/
	TermScaffold();
	
	return 1;
}


void DoThisButton()
{
	ControlPadEventData cped;
	Err					theError;

	do
	{
		GetControlPad (1, TRUE, &cped);   /* Wait for button press. */
		
		if ( (cped.cped_ButtonBits & (ControlA | ControlLeftShift | ControlRightShift)) == (ControlA | ControlLeftShift | ControlRightShift) )			/*	Filter Cutoff	*/
		{
			LogComment("Adjust Filter Cutoff");
			gSetting	= kFilterCutoff;
		} 
		else if ( (cped.cped_ButtonBits & (ControlB | ControlLeftShift | ControlRightShift)) == (ControlB | ControlLeftShift | ControlRightShift) )			/*	Filter Cutoff Modulation	*/
		{
			LogComment("Adjust Filter Cutoff Modulation");
			gSetting	= kCutoffModDepth;
		} 
		else if ( (cped.cped_ButtonBits & (ControlC | ControlLeftShift | ControlRightShift)) == (ControlC | ControlLeftShift | ControlRightShift) )			/*	Beep Time	*/
		{
			LogComment("The Beep Time is %d", GetBeepTime() ); 
		} 
		else if ( (cped.cped_ButtonBits & (ControlStart | ControlLeftShift)) == (ControlStart | ControlLeftShift) )		/*	Stop Channel	*/
		{
			printf("Channel %d.\t\tAmplitude:\t\t%f\n", channelNum, gAmp[channelNum]);
			printf("Channel %d.\t\tSample Rate:\t\t%f\n", channelNum, gSampRate[channelNum]);
			printf("Channel %d.\t\tAmplitude Rate:\t\t%f\n", channelNum, gAmpRate[channelNum]);
			printf("Channel %d.\t\tLFO rate:\t\t%f\n", channelNum, gLFORate[channelNum]);
			printf("Channel %d.\t\tAmplitude Modulation Depth:\t\t%f\n", channelNum, gAmpModDepth[channelNum]);
			printf("Channel %d.\t\tFrequency Modulation Depth:\t\t%f\n", channelNum, gFreqModDepth[channelNum]);
		}
		else if ( (cped.cped_ButtonBits & (ControlA | ControlLeftShift)) == (ControlA | ControlLeftShift) )			/*	Left Channel Amplitude	*/
		{
			LogComment("Adjust Left Channel Amplitude");
			gSetting	= kLeftAmp;
		}
		else if ( (cped.cped_ButtonBits & (ControlB | ControlLeftShift)) == (ControlB | ControlLeftShift) )			/*	Right Channel Amplitude	*/
		{
			LogComment("Adjust Right Channel Amplitude");
			gSetting	= kRightAmp;
		}
		else if ( (cped.cped_ButtonBits & (ControlC | ControlLeftShift)) == (ControlC | ControlLeftShift) )			/*	Beep Time	*/
		{
			LogComment("Adjust Global Amplitude");
			gSetting	= kGlobalAmp;
		}
		else if ( (cped.cped_ButtonBits & (ControlStart | ControlRightShift)) == (ControlStart | ControlRightShift) )		/*	Stop Channel	*/
		{
			theError	= StopBeepChannel( channelNum );
			LogComment("Stop Channel %d.\t\tStopBeepChannel returned %d", channelNum, theError);
		}
		else if ( (cped.cped_ButtonBits & (ControlA | ControlRightShift)) == (ControlA  | ControlRightShift ) )			/*	Amp Mod Depth Setting	*/
		{
			LogComment("Adjust Amplitude Modulation Depths");
			gSetting	= kAmpModDepth;
		}
		else if ( (cped.cped_ButtonBits & (ControlB | ControlRightShift)) == (ControlB  | ControlRightShift ) )			/*	Freq Mod Depth Setting	*/
		{
			LogComment("Adjust Frequency Modulation Depths");
			gSetting	= kFreqModDepth;
		}
		else if ( (cped.cped_ButtonBits & (ControlC | ControlRightShift)) == (ControlC  | ControlRightShift ) )			/*	Amp Rate Depth Setting	*/
		{
			LogComment("Adjust Amplitude Rates");
			gSetting	= kAmpRate;
		}
		else if ( (cped.cped_ButtonBits & ControlStart) == ControlStart )		/*	Start Channel	*/
		{
			if ( channelNum == 32)
			{
				theError	= SetBeepChannelData( channelNum, sampleInfo1->smpi_Data, sampleInfo1->smpi_NumFrames );
				
				if ( theError < 0 )
					LogComment("PASSED: SetBeepChannelData returned %d for channel 32", theError);
				else
					LogComment("FAILED: SetBeepChannelData returned %d for channel 32", theError);
			
				theError	= StartBeepChannel( channelNum );

				if ( theError < 0 )
					LogComment("PASSED: StartBeepChannel returned %d for channel 32", theError);
				else
					LogComment("FAILED: StartBeepChannel returned %d for channel 32", theError);
			}
			else if ( channelNum == 0)
			{
				SetBeepChannelData( channelNum, sampleInfo1->smpi_Data, sampleInfo1->smpi_NumFrames );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
			else if ( channelNum == 1)
			{
				SetBeepChannelData( channelNum, sampleInfo5->smpi_Data, sampleInfo5->smpi_NumFrames );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
			else if ( channelNum == 2)
			{
				SetBeepChannelData( channelNum, sampleInfo2->smpi_Data, sampleInfo2->smpi_NumFrames );
				SetBeepChannelDataNext( channelNum, sampleInfo3->smpi_Data, sampleInfo3->smpi_NumFrames, 0, gSignal1 );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
				
				WaitSignal( gSignal1 );
				SetBeepChannelDataNext( channelNum, sampleInfo4->smpi_Data, sampleInfo4->smpi_NumFrames, 0, gSignal2 );
				WaitSignal( gSignal2 );
				SetBeepChannelDataNext( channelNum, sampleInfo2->smpi_Data, sampleInfo2->smpi_NumFrames, 0, gSignal3 );
			}
			else if ( channelNum == 3)
			{
				SetBeepChannelDataNext( channelNum, sampleInfo4->smpi_Data, sampleInfo4->smpi_NumFrames, BEEP_F_IF_GO_FOREVER, 0 );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
			else if ( channelNum%4 == 0)
			{
				SetBeepChannelData( channelNum, sampleInfo1->smpi_Data, sampleInfo1->smpi_NumFrames );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
			else if ( (channelNum-1)%4 == 0)
			{
				SetBeepChannelData( channelNum, sampleInfo5->smpi_Data, sampleInfo5->smpi_NumFrames );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
			else if ( (channelNum-2)%2 == 0)
			{
				SetBeepChannelData( channelNum, sampleInfo2->smpi_Data, sampleInfo2->smpi_NumFrames );
				SetBeepChannelDataNext( channelNum, sampleInfo3->smpi_Data, sampleInfo3->smpi_NumFrames, 0, gSignal1 );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
				
				WaitSignal( gSignal1 );
				SetBeepChannelDataNext( channelNum, sampleInfo4->smpi_Data, sampleInfo4->smpi_NumFrames, 0, gSignal2 );
				WaitSignal( gSignal2 );
				SetBeepChannelDataNext( channelNum, sampleInfo2->smpi_Data, sampleInfo2->smpi_NumFrames, 0, gSignal3 );
			}
			else if ( (channelNum-3)%2 == 0)
			{
				SetBeepChannelDataNext( channelNum, sampleInfo4->smpi_Data, sampleInfo4->smpi_NumFrames, BEEP_F_IF_GO_FOREVER, 0 );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
			else
			{
				SetBeepChannelData( channelNum, sampleInfo1->smpi_Data, sampleInfo1->smpi_NumFrames );
			
				theError	= StartBeepChannel( channelNum );
				LogComment("Start Channel %d.\t\tStartBeepChannel returned %d", channelNum, theError);
			}
				
		}
		else if ( (cped.cped_ButtonBits & ControlDown) == ControlDown )		/*	Decrease Setting	*/
		{
			switch( gSetting )
			{
				case kAmp			:
										if ( gAmp[channelNum] > -1.0 )
											gAmp[channelNum]	-= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_AMPLITUDE,  gAmp[channelNum] );
										LogComment("Channel %2d:  New Amplitude: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gAmp[channelNum], theError );
										break;
				case kSampRate		:
										if ( gSampRate[channelNum] > 0.0 )
											gSampRate[channelNum]	-= 1000.0;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_SAMPLE_RATE,  gSampRate[channelNum] );
										LogComment("Channel %2d:  New Sample Rate: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gSampRate[channelNum], theError);
										break;
				case kLFORate		:
										if ( gLFORate[channelNum] > -2756.0 )
											gLFORate[channelNum]	-= 100.0;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_LFO_RATE,  gLFORate[channelNum] );
										LogComment("Channel %2d:  New LFO Rate: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gLFORate[channelNum], theError);
										break;
				case kAmpModDepth	:
										if ( gAmpModDepth[channelNum] > -1.0 )
											gAmpModDepth[channelNum]	-= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_AMP_MOD_DEPTH,  gAmpModDepth[channelNum] );
										LogComment("Channel %2d:  New Amplitude Mod Depth: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gAmpModDepth[channelNum], theError);
										break;
				case kFreqModDepth	:
										if ( gFreqModDepth[channelNum] > -44100.0 )
											gFreqModDepth[channelNum]	-= 1000.0;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_FREQ_MOD_DEPTH,  gFreqModDepth[channelNum] );
										LogComment("Channel %2d:  New Frequency Mod Depth: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gFreqModDepth[channelNum], theError);
										break;
				case kAmpRate		:
										if ( gAmpRate[channelNum] > 0.0 )
											gAmpRate[channelNum]	-= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_AMPLITUDE_RATE,  gAmpRate[channelNum] );
										LogComment("Channel %2d:  New Amplitude Rate: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gAmpRate[channelNum], theError);
										break;
				case kLeftAmp		:
										if ( gLeftAmp[channelNum] > -1.0 )
											gLeftAmp[channelNum]	-= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_LEFT_GAIN,  gLeftAmp[channelNum] );
										LogComment("Channel %2d:  New Left Channel Amplitude: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gLeftAmp[channelNum], theError);
										break;
				case kRightAmp		:
										if ( gRightAmp[channelNum] > -1.0 )
											gRightAmp[channelNum]	-= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_RIGHT_GAIN,  gRightAmp[channelNum] );
										LogComment("Channel %2d:  New Right Channel Amplitude: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gRightAmp[channelNum], theError);
										break;
				case kGlobalAmp		:
										if ( gGlobalAmp > -1.0 )
											gGlobalAmp	-= .1;
											
										theError	= SetBeepParameter( BMGP_AMPLITUDE,  gGlobalAmp );
										LogComment("New Global Amplitude: %f   SetBeepParameter returned 0x%x", gGlobalAmp, theError);
										break;
				case kFilterCutoff	:
										if ( gFilterCutoff > -1.0 )
											gFilterCutoff	-= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_FILTER_CUTOFF,  gFilterCutoff );
										LogComment("New Filter Cutoff: %f   SetBeepVoiceParameter returned 0x%x", gFilterCutoff, theError);
										break;
				case kCutoffModDepth:
										if ( gCutoffModDepth > -1.0 )
											gCutoffModDepth	-= .1;
											
/*										theError	= SetBeepVoiceParameter( channelNum, BMVP_CUTOFF_MOD_DEPTH,  gCutoffModDepth );
										LogComment("New Filter Cutoff Modulation Depth: %f   SetBeepVoiceParameter returned 0x%x", gCutoffModDepth, theError);
*/										break;
				default				:
										break;
			}
		}
		else if ( (cped.cped_ButtonBits & ControlUp) == ControlUp )			/*	Increase Setting	*/
		{
			switch( gSetting )
			{
				case kAmp			:
										if ( gAmp[channelNum] < 1.0 )
											gAmp[channelNum]	+= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_AMPLITUDE,  gAmp[channelNum] );
										LogComment("Channel %2d:  New Amplitude: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gAmp[channelNum], theError);
										break;
				case kSampRate		:
										if ( gSampRate[channelNum] <= 87200.0 )
											gSampRate[channelNum]	+= 1000.0;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_SAMPLE_RATE,  gSampRate[channelNum] );
										LogComment("Channel %2d:  New Sample Rate: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gSampRate[channelNum], theError);
										break;
				case kLFORate		:
										if ( gLFORate[channelNum] <= 2656.0 )
											gLFORate[channelNum]	+= 100.0;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_LFO_RATE,  gLFORate[channelNum] );
										LogComment("Channel %2d:  New LFO Rate: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gLFORate[channelNum], theError);
										break;
				case kAmpModDepth	:
										if ( gAmpModDepth[channelNum] < 1.0 )
											gAmpModDepth[channelNum]	+= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_AMP_MOD_DEPTH,  gAmpModDepth[channelNum] );
										LogComment("Channel %2d:  New Amplitude Mod Depth: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gAmpModDepth[channelNum], theError);
										break;
				case kFreqModDepth	:
										if ( gFreqModDepth[channelNum] <= 43100.0 )
											gFreqModDepth[channelNum]	+= 1000.0;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_FREQ_MOD_DEPTH,  gFreqModDepth[channelNum] );
										LogComment("Channel %2d:  New Frequency Mod Depth: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gFreqModDepth[channelNum], theError);
										break;
				case kAmpRate		:
										if ( gAmpRate[channelNum] < 1.0 )
											gAmpRate[channelNum]	+= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_AMPLITUDE_RATE,  gAmpRate[channelNum] );
										LogComment("Channel %2d:  New Amplitude Rate: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gAmpRate[channelNum], theError);
										break;
				case kLeftAmp		:
										if ( gLeftAmp[channelNum] < 1.0 )
											gLeftAmp[channelNum]	+= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_LEFT_GAIN,  gLeftAmp[channelNum] );
										LogComment("Channel %2d:  New Left Channel Amplitude: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gLeftAmp[channelNum], theError);
										break;
				case kRightAmp		:
										if ( gRightAmp[channelNum] < 1.0 )
											gRightAmp[channelNum]	+= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_RIGHT_GAIN,  gRightAmp[channelNum] );
										LogComment("Channel %2d:  New Right Channel Amplitude: %f   SetBeepVoiceParameter returned 0x%x", channelNum, gRightAmp[channelNum], theError);
										break;
				case kGlobalAmp		:
										if ( gGlobalAmp < 1.0 )
											gGlobalAmp	+= .1;
											
										theError	= SetBeepParameter( BMGP_AMPLITUDE,  gGlobalAmp );
										LogComment("New Global Amplitude: %f   SetBeepVoiceParameter returned 0x%x", gGlobalAmp, theError);
										break;
				case kFilterCutoff	:
										if ( gFilterCutoff < 1.0 )
											gFilterCutoff	+= .1;
											
										theError	= SetBeepVoiceParameter( channelNum, BMVP_FILTER_CUTOFF,  gFilterCutoff );
										LogComment("New Filter Cutoff: %f   SetBeepVoiceParameter returned 0x%x", gFilterCutoff, theError);
										break;
				case kCutoffModDepth:
										if ( gCutoffModDepth < 1.0 )
											gCutoffModDepth	+= .1;
											
/*										theError	= SetBeepVoiceParameter( channelNum, BMVP_CUTOFF_MOD_DEPTH,  gCutoffModDepth );
										LogComment("New Filter Cutoff Modulation Depth: %f   SetBeepVoiceParameter returned 0x%x", gCutoffModDepth, theError);
*/										break;
				default				:
										break;
			}
		}
		else if ( (cped.cped_ButtonBits & ControlLeft) == ControlLeft )		/*	Go to the previous channel	*/
		{
			if ( channelNum > 0 )
				LogComment("Select Channel %d", --channelNum);
		}
		else if ( (cped.cped_ButtonBits & ControlRight) == ControlRight )	/*	Go to the next channel	*/
		{
			if ( channelNum < 32 )
				LogComment("Select Channel %d", ++channelNum);
		}
		else if ( (cped.cped_ButtonBits & ControlA) == ControlA )			/*	Amplitude Setting	*/
		{
			LogComment("Adjust Amplitudes");
			gSetting	= kAmp;
		}
		else if ( (cped.cped_ButtonBits & ControlB) == ControlB )			/*	Sample rate Setting	*/
		{
			LogComment("Adjust Sample Rates");
			gSetting	= kSampRate;
		}
		else if ( (cped.cped_ButtonBits & ControlC) == ControlC )			/*	LFO rate Setting	*/
		{
			LogComment("Adjust LFO Rates");
			gSetting	= kLFORate;
		}
		
	}
	while( (cped.cped_ButtonBits & ControlX) == 0 );
}

