/* @(#) TestUtilities.c 96/07/22 1.1 */
/*
 *	File:				TestUtilities.c
 *	Author:				Eric L. Hayes
 *	Creation Date:		
 *
 *	Last Modified by:	Eric L. Hayes
 *	Modification Date:	1/25/95
 *
 *	Version:			M2 1.2
 *
 *	This file contains some utilities which are used in some of the Audio Tests
 */



#include <audio/audio.h>
#include <kernel/random.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <TestUtilities.h>


/*
 *	GetRandomNumber
 *
 *	Returns a random integer between upperLimit and lowerLimit. It calls ReadHardwareRandomNumber,
 *	which returns a random number between 0 and some MAXSIZE.  By modulo with the range, we can get a number
 *	with in a range.  Adding to that value the lower limit we get a random number within a the needed
 *	range
 *
 */
int32	GetRandomNumber( int lowerLimit, int upperLimit )
{
	int32		range, theAnswer;
	
	
	range	= upperLimit - lowerLimit;
	theAnswer	= (ReadHardwareRandomNumber() % range) + lowerLimit;
	
	return theAnswer;
	
}


/*
 *	atof32
 *
 *	Converts an ascii string to a 32 bit float
 *
 */
float32	atof32( char* string )
{
	Boolean		doCount	= FALSE;
	char		theNumber[255];
	float32		count	= 0.0, floatValue;
	int32		i, intValue;
	
	for ( i = 0; string[i] != '\0'; i++ )
	{
		if ( doCount )
			count	+= 	1.0;
			
		if (  string[i] == '.' )
			doCount	= TRUE;
		else
		{
			if ( doCount == TRUE )
				theNumber[i-1]	= string[i];
			else
				theNumber[i]	= string[i];
		}
	}
	
	if ( doCount == TRUE )
	{
		theNumber[i-1]	= '\0';
		intValue		= atol( theNumber );
		floatValue		= intValue / powf( 10.0, count );
	}
	else
	{
		theNumber[i]	= '\0';
		intValue		= atol( theNumber );
		floatValue		= intValue / 1.0;
	}

	
	return floatValue;
	
}

