/****************************************************************/
/* @(#) SimpleBlitIT.c 96/11/20 1.1						*/
/* Written by: Paige Wuestefeld x2773 10/96			*/
/* Modified:							*/
/*								*/
/* Simple Integration test of Blitter folio			*/
/* Data required: 	oxford-big.utf				*/
/*			panel01.utf				*/
/*			SimpleBlit.h				*/
/****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <graphics/clt/gstate.h>
#include <graphics/graphics.h>
#include <graphics/view.h>
#include <graphics/blitter.h>
#include <kernel/io.h>
#include <kernel/mem.h>
#include <kernel/task.h>
#include <kernel/random.h>
#include <assert.h>
#include <misc/event.h>
#include <device/te.h>
#include <string.h>

#include "SimpleBlit.h"

/* standard error checking */
#define CKERR(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y); PrintfSysErr(x); exit(1);}}
#define CKERRCONT(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y); PrintfSysErr(x); }}

void 
main (int argc, char *argv[])
{

  GState *gs;
  BlitObject *Q3Blit = NULL;
  BlitObject *myBigutf = NULL;
  BlitObject *Q2Blit = NULL;
  BlitObject *myutf = NULL;
  BlitObject *Q4Blit = NULL;
  ControlPadEventData cpad;
  VerticesSnippet *myVerts = NULL;
  VerticesSnippet *replacedSnippet;
  VerticesSnippet *Q3Verts = NULL;
  VerticesSnippet *Q2Verts = NULL;
  VerticesSnippet *Q4Verts = NULL;
  DBlendSnippet *replacedSnipDB;
  TxLoadSnippet *replacedSnipTXL;
  BltTxData *RTxSnp;
  PIPLoadSnippet *replacedSnipPIP;
  BlitScroll testScroll1, testScroll2, testScroll3, testScroll4, testScroll5,
    testScroll6, testScroll7, testScroll8;
  BlitObject *scrollBlit1, *scrollBlit2, *scrollBlit3, *scrollBlit4, *scrollBlit5,
   *scrollBlit6, *scrollBlit7, *scrollBlit8;
  void *replacedDB, *replacedTX;
  Err err;
  int32 i;
  uint32 fbindex = 0;
  int32 result;
  int frame = 0;
  uint32 clipped = 0;
  bool deform, fan, trans, memcheck;
  bool utilities = FALSE;

  printf ("\nUsage SimpleBlitIT [options: -strip -trans -memcheck ]\n");
  printf ("\n[-strip] makes main object a strip.\n");
  printf ("[-trans] causes *both* textured objects to be transparent.\n");
  printf ("[-memcheck] causes memdebug to be used.\n");
  printf ("Displays 5 blit objects - oxford-bug.utf [ Fan with texturing and w ] is the background blit object\n");
  printf ("Additional objects:\n");
  printf ("Centered object is main object: FAN or STRIP with texture only \n");
  printf ("Quadrant 2 [ Strip shaded with w but no texture - irregular shape ]\n");
  printf ("Quadrant 3 [ Fan shaded only ]\n");
  printf ("Quadrant 4 [ Fan shaded, textured, no w ]\n\n");

  printf ("Interact . . . \n\n");

  printf ("Arrows move centered object\n");

  printf ("Controller A turns on Outside Clipping for main object\n");
  printf ("Controller B turns on Inside Clipping for main object\n");
  printf ("Controller L rotates main object about center of view\n");
  printf ("Controller Right Shift and Arrows moves bottom right vertex of main object\n");
  printf ("Controller Left Shift and Arrows moves bottom left vertex of main object\n");

  printf ("Controller Pause renders only backgound\n");
  printf ("Control A after Pause Scrolls 8 rectangular areas around center of background towards center\n");

  fan = TRUE;
  trans = FALSE;
  memcheck = FALSE;

  /* can turn on STRIP/FAN or transparency via command line */
  for (i = 1; i < argc; i++) {
    if (strcmp (argv[i], "-strip") == 0)
      fan = FALSE;

    if (strcmp (argv[i], "-trans") == 0)
      trans = TRUE;

    if (strcmp (argv[i], "-memcheck") == 0)
      memcheck = TRUE;

  }

  if (memcheck) {
  err = CreateMemDebug (NULL);
  CKERR (err, "Create MemDebug");

  err = ControlMemDebug (MEMDEBUGF_PAD_COOKIES |
			 MEMDEBUGF_ALLOC_PATTERNS |
			 MEMDEBUGF_FREE_PATTERNS |
			 MEMDEBUGF_CHECK_ALLOC_FAILURES |
			 MEMDEBUGF_KEEP_TASK_DATA);
  CKERR (err, "Control MemDebug");
  }

  result = InitEventUtility (1, 0, LC_ISFOCUSED);
  CKERR (result, "InitEventUtil");

/**********************/
/* Setup graphics */
/**********************/

  gs = setupGS ();

/*******************************************/
/* set up scrolling structure info */
/*******************************************/
/* create BOs for scrolling */

  err = Blt_CreateBlitObject (&scrollBlit1, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit2, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit3, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit4, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit5, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit6, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit7, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

  err = Blt_CreateBlitObject (&scrollBlit8, NULL);
  CKERR (err, "Blt_CreateBlitObject scrollBlit");

/* create scrolling structures */

  SetupScrollStruct (&testScroll1, &testScroll2, &testScroll3, &testScroll4,
		     &testScroll5, &testScroll6, &testScroll7, &testScroll8);

/*******************************************/
/* utf BO */
/*******************************************/
  err = Blt_LoadUTF (&myutf, "panel01.utf");
  CKERR (err, "Could not load utf file");

  if (fan) {
    printf ("Fan\n");
    /* 4 vertices new, fan with texture no shading or w */
    err = Blt_CreateVertices (&myVerts, CLA_TRIANGLE (1, RC_FAN, 0, 1, 0, 4));
    CKERR (err, "Create Verts");

    replacedSnippet = Blt_SetSnippet (myutf, myVerts);
    if (replacedSnippet == NULL)
      printf ("ERROR: Returned NULL snippet for replacedSnippet\n");

    err = Blt_DeleteSnippet (replacedSnippet);
    CKERRCONT (err, "Delete Snippet replacedSnippet");

    err = Blt_SetVertices (myVerts, FanT);
    CKERR (err, "Set Verts");
  }
  else {

    printf ("Strip\n");
    /* 4 vertices new, strip with texture and shading and w */

    err = Blt_CreateVertices (&myVerts, CLA_TRIANGLE (1, RC_STRIP, 0, 1, 1, 4));
    CKERR (err, "Create Verts");

    replacedSnippet = Blt_SetSnippet (myutf, myVerts);
    if (replacedSnippet == NULL)
      printf ("ERROR: Returned NULL snippet for replacedSnippet\n");

    err = Blt_DeleteSnippet (replacedSnippet);
    CKERR (err, "Delete Snippet");

    err = Blt_SetVertices (myVerts, SquareStrip);
    CKERR (err, "Set Verts");

  }

/************/
/* re-use texture loading stuff */
/************/

  /* 4 vertices new, fan with texture and shading no w */
  err = Blt_CreateVertices (&Q4Verts, CLA_TRIANGLE (1, RC_FAN, 0, 1, 1, 4));
  CKERR (err, "Create Verts");

  err = Blt_CreateBlitObjectVA (&Q4Blit,
				BLIT_TAG_VERTICES, Q4Verts,
				TAG_END);
  CKERR (err, "Blt_CreateBlitObjectVA");

  err = Blt_SetVertices (Q4Verts, FanST);
  CKERR (err, "Set Verts");

  replacedTX = Blt_ReuseSnippet (myutf, Q4Blit, BLIT_TAG_TXDATA);
  err = Blt_DeleteSnippet (replacedTX);
  CKERRCONT (err, "DeleteSnippet replacedTX");

  replacedDB = Blt_ReuseSnippet (myutf, Q4Blit, BLIT_TAG_DBLEND);
  err = Blt_DeleteSnippet (replacedDB);
  CKERRCONT (err, "DeleteSnippet replacedDB");

/* memory leak here */
  err = Blt_CopySnippet (myutf, Q4Blit, &RTxSnp, BLIT_TAG_TBLEND);
  CKERR (err, "Blt_CopySnippet");

  err = Blt_DeleteSnippet (RTxSnp);
  CKERRCONT (err, "DeleteSnippet RTxSnp");

  if (Q4Blit->bo_txdata == myutf->bo_txdata)
    printf ("Reuse OK TxData\n");
  if (Q4Blit->bo_dbl == myutf->bo_dbl)
    printf ("Reuse OK DBLEND\n");

  /* turn on texturing and set LOD number and filter type */
  Q4Blit->bo_tbl->txb_addrCntl = (CLA_TXTADDRCNTL (1, RC_TXTADDRCNTL_MINFILTER_BILINEAR,
						   RC_TXTADDRCNTL_INTERFILTER_TRILINEAR, RC_TXTADDRCNTL_MAGFILTER_BILINEAR, (1 - 1)));

  /* Set texture application control register */
  Q4Blit->bo_tbl->txb_tabCntl = (CLA_TXTTABCNTL (RC_TXTTABCNTL_FIRSTCOLOR_TEXCOLOR,
   RC_TXTTABCNTL_SECONDCOLOR_PRIMCOLOR, RC_TXTTABCNTL_THIRDCOLOR_CONSTCOLOR,
     RC_TXTTABCNTL_FIRSTALPHA_TEXALPHA, RC_TXTTABCNTL_SECONDALPHA_PRIMALPHA,
		 RC_TXTTABCNTL_COLOROUT_BLEND, RC_TXTTABCNTL_ALPHAOUT_BLEND,
					       RC_TXTTABCNTL_BLENDOP_MULT));

  if (trans) {
    /* setting this affects both as they share this snippet */
    err = Blt_BlendBlitObject (myutf, 0.5, 0.5);
    CKERR (err, "Blt_BlendBlitObject");
  }

/*******************************************/
/* Created square with just rgba - regular create */
/*******************************************/
  err = Blt_CreateBlitObject (&Q3Blit, NULL);
  CKERR (err, "Blt_CreateBlitObject");

  err = Blt_CreateVertices (&Q3Verts, CLA_TRIANGLE (1, RC_FAN, 0, 0, 1, 4));
  CKERR (err, "Create Verts- shaded only");

  replacedSnippet = Blt_SetSnippet (Q3Blit, Q3Verts);
  if (replacedSnippet == NULL)
    printf ("Returned NULL snippet for replacedSnippet\n");

  err = Blt_DeleteSnippet (replacedSnippet);
  CKERRCONT (err, "Delete Snippet replacedSnippet");

  err = Blt_SetVertices (Q3Verts, FanS);
  CKERR (err, "Set Verts");

  Q3Blit->bo_tbl->txb_addrCntl = (CLA_TXTADDRCNTL (0, 0, 0, 0, 0));
  Q3Blit->bo_tbl->txb_tabCntl = (CLA_TXTTABCNTL (0, 0, 0, 0, 0,
    RC_TXTTABCNTL_COLOROUT_PRIMCOLOR, RC_TXTTABCNTL_ALPHAOUT_PRIMALPHA, 0));

/*******************************************/
/* Created with just rgba - create VA */
/*******************************************/
  err = Blt_CreateVertices (&Q2Verts, CLA_TRIANGLE (1, RC_STRIP, 1, 0, 1, 4));
  CKERR (err, "Create Verts- shaded only");

  err = Blt_SetVertices (Q2Verts, FreeStripSW);
  CKERR (err, "Set Verts");

  err = Blt_CreateBlitObjectVA (&Q2Blit,
				BLIT_TAG_VERTICES, Q2Verts,
				TAG_END);
  CKERR (err, "Blt_CreateBlitObjectVA");

  Q2Blit->bo_tbl->txb_addrCntl = (CLA_TXTADDRCNTL (0, 0, 0, 0, 0));
  Q2Blit->bo_tbl->txb_tabCntl = (CLA_TXTTABCNTL (0, 0, 0, 0, 0,
    RC_TXTTABCNTL_COLOROUT_PRIMCOLOR, RC_TXTTABCNTL_ALPHAOUT_PRIMALPHA, 0));

  replacedSnipDB = Blt_RemoveSnippet (Q2Blit, BLIT_TAG_DBLEND);
  if (replacedSnipDB == NULL)
    printf ("ERROR: replacedSnipDB is NULL\n");
  err = Blt_DeleteSnippet (replacedSnipDB);
  CKERRCONT (err, "Delete Snippet DBLEND");

  replacedSnipTXL = Blt_RemoveSnippet (Q2Blit, BLIT_TAG_TXLOAD);
  if (replacedSnipTXL == NULL)
    printf ("ERROR: replacedSnipTXL is NULL\n");
  err = Blt_DeleteSnippet (replacedSnipTXL);
  CKERRCONT (err, "Delete Snippet DBLEND");

  replacedSnipPIP = Blt_RemoveSnippet (Q2Blit, BLIT_TAG_PIP);
  if (replacedSnipPIP == NULL)
    printf ("ERROR: replacedSnipPIP is NULL\n");
  err = Blt_DeleteSnippet (replacedSnipPIP);
  CKERRCONT (err, "Delete Snippet DBLEND");

/*******************************************/
/* Big utf BO */
/*******************************************/
  err = Blt_LoadUTF (&myBigutf, "oxford-big.utf");
  CKERR (err, "Could not load oxford-big.utf texture file");

  err = Blt_SetVertices (myBigutf->bo_vertices, FanWT);
  CKERR (err, "Set Verts");

  while (1) {
    GS_BeginFrame (gs);

    GetControlPad (1, FALSE, &cpad);

    if (cpad.cped_ButtonBits & ControlX)
      break;

    if (cpad.cped_ButtonBits & ControlStart)
      utilities = TRUE;

    frame++;

    err = GS_ClearFBZ (gs, TRUE, TRUE);
    CKERR (err, "GS_ClearFBZ");

/****************************/
/* always render backdrop */
/****************************/
    err = Blt_BlitObjectToBitmap (gs, myBigutf, bitmaps[fbindex], 0);
    CKERR (err, "BlitObject Back");

    if (utilities) {

      /* Start then A Scrolls */

      if (cpad.cped_ButtonBits & ControlA) {
	err = Blt_Scroll (gs, scrollBlit1, bitmaps[fbindex], &testScroll1);
	CKERRCONT (err, "Blt_Scroll 1");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit2, bitmaps[fbindex], &testScroll2);
	CKERRCONT (err, "Blt_Scroll 2");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit3, bitmaps[fbindex], &testScroll3);
	CKERRCONT (err, "Blt_Scroll 3");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit4, bitmaps[fbindex], &testScroll4);
	CKERRCONT (err, "Blt_Scroll 4");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit5, bitmaps[fbindex], &testScroll5);
	CKERRCONT (err, "Blt_Scroll 5");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit6, bitmaps[fbindex], &testScroll6);
	CKERRCONT (err, "Blt_Scroll 6");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit7, bitmaps[fbindex], &testScroll7);
	CKERRCONT (err, "Blt_Scroll 7");
	err = GS_SendList (gs);
	CKERR (err, "SendList");

	err = Blt_Scroll (gs, scrollBlit8, bitmaps[fbindex], &testScroll8);
	CKERRCONT (err, "Blt_Scroll 8");
	err = GS_SendList (gs);
	CKERR (err, "SendList");
      }

    }				/* utilities */

/*****************************************************/
/* blit other BOs unless ControlStart is pressed */
/*****************************************************/

    if (!utilities) {

      deform = FALSE;

      if (cpad.cped_ButtonBits & ControlRightShift) {

	deform = TRUE;

	if (cpad.cped_ButtonBits & ControlRight)
	  (*BLITVERTEX_X (myutf->bo_vertices, 3)) = (*BLITVERTEX_X (myutf->bo_vertices, 3)) + 1.0;

	if (cpad.cped_ButtonBits & ControlUp)
	  (*BLITVERTEX_Y (myutf->bo_vertices, 3)) = (*BLITVERTEX_Y (myutf->bo_vertices, 3)) + 1.0;

	if (cpad.cped_ButtonBits & ControlLeft)
	  (*BLITVERTEX_X (myutf->bo_vertices, 3)) = (*BLITVERTEX_X (myutf->bo_vertices, 3)) - 1.0;

	if (cpad.cped_ButtonBits & ControlDown)
	  (*BLITVERTEX_Y (myutf->bo_vertices, 3)) = (*BLITVERTEX_Y (myutf->bo_vertices, 3)) - 1.0;

	err = Blt_SetVertices (myutf->bo_vertices, &(myutf->bo_vertices->vtx_vertex[0]));
	CKERR (err, "Set Vertices");

      }				/* Right Shift */

      if (cpad.cped_ButtonBits & ControlLeftShift) {

	deform = TRUE;

	if (cpad.cped_ButtonBits & ControlRight)
	  (*BLITVERTEX_X (myutf->bo_vertices, 2)) = (*BLITVERTEX_X (myutf->bo_vertices, 2)) + 1.0;

	if (cpad.cped_ButtonBits & ControlUp)
	  (*BLITVERTEX_Y (myutf->bo_vertices, 2)) = (*BLITVERTEX_Y (myutf->bo_vertices, 2)) + 1.0;

	if (cpad.cped_ButtonBits & ControlLeft)
	  (*BLITVERTEX_X (myutf->bo_vertices, 2)) = (*BLITVERTEX_X (myutf->bo_vertices, 2)) - 1.0;

	if (cpad.cped_ButtonBits & ControlDown)
	  (*BLITVERTEX_Y (myutf->bo_vertices, 2)) = (*BLITVERTEX_Y (myutf->bo_vertices, 2)) - 1.0;

	err = Blt_SetVertices (myutf->bo_vertices, &(myutf->bo_vertices->vtx_vertex[0]));
	CKERR (err, "Set Vertices");

      }				/* Left Shift */


      if (!deform) {

	if (cpad.cped_ButtonBits & ControlA) {
	  err = Blt_SetClipBox (myutf, TRUE, TRUE, &TOPL, &BOTTOMR);
	  CKERR (err, "Clip");
	  printf ("Clipping Outside ON for main object - Clips outside Width=295 and Height=215\n");
	}

	if (cpad.cped_ButtonBits & ControlB) {
	  err = Blt_SetClipBox (myutf, TRUE, FALSE, &TOPL, &BOTTOMR);
	  CKERR (err, "Clip");
	  printf ("Clipping Inside ON for main object- Clips inside Width=295 and Height=215\n");
	}

	if (cpad.cped_ButtonBits & ControlUp) {
	  err = Blt_MoveVertices (myutf->bo_vertices, 0.0, -3.0);
	  CKERR (err, "Move");
	}

	if (cpad.cped_ButtonBits & ControlDown) {
	  err = Blt_MoveVertices (myutf->bo_vertices, 0.0, 3.0);
	  CKERR (err, "Move");
	}

	if (cpad.cped_ButtonBits & ControlLeft) {
	  err = Blt_MoveVertices (myutf->bo_vertices, -3.0, 0.0);
	  CKERR (err, "Move");
	}

	if (cpad.cped_ButtonBits & ControlRight) {
	  err = Blt_MoveVertices (myutf->bo_vertices, 3.0, 0.0);
	  CKERR (err, "Move");
	}

	if (cpad.cped_ButtonBits & ControlF) {
	  err = Blt_RotateVertices (myutf->bo_vertices, 2.0, anchorX, anchorY);
	  CKERR (err, "Rotate");
	}

      }				/* deform */

      err = Blt_BlitObjectToBitmap (gs, Q3Blit, bitmaps[fbindex],
				    SKIP_SNIPPET_DBLEND |
	      SKIP_SNIPPET_TXLOAD | SKIP_SNIPPET_TXDATA | SKIP_SNIPPET_PIP);
      CKERR (err, "BlitObject 1");

      /* if replacing the snippets earlier worked correctly this will work otherwise 		*/
      /* it will get a timeout error unless we SKIP DBLEND, TXLOAD, TXDATA, and PIP snips */

      err = Blt_BlitObjectToBitmap (gs, Q2Blit, bitmaps[fbindex], 0);
      CKERR (err, "BlitObject 2");

      err = Blt_BlitObjectToBitmap (gs, myutf, bitmaps[fbindex], 0);
      CKERR (err, "BlitObject 3");

      err = Blt_BlitObjectToBitmap (gs, Q4Blit, bitmaps[fbindex], SKIP_SNIPPET_TXLOAD | SKIP_SNIPPET_PIP);
      CKERR (err, "BlitObject Re-used");

    }				/* utilities */

    if (Blt_BlitObjectClipped (myutf))
      clipped++;

    err = GS_SendList (gs);
    CKERR (err, "SendList");

    GS_EndFrame (gs);
    CKERR (err, "EndFrame");

    fbindex = 1 - fbindex;
    GS_SetDestBuffer (gs, bitmaps[fbindex]);

  }

  printf ("Finished \n");
  printf ("Finished. Clipped %i times \n", clipped);

  if (Blt_BlitObjectSliced (myBigutf))
    printf ("Correctly confirms texture was sliced\n");
  else
    printf ("Incorrectly thinks texture is not sliced\n");

  if (Blt_BlitObjectSliced (myutf))
    printf ("Incorrectly thinks texture was sliced\n");
  else
    printf ("Correctly confirms texture is not sliced\n");

  err = Blt_DeleteVertices (myVerts);
  if (err < 0)
    printf ("Correctly denied deletion of vertices snippet when still in use \n");
  else
    printf ("ERROR: Deleted inuse snippet\n");

/* Free memory associated with BOs */

  /* no err returned */
  Blt_DeleteBlitObject (Q2Blit);
  Blt_DeleteBlitObject (Q3Blit);
  Blt_DeleteBlitObject (Q4Blit);
  Blt_DeleteBlitObject (scrollBlit1);
  Blt_DeleteBlitObject (scrollBlit2);
  Blt_DeleteBlitObject (scrollBlit3);
  Blt_DeleteBlitObject (scrollBlit4);
  Blt_DeleteBlitObject (scrollBlit5);
  Blt_DeleteBlitObject (scrollBlit6);
  Blt_DeleteBlitObject (scrollBlit7);
  Blt_DeleteBlitObject (scrollBlit8);

  err = Blt_FreeUTF (myutf);
  CKERRCONT (err, "FreeUTF");

  err = Blt_FreeUTF (myBigutf);
  CKERRCONT (err, "FreeUTF");

  err = Blt_DeleteVertices (myVerts);
  CKERRCONT (err, "Delete Verts 1");

  err = Blt_DeleteVertices (Q3Verts);
  CKERRCONT (err, "Delete Verts 2");

  err = Blt_DeleteVertices (Q2Verts);
  CKERRCONT (err, "Delete Verts 3");

  err = Blt_DeleteVertices (Q4Verts);
  CKERRCONT (err, "Delete Verts 4");

/* Free memory associated with views, bitmaps */

  RemoveView (viewItem);
  /* no error returned */

  err = DeleteItem (viewItem);
  CKERRCONT (err, "DeleteItem");

  err = GS_FreeBitmaps (bitmaps, 3);
  CKERRCONT (err, "FreeBitmaps");

  err = GS_FreeLists (gs);
  CKERRCONT (err, "FreeLists");

  err = GS_Delete (gs);
  CKERRCONT (err, "DeleteItem");

  err = KillEventUtility ();
  CKERRCONT (err, "KillEventUtil");

  CloseBlitterFolio ();
  /* no error returned */

  err = CloseGraphicsFolio ();
  CKERRCONT (err, "CloseGraphicsFolio");

  if (memcheck) {
  DumpMemDebug (NULL);
  DeleteMemDebug ();
  }

}

void 
SetupScrollStruct (BlitScroll * testScroll1, BlitScroll * testScroll2, BlitScroll * testScroll3, BlitScroll * testScroll4,
		   BlitScroll * testScroll5, BlitScroll * testScroll6, BlitScroll * testScroll7, BlitScroll * testScroll8)
{

  testScroll1->bsc_region.min.x = 0.0;
  testScroll1->bsc_region.min.y = 0.0;
  testScroll1->bsc_region.max.x = 110;
  testScroll1->bsc_region.max.y = 80;
  testScroll1->bsc_dx = 1.0;
  testScroll1->bsc_dy = 1.0;
  testScroll1->bsc_replaceColor = 0x00ff0000;

  testScroll2->bsc_region.min.x = 0.0;
  testScroll2->bsc_region.min.y = 80.0;
  testScroll2->bsc_region.max.x = 110;
  testScroll2->bsc_region.max.y = 160;
  testScroll2->bsc_dx = 1.0;
  testScroll2->bsc_dy = 0.0;
  testScroll2->bsc_replaceColor = 0x0000ff00;

  testScroll3->bsc_region.min.x = 0.0;
  testScroll3->bsc_region.min.y = 160.0;
  testScroll3->bsc_region.max.x = 110;
  testScroll3->bsc_region.max.y = 240;
  testScroll3->bsc_dx = 1.0;
  testScroll3->bsc_dy = -1.0;
  testScroll3->bsc_replaceColor = 0x00ff0000;

  testScroll4->bsc_region.min.x = 110.0;
  testScroll4->bsc_region.min.y = 0.0;
  testScroll4->bsc_region.max.x = 210;
  testScroll4->bsc_region.max.y = 80;
  testScroll4->bsc_dx = 0.0;
  testScroll4->bsc_dy = 1.0;
  testScroll4->bsc_replaceColor = 0x0000ff00;

  testScroll5->bsc_region.min.x = 110.0;
  testScroll5->bsc_region.min.y = 160.0;
  testScroll5->bsc_region.max.x = 210;
  testScroll5->bsc_region.max.y = 240;
  testScroll5->bsc_dx = 0.0;
  testScroll5->bsc_dy = -1.0;
  testScroll5->bsc_replaceColor = 0x0000ff00;

  testScroll6->bsc_region.min.x = 210.0;
  testScroll6->bsc_region.min.y = 0.0;
  testScroll6->bsc_region.max.x = 320;
  testScroll6->bsc_region.max.y = 80;
  testScroll6->bsc_dx = -1.0;
  testScroll6->bsc_dy = 1.0;
  testScroll6->bsc_replaceColor = 0x00ff0000;

  testScroll7->bsc_region.min.x = 210.0;
  testScroll7->bsc_region.min.y = 80.0;
  testScroll7->bsc_region.max.x = 320;
  testScroll7->bsc_region.max.y = 160;
  testScroll7->bsc_dx = -1.0;
  testScroll7->bsc_dy = 0.0;
  testScroll7->bsc_replaceColor = 0x0000ff00;

  testScroll8->bsc_region.min.x = 210.0;
  testScroll8->bsc_region.min.y = 160.0;
  testScroll8->bsc_region.max.x = 320;
  testScroll8->bsc_region.max.y = 240;
  testScroll8->bsc_dx = -1.0;
  testScroll8->bsc_dy = -1.0;
  testScroll8->bsc_replaceColor = 0x00ff0000;

}

GState *
setupGS (void)
{

  GState *gs;
  Err err;
  int32 buffSignal;

  buffSignal = AllocSignal (0);

  err = OpenGraphicsFolio ();
  CKERR (err, "OpenGraphicsFolio");

  err = OpenBlitterFolio ();
  CKERR (err, "OpenBlitterFolio");

  gs = GS_Create ();
  if (gs == NULL) {
    printf ("Unable to create GState\n");
    exit (1);
  }

  err = GS_AllocBitmaps (bitmaps, BMWIDTH, BMHEIGHT, BMTYPE_16, 2, 1);
  CKERR (err, "AllocBitmaps");

  err = GS_AllocLists (gs, 2, (32 * 1024));
  CKERR (err, "AllocLists");

  err = GS_SetZBuffer (gs, bitmaps[2]);
  CKERR (err, "GSSetZBuffer");

  err = GS_SetVidSignal (gs, buffSignal);
  CKERR (err, "GSSetVidSignal");

  err = GS_SetClearValues (gs, .3, 0.3, 0.3, 1.0, 1.0);
  CKERR (err, "GSSetVidSignal");

  err = GS_SetDestBuffer (gs, bitmaps[0]);
  CKERR (err, "GSSetClearValues");

  /* create a view item */
  viewItem = CreateItemVA (MKNODEID (NST_GRAPHICS, GFX_VIEW_NODE),
			   VIEWTAG_VIEWTYPE, VIEWTYPE_16,
			   VIEWTAG_BITMAP, bitmaps[0],
			   VIEWTAG_RENDERSIGNAL, buffSignal,
			   TAG_END);

  AddViewToViewList (viewItem, 0);

  err = GS_SetView (gs, viewItem);
  CKERR (err, "SetView");

  return (gs);

}
