/**********************************************************************************************
**
**  Copyright (c) 1993-1996, an unpublished work by The 3DO Company.
**  All rights reserved. This material contains confidential
**  information that is the property of The 3DO Company. Any
**  unauthorized duplication, disclosure or use is prohibited.
**
**  CLT integration test
**  Written by:		       Paige Wuestefeld  
**
*************************************************************************************************/

/************************************************************************************************/
/* @(#) cltIT3.c 96/10/29 1.8									*/
/* This program displays a textured fan with perspective correction and shading at		*/
/* each of the vertices.  The shading and texture are combined using MULT blending.		*/
/* It uses the utility program texload.c to load in the textures.				*/
/*												*/
/* In the absence of any warnings, the GState tests have passed.				*/
/************************************************************************************************/

#ifdef MACINTOSH
#include <stdio.h>
#include <stdlib.h>
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#include <graphics:clt:gstate.h>
#include <graphics:graphics.h>
#include <graphics:view.h>
#include <kernel:io.h>
#include <kernel:mem.h>
#include <kernel:random.h>
#include <assert.h>
#include <misc:event.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <graphics/clt/gstate.h>
#include <graphics/graphics.h>
#include <graphics/view.h>
#include <kernel/io.h>
#include <kernel/mem.h>
#include <kernel/random.h>
#include <assert.h>
#include <misc/event.h>
#endif

#include "texload.h"

#define CKERR(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y);PrintfSysErr(x); exit(1);}}

#define MAXVERT	75
#define BMWIDTH 320
#define BMHEIGHT 240
#define numvert 6
#define mySize 90

typedef	struct	mycoords {
	gfloat	x;
	gfloat	y;
	gfloat	u;
	gfloat	v;
	gfloat	w;
	gfloat	r;
	gfloat	g;
	gfloat	b;
	gfloat	a;
} mycoords;

void main(int argc, char **argv)

{

mycoords*				myvtx;
int32					result, flag;
uint32					buttons;
ControlPadEventData			cpad;
Err					err;
int32					buffSignal;
int 					cnt;
GState 					*gs;     
Item					viewItem; 
Item 					bitmaps[3]; 
uint32					fbindex = 0;
float					xs, ys;
TextureSnippets 			snips;		/* cmd lists to load the texture */

TOUCH(argc);
TOUCH(argv);

myvtx=AllocMem(((numvert+1)*sizeof(mycoords)), MEMTYPE_NORMAL);
if (myvtx==NULL) printf("Memory Allocation Failure\n");
	
result = InitEventUtility(1,0,LC_ISFOCUSED);
CKERR( result, "InitEventUtility");

buffSignal = AllocSignal(0);

err = OpenGraphicsFolio (); 						
CKERR( err, "OpenGraphicsFolio");

gs= GS_Create(); 							
if (gs==NULL) {printf("Unable to create GState\n"); exit(1);}

err = GS_AllocBitmaps(bitmaps, BMWIDTH, BMHEIGHT, BMTYPE_16 , 2, 1); 
CKERR( err, "AllocBM");

err = GS_AllocLists(gs, 2, (32*1024));
CKERR( err, "AllocLists");

err=GS_SetZBuffer(gs, bitmaps[2]);
CKERR( err, "GSSetZBuffer");
	
err=GS_SetVidSignal(gs, buffSignal);
CKERR( err, "GSSetVidSignal");
	
err=GS_SetDestBuffer(gs, bitmaps[0]); 	
CKERR( err, "GSSetDestBuffer");
	
/* create a view item and assoc signal bit for tear-free rendering */
viewItem = CreateItemVA(MKNODEID(NST_GRAPHICS, GFX_VIEW_NODE),
						VIEWTAG_VIEWTYPE, VIEWTYPE_16,
						VIEWTAG_BITMAP, bitmaps[0],
						VIEWTAG_RENDERSIGNAL, buffSignal,
						TAG_END );

AddViewToViewList( viewItem, 0 );

err=GS_SetView(gs, viewItem);
CKERR( err, "GSSetView");

/********************************************************************/
/* Simple VertexRgbaUvW test	TEXTURED FAN			    */
/********************************************************************/

/* Load texture from disk into TRAM, including its PIP if there is one */
CLT_InitSnippet( &snips.dab );
CLT_InitSnippet( &snips.tab );
snips.lcb = NULL;
snips.pip = NULL;
snips.txdata = NULL;

/* parse and load specified utf file */
LoadTexture( &snips, ("cow.utf") );
UseTxLoadCmdLists( gs, &snips );
	
xs=ys=((snips.lcb->XSize * snips.txdata->maxLOD * 2) - 1.0);	/* max X of LOD 0 */

/* initial values for textured fan (rectangular) */

myvtx->x=50.0; 		
myvtx->y=100.0; 		
myvtx->w=0.80;		
myvtx->u=xs*.33*(myvtx->w); 	
myvtx->v=0.0*(myvtx->w);			
myvtx->r=0.2; 		
myvtx->g=0.2; 			
myvtx->b=0.2; 		
myvtx->a=1.0;
myvtx++;

myvtx->x=100.0; 	
myvtx->y=100.0; 		
myvtx->w=0.80;		
myvtx->u=xs*.33*(myvtx->w); 	
myvtx->v=ys*(myvtx->w);			
myvtx->r=0.2; 		
myvtx->g=0.2; 			
myvtx->b=0.2; 		
myvtx->a=1.0;
myvtx++;

myvtx->x=50.0; 		
myvtx->y=125.0; 		
myvtx->w=0.90;		
myvtx->u=xs*.66*(myvtx->w); 	
myvtx->v=0.0*(myvtx->w);			
myvtx->r=0.4; 		
myvtx->g=0.4; 			
myvtx->b=0.4; 		
myvtx->a=1.0;
myvtx++;

myvtx->x=100.0; 	
myvtx->y=125.0; 		
myvtx->w=0.90;		
myvtx->u=xs*.66*(myvtx->w); 	
myvtx->v=ys*(myvtx->w);			
myvtx->r=0.4; 		
myvtx->g=0.4; 			
myvtx->b=0.4; 		
myvtx->a=1.0;
myvtx++;

myvtx->x=50.0; 		
myvtx->y=150.0; 		
myvtx->w=0.95;		
myvtx->u=xs*(myvtx->w); 		
myvtx->v=0.0*(myvtx->w);			
myvtx->r=1.0; 		
myvtx->g=1.0; 			
myvtx->b=1.0; 		
myvtx->a=1.0;
myvtx++;

myvtx->x=100.0; 	
myvtx->y=150.0; 		
myvtx->w=0.95;		
myvtx->u=xs*(myvtx->w); 		
myvtx->v=ys*(myvtx->w);			
myvtx->r=1.0; 		
myvtx->g=1.0; 			
myvtx->b=1.0; 		
myvtx->a=1.0;
myvtx++;

flag=0;

/* turn on perspective correction */
CLT_ESCNTL( GS_Ptr(gs), 0, 0, 0);

printf("\nDisplaying textured rectangular fan with slight perspective correction\n");
printf("with shading at each of the vertices. The type of blending is MULT. \n\n");
printf("Press X on the controller to end display.\n\n");


while(1) {

	GS_BeginFrame(gs);

	/* clear frame buffer to blue */
	CLT_ClearFrameBuffer(gs, 0.0, 0.2, 0.5, 1.0, TRUE, TRUE);
	CLT_Sync(GS_Ptr(gs));

/***************************************************************************/
/* Display textured triangle fan					   */
/***************************************************************************/

/* texture mapping registers */

/* turn on texturing and set LOD number and filter type */
	CLT_TXTADDRCNTL(GS_Ptr(gs), 1, RC_TXTADDRCNTL_MINFILTER_BILINEAR, 
		RC_TXTADDRCNTL_INTERFILTER_TRILINEAR, RC_TXTADDRCNTL_MAGFILTER_BILINEAR, 3);

/* Set texture application control register */
	CLT_TXTTABCNTL(GS_Ptr(gs), RC_TXTTABCNTL_FIRSTCOLOR_TEXCOLOR, 
		RC_TXTTABCNTL_SECONDCOLOR_PRIMCOLOR, RC_TXTTABCNTL_THIRDCOLOR_CONSTCOLOR, 
		RC_TXTTABCNTL_FIRSTALPHA_TEXALPHA, RC_TXTTABCNTL_SECONDALPHA_PRIMALPHA, 
		RC_TXTTABCNTL_COLOROUT_BLEND, RC_TXTTABCNTL_ALPHAOUT_BLEND, 
		RC_TXTTABCNTL_BLENDOP_MULT);

/* destination blending registers */

		/* enable z buffering, z buffer output disable destination and source blending */
	CLT_DBUSERCONTROL(GS_Ptr(gs), 1, 1, 0, 0, 0, 0, 0, 
		  CLT_Const(DBUSERCONTROL, DESTOUTMASK, ALPHA) |
		  CLT_Const(DBUSERCONTROL, DESTOUTMASK, RED) |
		  CLT_Const(DBUSERCONTROL, DESTOUTMASK, GREEN) |
		  CLT_Const(DBUSERCONTROL, DESTOUTMASK, BLUE) );

		myvtx=(myvtx-numvert);			/* back to beginning of vertices */

/* Simple logic to move the object back and forth */
		for (cnt=0; cnt<numvert; cnt++)	{				
			if ( (!flag) && ((myvtx->x+(mySize/2))>BMWIDTH) ) { flag=1;
									     break; }
			myvtx++; 
		}		
		myvtx=(myvtx-cnt);

		for (cnt=0; cnt<numvert; cnt++) {						
			if ( (flag) && ((myvtx->x-(mySize/2))<1) ) {	flag=0; 
									break;}
			myvtx++;
		}
		myvtx=(myvtx-cnt);

/* Describe the FAN to the TE */
      	CLT_TRIANGLE(GS_Ptr(gs), 1, RC_STRIP, 1, 1, 1, numvert );

		for (cnt=0; cnt<numvert; cnt++) {			
		    /* Move the object */
		if (flag) myvtx->x=myvtx->x-1.0;
		else myvtx->x=myvtx->x+1.0;

/* Object is textured, shaded and has perspective correction */
		CLT_VertexRgbaUvW(GS_Ptr(gs), myvtx->x, myvtx->y, 
			myvtx->r, myvtx->g, myvtx->b, myvtx->a, 
			myvtx->u, myvtx->v, myvtx->w); 
		myvtx++;
		}

       	CLT_Sync(GS_Ptr(gs));

	/* send the command list which creates both objects to the TE */

	err=gs->gs_SendList(gs);
	CKERR( err, "SendList");

	GS_EndFrame(gs);

	fbindex = 1 - fbindex;
	GS_SetDestBuffer(gs, bitmaps[fbindex]);

	GetControlPad(1, FALSE, &cpad);
	buttons = cpad.cped_ButtonBits;
	if( buttons & ControlX )
		break;

	}

/* Free memory associated with views, bitmaps, etc */

RemoveView(viewItem);

err=DeleteItem(viewItem);
CKERR( err, "DeleteItem");

err=GS_FreeBitmaps(bitmaps, 3);
CKERR( err, "FreeBitmaps");

err=GS_FreeLists(gs);
CKERR( err, "FreeLists");

err=GS_Delete(gs);
CKERR( err, "DeleteItem");

err=KillEventUtility();
CKERR( err, "KillEventUtil");
	
err=CloseGraphicsFolio();
CKERR( err, "CloseGraphicsFolio");

printf("Quitting \n");


 }




