/******************************************************************************
**
**  Copyright (c) 1993-1996, an unpublished work by The 3DO Company.
**  All rights reserved. This material contains confidential
**  information that is the property of The 3DO Company. Any
**  unauthorized duplication, disclosure or use is prohibited.
**  @(#) cltapiFT.c 96/10/29 1.3
**  Tests CLT api calls
**  Written by:		       Paige Wuestefeld  
**
******************************************************************************/

#ifdef MACINTOSH
#include <stdio.h>
#include <stdlib.h>
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#include <graphics:clt:gstate.h>
#include <graphics:graphics.h>
#include <graphics:view.h>
#include <tests:3do_scaf.h>
#include <assert.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <graphics/clt/gstate.h>
#include <graphics/graphics.h>
#include <graphics/view.h>
#include <tests/3do_scaf.h>
#include <assert.h>
#endif

/* standard error checking */
#define CKERR(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y); PrintfSysErr(x); exit(1);}}

/* since all the CLT api do is write the packed CLA word if the CLA_XXX test passes and the 	*/
/* CLT tests compile and execute without error then they are okay. In addition, the CLT		*/
/* macros were VISUALLY inspected in the cltmacros header to make sure that the ordering in  	*/
/* them was the same as the ordering in the CLA macros.		 				*/

#define checka(x)	BeginTest(FT, Clip( (#x) ) ); (x) ? LogResult(PASSED, "" ) : LogResult(FAILED, "" ); count++;
#define checks(x) 	BeginTest(FT, Clip( (#x) ) ); x; LogResult(PASSED, "" ); count++;

char*	Clip(char* x);


char	final[50];

main(int argc, char **argv)
{

	GState			*gs;
	int			count;
	
	Err			err;
	
	/*
	 * Initialize - creates GState, Allocs Lists, assigns bitmaps, etc (see PortInit.c)
	 */

	InitScaffold(GRAPHICS_CLT, argc, argv);

	gs= GS_Create(); 							
	if (gs==NULL) {printf("Unable to create GState\n"); exit(1);}

  	err = GS_AllocLists(gs, 2, (32*1024));
	CKERR( err, "AllocLists");
	
	count=1;
					/* sets destination blend user control register */

	checka(CLA_DBUSERCONTROL(0x1,0x0,0x0,0x1,0x0,0x0,0x0,0x1)==0x00000481);
	checks(CLT_DBUSERCONTROL(&(gs->gs_ListPtr),0x1,0x0,0x0,0x1,0x0,0x0,0x0,0x1));

					/* sets the pixel discard control register */

	checka(CLA_DBDISCARDCONTROL(0x1,0x1,0x1,0x0)==0x0000000E);
	checks(CLT_DBDISCARDCONTROL(&(gs->gs_ListPtr),0x1,0x1,0x1,0x0));

					/* sets the X window clip value register */

	checka(CLA_DBXWINCLIP(2,320)==0x00020140);
	checks(CLT_DBXWINCLIP(&(gs->gs_ListPtr),0,640));

					/* sets the Y window clip value register */

	checka(CLA_DBYWINCLIP(0,320)==0x00000140);
	checks(CLT_DBYWINCLIP(&(gs->gs_ListPtr),0,480));


					/* sets the source read control register */
					
	checka(CLA_DBSRCCNTL(1, 1)==0x00000003);			      /* one bit each */
	checks(CLT_DBSRCCNTL(&(gs->gs_ListPtr), 1, 1));			      /* one bit each */

					/* sets the source XStride register */

	checka(CLA_DBSRCXSTRIDE(2)==0x00000002);		/* distance in pix to next line */
	checks(CLT_DBSRCXSTRIDE(&(gs->gs_ListPtr), 2));		/* distance in pix to next line */

					/* sets the offsets register */

	checka(CLA_DBSRCOFFSET(4, 8)==0x00040008);	/* x and y offsets for source reads */
	checks(CLT_DBSRCOFFSET(&(gs->gs_ListPtr), 4, 8));   /* x and y offsets for source reads */

					/* sets the Z buffer control register */

	checka(CLA_DBZCNTL(1, 1, 0, 0, 0, 0)==0x00000030);	
	checks(CLT_DBZCNTL(&(gs->gs_ListPtr), 1, 1, 0, 0, 0, 0));	

					/* sets the offsets for Z register */

	checka(CLA_DBZOFFSET(0, 2)== 0x00000002);
	checks(CLT_DBZOFFSET(&(gs->gs_ListPtr), 0, 2));		 /* X AND Y Z OFFSET */

					/* sets the SSB/DSB control register */

	checka(CLA_DBSSBDSBCNTL(1, RC_DBSSBDSBCNTL_DSBSELECT_CONST)==0x00000005);
	checks(CLT_DBSSBDSBCNTL(&(gs->gs_ListPtr), 1, RC_DBSSBDSBCNTL_DSBSELECT_CONST));

					/* sets the CONSTIN register */

	checka(CLA_DBCONSTIN(0xFF, 0xFF, 0xFF)==0x00FFFFFF);
	checks(CLT_DBCONSTIN(&(gs->gs_ListPtr), 0xFF, 0xFF, 0xFF));

					/* sets the texture multiplication control register */

	checka(CLA_DBAMULTCNTL(RC_DBAMULTCNTL_AINPUTSELECT_CONSTCOLOR, 
			       RC_DBAMULTCNTL_AMULTCOEFSELECT_SRCCOLOR,
			       RC_DBAMULTCNTL_AMULTCONSTCONTROL_TEXSSB, 1)==0x00000071);
	checks(CLT_DBAMULTCNTL(&(gs->gs_ListPtr), RC_DBAMULTCNTL_AINPUTSELECT_CONSTCOLOR, 
						RC_DBAMULTCNTL_AMULTCOEFSELECT_SRCCOLOR,
						RC_DBAMULTCNTL_AMULTCONSTCONTROL_TEXSSB, 1));

					/* sets the texture coef constant0 register */

	checka(CLA_DBAMULTCONSTSSB0(0xff, 0xff, 0x00)==0x00ffff00);
	checks(CLT_DBAMULTCONSTSSB0(&(gs->gs_ListPtr),0xff, 0xff, 0x00));

					/* sets the texture coef constant1 register */

	checka(CLA_DBAMULTCONSTSSB1(0x00, 0xff, 0xff)==0x0000ffff);
	checks(CLT_DBAMULTCONSTSSB1(&(gs->gs_ListPtr), 0x00, 0xff, 0xff));

					/* sets the source multiplication control register */

	checka(CLA_DBBMULTCNTL(RC_DBBMULTCNTL_BINPUTSELECT_TEXCOLORCOMPLEMENT, 
					RC_DBBMULTCNTL_BMULTCOEFSELECT_TEXCOLORCOMPLEMENT, 
					RC_DBBMULTCNTL_BMULTCONSTCONTROL_SRCSSB, 0)==0x000000B6);
	checks(CLT_DBBMULTCNTL(&(gs->gs_ListPtr), RC_DBBMULTCNTL_BINPUTSELECT_TEXCOLORCOMPLEMENT, 
					RC_DBBMULTCNTL_BMULTCOEFSELECT_TEXCOLORCOMPLEMENT, 
					RC_DBBMULTCNTL_BMULTCONSTCONTROL_SRCSSB, 0));

					/* sets the source coef constant0 register */

	checka(CLA_DBBMULTCONSTSSB0(0x00, 0xff, 0x00)==0x0000ff00);
	checks(CLT_DBBMULTCONSTSSB0(&(gs->gs_ListPtr), 0, 0, 0));

					/* sets the source coef constant0 register */

	checka(CLA_DBBMULTCONSTSSB1(0xff, 0x00, 0x00)==0x00ff0000);
	checks(CLT_DBBMULTCONSTSSB1(&(gs->gs_ListPtr), 0, 0, 0));

					/* sets the ALU control register */

	checka(CLA_DBALUCNTL(RC_DBALUCNTL_ALUOPERATION_NOTB_AND_A, 0)==0x000000A0);
	checks(CLT_DBALUCNTL(&(gs->gs_ListPtr), RC_DBALUCNTL_ALUOPERATION_NOTB_AND_A, 0));

					/* sets the source alpha control register */

	/* you have to set all 6 bits */
	checka(CLA_DBSRCALPHACNTL(0x3)==0x00000003); 
	checks(CLT_DBSRCALPHACNTL(&(gs->gs_ListPtr), 0x3));

					/* sets the destination alpha control register */

	checka(CLA_DBDESTALPHACNTL(RC_DBDESTALPHACNTL_DESTCONSTSELECT_TEXALPHA)==0x00000000);
	checks(CLT_DBDESTALPHACNTL(&(gs->gs_ListPtr), RC_DBDESTALPHACNTL_DESTCONSTSELECT_TEXALPHA));

					/* sets the destination alpha constants register */

	checka(CLA_DBDESTALPHACONST(0x00, 0xff)==0x000000ff);
	checks(CLT_DBDESTALPHACONST(&(gs->gs_ListPtr), 0x00, 0xff));

					/* sets the texture loader control register */

	checka(CLA_TXTLDCNTL(1, RC_TXTLDCNTL_LOADMODE_TEXTURE, 0)==0x00000400);
	checks(CLT_TXTLDCNTL(&(gs->gs_ListPtr),1, RC_TXTLDCNTL_LOADMODE_TEXTURE, 0));

					/* sets the address control register */

	checka(CLA_TXTADDRCNTL(1, RC_TXTADDRCNTL_MINFILTER_BILINEAR, 
			       RC_TXTADDRCNTL_INTERFILTER_TRILINEAR, 
			       RC_TXTADDRCNTL_MAGFILTER_POINT, 3)==0x00002983);
	checks(CLT_TXTADDRCNTL(&(gs->gs_ListPtr), 1, RC_TXTADDRCNTL_MINFILTER_BILINEAR, 
			RC_TXTADDRCNTL_INTERFILTER_TRILINEAR, RC_TXTADDRCNTL_MAGFILTER_POINT, 4));

					/* sets the PIP control register */

	checka(CLA_TXTPIPCNTL(RC_TXTPIPCNTL_PIPSSBSELECT_CONSTANT, 
					RC_TXTPIPCNTL_PIPALPHASELECT_TEXTURE, 
					RC_TXTPIPCNTL_PIPCOLORSELECT_PIP, 0)==0x00000A00);
	checks(CLT_TXTPIPCNTL(&(gs->gs_ListPtr), RC_TXTPIPCNTL_PIPSSBSELECT_CONSTANT, 
					RC_TXTPIPCNTL_PIPALPHASELECT_TEXTURE, 
					RC_TXTPIPCNTL_PIPCOLORSELECT_PIP, 0));

					/* sets the texture application control register */

	checka(CLA_TXTTABCNTL(RC_TXTTABCNTL_FIRSTCOLOR_TEXCOLOR, 
			      RC_TXTTABCNTL_SECONDCOLOR_PRIMALPHA, 
			      RC_TXTTABCNTL_THIRDCOLOR_CONSTCOLOR, 
			      RC_TXTTABCNTL_FIRSTALPHA_TEXALPHA, 
			      RC_TXTTABCNTL_SECONDALPHA_PRIMALPHA, RC_TXTTABCNTL_COLOROUT_BLEND, 
			      RC_TXTTABCNTL_ALPHAOUT_TEXALPHA, 0)==0x00008D43);
	checks(CLT_TXTTABCNTL(&(gs->gs_ListPtr), RC_TXTTABCNTL_FIRSTCOLOR_TEXCOLOR, 
			      RC_TXTTABCNTL_SECONDCOLOR_PRIMALPHA, 
			      RC_TXTTABCNTL_THIRDCOLOR_CONSTCOLOR, 
			      RC_TXTTABCNTL_FIRSTALPHA_TEXALPHA, 
			      RC_TXTTABCNTL_SECONDALPHA_PRIMALPHA, 
			      RC_TXTTABCNTL_COLOROUT_BLEND, RC_TXTTABCNTL_ALPHAOUT_TEXALPHA, 0));

			/* sets mip-map addresses (0-3) (should be 32 bit word aligned) */
			/* no CLA packing needed				*/

	checks(CLT_TXTLODBASE0(&(gs->gs_ListPtr), 0));

	checks(CLT_TXTLODBASE1(&(gs->gs_ListPtr), 0xFF000000));

	checks(CLT_TXTLODBASE2(&(gs->gs_ListPtr), 0x00FF0000));

	checks(CLT_TXTLODBASE3(&(gs->gs_ListPtr), 0x0000FF00));

			/* sets the texture loader count register */
					
	checka(CLA_TXTCOUNT(16)==0x00000010);	/* do X 16 ; X depends on loader mode */
	checks(CLT_TXTCOUNT(&(gs->gs_ListPtr), 16));	/* do X 16 ; X depends on loader mode */

			/* sets the texture loader width register */

	checka(CLA_TXTUVMAX(160, 120)==0x00A00078); /* width of dest&source frame buffer in pix */
	checks(CLT_TXTUVMAX(&(gs->gs_ListPtr), 160, 120)); 	

					/* sets the texture mask register */

				/* values as per the programmers manual for proper operation */
	checka(CLA_TXTUVMASK(0x3FF, 0x3FF)==0x03FF03FF);
	checks(CLT_TXTUVMASK(&(gs->gs_ListPtr), 0x3FF, 0x3FF));

					/* sets the texture source type registers (0-3) 01, 23 */

	checka(CLA_TXTSRCTYPE01(0x5, 0x4, 0, 0, 0, 0, 0, 0x8, 0x7, 1, 1, 1, 1, 1)==0x1F780045);
	checks(CLT_TXTSRCTYPE01(&(gs->gs_ListPtr),  0x5, 0x4, 0, 0, 0, 0, 0, 0x8, 0x7, 1, 1, 1, 1, 1));

	checka(CLA_TXTSRCTYPE23(0x8, 0x7, 1, 1, 1, 1, 1, 0x5, 0x4, 0, 0, 0, 0, 0)==0x00451F78);
	checks(CLT_TXTSRCTYPE23(&(gs->gs_ListPtr),  0x8, 0x7, 1, 1, 1, 1, 1, 0x5, 0x4, 0, 0, 0, 0, 0));

	checka(CLA_TXTCONST0(0xff, 0xff, 0xff, 1, 1)==0x81ffffff);
	checks(CLT_TXTCONST0(&(gs->gs_ListPtr), 0xff, 0xff, 0xff, 1, 1));

	checka(CLA_TXTCONST1(0xff, 0x00, 0xff, 0, 0)==0x00ff00ff);
	checks(CLT_TXTCONST1(&(gs->gs_ListPtr), 0xff, 0x00, 0xff, 0, 0));

	checka(CLA_TXTCONST2(0x00, 0xff, 0xab, 1, 1)==0x8100ffab);
	checks(CLT_TXTCONST2(&(gs->gs_ListPtr), 0x00, 0xff, 0xab, 1, 1));

	checka(CLA_TXTCONST3(0x02, 0xff, 0xff, 1, 0)==0x0102ffff);
	checks(CLT_TXTCONST3(&(gs->gs_ListPtr), 0x02, 0xff, 0xff, 1, 0));

	checks(CLT_DbDitherMatrix(&(gs->gs_ListPtr),0x11111111,0x01010101));

					/*base addr for source */

	checks(CLT_DBSRCBASEADDR(&(gs->gs_ListPtr), 0x00000FFF));	 /*base addr for source */

printf("\nFinished! %d APIs were checked.\n", count);

	err=GS_FreeLists(gs);
	CKERR( err, "FreeLists");

	err=GS_Delete(gs);
	CKERR( err, "DeleteGS");


TermScaffold();
}

/* Function to Clip the portion of the testname which we need */

char*	Clip(char* x)
{
int		i=0;

strcpy(final," ");

while (x[i] != '(') i++;
strncat(final,x,i);

return (final);

}

