/* @(#) testAbort.c 96/10/29 1.3							*/
/* Written by Paige Wuestefeld, the 3DO Company x2773			*/
/*									*/
/* 8/96									*/
/*									*/
/* Data Required: brick.utf						*/
/*									*/
/* Tests puts up 2 identical displays - one on the top, one		*/
/* on the bottom. Each display consists of a textured object	 	*/
/* and a specified number of partially transparent triangles 		*/
/* (default=125) translating in X. 					*/
/*									*/
/* Abort feature is off by default. To turn it on use:			*/
/*									*/
/* testAbort -abort 1							*/
/*									*/
/* If the abort is working you should get ~60 fps and also		*/
/* not see the opaque red triangles in the upper display -  		*/
/* (because they are aborted)						*/
/*									*/

#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <graphics/clt/gstate.h>
#include <graphics/graphics.h>
#include <graphics/frame2d/f2d.h>
#include <graphics/view.h>
#include <kernel/io.h>
#include <kernel/mem.h>
#include <kernel/random.h>
#include <misc/event.h>
#include <device/te.h>
#include <kernel/time.h>
#include <kernel/task.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <strings.h>
#include "texload.h"

/* error checking */
#define CKERR(x,y) {Err ick=(x);if(ick<0){printf("Error - %s\n",y);PrintfSysErr(x); exit(1);}}

#define bufSize	150000
#define NUMSNIP 100
#define MAXVERT	500
#define BMWIDTH 320
#define BMHEIGHT 240
#define SHIFTY 120
#define numvert 7
#define mySize 90

  typedef struct mycoords {
    gfloat x;
    gfloat y;
    gfloat u;
    gfloat v;
    gfloat w;
  } mycoords;

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

/* prototypes */

GState *myGSInit (void);
int Create_Display (GState * gs, mycoords * myvtx, gfloat * TriX, gfloat * TriY, CltSnippet * mysnippet, int frame, gfloat inc);
mycoords *GetFan1 (TextureSnippets * snips);
mycoords *GetFan2 (TextureSnippets * snips);
CltSnippet *SetSnippet (void);
void RndVert (float *x, float *y, float *r, float *g, float *b, float *a);

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

/* globals */
Item bitmaps[3];
gfloat TriX[MAXVERT], TriY[MAXVERT], r[MAXVERT], g[MAXVERT], b[MAXVERT],
  a[MAXVERT], TriY2[MAXVERT];
int TRISPERLIST;

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

void
main (int argc, char *argv[])
{

  GState *gs, *gs2;
  mycoords *myvtx, *myvtx2;
  TextureSnippets snips;
  CltSnippet *mysnippet;
  ControlPadEventData cpad;
  Item viewItem;
  Err err;
  TimeVal *timesample1, *timesample2;
  int i, cnt;
  uint32 fbindex = 0;
  int32 frame = 0;
  uint32 result, buttons, buffSignal;
  gfloat fps, inc, inc2;
  int ABORTVBL;

  printf ("Usage: \n");
  printf ("testAbort -abort 1 <turns abort on>\n");
  printf ("testAbort -abort 0 <turns abort off>\n");

/* set defaults */
  gs=gs2=NULL;

  TOUCH(gs);
  TOUCH(gs2);

  TRISPERLIST = 125;
  ABORTVBL = 0;

  for (i = 1; i < argc; i++) {

    if (strcmp (argv[i], "-abort") == 0)
      ABORTVBL = (atoi (argv[i + 1]));

    if (strcmp (argv[i], "-tris") == 0)
      TRISPERLIST = (atoi (argv[i + 1]));

  }

/* print options selected */
  printf ("Displaying %i triangles in each pane\n", TRISPERLIST);

/* TimeVal structure defined in kernel:time.h */
  timesample1 = AllocMem (sizeof (TimeVal), MEMTYPE_NORMAL);
  timesample2 = AllocMem (sizeof (TimeVal), MEMTYPE_NORMAL);

  myvtx = AllocMem (((numvert + 2) * sizeof (mycoords)), MEMTYPE_NORMAL);
  if (myvtx == NULL)
    printf ("Memory Allocation Failure\n");

  myvtx2 = AllocMem (((numvert + 2) * sizeof (mycoords)), MEMTYPE_NORMAL);
  if (myvtx2 == NULL)
    printf ("Memory Allocation Failure\n");

  result = InitEventUtility (1, 0, LC_ISFOCUSED);
  CKERR (result, "InitEventUtility");

  err = OpenGraphicsFolio ();
  CKERR (err, "OpenGraphicsFolio");

  gs = GS_Create ();
  if (gs == NULL) {
    printf ("Unable to create GState\n");
    exit (1);
  }

  err = GS_AllocBitmaps (bitmaps, BMWIDTH, BMHEIGHT, BMTYPE_16, 2, 1);
  CKERR (err, "AllocBitmaps");

  err = GS_AllocLists (gs, 2, bufSize);
  CKERR (err, "AllocLists");

  err = GS_SetZBuffer (gs, bitmaps[2]);
  CKERR (err, "GSSetZBuffer");

  buffSignal = AllocSignal (0);

  err = GS_SetVidSignal (gs, buffSignal);
  CKERR (err, "GSSetVidSignal");

  err = GS_SetDestBuffer (gs, bitmaps[0]);
  CKERR (err, "GSSetDestBuffer");

  viewItem = CreateItemVA (MKNODEID (NST_GRAPHICS, GFX_VIEW_NODE),
			   VIEWTAG_VIEWTYPE, VIEWTYPE_16,
			   VIEWTAG_BITMAP, bitmaps[0],
			   VIEWTAG_RENDERSIGNAL, buffSignal,
			   TAG_END);
  CKERR (viewItem, "ViewItem");

  AddViewToViewList (viewItem, 0);

  err = GS_SetView (gs, viewItem);
  CKERR (err, "GS_SetView");

  err = GS_SetClearValues (gs, 0.0, 0.2, 0.5, 1.0, 1.0);
  CKERR (err, "GS_SetClearValues");

/* Load texture from disk into TRAM, including its PIP if there is one */
  CLT_InitSnippet (&snips.dab);
  CLT_InitSnippet (&snips.tab);
  snips.lcb = NULL;
  snips.pip = NULL;
  snips.txdata = NULL;

/* get command list snippets for specified texture */
  LoadTexture (&snips, ("brick.utf"));

  if (&snips == NULL) {
    printf ("Snips NULL... exiting ...\n");
    exit (-1);
  }

/* get fan vertices */
  myvtx = GetFan1 (&snips);
  if (myvtx == NULL) {
    printf ("Problem getting vertices... exiting ...\n");
    exit (-1);
  }
  myvtx2 = GetFan2 (&snips);
  if (myvtx2 == NULL) {
    printf ("Problem getting vertices... exiting ...\n");
    exit (-1);
  }
/* get snippet for src blending */
  mysnippet = SetSnippet ();

/* turn on perspective correction */
  CLT_ESCNTL (&(gs->gs_ListPtr), 0, 0, 0);

/* get vertice values */
  RndVert (TriX, TriY, r, g, b, a);

/* shift triangles in Y */
  for (cnt = 0; cnt < (TRISPERLIST * 3); cnt++) {
    TriY2[cnt] = TriY[cnt] - (240 / 2);
  }

  inc = inc2 = 1.0;

  SendSignal (viewItem, buffSignal);

  SampleSystemTimeTV (timesample1);

  while (1) {

    frame++;

    GS_BeginFrame (gs);
    CKERR (err, "GS_BeginFrame");

    /* load the texture - need to do this per frame as the TE is reset with the abort */
    UseTxLoadCmdLists (gs, &snips);

    if (ABORTVBL) {
      err = GS_SetAbortVblank (gs, FALSE);
      CKERR (err, "SetAbortVblank");
    }

    err = GS_ClearFBZ (gs, TRUE, FALSE);
    CKERR (err, "GS_ClearFBZ");

    inc = Create_Display (gs, myvtx, TriX, TriY, mysnippet, frame, inc);
    err = gs->gs_SendList (gs);	/* bottom */
    CKERR (err, "SendList 1");

    if ((gs->gs_EndList - gs->gs_ListPtr) < 1) {
      printf ("Over-Ran Command List... EXITING ....\n");
      printf ("Start: %p Current: %p End: %p \n", GS_GetCurListStart (gs), gs->gs_ListPtr, gs->gs_EndList);
      break;
    }

    GS_WaitIO (gs);
    ModifyGraphicsItemVA (viewItem,
			  VIEWTAG_BITMAP, bitmaps[fbindex],
			  TAG_END);

    if (ABORTVBL) {
      err = GS_SetAbortVblank (gs, TRUE);
      CKERR (err, "SetAbortVblank");
    }

    /* this is the list i am willing to have aborted */
    inc2 = Create_Display (gs, myvtx2, TriX, TriY2, mysnippet, frame, inc2);
    err = gs->gs_SendList (gs);	/* top */
    CKERR (err, "SendList 2");

    if (ABORTVBL) {
      err = GS_SetAbortVblank (gs, FALSE);
      CKERR (err, "SetAbortVblank");
    }

    fbindex = 1 - fbindex;
    GS_SetDestBuffer (gs, bitmaps[fbindex]);

    SampleSystemTimeTV (timesample2);

    GetControlPad (1, FALSE, &cpad);
    buttons = cpad.cped_ButtonBits;
    if (buttons & ControlX)
      break;

  }

  fps = (float) frame / (float) (timesample2->tv_sec - timesample1->tv_sec);
  printf ("Frames/second= %f\n", fps);

/* Free memory associated with views, bitmaps, etc */

  RemoveView (viewItem);

  err = DeleteItem (viewItem);
  CKERR (err, "DeleteItem");

  err = GS_FreeBitmaps (bitmaps, 3);
  CKERR (err, "FreeBitmaps");

  err = GS_FreeLists (gs);
  CKERR (err, "FreeLists");

  err = GS_Delete (gs);
  CKERR (err, "DeleteGS1");

  err = KillEventUtility ();
  CKERR (err, "KillEventUtil");

  err = CloseGraphicsFolio ();
  CKERR (err, "CloseGraphicsFolio");

  CLT_FreeSnippet (mysnippet);
  FreeMem (myvtx, ((numvert + 1) * sizeof (mycoords)));
  FreeMem (myvtx2, ((numvert + 1) * sizeof (mycoords)));

  printf ("\nFinished! \n");


}

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

GState *
myGSInit (void)
{
  Err err;
  int32 buffSignal;
  GState *gs;


  gs = GS_Create ();
  if (gs == NULL) {
    printf ("Unable to create GState\n");
    exit (1);
  }
  err = GS_AllocBitmaps (bitmaps, BMWIDTH, BMHEIGHT, BMTYPE_16, 2, 1);
  CKERR (err, "AllocBitmaps");

  err = GS_AllocLists (gs, 2, bufSize);
  CKERR (err, "AllocLists");

  err = GS_SetZBuffer (gs, bitmaps[2]);
  CKERR (err, "GSSetZBuffer");

  buffSignal = AllocSignal (0);

  err = GS_SetVidSignal (gs, buffSignal);
  CKERR (err, "GSSetVidSignal");

  err = GS_SetDestBuffer (gs, bitmaps[0]);
  CKERR (err, "GSSetDestBuffer");


  return (gs);

}

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

int
Create_Display (GState * gs, mycoords * myvtx, gfloat * TriX, gfloat * TriY, 
CltSnippet * mysnippet, int frame, gfloat inc)
{

  int cnt;

/* this creates a command list for the display 				*/
/* the display is a textured fan with a designated number of partially 	*/
/* transparent triangles which blend. everything is translating in X	*/

/* texture mapping registers */

  /* use global snippet to turn on texturing since ClearFrameBuffer
	 * turns it off */
  CLT_CopySnippetData (&(gs->gs_ListPtr), &CltEnableTextureSnippet);

  /* clear colorout and alphaout and set from texture */
  CLT_ClearRegister (gs->gs_ListPtr, TXTTABCNTL,
		     FV_TXTTABCNTL_COLOROUT_MASK |
		     FV_TXTTABCNTL_ALPHAOUT_MASK);

  CLT_SetRegister (gs->gs_ListPtr, TXTTABCNTL,
		   CLT_SetConst (TXTTABCNTL, COLOROUT, TEXCOLOR) |
		   CLT_SetConst (TXTTABCNTL, ALPHAOUT, TEXALPHA));

/* destination blending registers */

  /* disable Destination and SCR blending, enable z buffering and Z
	 * buffer output */
  CLT_DBUSERCONTROL (&(gs->gs_ListPtr), 1, 1, 0, 0, 0, 0, 0,
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, ALPHA) |
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, RED) |
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, GREEN) |
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, BLUE));

  CLT_DBZCNTL (&(gs->gs_ListPtr), 1, 1, 1, 1, 1, 1);

  CLT_TRIANGLE (&(gs->gs_ListPtr), 1, RC_FAN, 1, 1, 0, numvert);

  for (cnt = 0; cnt < numvert; cnt++) {
    if (((myvtx->x + inc) >= (float) (BMWIDTH - 50)) || ((myvtx->x + inc) <= (float) (0 + 50))) {	/* printf("SWITCH at %f\n", myvtx->x); */
      inc = (-1.0 * inc);
      break;
    }
  }

  for (cnt = 0; cnt < numvert; cnt++) {
    myvtx->x = myvtx->x + inc;
    CLT_VertexUvW (&(gs->gs_ListPtr), myvtx->x, myvtx->y, myvtx->u, myvtx->v, myvtx->w);
    myvtx++;
  }

  CLT_Sync (&(gs->gs_ListPtr));

/* Add simple shaded triangle which is transparent and dest blends */

/* texture mapping registers */

  /* use global snippet to turn off texturing */
  CLT_CopySnippetData (&(gs->gs_ListPtr), &CltNoTextureSnippet);

  /* sets src base address to current destination buffer, xstride, and
	 * offset */
  CLT_SetSrcToCurrentDest (gs);

  CLT_CopySnippetData (&(gs->gs_ListPtr), mysnippet);

  /* this is a cheap wrap around - just reposition the offender */
  for (cnt = 0; cnt < (TRISPERLIST * 3); cnt++) {
    TriX[cnt] += 1.0;
    if (TriX[cnt] > (BMWIDTH))
      TriX[cnt] = 1.0;
  }

  cnt = 0;
  while (cnt < (TRISPERLIST * 3)) {
    CLT_TRIANGLE (&(gs->gs_ListPtr), 1, RC_STRIP, 0, 0, 1, 3);
    CLT_VertexRgba (&(gs->gs_ListPtr), TriX[cnt], TriY[cnt], r[cnt], g[cnt], b[cnt], a[cnt]);
    cnt++;
    CLT_VertexRgba (&(gs->gs_ListPtr), TriX[cnt], TriY[cnt], r[cnt], g[cnt], b[cnt], a[cnt]);
    cnt++;
    CLT_VertexRgba (&(gs->gs_ListPtr), TriX[cnt], TriY[cnt], r[cnt], g[cnt], b[cnt], a[cnt]);
    cnt++;
  }

  CLT_Sync (&(gs->gs_ListPtr));
  TOUCH(frame);

  return (inc);
}

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

CltSnippet *
SetSnippet (void)
{
  CltSnippet *mysnippet;
  Err err;

/* create a CLT snippet for translucent destination blending case */

  mysnippet = AllocMem ((NUMSNIP * (sizeof (CltSnippet))), MEMTYPE_NORMAL);
  if (mysnippet == NULL)
    printf ("Failed to allocate memory for snippet\n");

  CLT_InitSnippet (mysnippet);
  err = CLT_AllocSnippet (mysnippet, NUMSNIP);	/* NUMSNIP words may be
							 * put into mysnippet */
  CKERR (err, "AllocSnippet");

  mysnippet->size = 5 * 2;

  /* enable destination blending and source blending, z buffering but
	 * not  */
  /* z buffering output - for transparency to work correctly		 */
  CLT_DBUSERCONTROL (&(mysnippet->data), 1, 0, 0, 0, 1, 1, 0,
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, ALPHA) |
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, RED) |
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, GREEN) |
		     CLT_Const (DBUSERCONTROL, DESTOUTMASK, BLUE));

  /* need to set the Z buffer control or ClearFBZ won't work */
  CLT_DBZCNTL (&(mysnippet->data), 1, 1, 1, 1, 1, 1);

  CLT_DBAMULTCNTL (&(mysnippet->data),
		   CLT_Const (DBAMULTCNTL, AINPUTSELECT, TEXCOLOR),
		   CLT_Const (DBAMULTCNTL, AMULTCOEFSELECT, TEXALPHA),
		   CLT_Const (DBAMULTCNTL, AMULTCONSTCONTROL, TEXSSB),
		   0);

  /* B is the SRC blending buffer		 */
  CLT_DBBMULTCNTL (&(mysnippet->data),
		   CLT_Const (DBBMULTCNTL, BINPUTSELECT, SRCCOLOR),
	       CLT_Const (DBBMULTCNTL, BMULTCOEFSELECT, TEXALPHACOMPLEMENT),
		   CLT_Const (DBBMULTCNTL, BMULTCONSTCONTROL, SRCSSB),
		   0);

  /* Add the two and clamp on overrun */
  CLT_DBALUCNTL (&(mysnippet->data), CLT_Const (DBALUCNTL, ALUOPERATION, A_PLUS_BCLAMP), 0);

  mysnippet->data -= mysnippet->size;

  return (mysnippet);
}

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

void
RndVert (float *x, float *y, float *r, float *g, float *b, float *a)
{

  int cnt;

/* generates initial vertex values and alpha for each triangle 	*/

  for (cnt = 0; cnt < (TRISPERLIST * 3); cnt++) {
    x[cnt] = (gfloat) (rand () % BMWIDTH);
    y[cnt] = (gfloat) (rand () % (BMHEIGHT / 2) + (BMHEIGHT / 2));
    r[cnt] = (gfloat) (rand () % 10000) / 10000.0;
    g[cnt] = (gfloat) (rand () % 10000) / 10000.0;
    b[cnt] = (gfloat) (rand () % 10000) / 10000.0;
    a[cnt] = 0.3;
  }

/* make the last three opaque red - that way we know if the list is being aborted */
  for (cnt = ((TRISPERLIST - 3) * 3); cnt < (TRISPERLIST * 3); cnt++) {
    r[cnt] = 1.0;
    g[cnt] = 0.0;
    b[cnt] = 0.0;
    a[cnt] = 1.0;
  }
}

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

mycoords *
GetFan1 (TextureSnippets * snips)
{
  gfloat xs, ys, dev1, dev2;
  mycoords *myvtx;

/* initial values for textured FAN (rectangular) with perspective correction 		*/
/* to get perspective correction working with textures you need to send u*w and v*w */
/* also on vertices 3, 4, 5, 6 there is a little "cheat" to position the far points	*/
/* it is not meant to be used in serious endeavor-a real projection should be used 	*/

  myvtx = AllocMem (((numvert + 2) * sizeof (mycoords)), MEMTYPE_NORMAL);
  if (myvtx == NULL)
    printf ("Memory Allocation Failure\n");

  xs = ys = ((snips->lcb->XSize * snips->txdata->maxLOD * 2) - 1.0);	/* max X of LOD 0 */

  dev1 = 45.0;
  dev2 = 15.0;

/* BOTTOM */
/* assign first set of vertices */
  myvtx->w = 0.98;
  myvtx->x = 265.0;
  myvtx->y = 205.0;
  myvtx->u = xs * .5 * myvtx->w;
  myvtx->v = ys * myvtx->w;
  myvtx++;
  myvtx->w = 0.98;
  myvtx->x = 220.0;
  myvtx->y = 205.0;
  myvtx->u = 0.0 * myvtx->w;
  myvtx->v = ys * myvtx->w;
  myvtx++;
  myvtx->w = 0.80;
  myvtx->x = 220.0 + ((dev1 * myvtx->w) / 5.0);
  myvtx->y = 155.0;
  myvtx->u = 0.0 * myvtx->w;
  myvtx->v = 0.0 * myvtx->w;
  myvtx++;
  myvtx->w = 0.80;
  myvtx->x = 250.0 + ((dev2 * myvtx->w) / 5.0);
  myvtx->y = 155.0;
  myvtx->u = xs * .33 * myvtx->w;
  myvtx->v = 0.0 * myvtx->w;
  myvtx++;
  myvtx->w = 0.80;
  myvtx->x = 280.0 - ((dev2 * myvtx->w) / 5.0);
  myvtx->y = 155.0;
  myvtx->u = xs * .67 * myvtx->w;
  myvtx->v = 0.0 * myvtx->w;
  myvtx++;
  myvtx->w = 0.80;
  myvtx->x = 310.0 - ((dev1 * myvtx->w) / 5.0);
  myvtx->y = 155.0;
  myvtx->u = xs * myvtx->w;
  myvtx->v = 0.0 * myvtx->w;
  myvtx++;
  myvtx->w = 0.98;
  myvtx->x = 310.0;
  myvtx->y = 205.0;
  myvtx->u = xs * myvtx->w;
  myvtx->v = ys * myvtx->w;
  myvtx++;

  myvtx = (myvtx - numvert);	/* back to beginning of vertices */

  return (myvtx);
}

/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

mycoords *
GetFan2 (TextureSnippets * snips)
{
  gfloat xs, ys, dev1, dev2;
  mycoords *myvtx2;

  myvtx2 = AllocMem (((numvert + 2) * sizeof (mycoords)), MEMTYPE_NORMAL);
  if (myvtx2 == NULL)
    printf ("Memory Allocation Failure\n");


/* initial values for textured FAN (rectangular) with perspective correction 		*/
/* to get perspective correction working with textures you need to send u*w and v*w */
/* also on vertices 3, 4, 5, 6 there is a little "cheat" to position the far points	*/
/* it is not meant to be used in serious endeavor-a real projection should be used 	*/

  xs = ys = ((snips->lcb->XSize * snips->txdata->maxLOD * 2) - 1.0);	/* max X of LOD 0 */

  dev1 = 45.0;
  dev2 = 15.0;

/* TOP */
/* assign second set of vertices */
  myvtx2->w = 0.98;
  myvtx2->x = 265.0;
  myvtx2->y = 205.0 - SHIFTY;
  myvtx2->u = xs * .5 * myvtx2->w;
  myvtx2->v = ys * myvtx2->w;
  myvtx2++;
  myvtx2->w = 0.98;
  myvtx2->x = 220.0;
  myvtx2->y = 205.0 - SHIFTY;
  myvtx2->u = 0.0 * myvtx2->w;
  myvtx2->v = ys * myvtx2->w;
  myvtx2++;
  myvtx2->w = 0.80;
  myvtx2->x = 220.0 + ((dev1 * myvtx2->w) / 5.0);
  myvtx2->y = 155.0 - SHIFTY;
  myvtx2->u = 0.0 * myvtx2->w;
  myvtx2->v = 0.0 * myvtx2->w;
  myvtx2++;
  myvtx2->w = 0.80;
  myvtx2->x = 250.0 + ((dev2 * myvtx2->w) / 5.0);
  myvtx2->y = 155.0 - SHIFTY;
  myvtx2->u = xs * .33 * myvtx2->w;
  myvtx2->v = 0.0 * myvtx2->w;
  myvtx2++;
  myvtx2->w = 0.80;
  myvtx2->x = 280.0 - ((dev2 * myvtx2->w) / 5.0);
  myvtx2->y = 155.0 - SHIFTY;
  myvtx2->u = xs * .67 * myvtx2->w;
  myvtx2->v = 0.0 * myvtx2->w;
  myvtx2++;
  myvtx2->w = 0.80;
  myvtx2->x = 310.0 - ((dev1 * myvtx2->w) / 5.0);
  myvtx2->y = 155.0 - SHIFTY;
  myvtx2->u = xs * myvtx2->w;
  myvtx2->v = 0.0 * myvtx2->w;
  myvtx2++;
  myvtx2->w = 0.98;
  myvtx2->x = 310.0;
  myvtx2->y = 205.0 - SHIFTY;
  myvtx2->u = xs * myvtx2->w;
  myvtx2->v = ys * myvtx2->w;
  myvtx2++;

  myvtx2 = (myvtx2 - numvert);	/* back to beginning of vertices */

  return (myvtx2);
}
