/*----------------------------------------------------------------------------------
 *	
 *	FileName:	Test2DAPI.c
 *	Date:		October 10, 1995
 *	By:			Ali Latifi	x3487
 *
 *  Copyright  1995 by The 3DO Company.  All rights reserved.
 *
 *  Modification History:
 *
 *	By:			Date:		Changed:
 *				
 *--------------------------------------------------------------------------------*/

#include "Test2D_API.h"

#define FBWIDTH		640
#define FBHEIGHT	480
#define DEPTH		32

Boolean		gInteractive = TRUE;			/* required for scaffold */
GState*		gGState;
Err			gErr;
Item		gBitmaps[3];
Item		gViewItem;
char		*fname;
Boolean		gCPadFlag = false;

enum {	eTest2D_CLT,
		eTest2D_F2,
		eTest2D_GRO,
		eTest2D_SPR,

		eTest2DAPI_End};

char* gGroupNames[] = {
		"Test2D_CLT",
		"Test2D_F2",
		"Test2D_GRO",
		"Test2D_SPR",

		""};


/* -------------------------------------------------------------------------------- */
TEST2DAPI* Test2DAPI_Construct( int argc, char* argv[] )
{ 
	TEST2DAPI*		Test2DNew;

	Test2DNew = (TEST2DAPI*) malloc(sizeof(TEST2DAPI));
	assert(Test2DNew);

	Test2DAPI_Initialize( Test2DNew, argc, argv );

	return Test2DNew;
}  

/* -------------------------------------------------------------------------------- */
void Test2DAPI_Initialize( TEST2DAPI* theTest2DAPI, int argc, char* argv[] )
{
	theTest2DAPI->gp = NULL;
	theTest2DAPI->scene = NULL;
	theTest2DAPI->group = NULL;
	theTest2DAPI->subject = NULL;
	theTest2DAPI->target = NULL;
	theTest2DAPI->theErr = 0;
	theTest2DAPI->gShouldBuildStandards = FALSE;
	theTest2DAPI->gInteractive = FALSE;

	gInteractive = theTest2DAPI->gInteractive;

	theTest2DAPI->gShouldDisplay = TRUE;
	theTest2DAPI->gRunCompleteTests = TRUE;
	theTest2DAPI->gRunCompleteGroup = TRUE;
	theTest2DAPI->groupNumber = 0;
	theTest2DAPI->testNumber = 0;

	argc=InitScaffold(GRAPHICS_2D, argc, argv);

 	/* Changes defaults based on command-line */
 	Test2DAPI_ParseArgs( theTest2DAPI, argc, argv );		

#if 1 
 	/* Scafold Initialize */
 	theTest2DAPI->scaffold = Scaffold_Construct( argc, argv);
	assert( theTest2DAPI->scaffold );

#endif
}

/* -------------------------------------------------------------------------------- */
uint32 Do_BestViewType (uint32 xres, uint32 yres, uint32 depth)
{
	uint32		aViewType;

	aViewType = VIEWTYPE_INVALID;

	if ((xres==320) && (yres==240) && (depth==16)) aViewType = VIEWTYPE_16;
	if ((xres==320) && (yres==480) && (depth==16)) aViewType = VIEWTYPE_16_LACE;
	if ((xres==320) && (yres==240) && (depth==32)) aViewType = VIEWTYPE_32;
	if ((xres==320) && (yres==480) && (depth==32)) aViewType = VIEWTYPE_32_LACE;
	if ((xres==640) && (yres==240) && (depth==16)) aViewType = VIEWTYPE_16_640;
	if ((xres==640) && (yres==480) && (depth==16)) aViewType = VIEWTYPE_16_640_LACE;
	if ((xres==640) && (yres==240) && (depth==32)) aViewType = VIEWTYPE_32_640;
	if ((xres==640) && (yres==480) && (depth==32)) aViewType = VIEWTYPE_32_640_LACE;

	return aViewType;
}

/*--------------------------------------------------------------------------------*/
void Do_DefaultTabInit (SpriteObj *sp)
{
	Spr_SetTextureAttr (sp, TXA_ColorOut, 1);
	Spr_SetTextureAttr (sp, TXA_BlendOp, 1);
	Spr_SetTextureAttr (sp, TXA_FirstColor, 1);
	Spr_SetTextureAttr (sp, TXA_SecondColor, 3);
	Spr_SetTextureAttr (sp, TXA_ThirdColor, 2);
	Spr_SetTextureAttr (sp, TXA_AlphaOut, 1);
	Spr_SetTextureAttr (sp, TXA_FirstAlpha, 2);
	Spr_SetTextureAttr (sp, TXA_SecondAlpha, 1);
	Spr_SetTextureAttr (sp, TXA_TextureEnable, 1);
	Spr_SetTextureAttr (sp, TXA_BlendColorSSB0, 0xffffffff);
	Spr_SetTextureAttr (sp, TXA_BlendColorSSB1, 0xffffffff);
	
	if (Spr_GetPIP(sp)) {
		Spr_SetTextureAttr (sp, TXA_PipColorSelect, 2);	/* If we have a PIP, use the PIP color */
	} else {
		Spr_SetTextureAttr (sp, TXA_PipColorSelect, 1);	/* If no PIP, use texture color */
	}
	
	Spr_SetTextureAttr (sp, TXA_PipAlphaSelect, 0);		/* Default to using constant alpha */
	Spr_SetTextureAttr (sp, TXA_PipSSBSelect, 0);		/* Default to using constant SSB */
}

/*--------------------------------------------------------------------------------*/
void Do_AlphaInit (SpriteObj *sp)
{
	Spr_SetDBlendAttr (sp, DBLA_EnableAttrs, DBL_BlendEnable|DBL_SrcInputEnable|DBL_AlphaDestOut|DBL_RGBDestOut);
	Spr_SetDBlendAttr (sp, DBLA_Discard, DBL_DiscardAlpha0);
	Spr_SetDBlendAttr (sp, DBLA_DSBSelect, DBL_DSBSelectObjSSB);
	Spr_SetDBlendAttr (sp, DBLA_AInputSelect, DBL_ASelectTexColor);
	Spr_SetDBlendAttr (sp, DBLA_AMultCoefSelect, DBL_MASelectTexAlpha);
	Spr_SetDBlendAttr (sp, DBLA_AMultRtJustify, 0);
	Spr_SetDBlendAttr (sp, DBLA_BInputSelect, DBL_BSelectSrcColor);
	Spr_SetDBlendAttr (sp, DBLA_BMultCoefSelect, DBL_MASelectTexAlphaComplement);
	Spr_SetDBlendAttr (sp, DBLA_BMultRtJustify, 0);
	Spr_SetDBlendAttr (sp, DBLA_ALUOperation, DBL_AddClamp);

	if (sp->spr_TexBlend && sp->spr_TexBlend->tb_txData && (sp->spr_TexBlend->tb_txData->expansionFormat&EXP_HASALPHA)) {
		Spr_SetTextureAttr (sp, TXA_PipAlphaSelect, 1);				/* Use texture alpha if we have texture alpha */
	} else if (Spr_GetPIP(sp)) {
				Spr_SetTextureAttr (sp, TXA_PipAlphaSelect, 2);		/* If we have a PIP, use the PIP alpha */
		} else {
			Spr_SetTextureAttr (sp, TXA_PipAlphaSelect, 0);			/* If no PIP, use constant alpha */
		}
}

/*--------------------------------------------------------------------------------*/
Boolean Test2DAPI_ConstructScene( TEST2DAPI* theTest2DAPI )
{ 	
	int32					theBuffSignal;
	uint32					theViewType = Do_BestViewType(FBWIDTH, FBHEIGHT, DEPTH);
	uint32					bmType = (DEPTH==16)?BMTYPE_16:BMTYPE_32;
	SpriteObj				*tempSprite;

	OpenGraphicsFolio ();
	theBuffSignal = AllocSignal (0);
	gGState = GS_Create();
	if (!gGState) {
		printf ("Error creating GState\n");
		exit (1);
	}

	if( InitEventUtility(1, 0, LC_ISFOCUSED) < 0 ) {			/* Initialize the EventBroker. */
		printf("Error in InitEventUtility\n");
		exit(1);
	}
  
	GS_AllocLists(gGState, 2, 2048);
	gErr = GS_AllocBitmaps(gBitmaps, FBWIDTH, FBHEIGHT, bmType, 2, 1);
	if(gErr < 0)
		printf("Error returned GS_AllocBitmaps. %d\n", gErr);
	GS_SetDestBuffer(gGState, gBitmaps[0]);
	GS_SetZBuffer(gGState, gBitmaps[2]);

	gViewItem = CreateItemVA(MKNODEID(NST_GRAPHICS, GFX_VIEW_NODE),
											VIEWTAG_VIEWTYPE, theViewType,
												VIEWTAG_DISPLAYSIGNAL, theBuffSignal,
													VIEWTAG_BITMAP, gBitmaps[0], TAG_END );
	if (!gViewItem)
		printf("Unable to create ItemVA. %d\n", gViewItem);
	AddViewToViewList( gViewItem, 0 );
	CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);	/* Clear the frame buffer to black and make it visible */

	GS_SendList(gGState);
	GS_WaitIO(gGState);
	ModifyGraphicsItemVA(gViewItem, VIEWTAG_BITMAP, gBitmaps[0], TAG_END);

	tempSprite = Spr_Create(0);
	Spr_LoadUTF (tempSprite, "Test2D.utf");			/* display the opening texture/logo */
	Do_DefaultTabInit (tempSprite);
	Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
	Spr_Translate (tempSprite, 150, 110);
	Do_AlphaInit (tempSprite);
	F2_Draw (gGState, tempSprite);
	GS_SendList(gGState);
	GS_WaitIO(gGState);
	Spr_Delete(tempSprite);

	TOUCH(theTest2DAPI);

	return TRUE;
}

/* -------------------------------------------------------------------------------- */
void Test2DAPI_ParseArgs( TEST2DAPI* theTest2DAPI, int argc, char* argv[] )
{
	uint32	i, newIndex;
	char*	strptr;

	for (i=1; i<argc; i++) {
		strptr = argv[i];
		while (*strptr) {
			if ( (*strptr >= 'A') && (*strptr <= 'Z') )
				*strptr += 32;		/* convert to lowercase */
			strptr ++;
		}

		if (strncmp( argv[i], "-buildfiles", 11 ) == 0) {
			theTest2DAPI->gShouldBuildStandards = TRUE;
		}

		if (strncmp( argv[i], "-nodisplay", 10 ) == 0) {
			theTest2DAPI->gShouldDisplay = FALSE;
		}

		if (strncmp( argv[i], "-nointeractive", 14 ) == 0) {
			theTest2DAPI->gInteractive = FALSE;
			gInteractive = theTest2DAPI->gInteractive;
		}

		if (strncmp( argv[i], "-group", 6 ) == 0) {
			i++;
			if (i > argc ) {
				printf("Error: Group number not on command-line.\n");
				i--;
			} else {
				newIndex = atoi( argv[i] );
				theTest2DAPI->gRunCompleteTests = FALSE;
				theTest2DAPI->groupNumber = newIndex;
			}
		}

		if (strncmp( argv[i], "-test", 5 ) == 0) {
			i++;
			if (i > argc ) {
				printf("Error: Test number not on command-line.\n");
				i--;
			} else {
				newIndex = atoi( argv[i] );
				theTest2DAPI->gRunCompleteTests = FALSE;
				theTest2DAPI->gRunCompleteGroup = FALSE;
				theTest2DAPI->testNumber = newIndex;
			}
		}
	}
}

/* -------------------------------------------------------------------------------- */
void Test2DAPI_RunTests(TEST2DAPI* theTest2DAPI)
{
	int32		groupNumber, groupNumberEnd;
	int32		testNumber;
	int32		testCount;

 	SCAFFOLD	*scaffold = theTest2DAPI->scaffold;

	if ( Test2DAPI_ConstructScene( theTest2DAPI ) ) {

		if ( theTest2DAPI->gRunCompleteTests ) {
			groupNumber = 0;
			groupNumberEnd = eTest2DAPI_End;
		} else {
			groupNumber = theTest2DAPI->groupNumber;
			groupNumberEnd = groupNumber + 1;
		}
		

		do {

			switch (groupNumber) {

				case eTest2D_CLT:
					Test2D_CLT_Initialize();
					if ( theTest2DAPI->gRunCompleteGroup ) {
						testNumber = 0;
						testCount = Test2D_CLT_Count();
					} else {
						testNumber = theTest2DAPI->testNumber;
						testCount = testNumber;
					}					
					do {
						printf("Test [%d] ",testNumber);

						Test2D_CLT( theTest2DAPI, testNumber );
					} while (++testNumber <= testCount);
					break;

				case eTest2D_F2:
					Test2D_F2_Initialize();
					if ( theTest2DAPI->gRunCompleteGroup ) {
						testNumber = 0;
						testCount = Test2D_F2_Count();
					} else {
						testNumber = theTest2DAPI->testNumber;
						testCount = testNumber;
					}					
					do {
						printf("Test [%d] ",testNumber);

						Test2D_F2( theTest2DAPI, testNumber );
					} while (++testNumber <= testCount);
					break;

				case eTest2D_GRO:
					Test2D_GRO_Initialize();
					if ( theTest2DAPI->gRunCompleteGroup ) {
						testNumber = 0;
						testCount = Test2D_GRO_Count();
					} else {
						testNumber = theTest2DAPI->testNumber;
						testCount = testNumber;
					}					
					do {
						printf("Test [%d] ",testNumber);

						Test2D_GRO( theTest2DAPI, testNumber );
					} while (++testNumber <= testCount);
					break;

				case eTest2D_SPR:
					Test2D_SPR_Initialize();
					if ( theTest2DAPI->gRunCompleteGroup ) {
						testNumber = 0;
						testCount = Test2D_SPR_Count();
					} else {
						testNumber = theTest2DAPI->testNumber;
						testCount = testNumber;
					}					
					do {
						printf("Test [%d] ",testNumber);

						Test2D_SPR( theTest2DAPI, testNumber );
					} while (++testNumber <= testCount);
					break;

				default:
					Scaffold_ReportComment( scaffold, "No group defined for index(%d)", testNumber );
			}
		} while (++groupNumber < groupNumberEnd);

	} else {
		Scaffold_ReportComment( scaffold, " *** SETUP FAILURE *** " );
	}
}
 
/* -------------------------------------------------------------------------------- */
