/*----------------------------------------------------------------------------------
 *	
 *	FileName:	Test2D_SPR.c
 *	Date:		October 10, 1995
 *	By:			Ali Latifi		x3487
 *
 *  Copyright  1995 by The 3DO Company.  All rights reserved.
 *
 *  Modification History:
 *
 *	By:			Date:		Changed:
 *				
 *--------------------------------------------------------------------------------*/

#include "Test2D_SPR.h"

#define EXTERNAL_RELEASE	1
#undef NUPUPSIM

extern GState*		gGState;
extern Err			gErr;
extern char			*fname;
extern Boolean		gCPadFlag;
extern uint32 		gTexture[], ggTexture[];
extern uint32 		gPipData[4];

Color4 gColor[4] = {		1.00f, 0.00f, 0.00f, 1.00f,
							0.00f, 1.00f, 0.00f, 1.00f,
							0.00f, 0.00f, 1.00f, 1.00f,
							1.00f, 1.00f, 0.00f, 1.00f};
Point2 	gCorner[4] = {		{220., 240.},				/*   /\   */
							{320., 140.},				/*  /  \  */
							{420., 240.},				/*  \  /  */
							{320., 340.}, };			/*   \/   */

enum {
	eSpr_Create,
	eSpr_CreateExtended,
	eSpr_CreatePipControlBlock,
	eSpr_CreateShort,
	eSpr_CreateTxData,
	eSpr_Delete,
	eSpr_DisableClipping,
	eSpr_EnableClipping,
	eSpr_GetColors,
	eSpr_GetCorners,
	eSpr_GetDBlendAttr,
	eSpr_GetGeometryEnable,
	eSpr_GetHeight,
	eSpr_GetHSlice,
	eSpr_GetPIP,
	eSpr_GetPosition,
	eSpr_GetSkip,
	eSpr_GetTextureAttr,
	eSpr_GetTextureData,
	eSpr_GetVSlice,
	eSpr_GetWidth,
	eSpr_GetZValues,
	eSpr_LoadTexture,
	eSpr_LoadUTF,
	eSpr_MapCorners,
	eSpr_RemoveDBlendAttr,
	eSpr_RemovePIP,
	eSpr_RemoveTextureAttr,
	eSpr_RemoveTextureData,
	eSpr_ResetCorners,
	eSpr_Rotate,
	eSpr_Scale,
	eSpr_SetColors,
	eSpr_SetCorners,
	eSpr_SetDBlendAttr,
	eSpr_SetGeometryEnable,
	eSpr_SetHeight,
	eSpr_SetHSlice,
	eSpr_SetPIP,
	eSpr_SetPosition,
	eSpr_SetSkip,
	eSpr_SetTextureAttr,
	eSpr_SetTextureData,
	eSpr_SetVSlice,
	eSpr_SetWidth,
	eSpr_SetZValues,
	eSpr_Translate,
	eSpr_UnloadTexture,

	eSPR_Test_End
};

char* Test2D_SPR_Names[] = {
	"Spr_Create",
	"Spr_CreateExtended",
	"Spr_CreatePipControlBlock",
	"Spr_CreateShort",
	"Spr_CreateTxData",
	"Spr_Delete",
	"Spr_DisableClipping",
	"Spr_EnableClipping",
	"Spr_GetColors",
	"Spr_GetCorners",
	"Spr_GetDBlendAttr",
	"Spr_GetGeometryEnable",
	"Spr_GetHeight",
	"Spr_GetHSlice",
	"Spr_GetPIP",
	"Spr_GetPosition",
	"Spr_GetSkip",
	"Spr_GetTextureAttr",
	"Spr_GetTextureData",
	"Spr_GetVSlice",
	"Spr_GetWidth",
	"Spr_GetZValues",
	"Spr_LoadTexture",
	"Spr_LoadUTF",
	"Spr_MapCorners",
	"Spr_RemoveDBlendAttr",
	"Spr_RemovePIP",
	"Spr_RemoveTextureAttr",
	"Spr_RemoveTextureData",
	"Spr_ResetCorners",
	"Spr_Rotate",
	"Spr_Scale",
	"Spr_SetColors",
	"Spr_SetCorners",
	"Spr_SetDBlendAttr",
	"Spr_SetGeometryEnable",
	"Spr_SetHeight",
	"Spr_SetHSlice",
	"Spr_SetPIP",
	"Spr_SetPosition",
	"Spr_SetSkip",
	"Spr_SetTextureAttr",
	"Spr_SetTextureData",
	"Spr_SetVSlice",
	"Spr_SetWidth",
	"Spr_SetZValues",
	"Spr_Translate",
	"Spr_UnloadTexture",

	""
};

/* -------------------------------------------------------------------------------- */
int32 Test2D_SPR_Initialize(void)
{
	 return 0;
}

/* -------------------------------------------------------------------------------- */
/* Return the number of test's to preform */
int32 Test2D_SPR_Count(void)
{
	return eSPR_Test_End-1;
}

/* -------------------------------------------------------------------------------- */
void Do_Texturing(SpriteObj	*sp)
{
	Spr_SetTextureAttr (sp, TXA_ColorOut, TX_BlendOutSelectTex);
	Spr_SetTextureAttr (sp, TXA_AlphaOut, TX_AlphaSelectTexAlpha);
	Spr_SetTextureAttr (sp, TXA_TextureEnable, 1);
	Spr_SetTextureAttr (sp, TXA_MinFilter, TX_Nearest);
	Spr_SetTextureAttr (sp, TXA_MagFilter, TX_Nearest);
	Spr_SetTextureAttr (sp, TXA_InterFilter, TX_Nearest);
	if (Spr_GetPIP(sp)) 
	  Spr_SetTextureAttr (sp, TXA_PipColorSelect, RC_TXTPIPCNTL_PIPCOLORSELECT_PIP);	/* If there is PIP, use PIP color */
	else 
	  Spr_SetTextureAttr (sp, TXA_PipColorSelect, RC_TXTPIPCNTL_PIPCOLORSELECT_TEXTURE);/* If there is no PIP, use texture color */
	Spr_SetTextureAttr (sp, TXA_PipAlphaSelect, 0);
	Spr_SetTextureAttr (sp, TXA_PipSSBSelect, 0);
}

/* -------------------------------------------------------------------------------- */
void Do_Blends(TEST2DAPI* aTest2DAPI, SpriteObj *theSprite)
{
	uint32					tempValue;

/* ---------- */
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_EnableAttrs, 0);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_EnableAttrs to 0. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_EnableAttrs, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_EnableAttrs = 0. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_EnableAttrs				-->	00		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_ZBuffEnable, 1);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_ZBuffEnable to 1. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_ZBuffEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_ZBuffEnable = 1. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_ZBuffEnable				-->	01		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_ZBuffOutEnable, 2);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_ZBuffOutEnable to 2. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_ZBuffOutEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_ZBuffOutEnable = 2. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_ZBuffOutEnable			-->	02		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_WinClipInEnable, 3);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_WinClipInEnable to 3. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_WinClipInEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_WinClipInEnable = 3. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_WinClipInEnable			-->	03		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_WinClipOutEnable, 4);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_WinClipOutEnable to 4. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_WinClipOutEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_WinClipOutEnable = 4. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_WinClipOutEnable		-->	04		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BlendEnable, 5);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BlendEnable to 5. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BlendEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BlendEnable = 5. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BlendEnable				-->	05		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_SrcInputEnable, 6);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_SrcInputEnable to 6. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_SrcInputEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_SrcInputEnable = 6. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_SrcInputEnable			-->	06		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DitherEnable, 7);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DitherEnable to 7. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DitherEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DitherEnable = 7. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DitherEnable			-->	07		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_RGBADestOut, 8);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_RGBADestOut to 8. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_RGBADestOut, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_RGBADestOut = 8. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_RGBADestOut				-->	08		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_Discard, 9);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_Discard to 9. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_Discard, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_Discard = 9. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_Discard					-->	09		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DiscardZClipped, 10);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DiscardZClipped to 10. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DiscardZClipped, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DiscardZClipped = 10. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DiscardZClipped			-->	10		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DiscardSSB0, 11);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DiscardSSB0 to 11. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DiscardSSB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DiscardSSB0 = 11. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DiscardSSB0				-->	11		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DiscardRGB0, 12);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DiscardRGB0 to 12. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DiscardRGB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DiscardRGB0 = 12. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DiscardRGB0				-->	12		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DiscardAlpha0, 13);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DiscardAlpha0 to 13. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DiscardAlpha0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DiscardAlpha0 = 13. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DiscardAlpha0			-->	13		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_XWinClipMin, 14);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_XWinClipMin to 14. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_XWinClipMin, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_XWinClipMin = 14. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_XWinClipMin				-->	14		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_XWinClipMax, 15);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_XWinClipMax to 15. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_XWinClipMax, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_XWinClipMax = 15. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_XWinClipMax				-->	15		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DitherEnable, 16);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DitherEnable to 16. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DitherEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DitherEnable = 16. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DitherEnable			-->	16		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_YWinClipMin, 17);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_YWinClipMin to 17. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_YWinClipMin, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_YWinClipMin = 17. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_YWinClipMin				-->	17		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_YWinClipMax, 18);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_YWinClipMax to 18. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_YWinClipMax, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_YWinClipMax = 18. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_YWinClipMax				-->	18		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_ZCompareControl, 19);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_ZCompareControl to 19. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_ZCompareControl, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_ZCompareControl = 19. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_ZCompareControl			-->	19		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_ZXOffset, 20);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_ZXOffset to 20. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_ZXOffset, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_ZXOffset = 20. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_ZXOffset				-->	20		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_ZYOffset, 21);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_ZYOffset to 21. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_ZYOffset, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_ZYOffset = 21. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_ZYOffset				-->	21		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DSBConst, 22);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DSBConst to 22. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DSBConst, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DSBConst = 22. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DSBConst				-->	22		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DSBSelect, 23);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DSBSelect to 23. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DSBSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DSBSelect = 23. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DSBSelect				-->	23		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_RGBConstIn, 24);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_RGBConstIn to 24. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_RGBConstIn, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_RGBConstIn = 24. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_RGBConstIn				-->	24		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AInputSelect, 25);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AInputSelect to 25. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AInputSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AInputSelect = 25. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AInputSelect			-->	25		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AMultCoefSelect, 26);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AMultCoefSelect to 26. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AMultCoefSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AMultCoefSelect = 26. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AMultCoefSelect			-->	26		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AMultConstControl, 27);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AMultConstControl to 27. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AMultConstControl, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AMultConstControl = 27. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AMultConstControl		-->	27		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AMultRtJustify, 28);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AMultRtJustify to 28. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AMultRtJustify, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AMultRtJustify = 28. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AMultRtJustify			-->	28		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AMultConstSSB0, 29);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AMultConstSSB0 to 29. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AMultConstSSB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AMultConstSSB0 = 29. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AMultConstSSB0			-->	29		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AMultConstSSB1, 30);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AMultConstSSB1 to 30. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AMultConstSSB1, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AMultConstSSB1 = 30. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AMultConstSSB1			-->	30		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BInputSelect, 31);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BInputSelect to 31. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BInputSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BInputSelect = 31. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BInputSelect			-->	31		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BMultCoefSelect, 32);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BMultCoefSelect to 32. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BMultCoefSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BMultCoefSelect = 32. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BMultCoefSelect			-->	32		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BMultConstControl, 33);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BMultConstControl to 33. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BMultConstControl, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BMultConstControl = 33. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BMultConstControl		-->	33		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BMultRtJustify, 34);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BMultRtJustify to 34. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BMultRtJustify, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BMultRtJustify = 34. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BMultRtJustify			-->	34		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BMultConstSSB0, 35);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BMultConstSSB0 to 35. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BMultConstSSB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BMultConstSSB0 = 35. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BMultConstSSB0			-->	35		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_BMultConstSSB1, 36);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_BMultConstSSB1 to 36. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_BMultConstSSB1, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_BMultConstSSB1 = 36. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_BMultConstSSB1			-->	36		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_ALUOperation, 37);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_ALUOperation to 37. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_ALUOperation, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_ALUOperation = 37. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_ALUOperation			-->	37		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_FinalDivide, 38);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_FinalDivide to 38. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_FinalDivide, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_FinalDivide = 38. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_FinalDivide				-->	38		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_Alpha0ClampControl, 39);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_Alpha0ClampControl to 39. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_Alpha0ClampControl, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_Alpha0ClampControl = 39. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_Alpha0ClampControl		-->	39		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_Alpha1ClampControl, 40);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_Alpha1ClampControl to 40. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_Alpha1ClampControl, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_Alpha1ClampControl = 40. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_Alpha1ClampControl		-->	40		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_AlphaFracClampControl, 41);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_AlphaFracClampControl to 41. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_AlphaFracClampControl, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_AlphaFracClampControl = 41. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_AlphaFracClampControl	-->	41		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DestAlphaSelect, 42);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DestAlphaSelect to 42. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DestAlphaSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DestAlphaSelect = 42. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DestAlphaSelect			-->	42		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DestAlphaConstSSB0, 43);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DestAlphaConstSSB0 to 43. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DestAlphaConstSSB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DestAlphaConstSSB0 = 43. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DestAlphaConstSSB0		-->	43		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DestAlphaConstSSB1, 44);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DestAlphaConstSSB1 to 44. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DestAlphaConstSSB1, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DestAlphaConstSSB1 = 44. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DestAlphaConstSSB1		-->	44		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DitherMatrixA, 45);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DitherMatrixA to 45. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DitherMatrixA, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DitherMatrixA = 45. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DitherMatrixA			-->	45		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetDBlendAttr (theSprite, DBLA_DitherMatrixB, 46);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set DBLA_DitherMatrixB to 46. Error# %d", gErr);
	else
	{
		gErr = Spr_GetDBlendAttr (theSprite, DBLA_DitherMatrixB, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get DBLA_DitherMatrixB = 46. Error# %d", gErr);
		else
			LogResult(PASSED, "DBLA_DitherMatrixB			-->	46		Returned --> %d", tempValue);
	}

	TOUCH(aTest2DAPI);

}

/* -------------------------------------------------------------------------------- */
void Do_Textures(TEST2DAPI* aTest2DAPI, SpriteObj *theSprite)
{
 	uint32					tempValue;

/* ---------- */
	gErr = Spr_SetTextureAttr (theSprite, TXA_MinFilter, 0);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_MinFilter to 0. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_MinFilter, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_MinFilter. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_MinFilter				-->	00		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_MagFilter, 1);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_MagFilter to 1. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_MagFilter, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_MagFilter = 1. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_MagFilter				-->	01		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_InterFilter, 2);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_InterFilter to 2. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_InterFilter, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_InterFilter = 2. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_InterFilter				-->	02		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_TextureEnable, 3);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_TextureEnable to 3. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_TextureEnable, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_TextureEnable = 3. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_TextureEnable			-->	03		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_PipIndexOffset, 4);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_PipIndexOffset to 4. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_PipIndexOffset, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_PipIndexOffset = 4. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_PipIndexOffset			-->	04		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_PipColorSelect, 5);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_PipColorSelect to 5. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_PipColorSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_PipColorSelect = 5. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_PipColorSelect			-->	05		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_PipAlphaSelect, 6);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_PipAlphaSelect to 6. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_PipAlphaSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_PipAlphaSelect = 6. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_PipAlphaSelect			-->	06		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_PipSSBSelect, 7);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_PipSSBSelect to 7. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_PipSSBSelect, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_PipSSBSelect = 7. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_PipSSBSelect				-->	07		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_PipConstSSB0, 8);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_PipConstSSB0 to 8. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_PipConstSSB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_PipConstSSB0 = 8. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_PipConstSSB0				-->	08		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_PipConstSSB1, 9);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_PipConstSSB1 to 9. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_PipConstSSB1, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_PipConstSSB1 = 9. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_PipConstSSB1				-->	09		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_FirstColor, 10);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_FirstColor to 10. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_FirstColor, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_FirstColor = 10. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_FirstColor				-->	10		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_SecondColor, 11);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_SecondColor to 11. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_SecondColor, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_SecondColor = 11. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_SecondColor				-->	11		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_ThirdColor, 12);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_ThirdColor to 12. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_ThirdColor, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_ThirdColor = 12. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_ThirdColor				-->	12		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_FirstAlpha, 13);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_FirstAlpha to 13. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_FirstAlpha, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_FirstAlpha = 13. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_FirstAlpha				-->	13		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_SecondAlpha, 14);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_SecondAlpha to 14. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_SecondAlpha, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_SecondAlpha = 14. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_SecondAlpha				-->	14		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_ColorOut, 15);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_ColorOut to 15. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_ColorOut, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_ColorOut = 15. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_ColorOut					-->	15		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_AlphaOut, 16);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_AlphaOut to 16. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_AlphaOut, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_AlphaOut = 16. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_AlphaOut					-->	16		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_BlendOp, 17);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_BlendOp to 17. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_BlendOp, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_BlendOp = 17. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_BlendOp					-->	17		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_BlendColorSSB0, 18);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_BlendColorSSB0 to 18. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_BlendColorSSB0, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_BlendColorSSB0 = 18. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_BlendColorSSB0			-->	18		Returned --> %d", tempValue);
	}
/* ---------- */
	printf("          ");
	gErr = Spr_SetTextureAttr (theSprite, TXA_BlendColorSSB1, 19);
	if (gErr < 0)
		LogResult(FAILED, "Unable to set TXA_BlendColorSSB1 to 19. Error# %d", gErr);
	else
	{
		gErr = Spr_GetTextureAttr (theSprite, TXA_BlendColorSSB1, &tempValue);
		if (gErr < 0)
			LogResult(FAILED, "Unable to get TXA_BlendColorSSB1 = 19. Error# %d", gErr);
		else
			LogResult(PASSED, "TXA_BlendColorSSB1			-->	19		Returned --> %d", tempValue);
	}

	TOUCH(aTest2DAPI);

}

/* -------------------------------------------------------------------------------- */
void Test2D_SPR(TEST2DAPI* theTest2DAPI, int32 testNumber)
{
 	SpriteObj				*tempSprite,*temp1Sprite;
	ControlPadEventData		tempCPad;
	Color4					tempColor[4];
	gfloat 					tempZValue = 12.;
	Point2 					tempPoint;
	uint32					tempValue;
	CltPipControlBlock*		cltPipCtlBlock;
	CltTxData*				cltTxData;

	BeginTest(FT, Test2D_SPR_Names[testNumber]);

	switch( testNumber ) {

		case eSpr_Create:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					LogResult(PASSED, "");
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_CreateExtended:
			{
				tempSprite = Spr_CreateExtended(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					LogResult(PASSED, "");
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create an extended sprite");
			}
			break;

		case eSpr_CreatePipControlBlock:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_CreateTxData (tempSprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
					gErr = Spr_CreatePipControlBlock (tempSprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
					if (gErr < 0)
						LogResult(FAILED, "Unable to create pip control data.  Error # %d, gErr");
					else
						LogResult(PASSED, "");
					Do_Texturing(tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Spr_Translate (tempSprite, 50, 50);
					Spr_Scale (tempSprite, 5., 5.);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create an sprite");
			}
			break;

		case eSpr_CreateShort:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					LogResult(PASSED, "");
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_CreateTxData:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					gErr = Spr_CreateTxData (tempSprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
					if (gErr < 0)
						LogResult(FAILED, "Unable to create texture data.  Error # %d, gErr");
					else
						LogResult(PASSED, "");
					Spr_CreatePipControlBlock (tempSprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
					Do_Texturing(tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Spr_Translate (tempSprite, 75, 75);
					Spr_Scale (tempSprite, 10., 10.);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a short sprite");
			}
			break;

		case eSpr_Delete:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					gErr = Spr_Delete(tempSprite);
					if (gErr < 0)
						LogResult(FAILED, "Unabel to delete sprite.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_DisableClipping:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_DisableClipping(tempSprite);
					if (gErr < 0)
						LogResult(FAILED, "Unabel to disable clipping.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_EnableClipping:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_EnableClipping(tempSprite);
					if (gErr < 0)
						LogResult(FAILED, "Unabel to enable clipping.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetColors:
			{
				tempSprite = Spr_CreateExtended(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_GetColors(tempSprite, tempColor);
					if (gErr < 0)
						LogResult(FAILED, "Unable to get colors.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create an extended sprite");
			}
			break;

		case eSpr_GetCorners:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					Spr_GetCorners(tempSprite, gCorner);
					LogResult(PASSED, "");
					printf ("             [0.x] = %f\n", gCorner[0].x);
					printf ("             [0.y] = %f\n", gCorner[0].y);
					printf ("             [1.x] = %f\n", gCorner[1].x);
					printf ("             [1.y] = %f\n", gCorner[1].y);
					printf ("             [2.x] = %f\n", gCorner[2].x);
					printf ("             [2.y] = %f\n", gCorner[2].y);
					printf ("             [3.x] = %f\n", gCorner[3].x);
					printf ("             [3.y] = %f\n", gCorner[3].y);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetDBlendAttr:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					Do_Blends(theTest2DAPI, tempSprite);
					
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetGeometryEnable:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetGeometryEnable (tempSprite);
					if (tempValue == 1)
						LogResult(PASSED, "Geometry enabled.");
					else if (tempValue == 0)
						LogResult(PASSED, "Geometry disabled.");
					else
						LogResult(FAILED, "Geometry value = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_GetHeight:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetHeight(tempSprite);
					if (tempValue < 0)
						LogResult(FAILED, "Height = %d", tempValue);
					else
						LogResult(PASSED, "Height = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_GetHSlice:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetHSlice(tempSprite);
					if (tempValue < 0)
						LogResult(FAILED, "HSlice = %d", tempValue);
					else
						LogResult(PASSED, "HSlice = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetPIP:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					cltPipCtlBlock = Spr_GetPIP (tempSprite);
					LogResult(PASSED, "cltPipCtlBlock = 0x%x", cltPipCtlBlock);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetPosition:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					Spr_GetPosition (tempSprite, gCorner);
					LogResult(PASSED, "");
					printf ("              [0.x] = %f\n", gCorner[0].x);
					printf ("              [0.y] = %f\n", gCorner[0].y);
					printf ("              [1.x] = %f\n", gCorner[1].x);
					printf ("              [1.y] = %f\n", gCorner[1].y);
					printf ("              [2.x] = %f\n", gCorner[2].x);
					printf ("              [2.y] = %f\n", gCorner[2].y);
					printf ("              [3.x] = %f\n", gCorner[3].x);
					printf ("              [3.y] = %f\n", gCorner[3].y);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetSkip:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetSkip(tempSprite);
					LogResult(PASSED, "Skip = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetTextureAttr:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					Do_Textures(theTest2DAPI, tempSprite);

					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_GetTextureData:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					cltTxData = Spr_GetTextureData (tempSprite);
					LogResult(PASSED, "cltTxData = 0x%x", cltTxData);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_GetVSlice:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetVSlice(tempSprite);
					if (tempValue < 0)
						LogResult(FAILED, "VSlice = %d", tempValue);
					else
						LogResult(PASSED, "VSlice = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_GetWidth:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetWidth(tempSprite);
					if (tempValue < 0)
						LogResult(FAILED, "Width = %d", tempValue);
					else
						LogResult(PASSED, "Width = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_GetZValues:
			{
				tempSprite = Spr_CreateExtended(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					gErr = Spr_GetZValues(tempSprite, &tempZValue);
					if (gErr < 0)
						LogResult(FAILED, "Unable to get zValues.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create an extended sprite");
			}
			break;

		case eSpr_LoadTexture:
			{
				List		theList;

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				tempSprite = Spr_Create(0);
				Spr_CreateTxData (tempSprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_CreatePipControlBlock (tempSprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Do_Texturing(tempSprite);
				Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
				Spr_Translate (tempSprite, 50, 50);
				Spr_Scale (tempSprite, 5., 5.);
				F2_Draw (gGState, tempSprite);

				temp1Sprite = Spr_Create(0);
				Spr_CreateTxData (temp1Sprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_CreatePipControlBlock (temp1Sprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Do_Texturing(temp1Sprite);
				Spr_ResetCorners (temp1Sprite, SPR_TOPLEFT);
				Spr_Translate (temp1Sprite, 150, 150);
				Spr_Scale (temp1Sprite, 5., 5.);
				F2_Draw (gGState, temp1Sprite);

				PrepList(&theList);
				InsertNodeFromHead(&theList, &(tempSprite->spr_Node));
				InsertNodeFromHead(&theList, &(temp1Sprite->spr_Node));
				gErr = F2_DrawList (gGState, &theList);
				if (gErr >= 0)
				{
					if (gErr < 0)
						LogResult(FAILED, "Unable to load texture.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					GS_SendList(gGState);
					GS_WaitIO(gGState);
				} else {
					LogResult(FAILED, "Unable to draw list.  Error# %d", gErr);
				}
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				Spr_Delete(tempSprite);
				Spr_Delete(temp1Sprite);
			}
			break;
	
		case eSpr_LoadUTF:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					gErr = Spr_LoadUTF (tempSprite, fname);
					if (gErr < 0)
						LogResult(FAILED, "Unable to load UTF file.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_MapCorners:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);

					gErr = Spr_MapCorners (tempSprite, gCorner, SPR_TOPLEFT);
					if (gErr < 0)
						LogResult(FAILED, "Error# %d", gErr);
					else
					{
						LogResult(PASSED, "SPR_TOPLEFT");
						Do_DefaultTabInit (tempSprite);
						Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
						Do_AlphaInit (tempSprite); 
						F2_Draw (gGState, tempSprite);
						GS_SendList(gGState);
						GS_WaitIO(gGState);
						if (gCPadFlag)
						{
							printf ("Press a button...\n");
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
						}
					}
	
					printf("          ");
					gErr = Spr_MapCorners (tempSprite, gCorner, SPR_TOPRIGHT);
					if (gErr < 0)
						LogResult(FAILED, "Error# %d", gErr);
					else
					{
						LogResult(PASSED, "SPR_TOPRIGHT");
						Do_DefaultTabInit (tempSprite);
						Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
						Do_AlphaInit (tempSprite); 
						F2_Draw (gGState, tempSprite);
						GS_SendList(gGState);
						GS_WaitIO(gGState);
						if (gCPadFlag)
						{
							printf ("Press a button...\n");
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
						}
					}
	
					printf("          ");
					gErr = Spr_MapCorners (tempSprite, gCorner, SPR_BOTTOMLEFT);
					if (gErr < 0)
						LogResult(FAILED, "Error# %d", gErr);
					else
					{
						LogResult(PASSED, "SPR_BOTTOMLEFT");
						Do_DefaultTabInit (tempSprite);
						Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
						Do_AlphaInit (tempSprite); 
						F2_Draw (gGState, tempSprite);
						GS_SendList(gGState);
						GS_WaitIO(gGState);
						if (gCPadFlag)
						{
							printf ("Press a button...\n");
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
						}
					}
	
					printf("          ");
					gErr = Spr_MapCorners (tempSprite, gCorner, SPR_BOTTOMRIGHT);
					if (gErr < 0)
						LogResult(FAILED, "Error # %d", gErr);
					else
					{
						LogResult(PASSED, "SPR_BOTTOMRIGHT");
						Do_DefaultTabInit (tempSprite);
						Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
						Do_AlphaInit (tempSprite); 
						F2_Draw (gGState, tempSprite);
						GS_SendList(gGState);
						GS_WaitIO(gGState);
						if (gCPadFlag)
						{
							printf ("Press a button...\n");
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
						}
					}
	
					printf("          ");
					gErr = Spr_MapCorners (tempSprite, gCorner, SPR_CENTER);
					if (gErr < 0)
						LogResult(FAILED, "Error # %d", gErr);
					else
					{
						LogResult(PASSED, "SPR_CENTER");
						Do_DefaultTabInit (tempSprite);
						Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
						Do_AlphaInit (tempSprite); 
						F2_Draw (gGState, tempSprite);
						GS_SendList(gGState);
						GS_WaitIO(gGState);
						if (gCPadFlag)
						{
							printf ("Press a button...\n");
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
							while( !GetControlPad (1, FALSE, &tempCPad)) ;
						}
					}

					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_RemoveDBlendAttr:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					gErr = Spr_RemoveDBlendAttr (tempSprite, DBLA_EnableAttrs);
					if (gErr < 0)
						LogResult(FAILED, "Unable to remove DBLA_EnableAttrs.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_RemovePIP:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					gErr = Spr_RemovePIP (tempSprite);
					if (gErr < 0)
						LogResult(FAILED, "Unable to remove pip.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_RemoveTextureAttr:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					gErr = Spr_RemoveTextureAttr (tempSprite, TXA_MinFilter);
					if (gErr < 0)
						LogResult(FAILED, "Unable to remove texture attr.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_RemoveTextureData:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					gErr = Spr_RemoveTextureData (tempSprite);
					if (gErr < 0)
						LogResult(FAILED, "Unable to remove texture data.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_ResetCorners:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);

					gErr = Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					if (gErr < 0)
						LogResult(FAILED, "Unable to reset corner to SPR_TOPLEFT.  Error# %d", gErr);
					else
						LogResult(PASSED, "SPR_TOPLEFT");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					printf("          ");
					gErr = Spr_ResetCorners (tempSprite, SPR_TOPRIGHT);
					if (gErr < 0)
						LogResult(FAILED, "Unable to reset corner to SPR_TOPRIGHT.  Error# %d", gErr);
					else
						LogResult(PASSED, "SPR_TOPRIGHT");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					printf("          ");
					gErr = Spr_ResetCorners (tempSprite, SPR_BOTTOMLEFT);
					if (gErr < 0)
						LogResult(FAILED, "Unable to reset corner to SPR_BOTTOMLEFT.  Error# %d", gErr);
					else
						LogResult(PASSED, "SPR_BOTTOMLEFT");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					printf("          ");
					gErr = Spr_ResetCorners (tempSprite, SPR_BOTTOMRIGHT);
					if (gErr < 0)
						LogResult(FAILED, "Unable to reset corner to SPR_BOTTOMRIGHT.  Error# %d", gErr);
					else
						LogResult(PASSED, "SPR_BOTTOMRIGHT");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					printf("          ");
					gErr = Spr_ResetCorners (tempSprite, SPR_CENTER);
					if (gErr < 0)
						LogResult(FAILED, "Unable to reset corner to SPR_CENTER.  Error# %d", gErr);
					else
						LogResult(PASSED, "SPR_CENTER");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);

					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_Rotate:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_Rotate(tempSprite, 45.0);
					if (gErr < 0)
						LogResult(FAILED, "Unable to rotate sprite.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_Scale:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_Scale(tempSprite, 1.5, 1.5);
					if (gErr < 0)
						LogResult(FAILED, "Unable to rotate sprite.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_SetColors:
			{
				tempSprite = Spr_CreateExtended(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_SetColors (tempSprite, gColor);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set new colors.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create an extended sprite");
			}
			break;
			
		case eSpr_SetCorners:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_SetCorners (tempSprite, gCorner);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set corners.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_SetDBlendAttr:
			{
				LogResult(PASSED, "Set DBlends integrated in GetDBlendAttr tests.");
			}
			break;
			
		case eSpr_SetGeometryEnable:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite);
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					tempValue = Spr_GetGeometryEnable (tempSprite);
					if (tempValue == 0 || tempValue ==1)
					{
						if (tempValue == 0)
						{
							gErr = Spr_SetGeometryEnable(tempSprite, 1);
							if (gErr < 0)
								LogResult(FAILED, "Can't enable geometry. Error# %d", gErr);
							else
							{
								tempValue = Spr_GetGeometryEnable (tempSprite);
								LogResult(PASSED, "Geometry was disabled, but now it is enabled.");
							}
						}
						if (tempValue == 1)
						{
							gErr = Spr_SetGeometryEnable(tempSprite, 0);
							if (gErr < 0)
								LogResult(FAILED, "Can't disable geometry. Error# %d", gErr);
							else
							{
								tempValue = Spr_GetGeometryEnable (tempSprite);
								LogResult(PASSED, "Geometry was enabled, but now it is disabled.");
							}
						}
					}
					else
						LogResult(FAILED, "Unable to get geometry value = %d", tempValue);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			TOUCH(tempValue);
			}
			break;
			
		case eSpr_SetHeight:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					tempValue = Spr_GetHeight(tempSprite);
					gErr = Spr_SetHeight (tempSprite, tempValue*2);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set height to double.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_SetHSlice:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					tempValue = Spr_GetHSlice(tempSprite);
					gErr = Spr_SetHSlice(tempSprite, tempValue*2);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set HSlice to double.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_SetPIP:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					cltPipCtlBlock = Spr_GetPIP (tempSprite);
					gErr = Spr_SetPIP (tempSprite, cltPipCtlBlock);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set pip.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_SetPosition:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					tempPoint.x = 100.;
					tempPoint.y = 100.;
					gErr = Spr_SetPosition (tempSprite, &tempPoint);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set position.  Error # %d", gErr);
					else
						LogResult(PASSED, "Position set to 100, 100.");
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_SetSkip:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					tempValue = Spr_GetSkip(tempSprite);
					if (tempValue)
						gErr = Spr_SetSkip (tempSprite, 0);
					else
						gErr = Spr_SetSkip (tempSprite, 1);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set Skip.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_SetTextureAttr:
			{
				LogResult(PASSED, "Set Textures integrated in GetTextureAttr tests.");
			}
			break;

		case eSpr_SetTextureData:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					cltTxData = Spr_GetTextureData (tempSprite);
					gErr = Spr_SetTextureData (tempSprite, cltTxData);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set texture data.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_SetVSlice:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					tempValue = Spr_GetVSlice(tempSprite);
					gErr = Spr_SetVSlice(tempSprite, tempValue*2);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set VSlice to double.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_SetWidth:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					tempValue = Spr_GetWidth(tempSprite);
					gErr = Spr_SetWidth (tempSprite, tempValue*2);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set Width to double.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;
			
		case eSpr_SetZValues:
			{
				tempSprite = Spr_CreateExtended(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_SetZValues (tempSprite, &tempZValue);
					if (gErr < 0)
						LogResult(FAILED, "Unable to set zvalues.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create an extended sprite");
			}
			break;
			
		case eSpr_Translate:
			{
				tempSprite = Spr_Create(0);
				if (tempSprite != 0L)
				{
					CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
					Spr_LoadUTF (tempSprite, fname);
					Do_DefaultTabInit (tempSprite);
					Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
					gErr = Spr_Translate (tempSprite, 100.0, 100.0);
					if (gErr < 0)
						LogResult(FAILED, "Unable to translate the sprite.  Error # %d", gErr);
					else
						LogResult(PASSED, "");
					Do_AlphaInit (tempSprite); 
					F2_Draw (gGState, tempSprite);
					GS_SendList(gGState);
					GS_WaitIO(gGState);
					if (gCPadFlag)
					{
						printf ("Press a button...\n");
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
						while( !GetControlPad (1, FALSE, &tempCPad)) ;
					}
					Spr_Delete(tempSprite);
				}
				else
					LogResult(FAILED, "Unable to create a sprite");
			}
			break;

		case eSpr_UnloadTexture:
			{
				List		theList;

				CLT_ClearFrameBuffer (gGState, 0., 0., 0., 0., TRUE, TRUE);
				tempSprite = Spr_Create(0);
				Spr_CreateTxData (tempSprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_CreatePipControlBlock (tempSprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Do_Texturing(tempSprite);
				Spr_ResetCorners (tempSprite, SPR_TOPLEFT);
				Spr_Translate (tempSprite, 50, 50);
				Spr_Scale (tempSprite, 5., 5.);
				F2_Draw (gGState, tempSprite);

				temp1Sprite = Spr_Create(0);
				Spr_CreateTxData (temp1Sprite, gTexture, 16, 16, 2, MAKEEXPFORMAT (0, 0, 1, 0, 0, 0, 2));
				Spr_CreatePipControlBlock (temp1Sprite, gPipData, 0, sizeof(gPipData)/sizeof(gPipData[0]));
				Do_Texturing(temp1Sprite);
				Spr_ResetCorners (temp1Sprite, SPR_TOPLEFT);
				Spr_Translate (temp1Sprite, 150, 150);
				Spr_Scale (temp1Sprite, 5., 5.);
				F2_Draw (gGState, temp1Sprite);

				PrepList(&theList);
				InsertNodeFromHead(&theList, &(tempSprite->spr_Node));
				InsertNodeFromHead(&theList, &(temp1Sprite->spr_Node));
				gErr = F2_DrawList (gGState, &theList);
				if (gErr >= 0)
				{
					if (gErr < 0)
						LogResult(FAILED, "Unable to load texture.  Error# %d", gErr);
					else
						LogResult(PASSED, "");
					GS_SendList(gGState);
					GS_WaitIO(gGState);
				} else {
					LogResult(FAILED, "Unable to draw list.  Error# %d", gErr);
				}
				if (gCPadFlag)
				{
					printf ("Press a button...\n");
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
					while( !GetControlPad (1, FALSE, &tempCPad)) ;
				}
				Spr_Delete(tempSprite);
				Spr_Delete(temp1Sprite);
			}
			break;
	
		default:
			BeginTest(FT, "<Invalid Test>");
			LogComment("Test called with an invalid testNumber (%d)", testNumber );	
	}
}

/* -------------------------------------------------------------------------------- */
