/************************************************************************/
/* Written by Paige Wuestefeld						*/
/*									*/
/* Performs functional testing of Mercury Matrix utilities.		*/   
/* Designed to be used in conjunction with integration tests.		*/
/*									*/
/* Billboard tests are in testbboard.c by Greg Omi			*/
/************************************************************************/

#ifdef MACINTOSH
#include <:tests:3do_scaf.h>
#else
#include <tests/3do_scaf.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "matrix.h"
#include "scaf_MemUtils.h"
#include "Scaffold.h"
#include "MatrixTest.h"

#define tolerance (.00001f)
#define tolerance2 (.005f)

/* Constants */
  enum {
    eVector3D_Construct,
    eMatrix_Construct,
    eMatrix_Identity,
    eMatrix_Mult,
    eVector3D_Destruct,
    eVector3D_Set,
    eVector3D_GetX,
    eVector3D_GetY,
    eVector3D_GetZ,
    eVector3D_Average,
    eVector3D_Negate,
    eVector3D_Dot,
    eVector3D_Length,
    eVector3D_Normalize,
    eVector3D_Minimum,
    eVector3D_Maximum,
    eVector3D_Compare,
    eVector3D_CompareFuzzy,
    eVector3D_Copy,
    eVector3D_Add,
    eVector3D_Subtract,
    eVector3D_Scale,
    eVector3D_Multiply,
    eVector3D_Cross,
    eVector3D_Zero,
    eVector3D_Print,
    eVector3D_MultiplyByMatrix,
    eMatrix_Destruct,
    eMatrix_Normalize,
    eMatrix_Copy,
    eMatrix_Multiply,
    eMatrix_MultOrientation,
    eMatrix_MultiplyOrientation,
    eMatrix_Zero,
    eMatrix_GetTranslation,
    eMatrix_SetTranslationByVector,
    eMatrix_SetTranslation,
    eMatrix_TranslateByVector,
    eMatrix_Translate,
    eMatrix_MoveByVector,
    eMatrix_Move,
    eMatrix_Scale,
    eMatrix_ScaleByVector,
    eMatrix_ScaleLocal,
    eMatrix_ScaleLocalByVector,
    eMatrix_Rotate,
    eMatrix_RotateX,
    eMatrix_RotateY,
    eMatrix_RotateZ,
    eMatrix_RotateLocal,
    eMatrix_RotateXLocal,
    eMatrix_RotateYLocal,
    eMatrix_RotateZLocal,
    eMatrix_Turn,
    eMatrix_TurnX,
    eMatrix_TurnY,
    eMatrix_TurnZ,
    eMatrix_TurnLocal,
    eMatrix_TurnXLocal,
    eMatrix_TurnYLocal,
    eMatrix_TurnZLocal,
    eMatrix_Print,
    eMatrix_GetHeading,
    eMatrix_GetBank,
    eMatrix_GetElevation,
    eMatrix_LookAt,
    eMatrix_FullInvert,
    eMatrix_Invert,
    eMatrix_CopyOrientation,
    eMatrix_CopyTranslation,

    eMatrix_Test_End
  };

char *TestHg_Matrix_Names[] =
{
  "Vector3D_Construct",
  "Matrix_Construct",
  "Matrix_Identity",
  "Matrix_Mult",
  "Vector3D_Destruct",
  "Vector3D_Set",
  "Vector3D_GetX",
  "Vector3D_GetY",
  "Vector3D_GetZ",
  "Vector3D_Average",
  "Vector3D_Negate",
  "Vector3D_Dot",
  "Vector3D_Length",
  "Vector3D_Normalize",
  "Vector3D_Minimum",
  "Vector3D_Maximum",
  "Vector3D_Compare",
  "Vector3D_CompareFuzzy",
  "Vector3D_Copy",
  "Vector3D_Add",
  "Vector3D_Subtract",
  "Vector3D_Scale",
  "Vector3D_Multiply",
  "Vector3D_Cross",
  "Vector3D_Zero",
  "Vector3D_Print",
  "Vector3D_MultiplyByMatrix",
  "Matrix_Destruct",
  "Matrix_Normalize",
  "Matrix_Copy",
  "Matrix_Multiply",
  "Matrix_MultOrientation",
  "Matrix_MultiplyOrientation",
  "Matrix_Zero",
  "Matrix_GetTranslation",
  "Matrix_SetTranslationByVector",
  "Matrix_SetTranslation",
  "Matrix_TranslateByVector",
  "Matrix_Translate",
  "Matrix_MoveByVector",
  "Matrix_Move",
  "Matrix_Scale",
  "Matrix_ScaleByVector",
  "Matrix_ScaleLocal",
  "Matrix_ScaleLocalByVector",
  "Matrix_Rotate",
  "Matrix_RotateX",
  "Matrix_RotateY",
  "Matrix_RotateZ",
  "Matrix_RotateLocal",
  "Matrix_RotateXLocal",
  "Matrix_RotateYLocal",
  "Matrix_RotateZLocal",
  "Matrix_Turn",
  "Matrix_TurnX",
  "Matrix_TurnY",
  "Matrix_TurnZ",
  "Matrix_TurnLocal",
  "Matrix_TurnXLocal",
  "Matrix_TurnYLocal",
  "Matrix_TurnZLocal",
  "Matrix_Print",
  "Matrix_GetHeading",
  "Matrix_GetBank",
  "Matrix_GetElevation",
  "Matrix_LookAt",
  "Matrix_FullInvert",
  "Matrix_Invert",
  "Matrix_CopyOrientation",
  "Matrix_CopyTranslation",

  ""
};

/*--------------------------------------------------------------------------------*/

main (int argc, char **argv)
{

  uint32 testNumber, testCount;
  int flag, x, y, z;
  float mycalc[5][5];
  Vector3D *testV3;
  pMatrix testpm;

  printf("Usage:\n MatrixFT -R <release> <optional scaffold parameters>\n");

  InitScaffold (GRAPHICS_CLT, argc, argv);

  testCount = TestHgMatrix_Count ();	/* number of API tests to perform */

  for (testNumber = 0; testNumber <= testCount; testNumber++) {	/* for */
    printf ("Test [%d] ", testNumber);

    BeginTest (FT, TestHg_Matrix_Names[testNumber]);

    testV3 = Vector3D_Construct ();
    if (testV3 == NULL) {
      printf ("Setup error\n");
      exit (1);
    }

    testpm = Matrix_Construct ();
    if (testpm == NULL) {
      printf ("Setup error\n");
      exit (1);
    }

    switch (testNumber) {

    case eVector3D_Construct:
      {
	Vector3D *myV3;

	myV3 = Vector3D_Construct ();

	if (myV3 == NULL) {
	  Scaffold_Report (FAILED, "Null pointer");
	  exit (1);
	}
	else
	  Scaffold_ReportPassed ();

	Vector3D_Destruct (myV3);

      }
      break;

    case eVector3D_Destruct:
      {
	Vector3D *myV3;

	/* check memory */
	myV3 = Vector3D_Construct ();
	if (myV3 == NULL)
	  printf ("Setup error\n");
	Vector3D_Destruct (myV3);
	Scaffold_ReportPassed ();	/* use -SMemCheck=true in command line */
      }
      break;

    case eVector3D_Set:
      {
	Vector3D_Set (testV3, 10.0, 20.0, 30.0);
	if ((testV3->x == 10.0) && (testV3->y == 20.0) && (testV3->z == 30.0))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Data missmatch");
      }
      break;

    case eVector3D_GetX:
      {

	Vector3D_Set (testV3, 10.0, 20.0, 30.0);
	if ((testV3->x != 10.0) || (testV3->y != 20.0) || (testV3->z != 30.0))
	  printf ("Setup Error!\n");
	if (Vector3D_GetX (testV3) == 10.0)
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Data missmatch");

      }
      break;

    case eVector3D_GetY:
      {
	Vector3D_Set (testV3, 10.0, 20.0, 30.0);
	if ((testV3->x != 10.0) || (testV3->y != 20.0) || (testV3->z != 30.0))
	  printf ("Setup Error!\n");
	if (Vector3D_GetY (testV3) == 20.0)
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Data missmatch");

      }
      break;

    case eVector3D_GetZ:
      {
	Vector3D_Set (testV3, 10.0, 20.0, 30.0);
	if ((testV3->x != 10.0) || (testV3->y != 20.0) || (testV3->z != 30.0))
	  printf ("Setup Error!\n");
	if (Vector3D_GetZ (testV3) == 30.0)
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Data missmatch");

      }
      break;

    case eVector3D_Average:
      {
	Vector3D *result;

	result = Vector3D_Construct ();
	Vector3D_Average (result, &testVa1, &testVa2);
	if ((fabsf (result->x - (testVr.x / 2)) < tolerance) &&
	    (fabsf (result->y - (testVr.y / 2)) < tolerance) &&
	    (fabsf (result->z - (testVr.z / 2)) < tolerance))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Not expected");

	Vector3D_Destruct (result);

      }
      break;

    case eVector3D_Negate:
      {
	Vector3D *result;

	result = Vector3D_Construct ();
	Vector3D_Copy (result, testV3);
	Vector3D_Negate (result);
	if ((fabsf (result->x == -(result->x))) &&
	    (fabsf (result->y == -(result->y))) &&
	    (fabsf (result->z == -(result->z))))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Not expected");

	Vector3D_Destruct (result);
      }
      break;

    case eVector3D_Dot:
      {
	float dp;

	/* 24+56+96 */

	dp = Vector3D_Dot (&testVa1, &testVa2);
	if (fabsf (dp - 176.0) < tolerance)
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Not expected");

      }
      break;

    case eVector3D_Length:
      {
	if (fabsf (Vector3D_Length (&testVa1) - (sqrtf (56.0))) < tolerance)
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "Not expected");

      }
      break;

    case eVector3D_Normalize:
      {
	Vector3D *temp;

	temp = Vector3D_Construct ();
	Vector3D_Copy (temp, &testVa1);

	Vector3D_Normalize (temp);
	if ((fabsf (temp->x - (2.0 / sqrtf (56.0))) < tolerance) &&
	    (fabsf (temp->y - (4.0 / sqrtf (56.0))) < tolerance) &&
	    (fabsf (temp->z - (6.0 / sqrtf (56.0))) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (temp);
	}

	Vector3D_Destruct (temp);

      }
      break;

    case eVector3D_Minimum:
      {
	Vector3D *result;

	result = Vector3D_Construct ();
	Vector3D_Minimum (result, &testVa1, &testVMM);
	if ((fabsf (result->x - 2.0) < tolerance) &&
	    (fabsf (result->y - 2.0) < tolerance) &&
	    (fabsf (result->z - 6.0) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (result);
	}

      }
      break;

    case eVector3D_Maximum:
      {
	Vector3D *result;

	result = Vector3D_Construct ();
	Vector3D_Maximum (result, &testVa1, &testVMM);
	if ((fabsf (result->x - 4.0) < tolerance) &&
	    (fabsf (result->y - 4.0) < tolerance) &&
	    (fabsf (result->z - 6.0) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (result);
	}
      }
      break;

    case eVector3D_Compare:
      {
	if (Vector3D_Compare (&testVr, &testVa1))
	  Scaffold_Report (FAILED, "unexpected match");
	else
	  Scaffold_ReportPassed ();
      }
      break;

    case eVector3D_CompareFuzzy:
      {
	if (Vector3D_CompareFuzzy (&testVr, &testVrF, .001))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "missmatch");

      }
      break;

    case eVector3D_Copy:
      {
	Vector3D *temp;

	temp = Vector3D_Construct ();
	Vector3D_Copy (temp, &testVr);
	if ((temp->x == testVr.x) && (temp->y == testVr.y) && (temp->z == testVr.z))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

      }
      break;

    case eVector3D_Add:
      {
	Vector3D *temp;

	temp = Vector3D_Construct ();
	Vector3D_Add (temp, &testVa1, &testVa2);

	if ((temp->x == testVr.x) && (temp->y == testVr.y) && (temp->z == testVr.z))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

	Vector3D_Destruct (temp);
      }
      break;

    case eVector3D_Subtract:
      {
	Vector3D *temp;

	temp = Vector3D_Construct ();
	Vector3D_Subtract (temp, &testVa2, &testVa1);

	if ((fabsf (temp->x - 10.0) < tolerance) &&
	    (fabsf (temp->y - 10.0) < tolerance) &&
	    (fabsf (temp->y - 10.0) < tolerance))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

	Vector3D_Destruct (temp);

      }
      break;

    case eVector3D_Scale:
      {
	Vector3D *temp;

	temp = Vector3D_Construct ();
	Vector3D_Copy (temp, &testVa1);

	Vector3D_Scale (temp, 2.0, 2.0, 4.0);

	if ((fabsf (temp->x - 4.0) < tolerance) &&
	    (fabsf (temp->y - 8.0) < tolerance) &&
	    (fabsf (temp->z - 24.0) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (temp);
	}

	Vector3D_Destruct (temp);

      }
      break;

    case eVector3D_Multiply:
      {
	Vector3D *temp, *temp2, *result;

	temp = Vector3D_Construct ();
	Vector3D_Copy (temp, &testVa1);
	temp2 = Vector3D_Construct ();
	Vector3D_Copy (temp2, &testVa2);
	result = Vector3D_Construct ();

	Vector3D_Multiply (result, temp, temp2);

	if ((fabsf (result->x - 24.0) < tolerance) &&
	    (fabsf (result->y - 56.0) < tolerance) &&
	    (fabsf (result->z - 96.0) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (result);
	}

	Vector3D_Destruct (temp);
	Vector3D_Destruct (temp2);
	Vector3D_Destruct (result);

      }
      break;

    case eVector3D_Cross:
      {
	Vector3D *temp;

	temp = Vector3D_Construct ();
	Vector3D_Cross (temp, &testVa1, &testVa2);

	if ((fabsf (temp->x - (-20.0)) < tolerance) &&
	    (fabsf (temp->y - (40.0)) < tolerance) &&
	    (fabsf (temp->z - (-20.0)) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (temp);
	}

      }
      break;

    case eVector3D_Zero:
      {
	Vector3D_Zero (testV3);

	if ((testV3->x == 0.0) && (testV3->y == 0.0) && (testV3->z == 0.0))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

      }
      break;

    case eVector3D_Print:
      {
	Vector3D_Print (&testVa1);

      }
      break;

    case eVector3D_MultiplyByMatrix:
      {
	Vector3D *result;

	result = Vector3D_Construct ();
	Vector3D_Copy (result, &testVa1);
	Vector3D_MultiplyByMatrix (result, &testMatrixLocalX);
	if ((fabsf (result->x - 102.0) < tolerance) &&
	    (fabsf (result->y - 206.0) < tolerance) &&
	    (fabsf (result->z - 46.0) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (result);
	}

	Vector3D_Destruct (result);

      }
      break;

    case eMatrix_Construct:
      {
	pMatrix pM;

	pM = Matrix_Construct ();
	if (pM == NULL) {
	  Scaffold_Report (FAILED, "Null pointer");
	  exit (1);
	}
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (pM);
      }
      break;

    case eMatrix_Destruct:
      {
	Matrix *test;

	/* check memory by using -SMemCheck=true switch */
	test = Matrix_Construct ();
	if (test == NULL)
	  printf ("Setup error!\n");
	Matrix_Destruct (test);
	Scaffold_ReportPassed ();

      }
      break;

    case eMatrix_Normalize:
      {
	Matrix *result;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testIdentScaled);
	Matrix_Normalize (result);

	if (!CompareMtx (result, &testIdent)) {
	  Scaffold_Report (FAILED, "missmatch");
	  exit (1);
	}
	else
	  Scaffold_ReportPassed ();
      }
      break;

    case eMatrix_Copy:
      {
	pMatrix pM;

	pM = Matrix_Construct ();
	Matrix_Copy (pM, &gtestMatrixR);

	if (!CompareMtx (pM, &gtestMatrixR))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (pM);

      }
      break;

    case eMatrix_Mult:
      {
	pMatrix result;

	result = Matrix_Construct ();

	Matrix_Mult (result, &gtestMatrixR, &gtestMatrixRT);

	for (x = 0; x < 4; ++x) {
	  for (y = 0; y < 4; ++y) {
	    double sum = 0;

	    for (z = 0; z < 4; ++z)
	      sum += gtestMatrixR44.mat[x][z] * gtestMatrixRT44.mat[z][y];
	    mycalc[x][y] = sum;
	  }
	}

	flag = 1;
	for (x = 0; x < 4; ++x) {
	  for (y = 0; y < 3; ++y) {
	    if ((mycalc[x][y] - result->mat[x][y]) > tolerance) {
	      flag = 0;
	      break;
	    }
	  }
	}

	if (flag == 0) {
	  Scaffold_Report (FAILED, " ");
	  exit (1);
	}
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_Multiply:
      {
	pMatrix myCopy;

	myCopy = Matrix_Construct ();
	Matrix_Copy (myCopy, &gtestMatrixR);

	Matrix_Multiply (myCopy, &gtestMatrixRT);

	for (x = 0; x < 4; ++x) {
	  for (y = 0; y < 4; ++y) {
	    double sum = 0;

	    for (z = 0; z < 4; ++z)
	      sum += gtestMatrixR44.mat[x][z] * gtestMatrixRT44.mat[z][y];
	    mycalc[x][y] = sum;
	  }
	}

	flag = 1;
	for (x = 0; x < 4; ++x) {
	  for (y = 0; y < 3; ++y) {
	    if ((mycalc[x][y] - myCopy->mat[x][y]) > tolerance) {
	      flag = 0;
	      break;
	    }
	  }
	}

	if (flag == 0)
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (myCopy);

      }
      break;

    case eMatrix_MultiplyOrientation:
      {
	Matrix *result;

	/* r c */

	result = Matrix_Construct ();
	Matrix_Identity (result);

	Matrix_MultiplyOrientation (result, &gtestMatrixRT);

	if (!CompareMtx (&gtestMatrixR, result))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_MultOrientation:
      {
	Matrix *result;

	/* r c */

	result = Matrix_Construct ();
	Matrix_Identity (result);

	Matrix_MultOrientation (result, &gtestMatrixR, &gtestMatrixRT);

	for (x = 0; x < 3; ++x) {
	  for (y = 0; y < 3; ++y) {
	    double sum = 0;

	    for (z = 0; z < 3; ++z)
	      sum += gtestMatrixR.mat[x][z] * gtestMatrixRT.mat[z][y];
	    mycalc[x][y] = sum;
	  }
	}

	flag = 1;
	for (x = 0; x < 3; ++x) {
	  for (y = 0; y < 3; ++y) {
	    if ((mycalc[x][y] - result->mat[x][y]) > tolerance) {
	      flag = 0;
	      break;
	    }
	  }
	}

	if ((flag == 0))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_Zero:
      {
	Matrix *temp;

	temp = Matrix_Construct ();
	Matrix_Identity (temp);

	Matrix_Zero(temp);

	flag = 1;
	for (x = 0; x < 4; ++x) {
	  for (y = 0; y < 3; ++y) {
	    if (temp->mat[x][y] != 0.0) {
	      flag = 0;
	      break;
	    }
	  }
	}

	if (flag == 0)
	  Scaffold_Report (FAILED, "");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (temp);

      }
      break;

    case eMatrix_Identity:
      {
	Matrix_Identity (testpm);

	if (!CompareMtx (testpm, &testIdent)) {
	  Scaffold_Report (FAILED, "missmatch");
	  exit (1);
	}
	else
	  Scaffold_ReportPassed ();
      }
      break;

    case eMatrix_GetTranslation:
      {
	Vector3D *myLoc;

	myLoc = Vector3D_Construct ();
	Matrix_GetTranslation (&testMatrixST, myLoc);
	if ((fabsf (myLoc->x - 10.0) < tolerance) &&
	    (fabsf (myLoc->y - 5.0) < tolerance) &&
	    (fabsf (myLoc->z - 2.0) < tolerance))
	  Scaffold_ReportPassed ();
	else {
	  Scaffold_Report (FAILED, "data missmatch");
	  Vector3D_Print (myLoc);
	}

	Vector3D_Destruct (myLoc);
      }
      break;

    case eMatrix_SetTranslationByVector:
      {
	Vector3D *myLoc;
	Matrix *temp;

	myLoc = Vector3D_Construct ();
	myLoc->x = 50.0;
	myLoc->y = 60.0;
	myLoc->z = 70.0;

	temp = Matrix_Construct ();
	Matrix_Copy (temp, &testMatrixST);

	Matrix_SetTranslationByVector (temp, myLoc);
	if ((fabsf (temp->mat[3][0] - 50.0) < tolerance) &&
	    (fabsf (temp->mat[3][1] - 60.0) < tolerance) &&
	    (fabsf (temp->mat[3][2] - 70.0) < tolerance))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

	Vector3D_Destruct (myLoc);
	Matrix_Destruct (temp);
      }
      break;

    case eMatrix_SetTranslation:
      {
	Matrix *temp;

	temp = Matrix_Construct ();
	Matrix_Copy (temp, &testMatrixST);

	Matrix_SetTranslation (temp, 200.0, 300.0, 400.0);
	if ((fabsf (temp->mat[3][0] - 200.0) < tolerance) &&
	    (fabsf (temp->mat[3][1] - 300.0) < tolerance) &&
	    (fabsf (temp->mat[3][2] - 400.0) < tolerance))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

	Matrix_Destruct (temp);

      }
      break;

    case eMatrix_TranslateByVector:
      {
	Matrix *temp;

	temp = Matrix_Construct ();
	Matrix_Copy (temp, &gtestMatrixRT);

	Matrix_TranslateByVector (temp, &testVa2);
	if ((fabsf (temp->mat[3][0] - 12.0) < tolerance) &&
	    (fabsf (temp->mat[3][1] - 20.0) < tolerance) &&
	    (fabsf (temp->mat[3][2] - 16.0) < tolerance))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

	Matrix_Destruct (temp);
      }
      break;

    case eMatrix_Translate:
      {
	Matrix *temp;

	temp = Matrix_Construct ();
	Matrix_Identity (temp);

	Matrix_Translate (temp, 10.0, 100.0, 50.5);
	if ((fabsf (temp->mat[3][0] - 10.0) < tolerance) &&
	    (fabsf (temp->mat[3][1] - 100.0) < tolerance) &&
	    (fabsf (temp->mat[3][2] - 50.5) < tolerance))
	  Scaffold_ReportPassed ();
	else
	  Scaffold_Report (FAILED, "data missmatch");

      }
      break;

    case eMatrix_MoveByVector:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[3][0] = (12.0);
	check->mat[3][1] = (14.0);
	check->mat[3][2] = (16.0);

	Matrix_MoveByVector (result, &testVa2);

	flag = 1;
	for (y = 0; y < 3; ++y) {
	  if (fabsf (check->mat[3][y] - result->mat[3][y]) > tolerance) {
	    flag = 0;
	    break;
	  }
	}

	if (flag == 0)
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();


	Matrix_Destruct (result);

      }
      break;

    case eMatrix_Move:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[3][0] = (5.0);
	check->mat[3][1] = (10.0);
	check->mat[3][2] = (15.0);

	Matrix_Move (result, 5.0, 10.0, 15.0);

	flag = 1;
	for (y = 0; y < 3; ++y) {
	  if (fabsf (check->mat[3][y] - result->mat[3][y]) > tolerance) {
	    flag = 0;
	    break;
	  }
	}

	if (flag == 0)
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_Scale:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[0][0] = (0.5);
	check->mat[1][1] = (2.0);
	check->mat[2][2] = (0.25);

	Matrix_Scale (result, 0.5, 2.0, 0.25);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_ScaleByVector:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[0][0] = (2.0);
	check->mat[1][1] = (4.0);
	check->mat[2][2] = (6.0);

	Matrix_ScaleByVector (result, &testVa1);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_ScaleLocal:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testIdentScaled);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[0][0] = (10.0);
	check->mat[1][1] = (20.0);
	check->mat[2][2] = (30.0);

	Matrix_ScaleLocal (result, 20.0, 40.0, 60.0);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
      }
      break;

    case eMatrix_ScaleLocalByVector:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testIdentScaled);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[0][0] = (1.0);
	check->mat[1][1] = (2.0);
	check->mat[2][2] = (3.0);

	Matrix_ScaleLocalByVector (result, &testVa1);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);

      }
      break;

    case eMatrix_Rotate:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[1][1] = (0.0);
	check->mat[2][1] = (-1.0);
	check->mat[1][2] = (1.0);
	check->mat[2][2] = (0.0);

	Matrix_Rotate (result, 'X', (RAD (90)));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_RotateX:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[1][1] = (0.0);
	check->mat[2][1] = (1.0);
	check->mat[1][2] = (-1.0);
	check->mat[2][2] = (0.0);

	Matrix_RotateX (result, (-RAD (90)));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_RotateY:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[0][0] = (0.0);
	check->mat[2][0] = (-1.0);
	check->mat[0][2] = (1.0);
	check->mat[2][2] = (0.0);

	Matrix_RotateY (result, (-(90 * PI) / 180));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_RotateZ:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[0][0] = (0.0);
	check->mat[1][0] = (1.0);
	check->mat[0][1] = (-1.0);
	check->mat[1][1] = (0.0);

	Matrix_RotateZ (result, (-(90 * PI) / 180));


	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_RotateLocal:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	/* rotation values */
	check->mat[1][1] = (0.0);
	check->mat[2][1] = (1.0);
	check->mat[1][2] = (-1.0);
	check->mat[2][2] = (0.0);

	/* should also have same x, y, z values */
	check->mat[3][0] = (10.0);
	check->mat[3][1] = (5.0);
	check->mat[3][2] = (2.0);

	Matrix_RotateLocal (result, 'X', -RAD (90));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_RotateXLocal:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* rotation values */
	check->mat[1][1] = (0.0);
	check->mat[2][1] = (-1.0);
	check->mat[1][2] = (1.0);
	check->mat[2][2] = (0.0);

/* should also have same x, y, z values */
	check->mat[3][0] = (10.0);
	check->mat[3][1] = (5.0);
	check->mat[3][2] = (2.0);

	Matrix_RotateXLocal (result, (RAD (90)));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);


      }
      break;

    case eMatrix_RotateYLocal:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* rotation values */
	check->mat[0][0] = (0.0);
	check->mat[2][0] = (-1.0);
	check->mat[0][2] = (1.0);
	check->mat[2][2] = (0.0);

/* should also have same x, y, z values */
	check->mat[3][0] = (10.0);
	check->mat[3][1] = (5.0);
	check->mat[3][2] = (2.0);

	Matrix_RotateYLocal (result, (-RAD (90)));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);


      }
      break;

    case eMatrix_RotateZLocal:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* rotation values */
	check->mat[0][0] = (0.0);
	check->mat[1][0] = (1.0);
	check->mat[0][1] = (-1.0);
	check->mat[1][1] = (0.0);

/* should also have same x, y, z values */
	check->mat[3][0] = (10.0);
	check->mat[3][1] = (5.0);
	check->mat[3][2] = (2.0);

	Matrix_RotateZLocal (result, (-RAD (90)));

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);



      }
      break;

    case eMatrix_Turn:
      {
	Matrix *result;
	Matrix *check;
	Matrix *final;

/* rotation in local coordinate space */

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	Matrix_Turn (result, 'Y', RAD (90));

/* rotate in local coordinates */
/* 90 degree rotation of ID on Y */
	check = Matrix_Construct ();
	Matrix_Identity (check);
	check->mat[0][0] = (0.0);
	check->mat[2][0] = (1.0);
	check->mat[0][2] = (-1.0);
	check->mat[2][2] = (0.0);

/* multiply by original matrix */
	final = Matrix_Construct ();
	Matrix_Mult (final, check, &testMatrixST);

	if (!CompareMtx (result, final))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);
	Matrix_Destruct (final);

      }
      break;

    case eMatrix_TurnX:
      {
	Matrix *result;
	Matrix *check;
	Matrix *final;

/* rotation in local coordinate space */

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	Matrix_TurnX (result, (-RAD (90)));

/* rotate in local coordinates */
/* -90 degree rotation of ID on X */
	check = Matrix_Construct ();
	Matrix_Identity (check);
	check->mat[1][1] = (0.0);
	check->mat[2][1] = (1.0);
	check->mat[1][2] = (-1.0);
	check->mat[2][2] = (0.0);

/* multiply by original matrix */
	final = Matrix_Construct ();
	Matrix_Mult (final, check, &testMatrixST);

	if (!CompareMtx (result, final))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);
	Matrix_Destruct (final);

      }
      break;

    case eMatrix_TurnY:
      {
	Matrix *result;
	Matrix *check;
	Matrix *final;

/* rotation in local coordinate space */

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	Matrix_TurnY (result, (-RAD (90)));

/* rotate in local coordinates */
/* -90 degree rotation of ID on Y */
	check = Matrix_Construct ();
	Matrix_Identity (check);
	check->mat[0][0] = (0.0);
	check->mat[2][0] = (-1.0);
	check->mat[0][2] = (1.0);
	check->mat[2][2] = (0.0);

/* multiply by original matrix */
	final = Matrix_Construct ();
	Matrix_Mult (final, check, &testMatrixST);

	if (!CompareMtx (result, final))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);
	Matrix_Destruct (final);


      }
      break;

    case eMatrix_TurnZ:
      {
	Matrix *result;
	Matrix *check;
	Matrix *final;

/* rotation in local coordinate space */

	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixST);

	Matrix_TurnZ (result, (-RAD (90)));

/* rotate in local coordinates */
/* -90 degree rotation of ID on Z */
	check = Matrix_Construct ();
	Matrix_Identity (check);
	check->mat[0][0] = (0.0);
	check->mat[1][0] = (1.0);
	check->mat[0][1] = (-1.0);
	check->mat[1][1] = (0.0);

/* multiply by original matrix */
	final = Matrix_Construct ();
	Matrix_Mult (final, check, &testMatrixST);

	if (!CompareMtx (result, final))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);
	Matrix_Destruct (final);

      }
      break;

    case eMatrix_TurnLocal:
      {
	Matrix *result;
	Matrix *check;

/* test martix local is a -90 RotateX on the Identity matrix */
	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixLocalZ);

/* turning the test matrix +90 deg should yield the Identity 	*/
/* matrix again for the orientation 				*/
	Matrix_TurnLocal (result, 'Z', RAD (90));

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* should have same x, y, z values */
	check->mat[3][0] = (100.0);
	check->mat[3][1] = (200.0);
	check->mat[3][2] = (50.0);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_TurnXLocal:
      {
	Matrix *result;
	Matrix *check;

/* test martix local is a -90 RotateX on the Identity matrix */
	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixLocalX);

/* turning the test matrix +90 deg should yield the Identity 	*/
	Matrix_TurnXLocal (result, (RAD (90)));

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* should have same x, y, z values */
	check->mat[3][0] = (100.0);
	check->mat[3][1] = (200.0);
	check->mat[3][2] = (50.0);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);


      }
      break;

    case eMatrix_TurnYLocal:
      {
	Matrix *result;
	Matrix *check;

/* test martix local is a -90 Rotate on the Identity matrix */
	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixLocalY);

/* turning the test matrix +90 deg should yield the Identity 	*/
	Matrix_TurnYLocal (result, (RAD (90)));

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* should have same x, y, z values */
	check->mat[3][0] = (100.0);
	check->mat[3][1] = (200.0);
	check->mat[3][2] = (50.0);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);


      }
      break;

    case eMatrix_TurnZLocal:
      {
	Matrix *result;
	Matrix *check;

/* test martix local is a -90 RotateX on the Identity matrix */
	result = Matrix_Construct ();
	Matrix_Copy (result, &testMatrixLocalZ);

/* turning the test matrix +90 deg should yield the Identity 	*/
	Matrix_TurnZLocal (result, (RAD (90)));

	check = Matrix_Construct ();
	Matrix_Identity (check);

/* should have same x, y, z values */
	check->mat[3][0] = (100.0);
	check->mat[3][1] = (200.0);
	check->mat[3][2] = (50.0);

	if (!CompareMtx (result, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);


      }
      break;

    case eMatrix_Print:
      {
	printf ("Test Translation Matrix:\n");
	Matrix_Print (&testMatrixST);
	Scaffold_ReportPassed ();
      }
      break;

    case eMatrix_GetHeading:
      {				/* -rot->clock->+heading */
	float heading;

	heading = RAD_TO_DEG * Matrix_GetHeading (&testMatrixLocalY);
	if (heading - (90.0) > tolerance2) {
	  Scaffold_Report (FAILED, " ");
	  printf ("heading: %f\n", heading);
	}
	else
	  Scaffold_ReportPassed ();

      }
      break;

    case eMatrix_GetBank:
      {				/* - rot->clockwise-> -bank */
	float bank;

	bank = RAD_TO_DEG * Matrix_GetBank (&testMatrixLocalZ);
	if (bank - (-90.0) > tolerance2) {
	  Scaffold_Report (FAILED, " ");
	  printf ("bank: %f\n", bank);
	}
	else
	  Scaffold_ReportPassed ();

      }
      break;

    case eMatrix_GetElevation:
      {				/* - rot->clockwise-> -elevation */
	float elevation;

	elevation = RAD_TO_DEG * Matrix_GetElevation (&testMatrixLocalX);
	if (elevation - (-90.0) > tolerance2) {
	  Scaffold_Report (FAILED, " ");
	  printf ("elevation: %f\n", elevation);
	}
	else
	  Scaffold_ReportPassed ();
      }
      break;

    case eMatrix_LookAt:
      {
	Vector3D *where, *from;
	Matrix *result;

	result = Matrix_Construct ();
	where = Vector3D_Construct ();
	from = Vector3D_Construct ();

	Vector3D_Set (where, 0.0, 0.0, 0.0);
	Vector3D_Set (from, 0.0, 0.0, 10.0);
	Matrix_LookAt (result, where, from, RAD (-90));

	if (!CompareMtx (result, &lookMatrix)) {
	  Scaffold_Report (FAILED, "missmatch");
	  Matrix_Print (result);
	  Matrix_Print (&lookMatrix);
	}
	else
	  Scaffold_ReportPassed ();

      }
      break;

    case eMatrix_FullInvert:
      {
	Matrix *temp, *result, *check;
	Err theErr;

/* rotation and translation */
	result = Matrix_Construct ();
	temp = Matrix_Construct ();

	theErr = Matrix_FullInvert (result, &gtestMatrixRT);
	if (!theErr)
	  PrintfSysErr (theErr);

/*	printf ("Test Matrix:\n");
	Matrix_Print (&gtestMatrixRT);
	printf ("This is the inverse:\n");
	Matrix_Print (result);*/

	Matrix_Mult (temp, result, &gtestMatrixRT);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	if (!CompareMtx (temp, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (temp);
	Matrix_Destruct (check);

      }
      break;

    case eMatrix_Invert:
      {
	Matrix *temp, *result, *check;

	result = Matrix_Construct ();
	temp = Matrix_Construct ();

/* rotation and translation */
	Matrix_Invert (result, &gtestMatrixRT);

	Matrix_Mult (temp, result, &gtestMatrixRT);

	check = Matrix_Construct ();
	Matrix_Identity (check);

	if (!CompareMtx (temp, check))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (temp);
	Matrix_Destruct (check);
      }
      break;

    case eMatrix_CopyOrientation:
      {
	Matrix *result;
	bool flag = TRUE;
	int x, y;

	result = Matrix_Construct ();
	Matrix_Identity (result);

	Matrix_CopyOrientation (result, &gtestMatrixRT);

	for (x = 0; x < 3; x++) {
	  for (y = 0; y < 3; y++) {
	    if (fabsf ((result->mat[x][y] - gtestMatrixRT.mat[x][y])) > tolerance) {
	      flag = FALSE;
	      break;
	    }
	  }
	}

	if (!flag)
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
      }
      break;

    case eMatrix_CopyTranslation:
      {
	Matrix *result;
	Matrix *check;

	result = Matrix_Construct ();
	check = Matrix_Construct ();
	Matrix_Identity (check);

	check->mat[3][0] = 10.0;
	check->mat[3][1] = 5.0;
	check->mat[3][2] = 2.0;

	Matrix_CopyTranslation (result, &testMatrixST);

	if (!CompareMtx (result, &testMatrixST))
	  Scaffold_Report (FAILED, " ");
	else
	  Scaffold_ReportPassed ();

	Matrix_Destruct (result);
	Matrix_Destruct (check);

      }
      break;

    }				/* switch */

    Matrix_Destruct (testpm);
    Vector3D_Destruct (testV3);

    testpm = Matrix_Construct ();
    if (testpm == NULL) {
      printf ("Setup error\n");
      exit (1);
    }


  }

  TermScaffold ();
  printf ("Finished %d tests\n", testNumber);

}				/* main */

/*--------------------------------------------------------------------------------*/
int32 
TestHgMatrix_Count ()
{
  /*
	 * Return the number of test's to preform
	 */
  return eMatrix_Test_End - 1;
}

/*--------------------------------------------------------------------------------*/
bool 
CompareMtx (Matrix * m1, Matrix * m2)
{

  bool flag = TRUE;
  int x, y;

/* check 4x3 members of resultant matrix */

  for (x = 0; x < 4; ++x) {
    for (y = 0; y < 3; ++y) {
      if (fabsf (m1->mat[x][y] - m2->mat[x][y]) > tolerance2) {
	flag = FALSE;
	break;
      }
    }
  }
  return (flag);

}
