#ifdef MACINTOSH
#include <kernel:types.h>
#include <graphics:clt:clt.h>
#else
#include <kernel/types.h>
#include <graphics/clt/clt.h>
#endif
#include "mercury.h"
#include "SData.h"

#define kFloatsPerVertex       6
#define kSizePerVertex         (kFloatsPerVertex*sizeof(float))

float gHitherPlane = 10.0;

/****************************************************************************/
/* Box																		*/
/****************************************************************************/

static float gBoxVertices[] =
{

  100, 100, 0, -1, 0, 0,	/* 0	-x */
  100, 100, 25, 0, -1, 0,	/* 1	-y */
  100, 125, 0, 0, 0, -1,	/* 2	-z */
  100, 125, 25, 0, 1, 0,	/* 3	+y */
  125, 100, 0, 1, 0, 0,		/* 4	+x */
  125, 100, 25, 0, 0, 1,	/* 5	+z */
  125, 125, 0, 1, 0, 0,		/* 6	+x */
  125, 125, 25, 1, 0, 0		/* 7	+x */
};

/* location from first, normals (or color) from second */

static short gBoxSharedVerts[] =
{
  0, 1,				/* 8	-y */
  0, 2,				/* 9	-z */
  1, 0,				/* 10	-x */
  1, 5,				/* 11	+z */
  2, 0,				/* 12	-x */
  2, 3,				/* 13	+y */
  3, 0,				/* 14	-x */
  3, 5,				/* 15	+z */
  4, 1,				/* 16	-y */
  4, 2,				/* 17	-z */
  5, 4,				/* 18	+x */
  5, 1,				/* 19	-y */
  6, 3,				/* 20	+y */
  6, 2,				/* 21	-z */
  7, 3,				/* 22	+y */
  7, 5				/* 23	+z */
};


static short gBoxIndices[] =
{
  0 + STXT + NEWS + CFAN + PCLK, 2, 10, 12, 14,	/* -x */
  8 + NEWS + CFAN + PCLK, 16, 1, 19,	/* -y */
  9 + NEWS + CFAN + PCLK, 2, 17, 21,	/* -z */
  13 + NEWS + CFAN + PCLK, 3, 20, 22,	/* +y */
  6 + NEWS + CFAN + PCLK, 7, 4, 18,	/* +x */
  11 + NEWS + CFAN + PCLK, 5, 15, 23,	/* +z */
  0 + STXT + NEWS + CFAN + PCLK, -1
};

static PodGeometry gBoxGeometry =
{
  /* float fxmin, fymin, fzmin */ 100, 100, 0,
  /* float fxextent, fyextent, fzextent */ 25, 25, 25,
  /* float *pvertex */ gBoxVertices,
  /* short *pshared */ gBoxSharedVerts,
  /* uint16 vertexcount */ 8,
  /* uint16 sharedcount */ 16,
  /* short *pinxex */ gBoxIndices,
  /* float *puv */ NULL
};

Matrix gPodMatrix =
{
  1.0, 0.0, 0.0,
  0.0, 1.0, 0.0,
  0.0, 0.0, 1.0,
  0.0, 0.0, 0.0
};

Material gTransMaterial =
{
  /* base color */ 
 {0.1, 0.1, 0.1, 0.5},
 /* diffuse color */ 
 {0.85, 0.0, 0.0},
 /* shine */ 0.05,
 /* specular color */ 
 {0.0, 0.0, 0.0}
};

static LightDir LightDirDataW =
{
  -0.577, -0.577, -0.577,
  0.6, 0.6, 0.6
};

static LightDir LightDirDataW2 =
{
  0.577, 0.577, 0.577,
  1.0, 1.0, 1.0
};

static uint32 gLightDirW[] =
{
  (uint32) & M_LightDir,
  (uint32) & LightDirDataW,
  0
};

Pod gBoxDynPod =
{
  /* uint32 flags */ 0,
  /* struct Pod *pnext */ NULL,
  /* void (*pcase)(CloseData*) */ M_SetupDynLitTrans,
  /* struct PodTexture *ptexture */ NULL,
  /* struct PodGeometry *pgeometry */ &gBoxGeometry,
  /* Matrix *pmatrix */ &gPodMatrix,
  /* uint32 *plights */ gLightDirW,
  /* uint32 *puserdata */ NULL,
  /* Material *pmaterial */ &gTransMaterial
};

Material gTransMaterial2 =
{
  /* base color */ 
 {0.9, 0.9, 0.9, 0.7},
 /* diffuse color */ 
 {0.9, 0.9, 0.9},
 /* shine */ 0.05,
 /* specular color */ 
 {1.0, 1.0, 1.0}
};

Material gOpaqueMat =
{
  /* base color */ 
 {0.9, 0.9, 0.9, 1.0},
 /* diffuse color */ 
 {0.9, 0.9, 0.9},
 /* shine */ 0.5,
 /* specular color */ 
 {1.0, 1.0, 1.0}
};

static LightDir LightDirData2 =
{
  0.0, 0.0, -1.0,
  0.8, 0.8, 0.8
};


/*		fog0 = 1.0/(fogfar - fognear) 		*/
/*		fog1 = fognear/(fogfar - fognear)	*/
static LightFog LightFogData =
{
/*	1.0/(100.0 - 10.0),	10.0/(100.0 - 50.0)	*/
  1.0 / (125.0 - 10.0), 10.0 / (125.0 - 10.0)
};


LightPoint LightPointData =
{
  0.0, 0.0, 300.0,		/* location */
  100000.0 * 32.0 * 1.0,	/* maxdist */
  100000.0,			/* intensity */
  0.8, 0.8, 0.8			/* white */
};

uint32 gDirPointFogLightList[] =
{
  (uint32) & M_LightPoint,
  (uint32) & LightPointData,
  (uint32) & M_LightDir,
  (uint32) & LightDirData2,
  (uint32) & M_LightFog,
  (uint32) & LightFogData,
  0
};

uint32 gNoLightList[] = {
    				0
		};

uint32 gEnvFogLightList[] = 
{
	(uint32)&M_LightEnv,
	0,
  	(uint32) &M_LightFog,
  	(uint32) &LightFogData,
	0,
};


uint32 podenvlights[] = 
{
	(uint32)&M_LightEnv,
	0,
	0,
};
