
/************************************************************************/
/* Written by: Paige Wuestefeld						*/
/* Note: Pieces taken from the viewer example				*/
/* Created 10/3/96							*/
/* Modified:								*/
/*									*/
/* SimpleEnvMapIT - tests environmental mapping cases in Hg		*/
/* Cases tested:							*/
/*		DynLitEnv, DynLitTransEnv, DynLitSpecEnv, DynLitFogEnv	*/
/*									*/
/* Models required: guard.bsf, guard.utf, testEnv.bsf, testEnv,utf	*/
/* Guard model by John Cumming, sphere by Mike Rosefield		*/
/*									*/
/* Texture: panel01.utf							*/
/************************************************************************/

#ifdef MACINTOSH
#include <kernel:types.h>
#include <kernel:mem.h>
#include <graphics:graphics.h>
#include <graphics:view.h>
#include <graphics:clt:gstate.h>
#include <graphics:blitter.h>
#include <misc:event.h>
#else
#include <kernel/types.h>
#include <kernel/mem.h>
#include <graphics/graphics.h>
#include <graphics/view.h>
#include <graphics/clt/gstate.h>
#include <graphics/blitter.h>
#include <misc/event.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "graphicsenv.h"
#include "mercury.h"
#include "matrix.h"
#include "bsdf_read.h"
#include "filepod.h"
#include "controller.h"
#include "datatorus.h"
#include "scalemodel.h"

  typedef struct AppData {
    CloseData *close;
    Matrix *matrixSkew;
    Matrix *matrixCamera;

    uint32 curRenderScreen;

    CONTROLLER controls;

  } AppData;

/*------------------------------------------------------*/
/* Pod data	*/

extern Pod gTorusDynPod;

Matrix gMatrix =
{
  1.0, 0.0, 0.0,
  0.0, 1.0, 0.0,
  0.0, 0.0, 1.0,
  0.0, 0.0, 0.0
};

Material gTransMat =
{
 /* base color */
  {0.5, 0.5, 0.5, 0.4},
 /* diffuse color */
  {0.8, 0.8, 0.8},
  /* shine */ 0.05,
 /* specular color */
  {1.0, 0.0, 0.0}
};

LightDir LightDirDataEnv =
{
  0.0, 0.0, -1.0,
  0.2, 0.0, 0.0
};

LightDir LightDirDataSpecEnv1 =
{
  0.0, 0.0, -1.0,
  0.2, 0.0, 0.0
};

/* fog: near=10 far=100 */
static LightFog LightFogData =
{
  1.0 / (100.0 - 10.0), 10.0 / (100.0 - 10.0)
};

static uint32 podenvlightsFog[] =
{
  (uint32) & M_LightEnv,
  0,
  (uint32) & M_LightFog,
  (uint32) & LightFogData,
  0,
};

static uint32 podenvlights[] =
{
  (uint32) & M_LightEnv,
  0,
  0,
};

static uint32 podenvlightsSpec[] =
{
  (uint32) & M_LightEnv,
  0,
  (uint32) & M_LightDirSpecTex,
  (uint32) & LightDirDataSpecEnv1,
  0,
};

/*------------------------------------------------------*/


#define kMaxCmdListWords    400
#define kMaxVertsPerPod     4000

#define Pi                  	3.141592653589793
#define kRadian             	(Pi/180.0)
#define kMoveIncrement          4.0
#define kMoveCameraIncrement    (7.5f)

#define BMHEIGHT	480
#define BMWIDTH		640
#define TEXHEIGHT 	64
#define TEXWIDTH 	64
#define VIEWHEIGHT	200
#define VIEWWIDTH	200
#define CX			(BMWIDTH/2)
#define CY			(BMHEIGHT/2)

void Program_Begin (GraphicsEnv * genv, AppData * app);

static void Program_UserControl (AppData * appData, Matrix * matrixGuard, Matrix * matrixTorus,
				 Matrix * matrixSphere);

void AddFog (Pod * modelPod);

void AddSpecular (Pod * modelPod);

gfloat ViewScreen[] =
{
  (CX - (VIEWWIDTH / 2)), (CY - (VIEWHEIGHT / 2)), W_2D, 0, 0,
  (CX + (VIEWWIDTH / 2)), (CY - (VIEWHEIGHT / 2)), W_2D, (TEXWIDTH - 1), 0,
  (CX + (VIEWWIDTH / 2)), (CY + (VIEWHEIGHT / 2)), W_2D, (TEXWIDTH - 1), (TEXHEIGHT - 1),
  (CX - (VIEWWIDTH / 2)), (CY + (VIEWHEIGHT / 2)), W_2D, 0, (TEXHEIGHT - 1)
};


/* --------------------------------------------------------------------- */


int
main (int argc, char *argv[])
{
  Err err;
  GraphicsEnv *genv;
  AppData *appData;

  TOUCH (argc);
  TOUCH (argv);

  /* Print usage */

  printf ("\n");

  /* Setup Display, GState */

  genv = GraphicsEnv_Create ();
  if (genv == NULL) {
    printf ("Couldn't allocate graphics environment.  Exiting.\n");
    exit (1);
  }
  err = GraphicsEnv_Init (genv, 640, 480, 32);	/* "Default" environment */
  if (err < 0) {
    PrintfSysErr (err);
    exit (err);
  }

  /* Setup the EventBroker */

  err = InitEventUtility (1, 0, 0);
  if (err < 0) {
    PrintfSysErr (err);
    exit (err);
  }

  /* Create an instance of AppData */
  appData = AllocMem (sizeof (AppData), MEMTYPE_NORMAL);
  if (appData == NULL) {
    printf ("Couldn't create AppData structure.  Exiting\n");
    exit (1);
  }

  /* Load & initialize data, then begin */

  Program_Begin (genv, appData);

  /* Cleanup */

  printf ("Program exiting.\n");
  GraphicsEnv_Delete (genv);

  return 0;
}


/* --------------------------------------------------------------------- */


static
void
Program_UserControl (AppData * appData, Matrix * matrixGuard, Matrix * matrixTorus,
		     Matrix * matrixSphere)
{

  uint32 buttons = appData->controls.currentButtons;
  Vector3D moveGuard;
  Vector3D moveTorus;
  Vector3D moveSphere;
  Vector3D moveCamera;

  Vector3D_Zero (&moveGuard);
  Vector3D_Zero (&moveTorus);
  Vector3D_Zero (&moveSphere);
  Vector3D_Zero (&moveCamera);

/* guard */
  if (buttons & ControlA) {
    if (buttons & ControlUp)
      moveGuard.y = kMoveIncrement;
    if (buttons & ControlDown)
      moveGuard.y = -kMoveIncrement;
    if (buttons & ControlRight)
      moveGuard.x = kMoveIncrement;
    if (buttons & ControlLeft)
      moveGuard.x = -kMoveIncrement;
    if (buttons & ControlLeftShift)
      moveGuard.z = kMoveIncrement;
    if (buttons & ControlRightShift)
      moveGuard.z = -kMoveIncrement;
  }

/* torus */
  if (buttons & ControlB) {
    if (buttons & ControlUp)
      moveTorus.y = kMoveIncrement;
    if (buttons & ControlDown)
      moveTorus.y = -kMoveIncrement;
    if (buttons & ControlRight)
      moveTorus.x = kMoveIncrement;
    if (buttons & ControlLeft)
      moveTorus.x = -kMoveIncrement;
    if (buttons & ControlLeftShift)
      moveTorus.z = kMoveIncrement;
    if (buttons & ControlRightShift)
      moveTorus.z = -kMoveIncrement;
  }

/* sphere */
  if (buttons & ControlC) {
    if (buttons & ControlUp)
      moveSphere.y = kMoveIncrement;
    if (buttons & ControlDown)
      moveSphere.y = -kMoveIncrement;
    if (buttons & ControlRight)
      moveSphere.x = kMoveIncrement;
    if (buttons & ControlLeft)
      moveSphere.x = -kMoveIncrement;
    if (buttons & ControlLeftShift)
      moveSphere.z = kMoveIncrement;
    if (buttons & ControlRightShift)
      moveSphere.z = -kMoveIncrement;
  }

  if ((buttons & ControlD)) {
    Matrix_RotateXLocal (matrixGuard, kRadian * -2.0);
  }

  if ((buttons & ControlE)) {
    Matrix_RotateYLocal (matrixGuard, kRadian * 2.0);
  }

  if ((!(buttons & ControlA) && !(buttons & ControlB) && !(buttons & ControlC))) {
    if (buttons & ControlUp)
      moveCamera.y = kMoveCameraIncrement;
    if (buttons & ControlDown)
      moveCamera.y = -kMoveCameraIncrement;

    if (buttons & ControlLeft)
      moveCamera.x = kMoveCameraIncrement;
    if (buttons & ControlRight)
      moveCamera.x = -kMoveCameraIncrement;

    if (buttons & ControlLeftShift)
      moveCamera.z = kMoveCameraIncrement;
    if (buttons & ControlRightShift)
      moveCamera.z = -kMoveCameraIncrement;
  }


  Matrix_TranslateByVector (matrixGuard, &moveGuard);
  Matrix_TranslateByVector (matrixTorus, &moveTorus);
  Matrix_TranslateByVector (matrixSphere, &moveSphere);
  Matrix_TranslateByVector (appData->matrixCamera, &moveCamera);

}


/* --------------------------------------------------------------------- */


void
Program_Begin (GraphicsEnv * genv, AppData * appData)
{
  Pod *torusPod, *model1Pod, *model2Pod, *curPod;
  bool done = FALSE;
  Bitmap *MyDestBM;
  BlitObject *back = NULL;
  Err err;
  int i;
  ViewPyramid vp;
  uint32 maxPodVerts1, maxPodVerts2, modelPodCount1, modelPodCount2;
  float HitherPlane = 10.0;
  Vector3D center, position;

  Matrix camMtx =
  {
    {
      1.0, 0.0, 0.0,
      0.0, 1.0, 0.0,
      0.0, 0.0, 1.0,
      0.0, 0.0, 0.0
    }
  };

  maxPodVerts1 = maxPodVerts2 = 0;

  /* Load Models from disk */
  Model_LoadFromDisk ("guard", &modelPodCount1, &maxPodVerts1);
  model1Pod = gBSDF->pods;

  Model_LoadFromDisk ("testEnv", &modelPodCount2, &maxPodVerts2);
  model2Pod = gBSDF->pods;

  /* Initialize Mercury */
  appData->close = M_Init (kMaxVertsPerPod, kMaxCmdListWords, genv->gs);
  M_InitMP (appData->close, gBSDF->numPods, 100000, 650);

  if (appData->close == NULL) {
    printf ("Couldn't init Mercury.  Exiting\n");
    exit (1);
  }
  appData->close->fwclose = 1.01;
  appData->close->fwfar = 100000.0;
  appData->close->fscreenwidth = genv->d->width;
  appData->close->fscreenheight = genv->d->height;
  appData->close->depth = genv->d->depth;

  /* Setup camera */
  appData->matrixSkew = AllocMem (sizeof (Matrix), MEMTYPE_NORMAL);
  if (appData->matrixSkew == NULL) {
    printf ("Couldn't allocate skew matrix.  Exiting\n");
    exit (1);
  }

  vp.left = -1.0;
  vp.right = 1.0;
  vp.top = 0.75;		/* Basic 4/3 aspect ratio, with positive Y UP */
  vp.bottom = -0.75;
  vp.hither = 1.2;
  Matrix_Perspective (appData->matrixSkew, &vp, 0.0, (float) (genv->d->width),
		      0.0, (float) (genv->d->height), 1.0 / HitherPlane);


  /*
     * Additional camera setup
     */
  printf ("Additional camera setup...\n");
  appData->matrixCamera = Matrix_Construct ();
  if (appData->matrixCamera == NULL) {
    printf ("Couldn't allocate camera matrix.  Exiting\n");
    exit (1);
  }

  Vector3D_Set (&center, 0.0f, 0.0f, 0.0f);
  Matrix_Identity (appData->matrixCamera);
  Matrix_Translate (appData->matrixCamera,
		    0.0, 0.0, 300.0);
  Matrix_GetTranslation (appData->matrixCamera, &position);
  Matrix_LookAt (appData->matrixCamera, (Point3D *) & center,
		 (Point3D *) & position, 0.0f);

  M_SetCamera (appData->close, appData->matrixCamera, appData->matrixSkew);

  printf ("Setup controller...\n");
  Controller_Construct (&appData->controls);

  err = Blt_LoadUTF (&back, "panel01.utf");
  if (err < 0)
    printf ("Blt not loaded\n");

  err = Blt_SetVertices (back->bo_vertices, ViewScreen);
  if (err < 0)
    printf ("Blt not loaded\n");

  if (model1Pod == NULL) {
    printf ("NULL Pod 1!\n");
    exit (1);
  }

  if (model2Pod == NULL) {
    printf ("NULL Pod 2!\n");
    exit (1);
  }

  curPod = model1Pod;
  printf("For Guard setting case and lights to Dynamically lit Env case\n");

  for (i = 0; i < modelPodCount1; i++) {
    curPod->pmatrix = &gMatrix;
    curPod->pcase = &M_SetupDynLitEnv;
    curPod->plights = podenvlights;

    curPod = curPod->pnext;
  }				/* for */

  printf ("Changing Sphere case to DynLitTransEnv Pod\n");
  Matrix_Translate (model2Pod->pmatrix, -100.0, -100.0, 0.0);
  model2Pod->pcase = &M_SetupDynLitTransEnv;
  model2Pod->plights = podenvlights;
  model2Pod->pmaterial = &gTransMat;

  /* scale each vertex in the torus */
  Model_Scale (&gTorusPod, .5);
  torusPod = &gTorusPod;

  /* First setup DBlender so that the clear will also clear Z, etc. */
  GS_Reserve (genv->gs, M_DBInit_Size);
  M_DBInit (GS_Ptr (genv->gs), 0, 0, genv->d->width, genv->d->height);

  model1Pod = M_Sort (modelPodCount1, model1Pod, NULL);
  model2Pod = M_Sort (modelPodCount2, model2Pod, NULL);
  torusPod = M_Sort (1, torusPod, NULL);

  appData->curRenderScreen = 0;

  printf ("\n\nDISPLAY SHOULD CONTAIN...\n");
  printf ("Environmental mapped Guard \n");
  printf ("Environmental mapped partially transparent sphere\n");
  printf ("Simple Dynamically lit torus with red lighting\n");
  printf ("With textured blitter object in background\n\n");

  printf ("Controller A and arrows/shifts moves Guard\n");
  printf ("Controller L Rotates guard in X\n");
  printf ("Controller P Rotates guard in Y\n\n");
  printf ("Controller B and arrows/shifts moves Torus\n");
  printf ("Controller C and arrows/shifts moves Sphere\n");

  printf ("Controller P fogs Guard and adds specular reflections to the sphere\n");
  printf("Fog color matches background color and specular reflections are red\n");

  do {

    GS_BeginFrame (genv->gs);

    Controller_CollectEvents (&appData->controls);

    if (appData->controls.currentButtons & ControlX)
      done = TRUE;

    if (appData->controls.currentButtons & ControlStart) {
      printf("Adding fog to guard ...\n");
      AddFog (model1Pod);

      printf("Adding Red Specular reflections to sphere ...\n");
      AddSpecular (model2Pod);

      model1Pod = M_Sort (modelPodCount1, model1Pod, NULL);
      model2Pod = M_Sort (modelPodCount2, model2Pod, NULL);
    }

    Program_UserControl (appData, model1Pod->pmatrix, torusPod->pmatrix,
			 model2Pod->pmatrix);

    MyDestBM = (Bitmap *) LookupItem (GS_GetDestBuffer (genv->gs));
    appData->close->srcaddr = (uint32) MyDestBM->bm_Buffer;

    appData->close->fogcolor = M_PackColor (0.0, 0.3, 0.1);

    M_SetCamera (appData->close, appData->matrixCamera, appData->matrixSkew);

    /* Clear screen and Z-buffer */
    CLT_ClearFrameBuffer (genv->gs, 0.0, 0.3, 0.1, 1.0, TRUE, TRUE);

    GS_Reserve (genv->gs, M_DBInit_Size);
    M_DBInit (GS_Ptr (genv->gs), 0, 0, genv->d->width, genv->d->height);

    Blt_BlitObjectToBitmap (genv->gs, back, GS_GetDestBuffer (genv->gs), 0);

    genv->gs->gs_SendList (genv->gs);

    GS_Reserve (genv->gs, M_DBInit_Size);
    M_DBInit (GS_Ptr (genv->gs), 0, 0, genv->d->width, genv->d->height);

    /* it would be better to combine these and draw everthing once but then */
    /* it is difficult to deal with the objects separately after the sort */
    M_Draw (torusPod, appData->close);
    M_Draw (model1Pod, appData->close);
    M_Draw (model2Pod, appData->close);

    M_DrawEnd (appData->close);

    genv->gs->gs_SendList (genv->gs);

    GS_EndFrame (genv->gs);

    appData->curRenderScreen = (appData->curRenderScreen + 1) % 2;
    GS_SetDestBuffer (genv->gs, genv->bitmaps[appData->curRenderScreen]);
  } while (!done);

}				/* Program_Begin */

void
AddFog (Pod * modelPod)
{
  Pod *curPod;

/* NOTE: assumes all pods are textured */

  curPod = modelPod;

  do {

    curPod->pcase = &M_SetupDynLitFogEnv;
    curPod->pmaterial->base.a = 0.7;
    curPod->plights = podenvlightsFog;

    curPod = curPod->pnext;

  } while (curPod != NULL);

}				/* AddFog */

void
AddSpecular (Pod * modelPod) {

  Pod *curPod;

/* NOTE: assumes all pods are textured */

  curPod = modelPod;

  do {

	curPod->pmaterial->base.r = 0.5;
	curPod->pmaterial->base.g = 0.5;
	curPod->pmaterial->base.b = 0.5;
	curPod->pmaterial->shine = 0.5;
	curPod->pmaterial->specular.r = 1.0;
	curPod->pmaterial->specular.g = 0.0;
	curPod->pmaterial->specular.b = 0.0;
	curPod->pcase = &M_SetupDynLitSpecEnv;
	curPod->plights = podenvlightsSpec;
	curPod = curPod->pnext;

  } while (curPod != NULL);


 }


