/*
 * @(#) AllocMemUtils.c 96/10/21 1.2
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>



#include <kernel/types.h>
#include <kernel/item.h>
#include <kernel/debug.h>
#include <kernel/list.h>
#include <kernel/device.h>
#include <string.h>
#include <kernel/task.h>
#include <kernel/mem.h>
#include <stdio.h>
#include <stdlib.h>

#include <tests/3do_scaf.h>


/* ---------------------- */
/*       FORWARDS         */
/* ---------------------- */
	
void TestOneAllocMem(	int32			size,
						uint32			typebits,
						Boolean			expErr);

				
void TestAllocMemType(	uint32			typebits,
						Boolean			ql );
						
void TestOnemalloc(		int32			size,
						Boolean			expErr);
							
/* ---------------------- */
/*        GLOBALS         */
/* ---------------------- */

#define	MEM_OVERHEAD	63			/* maximum extra memory allocated allowed */
#define GAVAILOFF		1024

/* ---------------------- */
/*       UTILITIES        */
/* ---------------------- */

uint32	AMTGetFreeMem(uint32	typebits)
{

	MemInfo mi;
	int32	pSize;		/* page size */
	
	ScavengeMem();											/* return unused memory to system pool */
	
	pSize = GetPageSize( MEMTYPE_TYPESMASK&typebits );
	if (pSize < 0)
	{
		LogError(pSize );
		LogComment("Error #%d when attempting to get page size", pSize);
		return (0);
	}

	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_TYPESMASK&typebits );
	
	
	return(	(mi.minfo_TaskAllocatedPages * pSize) - mi.minfo_TaskAllocatedBytes +			/* free task mem */
			(mi.minfo_SystemAllocatedPages * pSize) - mi.minfo_SystemAllocatedBytes +		/* free sys mem */
			(mi.minfo_FreePages * pSize) );													/* unallocated mem */
}

/* --------------------------------------------------- */
/* returns the largest chunk of guaranteed to be able to allocate */
uint32	GetGuarLargestMem(uint32	typebits)
{

	MemInfo mi;
	int32	pSize;		/* page size */
		
	ScavengeMem();											/* return unused memory to system pool */
	
	pSize = GetPageSize( MEMTYPE_TYPESMASK&typebits );
	if (pSize < 0)
	{
		LogError(pSize );
		LogComment("Error #%d when attempting to get page size", pSize);
		return (0);
	}

	
	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_TYPESMASK&typebits );
	
	return(mi.minfo_LargestFreePageSpan);
}

/* --------------------------------------------------- */
/* returns the largest possible chunk (not guaranteeed) */
/* actually 1+this amount should guarantee failure */
uint32	GetMaxLargestMem(uint32	typebits)
{

	MemInfo mi;
	int32	pSize;		/* page size */
	
	ScavengeMem();											/* return unused memory to system pool */

	pSize = GetPageSize( MEMTYPE_TYPESMASK&typebits );
	if (pSize < 0)
	{
		LogError(pSize );
		LogComment("Error #%d when attempting to get page size", pSize);
		return (0);
	}
		
			
	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_TYPESMASK&typebits );
	
	return((mi.minfo_FreePages+1) * pSize);
}

/* --------------------------------------------------- */
Boolean CheckMemFill(	char	*result,
						char	fill,
						uint32	size )
{						
	uint32	n;
	
	for (n = 0; n < size; n++)
		if (result[n] != fill)
		{
			LogComment("array[%d] = $%d  instead of $%d", n, result[n], fill );
			return false;
		}
	
	return true;
}
/* *********************************** */
/*   AllocMem Tests */
/* *********************************** */

/* --------------------------------------------------- */
void	TestOneAllocMem(	int32			size,
							uint32			typebits,
							Boolean			expErr)
{
	void	*result;
	uint32	memBefore;
	uint32	memAfter;
	uint32	memEnd;
	int32	trackedSize;
	char	testName[1024];
	
	sprintf( testName, "AllocMem( %d, $%x )", size, typebits );
	if (expErr) strcat(testName, " - ExpErr");
	BeginTest( BAT, testName);
	
	memBefore = AMTGetFreeMem( typebits );
	result = AllocMem( size, typebits );
	memAfter = AMTGetFreeMem( typebits );
	
	if ( memAfter > memBefore )
	{
		LogResult(FAILED, "AllocMem( %d, 0x%x )  Memory appeared -- before %d  after %d", size, typebits, memBefore, memAfter );
		return;
	}
	
	if ( memBefore-memAfter > size+MEM_OVERHEAD )
		LogComment("Actual memory allocated = %d", memBefore-memAfter);
	
	if( (result == NULL) && expErr )
		LogResult(PASSED, "AllocMem( %d, 0x%x )  returns EXPECTED NULL", size, typebits );
	else if( (result != NULL) && expErr )
	{
		if ( (typebits&MEMTYPE_TRACKSIZE) == MEMTYPE_TRACKSIZE)		/* see if we track size */
			FreeMem( result, TRACKED_SIZE );
		else
			FreeMem( result, size );
			
		LogResult(FAILED, "AllocMem( %d, 0x%x )  returns UNEXPECTED valid mem", size, typebits );
	}
	else if( result == NULL )
		LogResult(FAILED, "AllocMem( %d, 0x%x )  returns NULL", size, typebits );
	else
	{
		if ( ((MEMTYPE_FILL&typebits) == MEMTYPE_FILL) &&
				(!CheckMemFill( result, (char)(typebits&MEMTYPE_FILLMASK), size )))
			LogResult(FAILED, "AllocMem( %d, 0x%x )  not filled properly", size, typebits  );
			
			/* see if we track size and check it*/
		else if ( ( (typebits&MEMTYPE_TRACKSIZE) == MEMTYPE_TRACKSIZE) &&
						((trackedSize = GetMemTrackSize(result)) != size)  )
			LogResult(FAILED, "GetMemTrackSize returns %d -- alloc size %d", trackedSize, size);
		else
			LogResult(PASSED, "AllocMem( %d, 0x%x )  returns valid memory", size, typebits  );
			
			
		if ( (typebits&MEMTYPE_TRACKSIZE) == MEMTYPE_TRACKSIZE)		/* see if we track size */
			FreeMem( result, TRACKED_SIZE );
		else
			FreeMem( result, size );

		memEnd = AMTGetFreeMem( typebits );
		
		if (memEnd > memBefore)
			LogComment("Memory gained = %d", memEnd-memBefore);
		else if (memBefore > memEnd)
			LogComment("Memory lost = %d", memBefore-memEnd);
	}
}

/* --------------------------------------------------- */
void TestAllocMemType(	uint32			typebits,
						Boolean			ql )
{
	uint32			n;
	
	TestOneAllocMem( 	0, typebits, TRUE );
	TestOneAllocMem( 	1, typebits, FALSE );
	TestOneAllocMem( 	64, typebits, FALSE );
	TestOneAllocMem( 	1025, typebits, FALSE );
	TestOneAllocMem( 	23342, typebits, FALSE );
	TestOneAllocMem( 	GetGuarLargestMem(typebits)-1, typebits, FALSE );
	TestOneAllocMem( 	GetGuarLargestMem(typebits), typebits, FALSE );
	TestOneAllocMem( 	GetMaxLargestMem(typebits)+GAVAILOFF, typebits, TRUE );
	
	if (ql) return;
	
	for (n = 1; n <= GetMaxLargestMem(typebits)*3; n = n*2+1)
	{
		if (n > GetGuarLargestMem(typebits) )
			TestOneAllocMem( n, typebits, TRUE );
		else
			TestOneAllocMem( n, typebits, FALSE );
	}
}

/* --------------------------------------------------- */
void TestAllocMem( Boolean	ql )
{
	TestAllocMemType(  MEMTYPE_NORMAL, ql );
	TestAllocMemType(  MEMTYPE_NORMAL|MEMTYPE_TRACKSIZE, ql );

	
	TestAllocMemType(  MEMTYPE_NORMAL|MEMTYPE_FILL|(uint32)0x000000ff, ql );

	if (ql) return;	

	TestAllocMemType(  MEMTYPE_NORMAL|MEMTYPE_FILL|(uint32)0x00000087, ql );
}

/* *********************************** */
/*   AllocMemAligned Tests */
/* *********************************** */

/* --------------------------------------------------- */
void	TestOneAllocMemAligned(	int32			size,
								uint32			typebits,
								uint32			alignment,
								Boolean			expErr)
{
	void	*result;
	uint32	memBefore;
	uint32	memAfter;
	uint32	memEnd;
	int32	trackedSize;
	char	testName[1024];
	
	sprintf( testName, "AllocMemAligned( %d, $%x, %d )", size, typebits, alignment);
	if (expErr) strcat(testName, " - ExpErr");
	BeginTest( BAT, testName);
	
	memBefore = AMTGetFreeMem( typebits );
	result = AllocMemAligned( size, typebits, alignment );
	memAfter = AMTGetFreeMem( typebits );
	
	if ( memAfter > memBefore )
	{
		LogResult(FAILED, "AllocMemAligned( %d, 0x%x, %d )  Memory appeared -- before %d  after %d", size, typebits, alignment, memBefore, memAfter );
		return;
	}
	
	if ( memBefore-memAfter > size+MEM_OVERHEAD )
		LogComment("Actual memory allocated = %d", memBefore-memAfter);
	
	if( (result == NULL) && expErr )
		LogResult(PASSED, "AllocMemAligned( %d, 0x%x, %d ) returns EXPECTED NULL", size, typebits, alignment );
	else if( (result != NULL) && expErr )
	{
		if ( (typebits&MEMTYPE_TRACKSIZE) == MEMTYPE_TRACKSIZE)		/* see if we track size */
			FreeMem( result, TRACKED_SIZE );
		else
			FreeMem( result, size );
			
		LogResult(FAILED, "AllocMemAligned( %d, 0x%x, %d ) returns UNEXPECTED valid mem", size, typebits, alignment );
	}
	else if( result == NULL )
		LogResult(FAILED, "AllocMemAligned( %d, 0x%x, %d ) returns NULL", size, typebits, alignment );
	else
	{
		if ( ((MEMTYPE_FILL&typebits) == MEMTYPE_FILL) &&
				(!CheckMemFill( result, (char)(typebits&MEMTYPE_FILLMASK), size )))
			LogResult(FAILED, "AllocMem( %d, 0x%x, %d )  not filled properly", size, typebits, alignment  );
			
			/* see if we track size and check it*/
		else if ( ( (typebits&MEMTYPE_TRACKSIZE) == MEMTYPE_TRACKSIZE) &&
						((trackedSize = GetMemTrackSize(result)) != size)  )
			LogResult(FAILED, "GetMemTrackSize returns %d -- alloc size %d", trackedSize, size);
		else if ( ((uint32)result % alignment) != 0)
			LogResult(FAILED, "AllocMemAligned( %d, 0x%x, %d )  returns %d - NOT ALIGNED", size, typebits, alignment  );
		else
			LogResult(PASSED, "AllocMemAligned( %d, 0x%x, %d )  returns valid memory", size, typebits, alignment  );
			
			
		if ( (typebits&MEMTYPE_TRACKSIZE) == MEMTYPE_TRACKSIZE)		/* see if we track size */
			FreeMem( result, TRACKED_SIZE );
		else
			FreeMem( result, size );

		memEnd = AMTGetFreeMem( typebits );
		
		if (memEnd > memBefore)
			LogComment("Memory gained = %d", memEnd-memBefore);
		else if (memBefore > memEnd)
			LogComment("Memory lost = %d", memBefore-memEnd);
	}
}

/* --------------------------------------------------- */
void TestAllocMemAlignedType(	uint32		typebits,
								uint32		alignment,
								Boolean		ql )
{
	uint32			n;
	
	TestOneAllocMemAligned( 	0, typebits, alignment, TRUE );
	TestOneAllocMemAligned( 	1, typebits, alignment, FALSE );
	TestOneAllocMemAligned( 	64, typebits, alignment, FALSE );
	TestOneAllocMemAligned( 	1025, typebits, alignment, FALSE );
	TestOneAllocMemAligned( 	23342, typebits, alignment, FALSE );
	TestOneAllocMemAligned( 	GetGuarLargestMem(typebits)-1, typebits, alignment, FALSE );
	TestOneAllocMemAligned( 	GetGuarLargestMem(typebits), typebits, alignment, FALSE );
	TestOneAllocMemAligned( 	GetMaxLargestMem(typebits)+GAVAILOFF, typebits, alignment, TRUE );
	
	if (ql) return;
	
	for (n = 1; n <= GetMaxLargestMem(typebits)*3; n = n*2+1)
	{
		if (n > GetGuarLargestMem(typebits) )
			TestOneAllocMemAligned( 	n, typebits, alignment, TRUE );
		else
			TestOneAllocMemAligned( 	n, typebits, alignment, FALSE );
	}
}

/* --------------------------------------------------- */
void TestAllocMemAligned( Boolean	ql )
{
	uint32			n;
	
	
	TestAllocMemAlignedType(  MEMTYPE_NORMAL, 8, ql );
	TestAllocMemAlignedType(  MEMTYPE_NORMAL|MEMTYPE_TRACKSIZE, 16, ql );

	
	TestAllocMemAlignedType(  MEMTYPE_NORMAL|MEMTYPE_FILL|(uint32)0x000000ff, 32, ql );

	if (ql) return;	

	TestAllocMemAlignedType(  MEMTYPE_NORMAL|MEMTYPE_FILL|(uint32)0x00000087, 64, ql );
	
	for (n = 8; n < 70000; n*=2)
		TestAllocMemAlignedType(  MEMTYPE_NORMAL, n, ql );
}

/* *********************************** */
/*   malloc Tests */
/* *********************************** */

/* --------------------------------------------------- */
void	TestOnemalloc(		int32			size,
							Boolean			expErr)
{
	void	*result;
	uint32	memBefore;
	uint32	memAfter;
	uint32	memEnd;
	char	testName[1024];
	
	sprintf( testName, "malloc( %d )", size );
	if (expErr) strcat(testName, " - ExpErr");
	BeginTest( BAT, testName);
	
	memBefore = AMTGetFreeMem( MEMTYPE_NORMAL );
	result = malloc( size );
	memAfter = AMTGetFreeMem( MEMTYPE_NORMAL );
	
	if ( memAfter > memBefore )
	{
		LogResult(FAILED, "malloc( %d )  Memory appeared -- before %d  after %d", size, memBefore, memAfter );
		return;
	}
	
	if ( memBefore-memAfter > size+MEM_OVERHEAD )
		LogComment("Actual memory allocated = %d", memBefore-memAfter);
	
	if( (result == NULL) && expErr )
		LogResult(PASSED, "malloc( %d )  returns EXPECTED NULL", size );
	else if( (result != NULL) && expErr )
	{
		free( result );
		LogResult(FAILED, "malloc( %d )  returns UNEXPECTED valid mem", size );
	}
	else if( result == NULL )
		LogResult(FAILED, "malloc( %d )  returns NULL", size );
	else
	{
		LogResult(PASSED, "malloc( %d )  returns valid memory", size  );
		free( result );
		memEnd = AMTGetFreeMem( MEMTYPE_NORMAL );
		
		if (memEnd > memBefore)
			LogComment("Memory gained = %d", memEnd-memBefore);
		else if (memBefore > memEnd)
			LogComment("Memory lost = %d", memBefore-memEnd);
	}
}

/* --------------------------------------------------- */
void Testmalloc( Boolean	ql )
{
	uint32			n;
	
	
	TestOnemalloc( 	0, TRUE );
	TestOnemalloc( 	1, FALSE );
	TestOnemalloc( 	64, FALSE );
	TestOnemalloc( 	23342, FALSE );
	TestOnemalloc( 	GetGuarLargestMem(MEMTYPE_NORMAL)-1, FALSE );
	TestOnemalloc( 	GetGuarLargestMem(MEMTYPE_NORMAL), FALSE );
	TestOnemalloc( 	GetMaxLargestMem(MEMTYPE_NORMAL)+GAVAILOFF, TRUE );
	
	if (ql) return;
	
	for (n = 1; n <= GetMaxLargestMem(MEMTYPE_NORMAL)*3; n = n*2+1)
	{
		if (n > GetGuarLargestMem(MEMTYPE_NORMAL) )
			TestOnemalloc( 	n, TRUE );
		else
			TestOnemalloc( 	n, FALSE );
	}
}

