/*
 * @(#) CreateUniqueSemaphore.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		CreateUniqueSemaphore.c

	Contains:	test the CreateUniqueSemaphore call

	Written by:	Eric Hegstrom
	

	Copyright:	 1994 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 1/4/95		ELH		created

	To Do:
*/


#include "globals.h"


/*****************************************************************************/
Err TestCreateUniqueSemaphore( int32	indx)
{
	uint8	semPri;
	char	*semName;
	Boolean	expErr;
	Item	aItem;
	Item	aItem2;
	Item	luItem;
	
	semPri 		= TCREATEUNIQUESEMAPHORE[indx].semPri;
	semName 	= TCREATEUNIQUESEMAPHORE[indx].semName;
	expErr 		= TCREATEUNIQUESEMAPHORE[indx].expErr;
	
		
	if (semName == NULL) semName = "NULL_STRING";
	sprintf( gTestName, "CreateUniqueSemaphore | pri = %d; name = \"%s\"", semPri, semName );
	if (expErr == true) strcat(gTestName, kExpectErrorStr);
				
	BeginTest( BAT, gTestName);
	
	if (strcmp("NULL_STRING", semName) == 0) semName = NULL;	/* actually send a NULL pointer if that is what we want */
	aItem = CreateUniqueSemaphore( semName, semPri );	
	if (semName == NULL) semName = "NULL_STRING";

	if ( (aItem < 0)&&(!expErr) )
	{
		LogResult( FAILED, "Error creating semaphore name = '%s' pri = %d", semName, semPri);
		LogError( aItem );
		return aItem;
	}
	else if ( (aItem >= 0)&&(expErr) )
	{
		LogResult( FAILED, "Unexpected success creating semaphore item name = '%s' pri = %d", semName, semPri);
	}
	else if (aItem < 0)
	{
		LogResult( PASSED, "Expected Error opening item name = '%s' pri = %d", semName, semPri);
		return aItem;
	}
	
/* it seemed to create the semaphore properly but lets do a little more checking */
	else
	{
	/* check to see that this semaphore item was actually created */
		luItem = FindSemaphore( semName );
		if (luItem < 0)
		{
			LogError( luItem );
			LogResult( FAILED, "Cannot find the semaphore we just allegedly created");
			return luItem;
		}
		else if (luItem != aItem)
		{
			LogResult( FAILED, "Semaphore has a different item number than returned by CreateUniqueSemaphore");
			return aItem;
		}
	
	/* now we'll try to create another semaphore with the same name and it shouldn't let */
	/* us since this would no longer be a unique semaphore */
	
		aItem2 = CreateUniqueSemaphore( semName, semPri );
		if (aItem2 == ER_Kr_UniqueItemExists)
			LogResult( PASSED, "Opened item name = '%s' pri = %d", semName, semPri);
		else if (aItem2 < 0)
		{
			LogResult( PASSED, "INCORRECT ERROR RETURNED err #$%x; name = '%s' pri = %d", aItem2, semName, semPri);
			LogError( aItem2);
		}
		else
		{
			LogResult( FAILED, "Non-unique Semaphore created name = '%s' pri = %d", semName, semPri);
			LogError( DeleteSemaphore(aItem2) );
		}
	}
	LogError( DeleteSemaphore(aItem) );
	
	return aItem;
}

