
/*
 * @(#) CompTestData.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		CompTestData.c

	Contains:	

	Written by:	Eric Hegstrom

	Copyright:	 1994 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 10/19/94	ELH		Created

	To Do:
*/

#include "CompTestTypes.h"


/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "CompressTests.c";	/* contains the name of the program */
char	* gTestName= "CompressionTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;				/* contains what level of testing occurs in this test */
int32	gTestType = 0;				/* interactive or automated */

void	*compWorkPtr = NULL;
void	*decompWorkPtr = NULL;

Compressor		*myComp = NULL;
Decompressor	*myDecomp = NULL;


uint32	*compressedData = NULL;
int32	compressedIndex = 0;
int32	compressedMaxSize = 0;

uint32	*finalData = NULL;
int32  finalIndex = 0;
int32  finalMaxSize = 0;

Boolean	compErrorEncountered = FALSE;



/* This file contains the test definitions for the Compression Tests */

void			*zeroPtr = 0;

/* ************************************************** */
/*  CreateCompressorVA test descriptions */

TCCVAtype	TCCVA[numTCCVA] = 
{
	&myComp,	DPutCompressedWord,	&compWorkPtr,	0,				false, "CreateCompressorVA - workArea !data",
	&myComp,	DPutCompressedWord,	&compWorkPtr,	(void*)-123,	false, "CreateCompressorVA - workArea data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		0,				false, "CreateCompressorVA - !workArea !data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		(void*)-456,	false, "CreateCompressorVA - !workArea data",
	0,			DPutCompressedWord,	&zeroPtr,		0,				true, "CreateCompressorVA - BAD comp !workArea !data -EXP ERR"
};

/* ************************************************** */
/*  CreateCompressor test descriptions */

TCCtype	TCC[numTCC] = 
{
	&myComp,	DPutCompressedWord,	&compWorkPtr,	0,				false, "CreateCompressor - workArea !data",
	&myComp,	DPutCompressedWord,	&compWorkPtr,	(void*)-123,	false, "CreateCompressor - workArea data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		0,				false, "CreateCompressor - !workArea !data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		(void*)-456,	false, "CreateCompressor - !workArea data",
	0,			DPutCompressedWord,	&zeroPtr,		0,				true, "CreateCompressor - BAD comp !workArea !data -EXP ERR"
};


/* ************************************************** */
/*  CreateCompressor test descriptions */

TDCtype	TDC[numTDC] = 
{
	&myComp,	DPutCompressedWord,	&compWorkPtr,	0,				false, "DeleteCompressor - workArea !data",
	&myComp,	DPutCompressedWord,	&compWorkPtr,	(void*)-123,	false, "DeleteCompressor - workArea data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		0,				false, "DeleteCompressor - !workArea !data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		(void*)-456,	false, "DeleteCompressor - !workArea data",
	&myComp,	DPutCompressedWord,	&zeroPtr,		0,				false, "DeleteCompressor - !workArea !data"
};


/* ************************************************** */
/*  Compress test descriptions */

TCtype	TC[numTC] = 
{
	1,		1,		1,		1,		true, "Compress - 	1,		1,		1,		1 -EXP ERR",
	1,		1,		200,	1,		true, "Compress - 	1,		1,		64,		1 -EXP ERR",
	10000,	10000,	13000,	13000,	false, "Compress - 	10000,	10000,	13000,	13000",
	4000,	4000,	6000,	6000,	false, "Compress - 	4000,	4000,	6000,	6000",
	10000,	10000,	455,	455,	true, "Compress - 	10000,	10000,	455,	455 -EXP ERR",
	10000,	455,	455,	455,	true, "Compress - 	10000,	455,	455,	455 -EXP ERR"
};


/* ************************************************** */
/*  Decompress test descriptions */

TDtype	TD[numTD] = 
{
	1,			false, "Decompress - 	1",
	128,		false, "Decompress - 	128",
	1024,		false, "Decompress - 	1024",
	12468,		false, "Decompress - 	12468",
	32000,		false, "Decompress - 	32000",
	0,			false, "Decompress - 	0"
};

/* ************************************************** */
/*  FeedDecompressor test descriptions */

TFDtype	TFD[numTD] = 
{
/*	1	*/
	&myComp,	&myDecomp,	PutCompressedWord,	PutFinalWord,	&compWorkPtr,	0,	
	128,		false,	"FeedDecompressor - CC(workArea !data) - 128",
/*	2	*/
	&myComp,	&myDecomp,	PutCompressedWord,	PutFinalWord,	&compWorkPtr,	0,	
	127,		false,	"FeedDecompressor - CC(workArea !data) - 127",
/*	3	*/
	&myComp,	&myDecomp,	PutCompressedWord,	PutFinalWord,	&compWorkPtr,	0,	
	129,		false,	"FeedDecompressor - CC(workArea !data) - 129",
/*	4	*/
	&myComp,	&myDecomp,	PutCompressedWord,	PutFinalWord,	&zeroPtr,		0,	
	8012,		false,	"FeedDecompressor - CC(!workArea !data) - 8012",
/*	5	*/
	&myComp,	&myDecomp,	PutCompressedWord,	PutFinalWord,	&zeroPtr,		0,	
	12345,		false,	"FeedDecompressor - CC(!workArea !data) - 12345",
/*	6	*/
	&myComp,	&myDecomp,	PutCompressedWord,	PutFinalWord,	&zeroPtr,		0,	
	0,			false,	"FeedDecompressor - CC(!workArea !data) - 0",
};

/* ************************************************** */
/*  FeedCompressor test descriptions */

TFCtype	TFC[numTC] = 
{
/*	1	*/
	&myComp,	PutCompressedWord,	&compWorkPtr,	0,	
	128,		false,	"FeedCompressor - CC(workArea !data) - 128",
/*	2	*/
	&myComp,	PutCompressedWord,	&compWorkPtr,	0,	
	127,		false,	"FeedCompressor - CC(workArea !data) - 127",
/*	3	*/
	&myComp,	PutCompressedWord,	&compWorkPtr,	0,	
	129,		false,	"FeedCompressor - CC(workArea !data) - 129",
/*	4	*/
	&myComp,	PutCompressedWord,	&zeroPtr,		0,	
	8012,		false,	"FeedCompressor - CC(!workArea !data) - 8012",
/*	5	*/
	&myComp,	PutCompressedWord,	&zeroPtr,		0,	
	12345,		false,	"FeedCompressor - CC(!workArea !data) - 12345",
/*	6	*/
	&myComp,	PutCompressedWord,	&zeroPtr,		0,	
	0,			false,	"FeedCompressor - CC(!workArea !data) - 0",
};

