
/*
 * @(#) FeedDecompressor.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		FeedDecompressor.c

	Contains:	Test The Compression Folio

	Written by:	Eric Hegstrom
	

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 11/7/94	ELH		created

	To Do:
*/


#include "globals.h"
#include "CompressUtils.h"

#define	FD_CLEANUP1	if (srcData != NULL) free( srcData );			\
				compressedIndex = 0;						\
				finalIndex = 0;							\
				compErrorEncountered = FALSE;

#define	FD_CLEANUP	if (srcData != NULL) free( srcData );			\
					if (destData != NULL) free( destData );			\
					if (decompData != NULL) free( decompData );		\
					compressedIndex = 0;							\
					finalIndex = 0;									\
					compErrorEncountered = FALSE;


/*****************************************************************************/
Err TestFeedDecompressor( int32	indx)
{
	#define		compOverhead 2500

	Err				err = -1;
	void			*srcData;
	void			*destData;
	void			*decompData;


	gTestName = TFD[indx].name;
	BeginTest( BAT, gTestName);
	compressedMaxSize = TFD[indx].srcSize+compOverhead;
	finalMaxSize = TFD[indx].srcSize+compOverhead;
	
	srcData = (void *) malloc( TFD[indx].srcSize*sizeof(uint32) );
	if ( ( srcData == NULL )&&( TFD[indx].srcSize*sizeof(uint32) != 0 ))
	{
		LogResult(UNRESOLVED, "Error Allocating srcData # $%x",err );
		return -1;
	}

	destData = (void *) malloc( compressedMaxSize*sizeof(uint32) );
	if ( ( destData == NULL )&&( compressedMaxSize*sizeof(uint32) != 0 ))
	{
		LogResult(UNRESOLVED, "Error Allocating destData # $%x",err );
		FD_CLEANUP1
		return -1;
	}

	decompData = (void *) malloc( finalMaxSize*sizeof(uint32) );
	if ( ( decompData == NULL )&&( finalMaxSize*sizeof(uint32)  != 0 ))
	{
		LogResult(UNRESOLVED, "Error Allocating decompData # $%x",err );
		FD_CLEANUP
		return -1;
	}
	
/* do this so the compressor and decompressor functions know where to put the data */
	compressedData = (uint32 *)destData;
	finalData = (uint32 *)decompData;

	
	FillRandomWord( srcData, TFD[indx].srcSize );
	
/* create the compressor */

	err = CreateCompressorVA(	TFD[indx].comp,
								TFD[indx].cf,
								COMP_TAG_WORKBUFFER, *(TFD[indx].workBufHdl),
								COMP_TAG_USERDATA, TFD[indx].userData,
								TAG_END,0);
	if (err < 0)
	{
		LogResult(UNRESOLVED, "CreateCompressor Error # $%x",err );
		LogError(err);
		FD_CLEANUP
		return err;
	}
	
/* do the compression */
	err = FeedCompressor( *(TFD[indx].comp), srcData, TFD[indx].srcSize );

	if ( err < 0 )
	{
		LogResult(UNRESOLVED, "FeedCompressor Error # $%x",err );
		LogError(err);
		FD_CLEANUP
		return err;
	}
	
	err = DeleteCompressor( *(TFD[indx].comp) );
	if ( err < 0 )
	{
		LogResult(UNRESOLVED, "DeleteCompressor Error # $%x",err );
		LogError(err);
		FD_CLEANUP
		return err;
	}
	
	if (compErrorEncountered)
	{
		FD_CLEANUP
		return -1;
	}
	
/* create the decompressor */

	err = CreateDecompressorVA(	TFD[indx].deComp,
								TFD[indx].df,
								COMP_TAG_WORKBUFFER, *(TFD[indx].workBufHdl),
								COMP_TAG_USERDATA, TFD[indx].userData,
								TAG_END, 0);
								
	if (compErrorEncountered)
		err = -1;

	if (err < 0)
	{
		LogResult(UNRESOLVED, "CreateDecompressor Error # $%x",err );
		LogError(err);
		FD_CLEANUP
		return err;
	}
	
/* Do the Decompression */
	err = FeedDecompressor( *(TFD[indx].deComp), destData, compressedIndex );


	if ( (err >= 0)&&(!TFD[indx].expErr) )
	{
		if(	finalIndex != TFD[indx].srcSize )
			LogResult( FAILED, "decompressed size (%d) DOES NOT match original size (%d)", finalIndex, TFD[indx].srcSize );
		else if( CompareMem( srcData, decompData, finalIndex ) )
			LogResult( PASSED, "decompress data matches original" );
		else
			LogResult( FAILED, "decompress data DOES NOT match original" );
	}
	else if ( (err < 0)&&(TFD[indx].expErr) )
	{
		LogResult(PASSED, "Decompress Expected Error # $%x",err );
		/* LogError(err); */
	}
	else if (err < 0)
	{
		LogResult(FAILED, "Decompress Unexpected Error # $%x",err);
		LogError(err);
	}
	else if (err >= 0)
	{
		LogResult(FAILED, "Decompress Unexpexted No-Error ");
	}
	
	err = DeleteDecompressor( *(TFD[indx].deComp) );
	if (err < 0)
	{
		LogComment("DeleteDecompressor Error # $%x",err);
		LogError(err);
	}

	FD_CLEANUP

	return err;
}

