#include "FileUtils.h"


/* ====================================================================	*/
/*					FindFileAndIdentify		Tests						*/
/* ====================================================================	*/
Boolean	SetupFindFileAndIdentify( void )
{
	#define	TEST_DIR1	"/remote/FSTestDir1"
	#define	TEXT_FILE_TYPE	0x54455854

	Err			err;
	char		savedDir[128];
	Boolean		noErrors = TRUE;
	FileInfo	fInfo;
	
	
	if ( (err = CreateDirectory( TEST_DIR1 )) < 0)
	{
		LogError( err );
		LogComment( "Error $%x creating directory \"%s\"", err, TEST_DIR1);
		return false;
	}
	
	GetDirectory( savedDir, 128 );			/* save currect directory path */
	ChangeDirectory( TEST_DIR1 );
	
	
	if ( MyCreateFile( "FSTestFile1" ) < 0 )	noErrors = FALSE;
	if ( SetFileAttrsVA( "FSTestFile1", FILEATTRS_TAG_FILETYPE, TEXT_FILE_TYPE, TAG_END, 0) < 0 )	noErrors = FALSE;
	if ( MyCreateFile( "FSTestFile2" ) < 0 )	noErrors = FALSE;
	if ( MyCreateFile( "FSTestFile3" ) < 0 )	noErrors = FALSE;
	
	ChangeDirectory( savedDir );
	
	if ( GetFileInfo ( &fInfo, "/remote/FSTestDir1/FSTestFile1" ) >= 0 )
		LogComment( "file  /remote/FSTestDir1/FSTestFile1  has type $%x", fInfo.fi_FileType );

	if ( GetFileInfo ( &fInfo, "/remote/KernelScript" ) >= 0 )
		LogComment( "file  /remote/KernelScript  has type $%x", fInfo.fi_FileType );

	
	return noErrors;
}

/* --------------------------------------------------- */
void	TestOneFindAndIdentifyFile(	char	*partialPath,
									TagArg	*searchTags,
									char	*tagDescription,
									char	*expFoundPath)		/* send NULL if we expect error (or no find)*/

{
	char			testName[128];
	Err				err;
	Boolean			result;
	char			pathBuf[512];
	
	if( partialPath == NULL )	partialPath = "_NULL_PTR_";		/* check for NULL path string pointers */
	if( expFoundPath == NULL )	expFoundPath  = "_NULL_PTR_";

	sprintf( testName, "FindFileAndIdentify(\"%s\") [%s] expect to find \"%s\"", partialPath, tagDescription, expFoundPath );
		
	BeginTest(BAT, testName);
	
	
	if ( (err = FindFileAndIdentify( pathBuf, 512, partialPath, searchTags )) < 0)
	{
		if (expFoundPath != NULL)		/* if expPath is not NULL we expect to find a file */
		{
			LogError( err );
			LogResult(FAILED, "FindFileAndIdentify Returns UNEXPECTED error $%x", err);
		}
		else
			LogResult(PASSED, "FindFileAndIdentify Returns expected error $%x", err);
			
		return;
	}

	if ( (strcmp( expFoundPath, pathBuf ) ) == 0)	/* did we find the file we expected to */
		result = TRUE;								/* if so set result to true */
	else
		result = FALSE;
											/* check if result was what we expected */
	if( result )
		LogResult(PASSED, "Found expected file");
	else 
		LogResult(FAILED, "expected \"%s\" but found \"%s\"", expFoundPath, pathBuf );

}



/* --------------------------------------------------- */
void	TestFindFileAndIdentify(void )
{
	
	if ( !SetupFindFileAndIdentify( ))
	{
		LogComment("================================");
		LogComment("  FindFileAndIdentify Setup failed no FindFileAndIdentify tests will be run");
		LogComment("================================");
		return;
	}
#if 0
	TestOneFindFileAndIdentify( "b", "/remote/", "bMy_OpenFileTest", FALSE );
#endif
}

