#include "FileUtils.h"


/* ====================================================================	*/
/*							Rename		Tests							*/
/* ====================================================================	*/
/* ------------------------------- */
Boolean	SetupRename( Boolean	cleanUp )	/* if cleanup is true it does cleanup */
{
	#define	TEST_DIR1	"/remote/FSRN1"

	Err			err;
	char		savedDir[128];
	Boolean		noErrors = TRUE;
	Item		dItem;
	
	
	if (!cleanUp)											/* if setup create a test directory */
		if ( (err = CreateDirectory( TEST_DIR1 )) < 0)
		{
			LogError( err );
			LogComment( "Error $%x creating directory \"%s\"", err, TEST_DIR1);
			return false;
		}
	
	GetDirectory( savedDir, 128 );			/* save currect directory path */
	
	if ( (dItem = ChangeDirectory( TEST_DIR1 )) < 0)
	{
			LogError( dItem );
			LogComment( "Error $%x changing to directory \"%s\"", dItem, TEST_DIR1);
			return false;
	}
	
	
	if ( MyCreateDeleteFile( "FS1", cleanUp ) < 0 )	noErrors = FALSE;
	if ( MyCreateDeleteFile( "FS2", cleanUp ) < 0 )	noErrors = FALSE;
	if ( MyCreateDeleteFile( "FS3", cleanUp ) < 0 )	noErrors = FALSE;
	
	if ( (dItem = ChangeDirectory( savedDir )) < 0)
	{
			LogError( dItem );
			LogComment( "Error $%x changing to directory \"%s\"", dItem, savedDir);
			return false;
	}
		
	if (cleanUp)											/* if clean up delete the test directory */
		if ( (err = DeleteDirectory( TEST_DIR1 )) < 0)
		{
			LogError( err );
			LogComment( "Error $%x deleting directory \"%s\"", err, TEST_DIR1);
			return false;
		}
		
	return noErrors;
}

/* --------------------------------------------------- */
void	TestOneRename(	char	*oldPath,
						char	*newPath,
						Boolean	expErr)		/* set TRUE if we expect error */

{
	char			testName[128];
	Err				err;
	Boolean			result = TRUE;
	
	if( oldPath == NULL )	oldPath = "_NULL_PTR_";		/* check for NULL path string pointers */
	if( newPath == NULL )	newPath  = "_NULL_PTR_";

	sprintf( testName, "Rename(\"%s\", \"%s\") ", oldPath, newPath );
	
	if (expErr) strcat( testName, " -ExpErr" );
		
	BeginTest(BAT, testName);
	
	
	if ( (err = Rename( oldPath, newPath )) < 0)
	{
		if (!expErr)		/* if expPath is not NULL we expect to find a file */
		{
			LogError( err );
			LogResult(FAILED, "Rename Returns UNEXPECTED error $%x", err);
		}
		else
			LogResult(PASSED, "Rename Returns expected error $%x", err);
			
		return;
	}

	if ( FileExists ( oldPath ) )		/* check if the original didn't go away */
	{
		result = FALSE;				
		LogComment( "\"%s\" still exists", oldPath);
	}
	if ( !FileExists ( newPath ) )		/* check if the new file exists */
	{
		result = FALSE;				
		LogComment( "\"%s\" does NOT exist", newPath);
	}

											/* check if result was what we expected */
	if( result  && expErr )
		LogResult(FAILED, "Expected an error but non encountered");
	if( result  && !expErr )
		LogResult(PASSED, "Appeared to work properly");
	else if (expErr)
		LogResult(PASSED, "An expected error occured" );
	else
		LogResult(FAILED, "UNEXPECTED Error encountered" );
}



/* --------------------------------------------------- */
void	TestRename(void )
{
	
	if ( !SetupRename( FALSE ))
	{
		LogComment("=========================================================================");
		LogComment("|																		|");
		LogComment("|					Rename Setup failed 								|");
		LogComment("|			FindFileAndIdentify tests will be run.						|");
		LogComment("|																		|");
		LogComment("=========================================================================");
		return;
	}

	TestOneRename( "/remote/FSRN1/FS1", "/remote/FSRN1/goober", FALSE );

#if 0
	if ( !SetupRename( TRUE ))			/* do cleanup */
		LogComment("  *** Rename Cleanup failed.");
#endif
}

