
/*
 * @(#) SetFileAttrsTests.c 96/10/21 1.2
 */

/* ====================================================================	*/
/*						SetFileAttrs		Tests						*/
/* ====================================================================	*/

#include "FileUtils.h"


#define	NO_CHANGE	0xA9				/* if any atribute is this don't mess with it */
#define	TEXT_FILE_TYPE	0x54455854


/* ------------------------------- */
Boolean	SetupSetFileAttrs( Boolean	cleanUp )	/* if cleanup is true it does cleanup */
{
	#define	TEST_DIR1	"/remote/FSD2"

	Err			err;
	char		savedDir[128];
	Boolean		noErrors = TRUE;
	
	
	if (!cleanUp)											/* if setup create a test directory */
		if ( (err = CreateDirectory( TEST_DIR1 )) < 0)
		{
			LogError( err );
			LogComment( "Error $%x creating directory \"%s\"", err, TEST_DIR1);
			return false;
		}
	
	GetDirectory( savedDir, 128 );			/* save currect directory path */
	ChangeDirectory( TEST_DIR1 );
	
	
	if ( MyCreateDeleteFile( "FS1", cleanUp ) < 0 )	noErrors = FALSE;
	if ( MyCreateDeleteFile( "FS2", cleanUp ) < 0 )	noErrors = FALSE;
	if ( MyCreateDeleteFile( "FS3", cleanUp ) < 0 )	noErrors = FALSE;
	
	ChangeDirectory( savedDir );
		
	if (cleanUp)											/* if clean up delete the test directory */
		if ( (err = DeleteDirectory( TEST_DIR1 )) < 0)
		{
			LogError( err );
			LogComment( "Error $%x deleting directory \"%s\"", err, TEST_DIR1);
			return false;
		}
		
	return noErrors;
}

/* ----------------------------- */
void TestOneSetFileAttrs(	char		*path,			/* file name */
							PackedID	fType,			/* attributes to change */
							uint8		fVersion,		/* use NO_CHANGE if we don't want that particular */
							uint8		fRevision,		/* attribute to be set */
							uint32		fBlockSize,
							Boolean		expErr	)		/* TRUE if we expect an error or mismatch */
							
{
	TagArg		tags[6];
	int32		indx = 0;
	Err			err;
	FileInfo	fInfo;
	Boolean		error = FALSE;			/* set this true when we've encountered an error */
	char		testName[128];
	
	if( path == NULL )	path = "_NULL_PTR_";		/* check for NULL path string pointers */

	sprintf( testName, "SetFileAttr path \"%s\"; type $%x; vers $%x; rev $%x; bSize $%x; ",
												path, fType, fVersion, fRevision, fBlockSize );
	if ( expErr )		
		strcat ( testName, " - ExpErr" );
		
	BeginTest(BAT, testName);
	
	if ( fType != NO_CHANGE)								/* set up attributes tags for all atributes */
	{														/* that we will attempt to set (that is any */
		tags[indx].ta_Tag	=	FILEATTRS_TAG_FILETYPE;		/* attribute thats not value NO_CHANGE */
		tags[indx++].ta_Arg	=	(void *)fType;
	}
	if ( fVersion != NO_CHANGE)
	{
		tags[indx].ta_Tag	=	FILEATTRS_TAG_VERSION;
		tags[indx++].ta_Arg	=	(void *)fVersion;
	}
	if ( fRevision != NO_CHANGE)
	{
		tags[indx].ta_Tag	=	FILEATTRS_TAG_REVISION;
		tags[indx++].ta_Arg	=	(void *)fRevision;
	}
	if ( fBlockSize != NO_CHANGE)
	{
		tags[indx].ta_Tag	=	FILEATTRS_TAG_BLOCKSIZE;
		tags[indx++].ta_Arg	=	(void *)fBlockSize;
	}
	tags[indx].ta_Tag	=	TAG_END;						/* don't forget to end the tag array */
	tags[indx].ta_Arg	=	(void *)0;


	err = SetFileAttrs( path, tags );						/* attempt to set attributes */
	
	if (err < 0)								/* handle errors from by reporting if it was expected */
	{											/* or not and then returning */
		if (expErr)
			LogResult( PASSED, "SetFileAttr returned an expected error $%x", err);
		else
		{
			LogError( err );
			LogResult( FAILED, "SetFileAttr returned an UNEXPECTED error $%x", err);
		}
		return;
	}
	
															/* get the file attributes so that we can */
	if ( GetFileInfo ( &fInfo, path) < 0)			/* confirm that they were set properly */
	{
		LogResult( UNRESOLVED, "Could not get the file attributes to check them ");
		return;
	}
	
	
												/* check that any attributes we expected to set */
												/* actually get set to the values we expect */
												/* if they don't report them and flag an error */

	if ( (fType != NO_CHANGE) && (fInfo.fi_FileType != fType) )		/* check FileType */
	{	
		LogComment("Type: expected $%8x - actual $%8x ", fType, fInfo.fi_FileType );
		error = TRUE;
	}
	
	if ( (fVersion != NO_CHANGE) && (fInfo.fi_Version != fVersion) )		/* check fVersion */
	{
		LogComment("Version: expected $%8x - actual $%8x ", fVersion, fInfo.fi_Version );
		error = TRUE;
	}
	
	if ( (fRevision != NO_CHANGE) && (fInfo.fi_Revision != fRevision) )		/* check Revision */
	{
		LogComment("Revision: expected $%8x - actual $%8x ", fRevision, fInfo.fi_Revision );
		error = TRUE;
	}
	
	if ( (fBlockSize != NO_CHANGE) && (fInfo.fi_BlockSize != fBlockSize) )		/* check BlockSize */
	{
		LogComment("BlockSize: expected $%8x - actual $%8x ", fBlockSize, fInfo.fi_BlockSize );
		error = TRUE;
	}

	if( !error )
		LogResult(PASSED, "All attributes set properly");
	else if( error && expErr )
		LogResult(PASSED, "Error occured but was expected");
	else 
		LogResult(FAILED, "UNEXPECTED Error occured" );

}

/* -------------------------- */
void TestSetFileAttrs( void )
{
	if ( !SetupSetFileAttrs( FALSE ))
	{
		LogComment("=========================================================================");
		LogComment("|																		|");
		LogComment("|				SetFileAttrs Setup failed 								|");
		LogComment("|			FindFileAndIdentify tests will be run.						|");
		LogComment("|																		|");
		LogComment("=========================================================================");
		return;
	}

	TestOneSetFileAttrs( "/remote/FSD2/FS1", NO_CHANGE, NO_CHANGE, NO_CHANGE, NO_CHANGE, FALSE );
	TestOneSetFileAttrs( "/remote/FSD2/FS1", NO_CHANGE, NO_CHANGE, NO_CHANGE, NO_CHANGE, 128 );
	TestOneSetFileAttrs( "/remote/FSD2/FS1", TEXT_FILE_TYPE, NO_CHANGE, NO_CHANGE, NO_CHANGE, FALSE );
	TestOneSetFileAttrs( "/remote/FSD2/FS2", NO_CHANGE, 8, NO_CHANGE, NO_CHANGE, FALSE );
	TestOneSetFileAttrs( "/remote/FSD2/FS3", NO_CHANGE, NO_CHANGE, 8, NO_CHANGE, FALSE );

	if ( !SetupSetFileAttrs( TRUE ))			/* do cleanup */
		LogComment("  *** SetFileAttrs Cleanup failed.");
}

