
/*
 * @(#) TestFindItem.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestFindItem.c

	Contains:	Test routine for FindItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestFindItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
/* HINTS:

	 May change drastically depending on the call.
*/
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
/* HINTS:

	 May change drastically depending on the call.
	 May need several different versions of these.
*/
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeFindItemVerifyStr	"Before iteration (%i)/\ttype/ $%x,\tname/ %s"
#define		kAfterFindItemVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean VerifyFindItem ( Boolean beforeCall, long aIndx, int32 aResult, int32 itemType, TagArg *tags )
/* Functionality verification for FindItem */
{
	char *aName = "";
	
	if (beforeCall == kBeforeCall)
	{		
		if (tags[0].ta_Arg != NULL) aName = (char *) tags[0].ta_Arg;
		LogDiagnostic( 12,kBeforeFindItemVerifyStr, aIndx, itemType, printable(aName));
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterFindItemVerifyStr, aIndx, aResult);
		return ( aResult >= 0 );
	}
	
}

#define		CLEANUP

long	CallFindItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	int32	aType;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) gItemName,
		TAG_END,  0
	};
	
	/***** Preparation for call iterations *****/
	aType = MKNODEID(aFType,aNType);
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyFindItem ( kBeforeCall, minInnerIterations, aResult, aType, tags );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = FindItem( aType, tags );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyFindItem ( kAfterCall, indx, aResult, aType, tags );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneFindItem (  Boolean expectError, uint8 aFType, uint8	aNType, char *aName )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
 	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	int32	aType;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  0,
		TAG_END,  0
	};
	tags[0].ta_Arg = (void *) aName;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aType = MKNODEID(aFType,aNType);
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyFindItem ( kBeforeCall, gCurrentIndex, aResult, aType, tags );

	aResult = FindItem( aType, tags );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyFindItem ( kAfterCall, gCurrentIndex, aResult, aType, tags );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_FINDITEM(expVal, fTyp, nTyp, name)	if (name != NULL) sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'", gTestAreaName,fTyp, nTyp, name );	\
														else sprintf( gTestName, "%s | f= %d; n= %d; name= NULL", gTestAreaName,fTyp, nTyp );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneFindItem(expVal, fTyp, nTyp, name); \
														


void ExerciseFindItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "FindItem";
	STATUS(callName);
	
	/* params: ftype, ntype */
	
	TEST_ONE_FINDITEM(kExpectError, gFType, gNType, "");
	TEST_ONE_FINDITEM(kExpectError, gFType, gNType, " ");
	TEST_ONE_FINDITEM(kExpectError, gFType, gNType, "W");
	TEST_ONE_FINDITEM(kExpectError, gFType, gNType, "w");
	TEST_ONE_FINDITEM(kExpectError, gFType, gNType, "5");
	TEST_ONE_FINDITEM(kExpectError, gFType, gNType, "This is quite an unreasonably lengthy name");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "kernel");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, FOLIONODE, "Graphics");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "File");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, FOLIONODE, "audio");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, FOLIONODE, "Operamath");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, ERRORTEXTNODE, "Kernel Extended Error Table");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, SEMA4NODE, "Swi");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, FIRQNODE, "kernel default");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, TASKNODE, "Operator");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, TASKNODE, "shell");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, TASKNODE, "EventBroker");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "timer");
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, DEVICENODE, "timer");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, TIMERNODE, NULL);
	TEST_ONE_FINDITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "ram");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, DEVICENODE, "ram");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, DEVICENODE, "CD-ROM");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, IOREQNODE, NULL);
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, DRIVERNODE, "mac");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, DEVICENODE, "mac");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, DRIVERNODE, "SPORT");
	TEST_ONE_FINDITEM(kExpectError, KERNELNODE, DEVICENODE, "SPORT");
	TEST_ONE_FINDITEM(kExpectError, FILENODE, 2, "");
	TEST_ONE_FINDITEM(kExpectError, AUDIONODE, 3, NULL);

/*
	...
 Invalid type
 low memory
*/
}

void TestFindItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseFindItem();
	else
	{
		callName = "FindItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallFindItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


