
/*
 * @(#) TestFindNamedItem.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestFindNamedItem.c

	Contains:	Test routine for FindNamedItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestFindNamedItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeFindNamedItemVerifyStr		"Before iteration (%i)/\ttype/ $%x,\tname/ %s"
#define		kAfterFindNamedItemVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyFindNamedItem ( Boolean beforeCall, long aIndx, Item aResult, int32 itemType, char *aName )
/* Functionality verification for FindNamedItem */
{
	Boolean succeeded;
	ItemNodeP aItemNode;
	char *actualName;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeFindNamedItemVerifyStr, aIndx, itemType, printable(aName));
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterFindNamedItemVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;
			
		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		actualName = aItemNode->n_Name;
		succeeded = ( strcmp(actualName, aName) == 0 );
		if (!succeeded)
			LogComment("!! Actual name: %s", printable(actualName));
			
		return succeeded;
	}
	
}

#define		CLEANUP

long	CallFindNamedItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	int32	aType;
	char	*aName = gItemName;
	
	/***** Preparation for call iterations *****/
	aType = MKNODEID(aFType,aNType);
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyFindNamedItem ( kBeforeCall, minInnerIterations, aResult, aType, aName );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = FindNamedItem( aType, aName );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyFindNamedItem ( kAfterCall, indx, aResult, aType, aName );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneFindNamedItem ( Boolean expectError, uint8 aFType, uint8 aNType, char *aName )
/* Individual call for parameter-testing */
{
 	Boolean theError = FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	int32	aType;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aType = MKNODEID(aFType,aNType);
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyFindNamedItem ( kBeforeCall, gCurrentIndex, aResult, aType, aName );

	aResult = FindNamedItem( aType, aName );

	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyFindNamedItem ( kAfterCall, gCurrentIndex, aResult, aType, aName );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#define	TEST_ONE_FINDNAMEDITEM(expVal, fTyp, nTyp, name) if (name != NULL) sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'", gTestAreaName,fTyp, nTyp, name );	\
														else sprintf( gTestName, "%s | f= %d; n= %d; name= NULL", gTestAreaName,fTyp, nTyp );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneFindNamedItem(expVal, fTyp, nTyp, name); \
														

void ExerciseFindNamedItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "FindNamedItem";
	STATUS(callName);
	
	/* params: ftype, ntype, name */
	TEST_ONE_FINDNAMEDITEM(kExpectError, gFType, gNType, "");
	TEST_ONE_FINDNAMEDITEM(kExpectError, gFType, gNType, " ");
	TEST_ONE_FINDNAMEDITEM(kExpectError, gFType, gNType, "W");
	TEST_ONE_FINDNAMEDITEM(kExpectError, gFType, gNType, "w");
	TEST_ONE_FINDNAMEDITEM(kExpectError, gFType, gNType, "5");
	TEST_ONE_FINDNAMEDITEM(kExpectError, gFType, gNType, "This is quite an unreasonably lengthy name");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "kernel");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, FOLIONODE, "Graphics");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "File");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, FOLIONODE, "audio");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, FOLIONODE, "Operamath");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, ERRORTEXTNODE, "Kernel Extended Error Table");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, SEMA4NODE, "Swi");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, FIRQNODE, "kernel default");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, TASKNODE, "Operator");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, TASKNODE, "shell");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, TASKNODE, "EventBroker");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "timer");
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, DEVICENODE, "timer");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, TIMERNODE, NULL);
	TEST_ONE_FINDNAMEDITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "ram");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, DEVICENODE, "ram");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, DEVICENODE, "CD-ROM");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, IOREQNODE, NULL);
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, DRIVERNODE, "mac");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, DEVICENODE, "mac");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, DRIVERNODE, "SPORT");
	TEST_ONE_FINDNAMEDITEM(kExpectError, KERNELNODE, DEVICENODE, "SPORT");
	TEST_ONE_FINDNAMEDITEM(kExpectError, FILENODE, 2, "");
	TEST_ONE_FINDNAMEDITEM(kExpectError, AUDIONODE, 3, NULL);

/*
	...
 Invalid type
 low memory
*/
}

void TestFindNamedItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseFindNamedItem();
	else
	{
		callName = "FindNamedItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallFindNamedItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


