
/*
 * @(#) TestLookupItem.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"
#include	"itemUtils.h"

/*
	File:		TestLookupItem.c

	Contains:	Test routine for LookupItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestLookupItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeLookupItemVerifyStr		"Before iteration (%i)/\titem/ $%x,\tname/ %s"
#define		kAfterLookupItemVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyLookupItem ( Boolean beforeCall, long aIndx, void *aResult, Item aItem, char *aName )
/* Functionality verification for LookupItem */
{
	Boolean succeeded;
	Item actualItem;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeLookupItemVerifyStr, aIndx, aItem, printable(aName));
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterLookupItemVerifyStr, aIndx, aResult);
		if (aResult == NULL)
			return false;
		
		actualItem = ( (ItemNodeP) aResult )->n_Item;
		succeeded = ( actualItem == aItem );
		if (!succeeded)
			LogComment("!! Actual item number: $%x", actualItem);
			
		return succeeded;
	}
	
}

#define		CLEANUP

long	CallLookupItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	void	*aResult = NULL;
	Item	aItem;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	
	TOUCH(currentOuterIteration);

	/***** Preparation for call iterations *****/
	aItem = FindNamedItem( MKNODEID(aFType,aNType), gItemName );
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyLookupItem ( kBeforeCall, minInnerIterations, aResult, aItem, gItemName );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = LookupItem( aItem );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyLookupItem ( kAfterCall, indx, aResult, aItem, gItemName );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneLookupItem ( Boolean expectError, uint8 aFType, uint8 aNType, char *aName )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	void	*aResult = NULL;

	/* Ancillary variables */
	Item	aItem;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aItem = FindNamedItem( MKNODEID(aFType,aNType), aName );
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyLookupItem ( kBeforeCall, gCurrentIndex, aResult, aItem, aName );

	aResult = LookupItem( aItem );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyLookupItem ( kAfterCall, gCurrentIndex, aResult, aItem, aName );
		
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_LOOKUPITEM(expVal, fTyp, nTyp, name)	\
	if (name != NULL) \
	    sprintf( gTestName, "%s | f= %d; n= %d; name= '%s';", gTestAreaName, fTyp, nTyp, name); \
	else \
	    sprintf( gTestName, "%s | f= %d; n= %d; name= NULL;", gTestAreaName, fTyp, nTyp);	\
	if (expVal == kExpectError) \
	    strcat(gTestName, kExpectErrorStr);	\
	BeginTest( BAT, gTestName );	\
	OneLookupItem(expVal, fTyp, nTyp, name);
														


void ExerciseLookupItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "LookupItem";
	STATUS(callName);
	
	/* params: ftype, ntype */
	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "kernel");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, FOLIONODE, "Graphics");

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, FOLIONODE, "File");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, FOLIONODE, "audio");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, FOLIONODE, "Operamath");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, ERRORTEXTNODE, "Kernel Extended Error Table");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, SEMA4NODE, "Swi");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, FIRQNODE, "kernel default");

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, TASKNODE, "Operator");

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, TASKNODE, "shell");

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, TASKNODE, "EventBroker");

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "timer");

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, DEVICENODE, "timer");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, TIMERNODE, NULL);

	TEST_ONE_LOOKUPITEM(kExpectSuccess, KERNELNODE, DRIVERNODE, "ram");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, DEVICENODE, "ram");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, DEVICENODE, "CD-ROM");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, IOREQNODE, NULL);

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, DRIVERNODE, "mac");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, DEVICENODE, "mac");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, DRIVERNODE, "SPORT");

	TEST_ONE_LOOKUPITEM(kExpectError, KERNELNODE, DEVICENODE, "SPORT");

	TEST_ONE_LOOKUPITEM(kExpectError, FILENODE, 2, "");

	TEST_ONE_LOOKUPITEM(kExpectError, AUDIONODE, 3, NULL);

/*
	...
 Invalid type
 low memory
*/
}

void TestLookupItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseLookupItem();
	else
	{
		callName = "LookupItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallLookupItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


