
/*
 * @(#) IManagingTests.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		IManagingTests.c

	Contains:	Item Management call tests, kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <2>	 7/13/93	CDE		Derived from shell
		 <1>	 6/22/93	CDE		Version 1.0 of TestShell.c

	To Do:
*/
#define USE_4B1	1



/* HINTS:

	 If you don't know how to do something, try to write the syntax of the routine, and use the word
		UNIMPLEMENTED in the header so you can find it later.
*/

/***** Tool description for Intro, Exit, Help and Session Options displays *****/
#define		kToolDescription		"Item Management Tests"

/***** Version string *****/
#define		kVersionStr		"V1.0"

/***** Programmer name for Help and Session Options displays *****/
#define		kProgrammerName		"Charlie Eckhaus"

/***** Call constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kCreateItem				1
#define		kCreateSizedItem		2
#define		kDeleteItem				3
#define		kOpenItem				4
#define		kCloseItem				5
#define		kSetItemOwner			6
#define		kSetItemPri				7
#define		kMKNODEID				8
#define		kMkNodeID				9
#if USE_4B1
#define		kCreateThread			10
#define		kDeleteThread			11
#endif

/***** Default log filename: *****/
#define		kDefaultLogFile			"IManagingTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/* HINTS:

	 see usage of gIncrement1 in TestCallA()
	 Change the units to be meaningful for the tool, e.g., for a memory tool, kIncrementUnits = "bytes"
	 Change the default value to the minimum number of units for which the test is meaningful
*/
#define		kIncrement1Units	"units"
#define		kDefaultIncrement1	1


/***** Other test options *****/
#define kDefaultItemName "TestName"
#define kDefaultFType TEST_FTYPE
#define kDefaultNType TEST_NTYPE
#define kDefaultVers	0
#define kDefaultRev		0
#define kDefaultSize	0
#define	kDefaultSignalMask	0

/***** Include files *****/

#include "itemUtils.h"
#include "threadUtils.h"
#include "TestCreateItem.h"
#include "TestCreateSizedItem.h"
#include "TestDeleteItem.h"
#include "TestOpenItem.h"
#include "TestCloseItem.h"
#include "TestSetItemOwner.h"
#include "TestSetItemPri.h"
#include "TestMKNODEID.h"
#if 0
#include "TestMkNodeID.h"
#endif
#if USE_4B1
#include "TestCreateThread.h"
#include "TestDeleteThread.h"
#endif

/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "IManagingTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "IManagingTests.c";	/* contains the name of the program */
char	* gTestName= "IManagingTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */


/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
/* HINTS:

	 Add one global for each additional tool option
*/
char	gItemName[64];
uint8	gFType;
uint8	gNType;
uint8	gVers;
uint8	gRev;
int32	gSize;
uint32	gSignalMask;
uint32  gThreadSignal;

/***** Utilities *****/

char *printable(char *aName)
{
	if (aName == NULL) return "<NULL>";
	if (aName == " ") return "<space>";
	if (strlen(aName)) return aName;
	return "<empty>";
}

/**** (End Utilities) ****/


void	ShowHelp(void);
void	ShowHelp()
/* Show Help for this tool */
/* HINTS:

	 Incorporate your additional options, all of the category's call names
*/
{
		printf ( "#################################################################");
		printf ( "%s %s Help", kToolDescription, kVersionStr );
		printf ( "Written by %s for The 3DO Company", kProgrammerName );
		printf (    "#################################################################");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-out << no parameter>>\tDo not use a log file.");
		LogDiagnostic( 80, "     <<filename>>     \tUse specified log file.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80,"-icmin <<value>>\tIterate calls starting at <<value>>.");
		LogDiagnostic( 80,"-icmax <<value>>\tEnd call iterations at <<value>>.");
		LogDiagnostic( 80,"-itmin <<value>>\tIterate tests starting at <<value>>.");
		LogDiagnostic( 80,"-itmax <<value>>\tEnd test iterations at <<value>>.");
		LogDiagnostic( 80, "");

		printf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		printf ( "-ftype <<value>>\tUse <<value>> as folio type."); 
		LogDiagnostic( 80, "       k\tKERNELNODE.");
#if USE_4B1
		LogDiagnostic( 80, "       g\tGRAPHICSFOLIO.");
#else
		LogDiagnostic( 80, "       g\tNODE_GRAPHICS.");
#endif
		LogDiagnostic( 80, "       f\tFILEFOLIO.");
		LogDiagnostic( 80, "       a\tAUDIONODE.");
		LogDiagnostic( 80, "       m\tMATHFOLIO.");
		LogDiagnostic( 80, "");
		
		printf ( "-ntype <<value>>\tUse <<value>> as node type."); 
		LogDiagnostic( 80, "       memfree\tMEMFREENODE.");
		LogDiagnostic( 80, "       list\tLISTNODE.");
		LogDiagnostic( 80, "       memhdr\tMEMHDRNODE.");
		LogDiagnostic( 80, "       folio\tFOLIONODE.");
		LogDiagnostic( 80, "       task\tTASKNODE.");
		LogDiagnostic( 80, "       firq\tFIRQNODE.");
		LogDiagnostic( 80, "       sema4\tSEMA4NODE.");
		LogDiagnostic( 80, "       message\tMESSAGENODE.");
		LogDiagnostic( 80, "       msgport\tMSGPORTNODE.");
		LogDiagnostic( 80, "       memlist\tMEMLISTNODE.");
		LogDiagnostic( 80, "       romtag\tROMTAGNODE.");
		LogDiagnostic( 80, "       driver\tDRIVERNODE.");
		LogDiagnostic( 80, "       ioreq\tIOREQNODE.");
		LogDiagnostic( 80, "       device\tDEVICENODE.");
		LogDiagnostic( 80, "       timer\tTIMERNODE.");
		LogDiagnostic( 80, "       errortext\tERRORTEXTNODE.");
		LogDiagnostic( 80, "       template\tAUDIO_TEMPLATE_NODE.");
		LogDiagnostic( 80, "       instrument\tAUDIO_INSTRUMENT_NODE.");
		LogDiagnostic( 80, "       knob\tAUDIO_KNOB_NODE.");
		LogDiagnostic( 80, "       sample\tAUDIO_SAMPLE_NODE.");
		LogDiagnostic( 80, "       cue\tAUDIO_CUE_NODE.");
		LogDiagnostic( 80, "       envelope\tAUDIO_ENVELOPE_NODE.");
		LogDiagnostic( 80, "       attachment\tAUDIO_ATTACHMENT_NODE.");
#if USE_4B1
		LogDiagnostic( 80, "       tuning\tAUDIO_TUNING_NODE.");
#endif
		LogDiagnostic( 80, "       filesystem\tFILESYSTEMNODE.");
		LogDiagnostic( 80, "       file\tFILENODE.");
#if USE_4B1
		LogDiagnostic( 80, "       filealias\tFILEALIASNODE.");
#endif
		LogDiagnostic( 80, "       test\tTEST_NTYPE.");
		LogDiagnostic( 80, "");
		
		printf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-error none\tDon't check for errors.");
		LogDiagnostic( 80, "       cont\tWhen an error occurs, report it and continue.");
		LogDiagnostic( 80, "       stop\tWhen an error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-ierror none\tDon't check for incidental errors.");
		LogDiagnostic( 80, "        cont\tWhen an incidental error occurs, report it and continue.");
		LogDiagnostic( 80, "        stop\tWhen an incidental error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-time off\tDon't output timing information.");
		LogDiagnostic( 80, "      on \tOutput timing information.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-func off\tDon't verify functionality.");
		LogDiagnostic( 80,"      on \tVerify functionality.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-fault off\tMake known assumptions.");
		LogDiagnostic( 80,"       on \tSkip known assumptions.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-sel <<no parameter>>\tRun the entire test suite.");
		/* HINTS:

			 Replace the following with your category's call names and descriptions
		*/
		LogDiagnostic( 80,"     CreateItem		\tTest CreateItem.");
		LogDiagnostic( 80,"     CreateSizedItem	\tTest CreateSizedItem.");
		LogDiagnostic( 80,"     DeleteItem	    \tTest DeleteItem.");
		LogDiagnostic( 80,"     OpenItem	        \tTest OpenItem.");
		LogDiagnostic( 80,"     CloseItem	        \tTest CloseItem.");
		LogDiagnostic( 80,"     SetItemOwner	    \tTest SetItemOwner.");
		LogDiagnostic( 80,"     SetItemPri	    \tTest SetItemPri.");
		LogDiagnostic( 80,"     MKNODEID			\tTest MKNODEID.");
		LogDiagnostic( 80,"     MkNodeID			\tTest MkNodeID.");
#if USE_4B1
		LogDiagnostic( 80,"     CreateThread      \tTest CreateThread.");
		LogDiagnostic( 80,"     DeleteThread      \tTest DeleteThread.");
#endif
		LogDiagnostic( 80, "");
		
		printf ( "#################################################################");
}

void ShowOptions(void);
void ShowOptions()
/* Show the option settings for this session */
/* HINTS:

	 Incorporate your additional options and the category's calls
*/
{
		char	aString[64];
		
		printf ( "#################################################################");
		printf ( "%s %s - Session options", kToolDescription, kVersionStr);
		printf ( "Written by %s for The 3DO Company", kProgrammerName );
		printf (   "#################################################################");
		
		if (gUseLogFile) LogDiagnostic( 80, "-out: Use log file: %s.",gLogFile); else LogDiagnostic( 80,"Do not use a log file.");
		
		printf ( "Iterate each test from %i %s",gMinOuterIterations, kIncrement1Units);
		printf ( " to %i %s,", gMaxOuterIterations, kIncrement1Units);
		printf ( " in increments of %i %s.",gIncrement1,kIncrement1Units); 
		printf ( "Within each test, iterate the call from %i to %i.",gMinInnerIterations,gMaxInnerIterations);
		
		printf ("-name: Use %s as item name.", gItemName);

		FolioTypeToStr(gFType, aString);
		printf ("-ftype: Use %s as folio type.", aString);

		NodeTypeToStr(gFType, gNType, aString);
		printf ("-ntype: Use %s as node type.", aString);
		
		printf ("-vers: Use %i as version number.", gVers);

		printf ("-rev: Use %s as revision number.", gRev);

		printf ("-size: Use %s as item size.", gSize);

		LogDiagnostic( 80, "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80, "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for incidental errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an incidental error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first incidental error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80,"-time: ");
		if (gOutputTiming) LogDiagnostic( 80,"Output timing information."); else LogDiagnostic( 80,"Don't output timing information.");

		LogDiagnostic( 80,"-func: ");
		if (gVerify) LogDiagnostic( 80,"Verify functionality."); else LogDiagnostic( 80,"Don't verify functionality.");

		LogDiagnostic( 80,"-fault: ");
		if (gCheckFault) LogDiagnostic( 80,"Skip known assumptions."); else LogDiagnostic( 80,"Make known assumptions.");

		LogDiagnostic( 80,"-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				LogDiagnostic( 80,"Run the entire test suite.");
				break;
				
			/* HINTS:
			
				 Customize for your category's calls
			*/
			case kCreateItem:
				LogDiagnostic( 80,"Test CreateItem.");
				break;

			case kCreateSizedItem:
				LogDiagnostic( 80,"Test CreateSizedItem.");
				break;

			case kDeleteItem:
				LogDiagnostic( 80,"Test DeleteItem.");
				break;

			case kOpenItem:
				LogDiagnostic( 80,"Test OpenItem.");
				break;

			case kCloseItem:
				LogDiagnostic( 80,"Test CloseItem.");
				break;

			case kSetItemOwner:
				LogDiagnostic( 80,"Test SetItemOwner.");
				break;

			case kSetItemPri:
				LogDiagnostic( 80,"Test SetItemPri.");
				break;

			case kMKNODEID:
				LogDiagnostic( 80,"Test MKNODEID.");
				break;

			case kMkNodeID:
				LogDiagnostic( 80,"Test MkNodeID.");
				break;

#if USE_4B1
			case kCreateThread:
				LogDiagnostic( 80,"Test CreateThread.");
				break;

			case kDeleteThread:
				LogDiagnostic( 80,"Test DeleteThread.");
				break;

#endif
			} /* end switch */
		
		printf ( "#################################################################");
		
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	TOUCH(theArgCount);
	TOUCH(theArgList);

	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	strcpy(gItemName, kDefaultItemName);
	gFType = kDefaultFType;
	gNType = kDefaultNType;
	gVers = kDefaultVers;
	gRev = kDefaultRev;
	gSize = kDefaultSize;
	gSignalMask = kDefaultSignalMask;

	gVerify = true;
	gCheckFault = true;
	
	
	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory(void);
void TestCategory()
/* HINTS:

	 Customize for your category's calls
*/
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateItem ) )
	{
		gTestAreaName = "CreateItem";		/* contains the name of the test    */
		TestCreateItem(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateSizedItem ) )
	{
		gTestAreaName = "CreateSizedItem";		/* contains the name of the test    */
		TestCreateSizedItem(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kOpenItem ) )
	{
		gTestAreaName = "OpenItem";		/* contains the name of the test    */
		TestOpenItem(gMinOuterIterations, gMaxOuterIterations);
	}

#if 0
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCloseItem ) )
	{
		gTestAreaName = "CloseItem";		/* contains the name of the test    */
		TestCloseItem(gMinOuterIterations, gMaxOuterIterations);
	}

#endif
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kDeleteItem ) )
	{
		gTestAreaName = "DeleteItem";		/* contains the name of the test    */
		TestDeleteItem(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSetItemOwner ) )
	{
		gTestAreaName = "SetItemOwner";		/* contains the name of the test    */
		TestSetItemOwner(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSetItemPri ) )
	{
		gTestAreaName = "SetItemPri";		/* contains the name of the test    */
		TestSetItemPri(gMinOuterIterations, gMaxOuterIterations);
	}

#if USE_4B1
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateThread ) )
	{
		gTestAreaName = "CreateThread";		/* contains the name of the test    */
		TestCreateThread(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kDeleteThread ) )
	{
		gTestAreaName = "DeleteThread";		/* contains the name of the test    */
		TestDeleteThread(gMinOuterIterations, gMaxOuterIterations);
	}

#endif

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kMKNODEID ) )
	{
		gTestAreaName = "MKNODEID";		/* contains the name of the test    */
		TestMKNODEID(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kMkNodeID ) )
	{
		gTestAreaName = "MkNodeID";		/* contains the name of the test    */
		TestMkNodeID(gMinOuterIterations, gMaxOuterIterations);
	}



}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram(void);
Boolean	InitProgram()
{
#if 0
	int32 theError;
#endif
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogComment("Can't open the audio folio!");
		return FALSE;
	}
#endif	
#if 0
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
#endif
	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
/*	gTestName= "123456790123456790123456790123456790123456790123456790123456790123456790123456790123456790123456790123456790123456790";				*/ /* contains the name of the test    */
	gTestName = (char *)malloc( sizeof(char)*1024 );

	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
			
					
		}

	}
	TermScaffold();

	return(0);
 }






