
/*
 * @(#) TestCreateItem.c 96/10/24 1.5
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCreateItem.c

	Contains:	Test routine for CreateItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestCreateItem.h"

/*
 * Uncomment the next line to enable kMyDebug.  Implemented this way avoids warnings
 */
/*#define	KMYDEBUG	1 */
#define kMyDebug 0

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
/* HINTS:

	 May change drastically depending on the call.
*/
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
/* HINTS:

	 May change drastically depending on the call.
	 May need several different versions of these.
*/
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define TAGARG_STR(i)	((char *) tags[i].ta_Arg)
#define TAGARG_UINT8(i)	((uint8) tags[i].ta_Arg)

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCreateItemVerifyStr		"Before iteration (%i)/\ttype/ $%x,\ttags/$%x,\tname/ %s,\tVers.Rev/ %i.%i"
#define		kAfterCreateItemVerifyStr		"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr "!! Actual %s/ %s"
#define		kNumMismatchStr "!! Actual %s/ %i"

Boolean		VerifyCreateItem ( Boolean beforeCall, long aIndx, Item aResult, int32 itemType, TagArg *tags )
/* Functionality verification for CreateItem */
{
	ItemNodeP aItemNode;
	Boolean mismatch;
	Boolean anyMismatch;
	static char *name;
	static long version;
	static long revision;
	
	if (beforeCall == kBeforeCall)
	{		
		name = (char *) (tags[0].ta_Arg);
		version = (long) (tags[1].ta_Arg);
		revision = (long) (tags[2].ta_Arg);
		LogDiagnostic( 12,kBeforeCreateItemVerifyStr, aIndx, itemType, tags, name, version, revision);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCreateItemVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return ( false );

		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
		if ( ( aItemNode->n_Name == NULL ) && ( name != NULL ) )
			anyMismatch = TRUE;
		else if ( ( aItemNode->n_Name == NULL ) && ( name == NULL ) )
			anyMismatch = FALSE;
		else
		{
			anyMismatch = ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0) );
			if (mismatch) LogComment(kStrMismatchStr, "name", aItemNode->n_Name);
		}
		
		anyMismatch |= ( mismatch = ( version != aItemNode->n_Version ) );
		if (mismatch) LogComment(kNumMismatchStr, "version", aItemNode->n_Version);
		
		anyMismatch |= ( mismatch = ( revision != aItemNode->n_Revision ) );
		if ( mismatch ) LogComment(kNumMismatchStr, "revision", aItemNode->n_Revision);
		
		return ( !anyMismatch );
	}
	
}

#define		CLEANUP

long	CallCreateItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	int32	aType;
	char	*aName = gItemName;
	uint8	aVers = gVers;
	uint8	aRev  = gRev;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	
	TOUCH(currentOuterIteration);

	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	/***** Preparation for call iterations *****/
	aType = MKNODEID(aFType,aNType);
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCreateItem ( kBeforeCall, minInnerIterations, aResult, aType, tags );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = CreateItem( aType, tags );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCreateItem ( kAfterCall, indx, aResult, aType, tags );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneCreateItem ( Boolean expectError, uint8 aFType, uint8 aNType, char *aName, uint8 aVers, uint8 aRev )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	int32	aType;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	
 	gCurrentIndex++;
	
	/***** Call preparation *****/
	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aType = MKNODEID(aFType,aNType);
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCreateItem ( kBeforeCall, gCurrentIndex, aResult, aType, tags );

#ifdef KMYDEBUG
	if (kMyDebug) LogDiagnostic( 80,"CreateItem: Folio = %d; Num = %d; name \"%s\";",aFType, aNType, aName );
	if (kMyDebug) LogDiagnostic( 80,"vers %d; rev %d", aVers, aRev);
#endif
	aResult = CreateItem( aType, tags );
#ifdef KMYDEBUG
	if (kMyDebug) LogDiagnostic( 80,"Done CreateItem returns 0x%x", aResult);
#endif
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCreateItem ( kAfterCall, gCurrentIndex, aResult, aType, tags );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#if 0
#define	TEST_ONE_CREATEITEM(expVal, fTyp, nTyp, name, vers, rev)	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev=%d", gTestAreaName,fTyp, nTyp, name, vers, rev );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCreateItem(expVal, fTyp, nTyp, name, vers, rev); \
														

#else
void TEST_ONE_CREATEITEM(int32 expVal, int32 fTyp, int32 nTyp, char *name, int32 vers, int32 rev);
void TEST_ONE_CREATEITEM(int32 expVal, int32 fTyp, int32 nTyp, char *name, int32 vers, int32 rev)
{
    sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev=%d",
		gTestAreaName, fTyp, nTyp, name, vers, rev );
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneCreateItem(expVal, fTyp, nTyp, name, vers, rev);
}
														
#endif



void ExerciseCreateItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CreateItem";
	STATUS(callName);

	
	TEST_ONE_CREATEITEM(kExpectError, KERNELNODE, FOLIONODE, "FOLIONODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectSuccess, KERNELNODE, SEMA4NODE, "SEMA4NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectSuccess, KERNELNODE, MESSAGENODE, "MESSAGENODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectSuccess, KERNELNODE, MSGPORTNODE, "MSGPORTNODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, KERNELNODE, IOREQNODE, "IOREQNODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, KERNELNODE, ERRORTEXTNODE, "ERRORTEXTNODE_Item", gVers, gRev);


	TEST_ONE_CREATEITEM(kExpectError, FILENODE, FILENODE, "FILENODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, FILENODE, FILESYSTEMNODE, "FILESYSTEMNODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, FILENODE, FILEALIASNODE, "FILEALIASNODE_Item", gVers, gRev);

/*	TEST_ONE_CREATEITEM(kExpectSuccess, GRAPHICSNODE, SCREENGROUPNODE, "SCREENGROUPNODE_Item", gVers, gRev); */
/*	TEST_ONE_CREATEITEM(kExpectSuccess, GRAPHICSNODE, VDLNODE, "VDLNODE_Item", gVers, gRev); */

	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_TEMPLATE_NODE, "AUDIO_TEMPLATE_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_INSTRUMENT_NODE, "AUDIO_INSTRUMENT_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_KNOB_NODE, "AUDIO_KNOB_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_SAMPLE_NODE, "AUDIO_SAMPLE_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_CUE_NODE, "AUDIO_CUE_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_ENVELOPE_NODE, "AUDIO_ENVELOPE_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_ATTACHMENT_NODE, "AUDIO_ATTACHMENT_NODE_Item", gVers, gRev);
	TEST_ONE_CREATEITEM(kExpectError, AUDIONODE, AUDIO_TUNING_NODE, "AUDIO_TUNING_NODE_Item", gVers, gRev);

	/* params: ftype, ntype, name, version, revision */
/*	TEST_ONE_CREATEITEM(kExpectError, gFType, gNType, gItemName, gVers, gRev); */
/*	TEST_ONE_CREATEITEM(kExpectSuccess, gFType, gNType, gItemName, gVers, gRev); */
/*
 Invalid type
 low memory
*/
}

void TestCreateItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCreateItem();
	else
	{
		callName = "CreateItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCreateItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

