
/*
 * @(#) TestDeleteItem.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestDeleteItem.c

	Contains:	Test routine for DeleteItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestDeleteItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr		"Error initializing for %s"
#define		kIncCreateItemErrStr	"!! Unable to create item/ ftype/ %i, ntype/ %i, name %s"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define TAGARG_STR(i)	((char *) tags[i].ta_Arg)
#define TAGARG_UINT8(i)	((uint8) tags[i].ta_Arg)

/***** Test routines *****/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeDeleteItemVerifyStr	"Before iteration (%i)/\titem/ $%x"
#define		kAfterDeleteItemVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyDeleteItem ( Boolean beforeCall, long aIndx, Err aResult, Item aItem )
/* Functionality verification for DeleteItem */
{
	ItemNodeP	aItemNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeDeleteItemVerifyStr, aIndx, aItem);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterDeleteItemVerifyStr, aIndx, aResult);
		if ( aResult < 0 )
			return false;

		aItemNode = (ItemNodeP) LookupItem( aItem );
		
		return ( aItemNode == NULL );
	}
	
}

#define		CLEANUP

long	CallDeleteItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aItem;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	char	*aName = gItemName;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_END,  0
	};
	Err		aResult = 0;

	TOUCH(currentOuterIteration);

	/***** Preparation for call iterations *****/
	tags[0].ta_Arg = (void *) aName;
	
	aItem = CreateItem( MKNODEID(aFType,aNType), tags );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem == 0)
		{
			LogComment(kIncCreateItemErrStr, aFType, aNType, aName);
			LogError( aItem );
			LogResult( UNRESOLVED, "CreateItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyDeleteItem ( kBeforeCall, minInnerIterations, aResult, aItem );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = DeleteItem( aItem );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyDeleteItem ( kAfterCall, indx, aResult, aItem );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneDeleteItem ( Boolean expectError, uint8 aFType, uint8 aNType, char *aName )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Err		aResult = 0;

	/* Ancillary variables */
	Item	aItem;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_END,  0
	};
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	tags[0].ta_Arg = (void *) aName;
	
	aItem = CreateItem( MKNODEID(aFType,aNType), tags );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem == 0)
		{
			LogComment(kIncCreateItemErrStr, aFType, aNType, aName);
			LogError( aItem );
			LogResult( UNRESOLVED, "CreateItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyDeleteItem ( kBeforeCall, gCurrentIndex, aResult, aItem );

	aResult = DeleteItem( aItem );

	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyDeleteItem ( kAfterCall, gCurrentIndex, aResult, aItem );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#if 0
#define	TEST_ONE_DELETEITEM(expVal, fTyp, nTyp, name)	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'", gTestAreaName,fTyp, nTyp, name );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneDeleteItem(expVal, fTyp, nTyp, name);
														

#else
void TEST_ONE_DELETEITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name);
void TEST_ONE_DELETEITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name)
{
    sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'", gTestAreaName, fTyp, nTyp, name );
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneDeleteItem(expVal, fTyp, nTyp, name);
}
#endif


void ExerciseDeleteItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "DeleteItem";
	STATUS(callName);
	
	/* params: ftype, ntype, name */
	
	TEST_ONE_DELETEITEM(kExpectSuccess, 1, 10, "MSGPORTNODE_Item");
/*	TEST_ONE_DELETEITEM(kExpectError, gFType, gNType, gItemName);
	TEST_ONE_DELETEITEM(kExpectSuccess, gFType, gNType, gItemName);
*/
/*
 Invalid type
 low memory
*/
}

void TestDeleteItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseDeleteItem();
	else
	{
		callName = "DeleteItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallDeleteItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

