
/*
 * @(#) TestMKNODEID.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestMKNODEID.c

	Contains:	Test routine for MKNODEID() and MkNodeID().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestMKNODEID.h"

long	OneMkNodeID (  Boolean expectError, uint8 aFType, uint8 aNType );

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncLookupErrorStr	"Error looking up item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines for MKNODEID *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeMKNODEIDVerifyStr	"Before iteration (%i)/\ta/ $%x,\tb/ $%x"
#define		kAfterMKNODEIDVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyMKNODEID ( Boolean beforeCall, long aIndx, int32 aResult, int32 a, int32 b )
/* Functionality verification for MKNODEID */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeMKNODEIDVerifyStr, aIndx, a, b);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterMKNODEIDVerifyStr, aIndx, aResult);
		
		return ( aResult == ((int32) ( ((a)<<8) | (b) )) );
	}
	
}

#define		CLEANUP

long CallMKNODEID (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	int32	aResult = 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;

	TOUCH(currentOuterIteration);

	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyMKNODEID ( kBeforeCall, minInnerIterations, aResult, aFType, aNType );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = MKNODEID( aFType, aNType );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyMKNODEID ( kAfterCall, indx, aResult, aFType, aNType );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneMKNODEID ( Boolean expectError, uint8 aFType, uint8 aNType )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	int32	aResult = 0;

  	gCurrentIndex++;
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyMKNODEID ( kBeforeCall, gCurrentIndex, aResult, aFType, aNType );

	aResult = MKNODEID( aFType, aNType );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyMKNODEID ( kAfterCall, gCurrentIndex, aResult, aFType, aNType );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#if 0
#define	EXERCISE_ONE_NTYPE(OneProc, expVal, fTyp, nTyp)	sprintf( gTestName, "%s | f= %d; n= %d", gTestAreaName,fTyp, nTyp);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneProc(expVal, fTyp, nTyp );
														


#define EXERCISE_NTYPE( expect, OneProc, ftype ) gCurrentIndex = 0;\
												EXERCISE_ONE_NTYPE(OneProc, expect, ftype, 0);\
												EXERCISE_ONE_NTYPE(OneProc, expect, ftype, 1);\
												EXERCISE_ONE_NTYPE(OneProc, expect, ftype, 0x000000fe);\
												EXERCISE_ONE_NTYPE(OneProc, expect,	ftype, 0x000000ff);


#else
void EXERCISE_ONE_NTYPE( Boolean expVal, uint8 fTyp, uint8 nTyp);
void EXERCISE_ONE_NTYPE( Boolean expVal, uint8 fTyp, uint8 nTyp)
{
    sprintf( gTestName, "%s | f= %d; n= %d", gTestAreaName, fTyp, nTyp);
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneMKNODEID(expVal, fTyp, nTyp );
}


void EXERCISE_NTYPE( Boolean expect, uint8 ftype );
void EXERCISE_NTYPE( Boolean expect, uint8 ftype )
{
    gCurrentIndex = 0;
    EXERCISE_ONE_NTYPE(expect, ftype, 0);
    EXERCISE_ONE_NTYPE(expect, ftype, 1);
    EXERCISE_ONE_NTYPE(expect, ftype, 0x000000fe);
    EXERCISE_ONE_NTYPE(expect, ftype, 0x000000ff);
}


void Exercise_One_Ntype( Boolean expVal, uint8 fTyp, uint8 nTyp);
void Exercise_One_Ntype( Boolean expVal, uint8 fTyp, uint8 nTyp)
{
    sprintf( gTestName, "%s | f= %d; n= %d", gTestAreaName, fTyp, nTyp);
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneMkNodeID(expVal, fTyp, nTyp );
}


void Exercise_Ntype( Boolean expect, uint8 ftype );
void Exercise_Ntype( Boolean expect, uint8 ftype )
{
    gCurrentIndex = 0;
    Exercise_One_Ntype(expect, ftype, 0);
    Exercise_One_Ntype(expect, ftype, 1);
    Exercise_One_Ntype(expect, ftype, 0x000000fe);
    Exercise_One_Ntype(expect, ftype, 0x000000ff);
}
#endif


void ExerciseMKNODEID( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "MKNODEID";
	STATUS(callName);
	
	/* params: ftype, ntype */
	
	EXERCISE_NTYPE(kExpectSuccess, 0);
	EXERCISE_NTYPE(kExpectSuccess, 1);
	EXERCISE_NTYPE(kExpectSuccess, 17);
	EXERCISE_NTYPE(kExpectSuccess, 0x00ff);
	EXERCISE_NTYPE(kExpectSuccess, 0x0100);
/*
	...
 Invalid type
 low memory
*/
}

void TestMKNODEID( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseMKNODEID();
	else
	{
		callName = "MKNODEID";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallMKNODEID(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/***** Test routines for MkNodeID *****/
/* This seems silly since MkNodeID is a convenience macro defined identically to MKNODEID,
	but there may be a mistake in the headers someday */
#define		kBeforeMkNodeIDVerifyStr	kBeforeMKNODEIDVerifyStr
#define		kAfterMkNodeIDVerifyStr		kAfterMKNODEIDVerifyStr

Boolean		VerifyMkNodeID ( Boolean beforeCall, long aIndx, int32 aResult, int32 a, int32 b )
/* Functionality verification for MkNodeID */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeMkNodeIDVerifyStr, aIndx, a, b);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterMkNodeIDVerifyStr, aIndx, aResult);
		
		return ( aResult == ((int32) ( ((a)<<8) | (b) )) );
	}
	
}

long	CallMkNodeID (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	int32	aResult = 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;

	TOUCH(currentOuterIteration);

	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyMkNodeID ( kBeforeCall, minInnerIterations, aResult, aFType, aNType );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = MkNodeID( aFType, aNType );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyMkNodeID ( kAfterCall, indx, aResult, aFType, aNType );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneMkNodeID (  Boolean expectError, uint8 aFType, uint8 aNType )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	int32	aResult = 0;

  	gCurrentIndex++;
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyMkNodeID ( kBeforeCall, gCurrentIndex, aResult, aFType, aNType );

	aResult = MkNodeID( aFType, aNType );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyMkNodeID ( kAfterCall, gCurrentIndex, aResult, aFType, aNType );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

void ExerciseMkNodeID( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "MkNodeID";
	STATUS(callName);
	
	/* params: ftype, ntype */
	
	Exercise_Ntype(kExpectSuccess, 0);
	Exercise_Ntype(kExpectSuccess, 1);
	Exercise_Ntype(kExpectSuccess, 17);
	Exercise_Ntype(kExpectSuccess, 0x00ff);
	Exercise_Ntype(kExpectSuccess, 0x0100);
/*
	...
 Invalid type
 low memory
*/
}

void TestMkNodeID( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseMkNodeID();
	else
	{
		callName = "MkNodeID";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallMkNodeID(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

