
/*
 * @(#) TestOpenItem.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestOpenItem.c

	Contains:	Test routine for OpenItem().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/13/93	CDE		Derived from shell V1.6

	To Do:
*/


#include "TestOpenItem.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
/* HINTS:

	 May change drastically depending on the call.
*/
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
/* HINTS:

	 May change drastically depending on the call.
	 May need several different versions of these.
*/
#define		kIncidentalErrorStr		"Error initializing for %s"
#define		kIncFindNamedErrorStr	"!! Unable to find named item %s/ $%x"
#define		kIncLookupErrorStr		"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define TAGARG_STR(i)	((char *) tags[i].ta_Arg)
#define TAGARG_UINT8(i)	((uint8) tags[i].ta_Arg)

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeOpenItemVerifyStr	"Before iteration (%i)/\titem/ $%x,\ttags/ $%x"
#define		kAfterOpenItemVerifyStr		"After iteration (%i)//tresult/ $%x"
#define		kStrMismatchStr				"!! Actual %s/ %s"
#define		kNumMismatchStr				"!! Actual %s/ %i"

Boolean		VerifyOpenItem ( Boolean beforeCall, long aIndx, Item aResult, Item aItem, TagArg *tags )
/* Functionality verification for OpenItem */
{
	ItemNodeP aItemNode;
/*
	Boolean mismatch = FALSE;
	Boolean anyMismatch = FALSE;
	static char *name;
	static long version;
	static long revision;
*/
	
	if (beforeCall == kBeforeCall)
	{		
/*
		name = (char *) (tags[0].ta_Arg);
		version = (long) (tags[1].ta_Arg);
		revision = (long) (tags[2].ta_Arg);
*/
		LogDiagnostic( 12,kBeforeOpenItemVerifyStr, aIndx, aItem, tags);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterOpenItemVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return ( false );

		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
/* NEED TO REPLACE THIS WITH TESTS OF THE TAGARGS VS. THE ITEM'S FIELDS
		anyMismatch = ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0) );
		if (mismatch) LogDiagnostic( 12,kStrMismatchStr, "name", aItemNode->n_Name);
		
		anyMismatch |= ( mismatch = ( version != aItemNode->n_Version ) );
		if (mismatch) LogDiagnostic( 12,kNumMismatchStr, "version", aItemNode->n_Version);
		
		anyMismatch |= ( mismatch = ( revision != aItemNode->n_Revision ) );
		if ( mismatch ) LogDiagnostic( 12,kNumMismatchStr, "revision", aItemNode->n_Revision);

		return ( !anyMismatch );
*/
		return ( true );
	}
	
}

#define		CLEANUP

long	CallOpenItem (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	char	*aName = gItemName;
	uint8	aVers = gVers;
	uint8	aRev  = gRev;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	Item	aItem;

	TOUCH(currentOuterIteration);

	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aItem = FindNamedItem(MKNODEID(aFType,aNType),aName);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncFindNamedErrorStr, aName, aItem);
			LogError( aItem );
			LogResult( UNRESOLVED, "FindNamedItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyOpenItem ( kBeforeCall, minInnerIterations, aResult, aItem, tags );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0;
		
		aResult = OpenItem( MKNODEID(aFType,aNType), tags );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyOpenItem ( kAfterCall, indx, aResult, aItem, tags );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long	OneOpenItem (  Boolean expectError, uint8 aFType, uint8 aNType, char *aName, uint8 aVers, uint8 aRev )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	Item	aItem;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aItem = FindNamedItem(MKNODEID(aFType,aNType),aName);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncFindNamedErrorStr, aName, aItem);
			LogError( aItem );
			LogResult( UNRESOLVED, "FindNamedItem returned an error");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyOpenItem ( kBeforeCall, gCurrentIndex, aResult, aItem, tags );

	aResult = OpenItem( MKNODEID(aFType,aNType), tags );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyOpenItem ( kAfterCall, gCurrentIndex, aResult, aItem, tags );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#if 0
#define	TEST_ONE_OPENITEM(expVal, fTyp, nTyp, name, vers, rev)	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev=%d", gTestAreaName,fTyp, nTyp, name, vers, rev );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneOpenItem(expVal, fTyp, nTyp, name, vers, rev);
														
#else
void TEST_ONE_OPENITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name, uint8 vers, uint8 rev);
void TEST_ONE_OPENITEM( Boolean expVal, uint8 fTyp, uint8 nTyp, char *name, uint8 vers, uint8 rev)
{
    sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev=%d",
		gTestAreaName, fTyp, nTyp, name, vers, rev );
    if (expVal == kExpectError)
	strcat(gTestName, kExpectErrorStr);
    BeginTest( BAT, gTestName );
    OneOpenItem(expVal, fTyp, nTyp, name, vers, rev);
}
#endif


void ExerciseOpenItem( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "OpenItem";
	STATUS(callName);
	
	/* params: ftype, ntype, name, version, revision */
	
	TEST_ONE_OPENITEM(kExpectError, 1, 10, "MSGPORTNODE_Item", gVers, gRev);
/*
	...
 Invalid type
 low memory
*/
}

void TestOpenItem( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseOpenItem();
	else
	{
		callName = "OpenItem";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallOpenItem(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

