
/*
 * @(#) TestSetItemOwner.c 96/10/24 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestSetItemOwner.c

	Contains:	Test routine for SetItemOwner().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/16/93	CDE		Derived from TestCloseItem

	To Do:
*/


#include "TestSetItemOwner.h"
#include <loader/loader3do.h>
#include <kernel/task.h>

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr			"Error initializing for %s"
#define		kIncLookupErrorStr			"!! Unable to lookup item $%x"
#define		kIncCreateItemErrorStr		"!! Unable to create item/ $%x"
#define		kIncLoadProgramErrorStr		"!! Unable to load %s/ $%x"
#define		kIncDeleteItemErrorStr		"!! Unable to delete child/ $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

#define TAGARG_STR(i)	((char *) tags[i].ta_Arg)
#define TAGARG_UINT8(i)	((uint8) tags[i].ta_Arg)

ItemNodeP	VerifyItemForSetOwner( Item aItem )
{
	ItemNodeP aItemNode;

	aItemNode = (ItemNodeP) LookupItem( aItem );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItemNode == NULL)
		{
			LogComment(kIncLookupErrorStr, aItem);
			if (gCheckIncidentalErrors == kStop) return NULL;
		}
	}
	
	return aItemNode;
}

Item LoadProgram (char *path)
{
	Item	result;
	Item	mItem;
	
	mItem = OpenModule( path, OPENMODULE_FOR_TASK, NULL);
	if (mItem < 0)
		return mItem;
		
	result = CreateTask( mItem, path, NULL );

	return result;
}


/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeSetItemOwnerVerifyStr	"Before iteration (%i)/\titem/ $%x,\towner/ $%x"
#define		kAfterSetItemOwnerVerifyStr		"After iteration (%i)//tresult/ $%x"

Boolean		VerifySetItemOwner ( Boolean beforeCall, long aIndx, Err aResult, Item aItem, Item owner )
/* Functionality verification for SetItemOwner */
{
	Boolean succeeded;
	ItemNodeP	aItemNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeSetItemOwnerVerifyStr, aIndx, aItem, owner);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterSetItemOwnerVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;
		
		aItemNode = VerifyItemForSetOwner( aItem );			
		if (aItemNode == NULL)
			return false;
		
		succeeded = (aItemNode->n_Owner == owner);
		if (!succeeded)
			LogComment("!! Actual owner: $%x", aItemNode->n_Owner);
			
		return ( succeeded );
	}
	
}

#define		CLEANUP		KillChild(aChildItem)

long	CallSetItemOwner (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	uint8	aFType = gFType;
	uint8	aNType = gNType;
	char	*aName = gItemName;
	uint8	aVers = gVers;
	uint8	aRev  = gRev;
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	Item	aItem;
	Item aChildItem;

	TOUCH(currentOuterIteration);

	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aChildItem = LoadProgram( "/remote/sleeptask" );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aChildItem < 0)
		{
			LogComment(kIncLoadProgramErrorStr, "/remote/sleeptask", aChildItem);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aItem = CreateItem(MKNODEID(aFType,aNType),tags);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncCreateItemErrorStr, aItem);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}

		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifySetItemOwner ( kBeforeCall, minInnerIterations, aResult, aItem, aChildItem );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0;
		
		aResult = SetItemOwner( aItem, aChildItem );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifySetItemOwner ( kAfterCall, indx, aResult, aItem, aChildItem );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (!CLEANUP) return kBreakTime;
	
	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneSetItemOwner ( Boolean expectError, uint8 aFType, uint8 aNType, char *aName, uint8 aVers, uint8 aRev )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	TagArg tags[] =
	{
		TAG_ITEM_NAME,  (void *) 0,
		TAG_ITEM_VERSION, (void *) 0,
		TAG_ITEM_REVISION, (void *) 0,
		TAG_END,  0
	};
	Item	aItem;
	Item aChildItem;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	tags[0].ta_Arg = (void *) aName;
	tags[1].ta_Arg = (void *) aVers;
	tags[2].ta_Arg = (void *) aRev;
	
	aChildItem = LoadProgram( "/remote/sleeptask" );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aChildItem < 0)
		{
			LogComment(kIncLoadProgramErrorStr, "/remote/sleeptask", aChildItem);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aItem = CreateItem(MKNODEID(aFType,aNType),tags);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aItem < 0)
		{
			LogComment(kIncCreateItemErrorStr, aItem);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}

		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifySetItemOwner ( kBeforeCall, gCurrentIndex, aResult, aItem, aChildItem );

	aResult = SetItemOwner( aItem, aChildItem );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifySetItemOwner ( kAfterCall, gCurrentIndex, aResult, aItem, aChildItem );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_SETITEMOWNER(expVal, fTyp, nTyp, name, vers, rev)	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev=%d", gTestAreaName,fTyp, nTyp, name, vers, rev );	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneSetItemOwner(expVal, fTyp, nTyp, name, vers, rev);
														


void ExerciseSetItemOwner( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "SetItemOwner";
	STATUS(callName);
	
	/* params: ftype, ntype, name, version, revision */
	
#if 0
	TEST_ONE_SETITEMOWNER(kExpectSuccess, KERNELNODE, SEMA4NODE, gItemName, gVers, gRev);
#else
	sprintf( gTestName, "%s | f= %d; n= %d; name= '%s'; vers= %d; rev=%d",
		gTestAreaName, KERNELNODE, SEMA4NODE, gItemName, gVers, gRev );
	BeginTest( BAT, gTestName );
	OneSetItemOwner(kExpectSuccess, KERNELNODE, SEMA4NODE, gItemName, gVers, gRev);
														
#endif
/*
	...
 Invalid type
 low memory
*/
}

void TestSetItemOwner( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseSetItemOwner();
	else
	{
		callName = "SetItemOwner";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallSetItemOwner(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

