
#ifndef _BASEUTILS_H
#define _BASEUTILS_H

/*
 * @(#) BaseUtils.h 96/10/21 1.2
 */

/*
	File:		baseUtils.h

	Contains:	Base utilities for folio call tests (Header)

	Written by:	Robert Chinn, Charlie Eckhaus, Eric Hegstrom

	Copyright:	 1993-1996 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <9>	 3/28/96	ELH		Retrofit for 3dfo_scaf
		 <8>	 5/22/95	ELH		Retrofit for M2 diab
		 <7>	 8/3/93		CDE		Added kDefaultDuration
		 <6>	 7/22/93	CDE		Added _BASEUTILS_H
		 <5>	 7/10/93	CDE		Deleted USE_STRCASECMP in favor of
		 							makefile Define
		 <4>	 6/30/93	CDE		Added USE_STRCASECMP, changed #ifdef
		 <3>	 6/29/93	CDE		Added define strcasecmp for 4B1
		 <2>	 6/29/93	CDE		General fixup per Robert
		 							Removed DBUG, FULLDBUG macros
		 <1>	 6/22/93	CDE		Prepared for public consumption

	To Do:
*/

#include <tests/3do_scaf.h>
#include <stdio.h>

#include <ctype.h>
#include <kernel/types.h>
#include <kernel/item.h>
#include <kernel/list.h>
#include <kernel/task.h>
#include <kernel/kernelnodes.h>
#include <kernel/nodes.h>
/*#include <:misc:operamath.h>*/
#include <kernel/operror.h>
#include <kernel/mem.h>
#include <varargs.h>
#include <graphics/graphics.h>
#include <file/filesystem.h>

/* added 11.30.94 ELH so we log all errors */
#define PrintfSysErr(x)	LogError(x)

/* added 10.24.94 ELH because these disappeared from task.h when babs came around */
#define ITEM_WAS_OPENED	0x4000
#define ITEM_NOT_AN_ITEM	0x80000000

#define HIWRD(aLong)	(((aLong) >> 16) & 0xFFFF)
#define LOWRD(aLong)	((aLong) & 0xFFFF)

#define	kTickStr		" -> for %i ticks"

#define	kPrintTickStr	" -> for %i Audio ticks (240/second)"

#define	kPlusChar		"+"
#define	kMinusChar		"-"

/* switch constants */
#define		kOff	"off"
#define		kOn		"on"

/* Just showing help or running the test */
#define		kAbortTest	FALSE
#define		kShowHelp	FALSE
#define		kRunTest	TRUE

/* return this time value to break out of iterations */
#define		kBreakTime	-1L

/* functionality verification control */
#define		kBeforeCall	TRUE
#define		kAfterCall	FALSE

/* Error-checking constants */
#define		kNone	0
#define		kCont	1
#define		kStop	2

/* Functionality verification constants (kNone is also valid) */
#define		kNoVerify	0
#define		kSilent		1
#define		kReport		2

/* Call selection constants */
#define		kFullTest	0

/* Default values for standard tool options: */
#define		kDefaultMinInnerIterations		1
#define		kDefaultMaxInnerIterations		15
#define		kDefaultMinOuterIterations		1
#define		kDefaultMaxOuterIterations		1
#define		kDefaultDuration				120
#define		kDefaultAutomatic				TRUE
#define		kDefaultCheckErrors				kCont
#define		kDefaultCheckIncidentalErrors	kStop
#define		kDefaultVerify					kNone
#define		kDefaultOutputTiming			FALSE
#define		kDefaultCheckFault				FALSE
#define		kDefaultCallSelector			kFullTest


/* Report formats for unexpected behavior in parameter-testing */
#define		kUnexpectedErrorStr	"UNEXPECTED error on iteration %i"
#define		kUnexpectedSuccessStr "UNEXPECTED success on iteration %i"

/* this is appended to output on tests that expect an error from the call */
#define		kExpectErrorStr "- exp err"

/* Constants for parameter-testing */
#define		kExpectError	TRUE
#define		kExpectSuccess	FALSE


#define	STATUS(x)  LogDiagnostic( 72, "## %s",x );


#endif

