
/*
 * @(#) TestCreateIOReq.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCreateIOReq.c

	Contains:	Test routine for CreateIOReq().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/28/93	CDE		Derived from TestCreateItem

	To Do:
*/


#include "TestCreateIOReq.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCreateIOReqVerifyStr		"Before iteration (%i)/\tname/ %s,\tpri/ %i,\tdevice/ $%x,\tport/ $%x"
#define		kAfterCreateIOReqVerifyStr		"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr		"!! Actual %s/ %s"
#define		kNumMismatchStr		"!! Actual %s/ %i"

Boolean		VerifyCreateIOReq ( Boolean beforeCall, long aIndx, Item aResult, char *name, int32 pri, Item dev, Item mp )
/* Functionality verification for CreateIOReq */
{
	ItemNodeP aItemNode;
	Boolean mismatch = FALSE;
	Boolean anyMismatch = FALSE;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeCreateIOReqVerifyStr, aIndx, name, pri, dev, mp);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCreateIOReqVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return ( false );

		InspectIOReq(aResult);
		
		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				/* Already done in InspectIOReq (do this a better way!) */
				/* LogDiagnostic( 12,kIncLookupErrorStr, aResult); */
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
		anyMismatch = ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0) );
		if (mismatch) LogComment(kStrMismatchStr, "name", aItemNode->n_Name);
		
		anyMismatch |= ( mismatch = ( pri != aItemNode->n_Priority ) );
		if (mismatch) LogComment(kNumMismatchStr, "priority", aItemNode->n_Priority);
		
/* UNIMPLEMENTED: WANT TO CHECK DEVICE AND MESSAGEPORT */
		return ( !anyMismatch );
	}
	
}

long	CallCreateIOReq (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	char	*aName = gIOReqName;
	int32	aPriority = gPriority;
	long	aDevSpecifier = gDevSpecifier;
	Item	aDevice = (Item) 0;
	Item	aMessagePort = (Item) 0;
	
	/***** Functionality verification (pre-call) *****/
	aDevice = CreateTestDevice( aDevSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, aDevSpecifier);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}

	aMessagePort = CreateMessagePort( "TestMessagePort" );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	if (gVerify) VerifyCreateIOReq ( kBeforeCall, minInnerIterations, aResult, aName, aPriority, aDevice, aMessagePort );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = CreateIOReq( aName, aPriority, aDevice, aMessagePort  );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCreateIOReq ( kAfterCall, indx, aResult, aName, aPriority, aDevice, aMessagePort );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

#define CLEANUP
long	OneCreateIOReq (Boolean expectError, char *aName, int32 aPriority, long	aDevSpecifier )
/* Inner test */
{
	Boolean theError= FALSE;
	
  	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	Item	aDevice = (Item) 0;
	Item	aMessagePort = (Item) 0;
	
	gCurrentIndex++;

	/***** Call preparation *****/
	aDevice = CreateTestDevice( aDevSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, aDevSpecifier);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aMessagePort = CreateMessagePort( "TestMessagePort" );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCreateIOReq ( kBeforeCall, gCurrentIndex, aResult, aName, aPriority, aDevice, aMessagePort );

	aResult = CreateIOReq( aName, aPriority, aDevice, aMessagePort  );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCreateIOReq ( kAfterCall, gCurrentIndex, aResult, aName, aPriority, aDevice, aMessagePort );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogComment(kCallErrorStr, gCurrentIndex);
			else
			{
				LogComment(kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogComment(kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_CREATEIOREQ(expVal, name, pri, dev)	sprintf( gTestName, "%s - %s | pri= %d; dev= %d", gTestAreaName, name, pri, dev);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCreateIOReq(expVal, name, pri, dev); \
														


void ExerciseCreateIOReq( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CreateIOReq";
	STATUS(callName);
	
	/* params: name, priority, devSpecifier*/
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, " ", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "W", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "w", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "5", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "This is quite an unreasonably length name", CURRENT_TASK_PRIORITY, gDevSpecifier);

	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", 0, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", TASK_MIN_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", TASK_MAX_PRIORITY, gDevSpecifier);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", 255, gDevSpecifier);

	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, kCDROM);
	TEST_ONE_CREATEIOREQ(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, kTimer);

/*
 Name: empty, short, lengthy, special characters
 Pri: 0, minimum, current task, maximum, 255
 DevSpecifier: kCDROM, kTimer, kMac, kSPORT, <= 0, non-device
 IOReq population: none, 1, many
 low memory
*/
}

void TestCreateIOReq( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCreateIOReq();
	else
	{
		callName = "CreateIOReq";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCreateIOReq(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


