
/*
 * @(#) TestDeleteIOReq.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestDeleteIOReq.c

	Contains:	Test routine for DeleteIOReq().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/28/93	CDE		Derived from TestCreateIOReq

	To Do:
*/


#include "TestDeleteIOReq.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"Error/ %s"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeDeleteIOReqVerifyStr		"Before iteration (%i)/\titem/ $%x"
#define		kAfterDeleteIOReqVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyDeleteIOReq ( Boolean beforeCall, long aIndx, Err aResult, Item aItem )
/* Functionality verification for DeleteIOReq */
{
	Boolean succeeded;
	ItemNodeP aItemNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeDeleteIOReqVerifyStr, aIndx, aItem);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterDeleteIOReqVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return ( false );

		/* The item is deleted if we can't find it */
		aItemNode = (ItemNodeP) LookupItem( aItem );
		succeeded = (aItemNode == NULL);
		if (!succeeded)
			LogComment("!! IOReq still exists");

		return ( succeeded );
	}
	
}

#define		CLEANUP		DeleteMsgPort(aMessagePort)

long	CallDeleteIOReq (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Err	aResult = 0;
	char	*aName = gIOReqName;
	int32	aPriority = gPriority;
	long	aDevSpecifier = gDevSpecifier;
	Item	aDevice = (Item) 0;
	Item	aMessagePort = (Item) 0;
	Item	aIOReq = (Item) 0;
	
	/***** Functionality verification (pre-call) *****/
	aDevice = CreateTestDevice( aDevSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, aDevSpecifier);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aMessagePort = CreateMessagePort( "TestMessagePort" );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	aIOReq = CreateIOReq( aName, aPriority, aDevice, aMessagePort );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aIOReq <= 0)
		{
			LogComment(kIncCreateIOReqErrorStr, aIOReq);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	if (gVerify) VerifyDeleteIOReq ( kBeforeCall, minInnerIterations, aResult, aIOReq );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = DeleteIOReq( aIOReq  );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyDeleteIOReq ( kAfterCall, indx, aResult, aIOReq );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
					{
						CLEANUP;
						return kBreakTime;
					}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
		
	CLEANUP;
	
	return accumulator;
}

long OneDeleteIOReq ( Boolean expectError, char *aName, int32 aPriority, long aDevSpecifier )
/* Inner test */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Err	aResult = 0;
	
	/* Ancillary variables */
	Item	aDevice = (Item) 0;
	Item	aMessagePort = (Item) 0;
	Item	aIOReq = (Item) 0;
	
  	gCurrentIndex++;

	/***** Call preparation *****/
	aDevice = CreateTestDevice( aDevSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, aDevSpecifier);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aMessagePort = CreateMessagePort( "TestMessagePort" );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	aIOReq = CreateIOReq( aName, aPriority, aDevice, aMessagePort );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aIOReq <= 0)
		{
			LogComment(kIncCreateIOReqErrorStr, aIOReq);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyDeleteIOReq ( kBeforeCall, gCurrentIndex, aResult, aIOReq );

	aResult = DeleteIOReq( aIOReq  );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyDeleteIOReq ( kAfterCall, gCurrentIndex, aResult, aIOReq );
		
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogComment(kCallErrorStr, gCurrentIndex);
			else
			{
				LogComment(kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogComment(kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#define	TEST_ONE_DELETEIOREQ(expVal, name, pri, dev)	sprintf( gTestName, "%s - %s | pri= %d; dev= %d", gTestAreaName, name, pri, dev);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneDeleteIOReq(expVal, name, pri, dev); \
														

void ExerciseDeleteIOReq( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "DeleteIOReq";
	STATUS(callName);
	
	/* params: name, priority, devSpecifier */
	TEST_ONE_DELETEIOREQ(kExpectSuccess, "", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_DELETEIOREQ(kExpectSuccess, " ", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_DELETEIOREQ(kExpectSuccess, "W", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_DELETEIOREQ(kExpectSuccess, "w", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_DELETEIOREQ(kExpectSuccess, "5", CURRENT_TASK_PRIORITY, gDevSpecifier);
	TEST_ONE_DELETEIOREQ(kExpectSuccess, "This is quite an unreasonably length name", CURRENT_TASK_PRIORITY, gDevSpecifier);

	TEST_ONE_DELETEIOREQ(kExpectSuccess, "TestIOReq", TASK_MIN_PRIORITY, gDevSpecifier);

/*
 Name: empty, short, lengthy, special characters
 Pri: 0, minimum, current task, maximum, 255
 DevSpecifier: kCDROM, kTimer, kMac, kSPORT, <= 0, non-device
 IOReq population: none, 1, many
 low memory
*/
}

void TestDeleteIOReq( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseDeleteIOReq();
	else
	{
		callName = "DeleteIOReq";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallDeleteIOReq(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


