
/*
 * @(#) TestDoIO.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestDoIO.c

	Contains:	Test routines for DoIO().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/23/93	CDE		Derived from TestSendIO

	To Do:
*/


#include "TestDoIO.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeDoIOVerifyStr	"Before iteration (%i)/ ioReq/ $%x,\tioInfo/ $%x"
#define		kAfterDoIOVerifyStr		"After iteration (%i)/ result/ $%x"

Boolean		VerifyDoIO ( Boolean beforeCall, long aIndx, Err aResult, Item ioRequest, IOInfo *ioInfo )
/* Functionality verification for DoIO */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeDoIOVerifyStr, aIndx, ioRequest, ioInfo);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterDoIOVerifyStr, aIndx, aResult);
		InspectIOReq(ioRequest);
		return ( aResult == 0 );
	}
	
}

#define		CLEANUP		KillIOReq(aIOReq)

long	CallDoIO (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Err		aResult = 0;
	Item	aIOReq = 0;
	IOInfo  aIOInfo;
	char	*aName = "TestIOReq";
	int32	aPriority = CURRENT_TASK_PRIORITY;
	long	aDevSpecifier = gDevSpecifier;
	Item	aDevice = 0;
	int32	aIOCommand = gIOCommand;
	IOReq	*aIOReqP;
	int32	aLength = gLength;
	
	/* Preparation for call iterations */
	aDevice = CreateTestDevice ( aDevSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, aDevSpecifier);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aIOReq = CreateTestIOReq( aName, aPriority, aDevice, 0 );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aIOReq <= 0)
		{
			LogComment(kIncCreateIOReqErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
/***** FILL IN IOINFO *****/
	memset(&aIOInfo,0,sizeof(IOInfo));
	aIOInfo.ioi_Command = aIOCommand;
	aIOInfo.ioi_Send.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Send.iob_Len = aLength;
	aIOInfo.ioi_Recv.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Recv.iob_Len = aLength;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyDoIO ( kBeforeCall, minInnerIterations, aResult, aIOReq, &aIOInfo );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = DoIO( aIOReq, &aIOInfo );

		/* wack the error field into aResult if needed */
		aIOReqP = (IOReq *) LookupItem( aIOReq );
		if (aIOReqP != NULL)
			if (aIOReqP->io_Error < 0 ) aResult = aIOReqP->io_Error;
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyDoIO ( kAfterCall, indx, aResult, aIOReq, &aIOInfo );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	CLEANUP;

	return accumulator;
}

long OneDoIO (Boolean expectError, char *aName, int32 aPriority, long aDevSpecifier, int32 aIOCommand, int32 aLength)
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	IOReq	*aIOReqP;
	
	/* Return value */
	Err		aResult = 0;

	/* Ancillary variables */
	Item	aIOReq = 0;
	IOInfo  aIOInfo;
	Item	aDevice = 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aDevice = CreateTestDevice ( aDevSpecifier );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aDevice <= 0)
		{
			LogComment(kIncOpenDeviceErrorStr, aDevSpecifier);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aIOReq = CreateTestIOReq( aName, aPriority, aDevice, 0 );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aIOReq <= 0)
		{
			LogComment(kIncCreateIOReqErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
/***** FILL IN IOINFO *****/
	memset(&aIOInfo,0,sizeof(IOInfo));
	aIOInfo.ioi_Command = aIOCommand;
	aIOInfo.ioi_Send.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Send.iob_Len = aLength;
	aIOInfo.ioi_Recv.iob_Buffer = gIOBuffer;
	aIOInfo.ioi_Recv.iob_Len = aLength;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyDoIO ( kBeforeCall, gCurrentIndex, aResult, aIOReq, &aIOInfo );

	aResult = DoIO( aIOReq, &aIOInfo );

	/* wack the error field into aResult if needed */
	aIOReqP = (IOReq *) LookupItem( aIOReq );
	if (aIOReqP != NULL)
		if (aIOReqP->io_Error < 0 ) aResult = aIOReqP->io_Error;
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyDoIO ( kAfterCall, gCurrentIndex, aResult, aIOReq, &aIOInfo );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogComment(kCallErrorStr, gCurrentIndex);
			else
			{
				LogComment(kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogComment(kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_DOIO(expVal, name, pri, dev, com, len)	sprintf( gTestName, "%s - %s | pri= %d; dev= %d; com= %d; len= %d", gTestAreaName, name, pri, dev, com, len);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneDoIO(expVal, name, pri, dev, com, len); \
														


void ExerciseDoIO( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "DoIO";
	STATUS(callName);
	
	/* params: name, priority, devSpecifier, IOCommand, length */
	TEST_ONE_DOIO(kExpectSuccess, "", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, " ", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "W", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "w", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "5", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "This is quite an unreasonably lengthy name", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);

	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", 0, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", TASK_MIN_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", TASK_MAX_PRIORITY, gDevSpecifier, gIOCommand, gLength);
	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", 255, gDevSpecifier, gIOCommand, gLength);

	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, (gLength - 1));
	TEST_ONE_DOIO(kExpectSuccess, "TestIOReq", CURRENT_TASK_PRIORITY, gDevSpecifier, gIOCommand, (gLength + 1));
/*
 Name: empty, short, lengthy, special characters
 Pri: 0, minimum, current task, maximum, 255
 DevSpecifier: kCDROM, kTimer, kMac, kSPORT, <= 0, non-device
 I/O command: CMD_WRITE, CMD_READ, CMD_STATUS
 Length: buffer length
 IOReq population: none, 1, many
 low memory
*/
}

void TestDoIO( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseDoIO();
	else
	{
		callName = "DoIO";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallDoIO(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


