
/*
 * @(#) LDeletionTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		LDeletionTests.c

	Contains:	List Deletion call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/1/93		CDE		Derived from shell

	To Do:
*/


/* HINTS:

	 If you don't know how to do something, try to write the syntax of the routine, and use the word
		UNIMPLEMENTED in the header so you can find it later.
*/

/***** Version string *****/
#define		kVersionStr		"V1.0"


/***** Call selection constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kRemHead				14
#define		kRemNode				15
#define		kRemTail				16

/***** Default log filename: *****/
#define		kDefaultLogFile			"LDeletionTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/*#define		kIncrement1Units	"units" */
#define		kDefaultIncrement1	1


/***** Other test options *****/
/* HINTS:

	 Add one constant for each permissible value of each additional tool option.
	 Add a constant for the default value.
*/
#define		kDefaultListName			"TestList"
#define		kDefaultNodeName			"TestNode"
#define		kDefaultPriority			10


/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
/* HINTS:

	 May change drastically depending on the call.
*/
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
/* HINTS:

	 May change drastically depending on the call.
	 May need several different versions of these.
*/
#define		kIncidentalErrorStr		"Error initializing for %s"
#define		kIncAllocMemErrorStr	"Error allocating memory"
#define		kIncAllocListErrorStr	"Error allocating list"
#define		kIncAllocNodeErrorStr	"Error allocating node"
#define		kIncAddNodeErrorStr		"Error adding node to list"

#define FreeList(aList)		FreeMem( aList, sizeof(List) )

#define FreeNode(aNode)		FreeMem( aNode, sizeof(Node) )


/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr		"Error on iteration %i"

/***** Include files *****/





/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "LDeletionTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "LDeletionTests.c";	/* contains the name of the program */
char	* gTestName= "LDeletionTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */



/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
/* HINTS:

	 Add one global for each additional tool option
*/
char	gListName[64];
char	gNodeName[64];
uint8	gPriority;


/***** Utilities *****/
/* HINTS:

	 Anything you write that looks like it might be a generally useful call belongs here.
	 These will gradually migrate to our utilities libraries.
*/

Boolean FindNode( ListP aList, NodeP aTestNode )
{
	NodeP	aNode;

	for (aNode = FirstNode( aList ); IsNode( aList, aNode ); aNode = NextNode(aNode) )
     	if ( aNode == aTestNode ) return true;

	return false;
}

List *GetAList()
/* Allocate and initialize a list */
{
	List	*aList = NULL;
	int32	aSize = sizeof(List);
	uint32	aMemType = MEMTYPE_NORMAL;
	char	*aName = gListName;

	aList = (List *) AllocMem( aSize, aMemType );
	memset(aList, 0, aSize);
	InitList (aList, aName);
	return aList;
}

Node *GetANode()
/* Allocate and initialize a list */
{
	uint32	aMemType = MEMTYPE_NORMAL;
	Node	*aNode = NULL;
	int32	aSize = sizeof(Node);

	aNode = (Node *) AllocMem( aSize, aMemType );
	memset(aNode, 0, aSize);
	return (Node *)aNode;
}

#define		CLEANUP		CleanupList(aList, aNode)

void CleanupList(ListP aList, NodeP aNode)
{
	FreeList(aList);
	FreeNode(aNode);
}


bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	strcpy(gListName, kDefaultListName);
	strcpy(gNodeName, kDefaultNodeName);
	gPriority = kDefaultPriority;
	
	gVerify = true;
	gCheckFault = true;
	
	
	return kRunTest;
}	/* end GetParams */


Boolean SetupList(ListP *pList, NodeP *pNode)
/*
	Allocate a list and add a node to its tail.
	Return TRUE if successful.
*/
{
	ListP	aList;
	NodeP	aNode;
	
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;
	
	/* Add the node */
	AddTail( aList, aNode);
	
	*pList= aList;
	*pNode= aNode;
	
	return TRUE;
}
	
void InspectNode( Node *aNode )
{
	LogDiagnostic( 12,"Node:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"n_Next: $%x", aNode->n_Next);
	LogDiagnostic( 12,"n_Prev: $%x", aNode->n_Prev);
	LogDiagnostic( 12,"n_SubsysType: %i", aNode->n_SubsysType);
	LogDiagnostic( 12,"n_Type: %i", aNode->n_Type);
	LogDiagnostic( 12,"n_Priority: %i", aNode->n_Priority);
	LogDiagnostic( 12,"n_Flags: %i", aNode->n_Flags);
	LogDiagnostic( 12,"n_Size: %i", aNode->n_Size);
	LogDiagnostic( 12,"n_Name: %s", aNode->n_Name);
	LogDiagnostic( 12,"}");
}

void InspectLink( Link *aLink )
{
	LogDiagnostic( 12,"Link:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"   flink: $%x", aLink->flink);
	LogDiagnostic( 12,"   blink: $%x", aLink->blink);
	LogDiagnostic( 12,"}");
}

void InspectListAnchor( ListAnchor *aListAnchor )
{
	LogDiagnostic( 12,"ListAnchor:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"   head:");
	LogDiagnostic( 12,"      links: ");
	InspectLink(&aListAnchor->head.links);
	LogDiagnostic( 12,"      filler: $%x", aListAnchor->head.filler);
	LogDiagnostic( 12,"   tail:");
	LogDiagnostic( 12,"      filler: $%x", aListAnchor->tail.filler);
	LogDiagnostic( 12,"      links: ");
	InspectLink(&aListAnchor->tail.links);
	LogDiagnostic( 12,"}");
}

void InspectList( List *aList )
{
	LogDiagnostic( 12,"List: $%x:", aList);
	LogDiagnostic( 12,"{");

#if 0
/* removed for m2 compatibility ELH 6-26-95 */
	LogDiagnostic( 12,"   l:");
	InspectNode(&aList->l);
#endif

	LogDiagnostic( 12,"   ListAnchor:");
	InspectListAnchor(&aList->ListAnchor);
	LogDiagnostic( 12,"}");
}

/**** (End Utilities) ****/


/******************************
  *
  * Test routines for RemHead
  *
  ******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeRemVerifyStr		"Before iteration (%i)/\tlist/ $%x,\tnode/ $%x"
#define		kAfterRemVerifyStr		"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyRemHead ( Boolean beforeCall, long aIndx, NodeP aResult, ListP aList, NodeP aNode )
/* Functionality verification for RemHead */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeRemVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterRemVerifyStr, aIndx, aResult);
		return ( aResult == aNode );
	}
	
}

#define		CLEANUPRemHead

long	CallRemHead (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP aResult = NULL;
	
	/* Preparation for call iterations */
	if ( !SetupList(&aList, &aNode) ) return kBreakTime;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyRemHead ( kBeforeCall, minInnerIterations, aResult, aList, aNode );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = RemHead( aList);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyRemHead ( kAfterCall, indx, aResult, aList, aNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUPRemHead;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUPRemHead;
				
	return accumulator;
}

#define	TEST_ONE_REMHEAD(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneRemHead(expVal, list, node); \
											


long OneRemHead ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	NodeP aResult = NULL;

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* (none) */
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyRemHead ( kBeforeCall, gCurrentIndex, aResult, aList, aNode );

	aResult = RemHead( aList);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyRemHead ( kAfterCall, gCurrentIndex, aResult, aList, aNode );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPRemHead;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPRemHead;
	
	return 0;
}

Boolean ExerciseRemHead( void )
{
	char	*callName;

	/* Ancillary variables */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP a2ndNode = NULL;

	gCurrentIndex = 0;
	
	callName = "RemHead";
	STATUS(callName);
	
	/***** Initalized list with no nodes *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;


#if 0
/* this stuff removed because it blows up on M2 -- but it it supposed to ELH - 9-7-95   */
	/* DON'T add the node */
/* DEBUG: verify if this causes workstation to hang? */
	TEST_ONE_REMHEAD(kExpectError, aList, aNode);
	CleanupList(aList, aNode);
#endif

	/***** Simple case of list with 1 node *****/
	if ( !SetupList(&aList, &aNode) ) return FALSE;
	TEST_ONE_REMHEAD(kExpectSuccess, aList, aNode);
	CleanupList(aList, aNode);
	
	/***** 2 nodes *****/
	if ( !SetupList(&aList, &aNode) ) return FALSE;
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	AddTail(aList, a2ndNode);
	a2ndNode->n_Name = "SecondNode";
	a2ndNode->n_Priority = kDefaultPriority;
	TEST_ONE_REMHEAD(kExpectSuccess, aList, aNode);
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;

/*
	...
 low memory
*/
}

void TestRemHead( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseRemHead();
	else
	{
		callName = "RemHead";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallRemHead(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for RemNode
  *
  ******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeRemNodeVerifyStr		"Before iteration (%i)/\tnode/ $%x,\tlist/ $%x"
#define		kAfterRemNodeVerifyStr		"After iteration (%i)"

Boolean		VerifyRemNode ( Boolean beforeCall, long aIndx, NodeP aNode, ListP aList )
/* Functionality verification for RemNode */
{
	Boolean succeeded;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeRemNodeVerifyStr, aIndx, aNode, aList);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterRemNodeVerifyStr, aIndx);
		
		/* Remove worked if we can't find the node in the list */
		succeeded = ( !FindNode(aList, aNode) );
		if (!succeeded)
			LogComment("!! Node is still in list");
			
		return ( succeeded );
	}
	
}

long	CallRemNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	
	/* Preparation for call iterations */
	if ( !SetupList(&aList, &aNode) ) return kBreakTime;

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyRemNode ( kBeforeCall, minInnerIterations, aNode, aList );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		RemNode( aNode );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyRemNode ( kAfterCall, indx, aNode, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define	CLEANUPRemNode

#define	TEST_ONE_REMNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneRemNode(expVal, list, node); \
											


long OneRemNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
  	gCurrentIndex++;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyRemNode ( kBeforeCall, gCurrentIndex, aNode, aList );

	RemNode( aNode );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyRemNode ( kAfterCall, gCurrentIndex, aNode, aList );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPRemNode;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUPRemNode;
	return 0;
}

Boolean ExerciseRemNode( void )
{
	char	*callName;

	/* Ancillary variables */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP a2ndNode = NULL;

	gCurrentIndex = 0;
	
	callName = "RemNode";
	STATUS(callName);
	
	/***** Initalized list with no nodes *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();

	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;
	
#if 0
/* this stuff removed because it blows up on M2 -- but it it supposed to ELH - 9-7-95   */
	/***** Try to remove the node without adding it *****/
	TEST_ONE_REMNODE(kExpectSuccess, aList, aNode);
	/* ONE MIGHT EXPECT AN ERROR HERE, BUT SINCE THE NODE WAS NOT IN THE LIST TO BEGIN WITH,
		AND VERIFICATION IS SUCCESSFUL IF WE CAN'T FIND IT IN THE LIST AFTER THE CALL, THE
		BEST WE CAN DO IS SEE IF THE CALL CAUSES A CRASH */
	CleanupList(aList, aNode);
#endif

	/***** Simple case of list with 1 node *****/
	if ( !SetupList(&aList, &aNode) ) return FALSE;
	TEST_ONE_REMNODE(kExpectSuccess, aList, aNode);
	CleanupList(aList, aNode);
	
	/***** 2 nodes (remove both) *****/
	if ( !SetupList(&aList, &aNode) ) return FALSE;
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	AddTail(aList, a2ndNode);
	a2ndNode->n_Name = "SecondNode";
	a2ndNode->n_Priority = kDefaultPriority;
	TEST_ONE_REMNODE(kExpectSuccess, aList, aNode);
	TEST_ONE_REMNODE(kExpectSuccess, aList, a2ndNode);
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;

/*
	...
 low memory
*/
}

void TestRemNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseRemNode();
	else
	{
		callName = "RemNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallRemNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for RemTail
  *
  ******************************/

Boolean		VerifyRemTail ( Boolean beforeCall, long aIndx, NodeP aResult, ListP aList, NodeP aNode )
/* Functionality verification for RemTail */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeRemVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterRemVerifyStr, aIndx, aResult);
		return ( aResult == aNode );
	}
	
}

long	CallRemTail (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP aResult = NULL;
	
	/* Preparation for call iterations */
	if ( !SetupList(&aList, &aNode) ) return kBreakTime;

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyRemTail ( kBeforeCall, minInnerIterations, aResult, aList, aNode );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = RemTail( aList);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyRemTail ( kAfterCall, indx, aResult, aList, aNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define CLEANUPRemTail

#define	TEST_ONE_REMTAIL(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneRemTail(expVal, list, node); \
											


long OneRemTail( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	NodeP aResult = NULL;

  	gCurrentIndex++;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyRemTail ( kBeforeCall, gCurrentIndex, aResult, aList, aNode );

	aResult = RemTail( aList);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyRemTail ( kAfterCall, gCurrentIndex, aResult, aList, aNode );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPRemTail;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUPRemTail;
	
	return 0;
}

Boolean ExerciseRemTail( void )
{
	char	*callName;

	/* Ancillary variables */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP a2ndNode = NULL;

	gCurrentIndex = 0;
	
	callName = "RemTail";
	STATUS(callName);
	
	/***** Initalized list with no nodes *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;

#if 0
/* this stuff removed because it blows up on M2 -- but it it supposed to ELH - 9-7-95   */
	/* DON'T add the node */
	TEST_ONE_REMTAIL(kExpectError, aList, aNode);
	CleanupList(aList, aNode);
#endif

	/***** Simple case of list with 1 node *****/
	if ( !SetupList(&aList, &aNode) ) return FALSE;
	TEST_ONE_REMTAIL(kExpectSuccess, aList, aNode);
	CleanupList(aList, aNode);
	
	/***** 2 nodes *****/
	if ( !SetupList(&aList, &aNode) ) return FALSE;
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	AddTail(aList, a2ndNode);
	a2ndNode->n_Name = "SecondNode";
	a2ndNode->n_Priority = kDefaultPriority;
	TEST_ONE_REMTAIL(kExpectSuccess, aList, a2ndNode);
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;

/*
	...
 low memory
*/
}

void TestRemTail( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseRemTail();
	else
	{
		callName = "RemTail";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallRemTail(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory()
/* HINTS:

	 Add a conditional statement for each call test, substituting the specific call constant
		and test routine name
*/
{
	gCallSelector = kFullTest;
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kRemHead ) )
	{
		gTestAreaName = "RemHead";		/* contains the name of the test    */
		TestRemHead(gMinOuterIterations, gMaxOuterIterations);
	}
	
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kRemTail ) )
	{
		gTestAreaName = "RemTail";		/* contains the name of the test    */
		TestRemTail(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kRemNode ) )
	{
		gTestAreaName = "RemNode";		/* contains the name of the test    */
		TestRemNode(gMinOuterIterations, gMaxOuterIterations);
	}

}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram()
{
	int32 theError;
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
	return TRUE;
}

int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
		}
	TermScaffold();
	
 }







