
/*
 * @(#) LInsertionTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		LInsertionTests.c

	Contains:	List insertion call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/1/93		CDE		Derived from shell

	To Do:
*/


/* HINTS:

	 If you don't know how to do something, try to write the syntax of the routine, and use the word
		UNIMPLEMENTED in the header so you can find it later.
*/

/***** Version string *****/
#define		kVersionStr		"V1.0"

#define USE_INSERTNODEFROMHEAD	1
#define USE_SETNODEPRI			1

/***** Call selection constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kAddHead				1
#define		kAddTail				2

#if USE_INSERTNODEFROMHEAD
#define		kInsertNodeFromHead		6
#endif

#define		kInsertNodeFromTail		7

#if USE_SETNODEPRI
#define		kSetNodePri				17
#endif

#define		kUniversalInsertNode	18


/***** Default log filename: *****/
#define		kDefaultLogFile			"LInsertionTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/*#define		kIncrement1Units	"units" */
#define		kDefaultIncrement1	1


/***** Other test options *****/
/* HINTS:

	 Add one constant for each permissible value of each additional tool option.
	 Add a constant for the default value.
*/
#define		kDefaultListName			"TestList"
#define		kDefaultNodeName			"TestNode"
#define		kDefaultPriority			10


/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kInitListVerifyStr		"%s (%i)/ list/ $%x, name/ %s"
#define		kFindNamedNodeVerifyStr	"%s (%i)/ node/ expected/ $%x, actual/ $%x; name/ expected/ %s, actual/ %s"
#define		kTestNodeVerifyStr	"%s (%i)/ node/ expected/ $%x, actual/ $%x"

/* NOT UPGRADED BECAUSE CALL NOT IMPLEMENTED */
#if USE_SETNODEPRI
#define		kSetNodePriVerifyStr	"%s (%i)/ old priority/ expected/ %i, actual/ %i; new priority/ expected/ %i, actual/ %i"
#endif

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
/* HINTS:

	 May change drastically depending on the call.
	 May need several different versions of these.
*/
#define		kIncidentalErrorStr		"Error initializing for %s"
#define		kIncAllocMemErrorStr	"!! Unable to allocate memory"
#define		kIncAllocListErrorStr	"!! Unable to allocate list"
#define		kIncAllocNodeErrorStr	"!! Unable to allocate node"
#define		kIncAddNodeErrorStr		"!! Unable to add node to list"

#define FreeList(aList)		FreeMem( aList, sizeof(List) )

#define FreeNode(aNode)		FreeMem( aNode, sizeof(Node) )

/***** Include files *****/





/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "LInsertionTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "LInsertionTests.c";	/* contains the name of the program */
char	* gTestName= "LInsertionTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */




/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
/* HINTS:

	 Add one global for each additional tool option
*/
char	gListName[64];
char	gNodeName[64];
uint8	gPriority;


/***** Utilities *****/
/* HINTS:

	 Anything you write that looks like it might be a generally useful call belongs here.
	 These will gradually migrate to our utilities libraries.
*/

List *GetAList()
/* Allocate and initialize a list */
{
	List	*aList = NULL;
	int32	aSize = sizeof(List);
	uint32	aMemType = MEMTYPE_NORMAL;
	char	*aName = gListName;

	aList = (List *) AllocMem( aSize, aMemType );
	memset(aList, 0, aSize);
	InitList (aList, aName);
	return aList;
}

NodeP GetANode()
/* Allocate and initialize a list */
{
	uint32	aMemType = MEMTYPE_NORMAL;
	NodeP	aNode = NULL;
	int32	aSize = sizeof(Node);

	aNode = (NodeP) AllocMem( aSize, aMemType );
	memset(aNode, 0, aSize);
	return (NodeP) aNode;
}

NodeP GetValidNode()
{
	NodeP	aNode;

	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			return aNode;
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;
	
	return aNode;
}

void CleanupList(ListP aList, NodeP aNode)
{
	FreeList(aList);
	FreeNode(aNode);
}

Boolean SetupList(ListP *pList, NodeP *pNode)
/*
	Allocate a list and add a node to its tail.
	Return TRUE if successful.
*/
{
	ListP	aList;
	NodeP	aNode;
	
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;
	
	/* Add the node */
	AddTail( aList, aNode);
	
	*pList= aList;
	*pNode= aNode;
	
	return TRUE;
}
	
void InspectNode( Node *aNode )
{
	LogDiagnostic( 12,"Node:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"n_Next: $%x", aNode->n_Next);
	LogDiagnostic( 12,"n_Prev: $%x", aNode->n_Prev);
	LogDiagnostic( 12,"n_SubsysType: %i", aNode->n_SubsysType);
	LogDiagnostic( 12,"n_Type: %i", aNode->n_Type);
	LogDiagnostic( 12,"n_Priority: %i", aNode->n_Priority);
	LogDiagnostic( 12,"n_Flags: %i", aNode->n_Flags);
	LogDiagnostic( 12,"n_Size: %i", aNode->n_Size);
	LogDiagnostic( 12,"n_Name: %s", aNode->n_Name);
	LogDiagnostic( 12,"}");
}

void InspectLink( Link *aLink )
{
	LogDiagnostic( 12,"Link:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"   flink: $%x", aLink->flink);
	LogDiagnostic( 12,"   blink: $%x", aLink->blink);
	LogDiagnostic( 12,"}");
}

void InspectListAnchor( ListAnchor *aListAnchor )
{
	LogDiagnostic( 12,"ListAnchor:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"   head:");
	LogDiagnostic( 12,"      links: ");
	InspectLink(&aListAnchor->head.links);
	LogDiagnostic( 12,"      filler: $%x", aListAnchor->head.filler);
	LogDiagnostic( 12,"   tail:");
	LogDiagnostic( 12,"      filler: $%x", aListAnchor->tail.filler);
	LogDiagnostic( 12,"      links: ");
	InspectLink(&aListAnchor->tail.links);
	LogDiagnostic( 12,"}");
}

void InspectList( List *aList )
{
	LogDiagnostic( 12,"List: $%x:", aList);
	LogDiagnostic( 12,"{");
	
#if 0
/* this has disappeard 6-22-95 elh */
	LogDiagnostic( 12,"   l:");
	InspectNode(&aList->l);
#endif

	LogDiagnostic( 12,"   ListAnchor:");
	InspectListAnchor(&aList->ListAnchor);
	LogDiagnostic( 12,"}");
}

Boolean VerifyList( ListP aList, NodeP aTestNode )
{
	NodeP	aNode;
	uint8	aPriority = aTestNode->n_Priority;
	Boolean	afterNode= FALSE;

	for (aNode = FirstNode( aList ); IsNode( aList, aNode ); aNode = NextNode(aNode) )
    {
		if ( aNode == aTestNode )
		{
			if (afterNode)
			{
				LogComment("!! Redundant occurrence of node - but this is OK");
				return true;
			}
			afterNode = true;
		}
		else
		{
			if ( afterNode )
			{
				if ( aNode->n_Priority > aPriority )
				{
					LogComment("!! Later node has higher priority");
					return false;
				}
			}
			else
				if ( aNode->n_Priority < aPriority )
				{
					LogComment("!! Earlier node has lower priority");
					return false;
				}
			
		}
	}
	return true;
}

Boolean CompareNodes( NodeP nodeToInsert, NodeP existingNode )
/* Compare function for UniversalInsertNode */
{
	return ( (nodeToInsert->n_Priority) > (existingNode->n_Priority) );
}

/**** (End Utilities) ****/


void	ShowHelp()
/* Show Help for this tool */
{
		kprintf ( "#################################################################");
		kprintf ( "List Insertion Tests %s Help", kVersionStr );
		kprintf ( "Written by Charlie Eckhaus for The 3DO Company" );
		kprintf (    "#################################################################");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-out << no parameter>>\tDo not use a log file.");
		LogDiagnostic( 80, "     <<filename>>     \tUse specified log file.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80,"-icmin <<value>>\tIterate calls starting at <<value>>.");
		LogDiagnostic( 80,"-icmax <<value>>\tEnd call iterations at <<value>>.");
		LogDiagnostic( 80,"-itmin <<value>>\tIterate tests starting at <<value>>.");
		LogDiagnostic( 80,"-itmax <<value>>\tEnd test iterations at <<value>>.");
		LogDiagnostic( 80, "");

		kprintf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-name <<value>>\tUse <<value>> as list name.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-error none\tDon't check for errors.");
		LogDiagnostic( 80, "       cont\tWhen an error occurs, report it and continue.");
		LogDiagnostic( 80, "       none\tWhen an error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-ierror none\tDon't check for incidental errors.");
		LogDiagnostic( 80, "        cont\tWhen an incidental error occurs, report it and continue.");
		LogDiagnostic( 80, "        none\tWhen an incidental error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-time off\tDon't output timing information.");
		LogDiagnostic( 80, "      on \tOutput timing information.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-func off\tDon't verify functionality.");
		LogDiagnostic( 80,"      on \tVerify functionality.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-fault off\tMake known assumptions.");
		LogDiagnostic( 80,"       on \tSkip known assumptions.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-sel <<no parameter>>\tRun the entire test suite.");
		LogDiagnostic( 80,"     AddHead               \tTest AddHead.");
		LogDiagnostic( 80,"     AddTail               \tTest AddTail.");
#if USE_INSERTNODEFROMHEAD
		LogDiagnostic( 80,"     InsertNodeFromHead    \tTest InsertNodeFromHead.");
#endif
		LogDiagnostic( 80,"     InsertNodeFromTail    \tTest InsertNodeFromTail.");
#if USE_SETNODEPRI
		LogDiagnostic( 80,"     SetNodePri            \tTest SetNodePri.");
#endif
		LogDiagnostic( 80,"     UniversalInsertNode   \tTest UniversalInsertNode.");
		LogDiagnostic( 80, "");
		
		kprintf ( "#################################################################");
}

void ShowOptions()
/* Show the option settings for this session */
/* HINTS:

	 Incorporate your tool name, your name, additional options and the category's calls
*/
{
		kprintf ( "#################################################################");
		kprintf ( "List Insertion Tests %s - Session options", kVersionStr);
		kprintf ( "Written by Charlie Eckhaus for The 3DO Company" );
		kprintf (   "#################################################################");
		
		if (gUseLogFile) LogDiagnostic( 80, "-out: Use log file: %s.",gLogFile); else LogDiagnostic( 80,"Do not use a log file.");
		
		kprintf ( "Iterate each test from %i",gMinOuterIterations);
		kprintf ( " to %i,", gMaxOuterIterations);
		kprintf ( " in increments of %i.",gIncrement1); 
		kprintf ( "Within each test, iterate the call from %i to %i.",gMinInnerIterations,gMaxInnerIterations);
		
		kprintf ( "-name: Use %s as list name.", gListName);
		kprintf ( "-nodename: Use %s as node name.", gNodeName);
		kprintf ( "-priority: Use %i as new node priority.", gPriority);
		
		LogDiagnostic( 80, "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80, "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for incidental errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an incidental error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first incidental error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80,"-time: ");
		if (gOutputTiming) LogDiagnostic( 80,"Output timing information."); else LogDiagnostic( 80,"Don't output timing information.");

		LogDiagnostic( 80,"-func: ");
		if (gVerify) LogDiagnostic( 80,"Verify functionality."); else LogDiagnostic( 80,"Don't verify functionality.");

		LogDiagnostic( 80,"-fault: ");
		if (gCheckFault) LogDiagnostic( 80,"Skip known assumptions."); else LogDiagnostic( 80,"Make known assumptions.");

		LogDiagnostic( 80,"-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				LogDiagnostic( 80,"Run the entire test suite.");
				break;
				
			case kAddHead:
				LogDiagnostic( 80,"Test AddHead.");
				break;
				
			case kAddTail:
				LogDiagnostic( 80,"Test AddTail.");
				break;
				
#if USE_INSERTNODEFROMHEAD
			case kInsertNodeFromHead:
				LogDiagnostic( 80,"Test InsertNodeFromHead.");
				break;
#endif
				
			case kInsertNodeFromTail:
				LogDiagnostic( 80,"Test InsertNodeFromTail.");
				break;
				
#if USE_SETNODEPRI
			case kSetNodePri:
				LogDiagnostic( 80,"Test SetNodePri.");
				break;
#endif
				
			case kUniversalInsertNode:
				LogDiagnostic( 80,"Test UniversalInsertNode.");
				break;
				
			} /* end switch */
		
		kprintf ( "#################################################################");
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	strcpy(gListName, kDefaultListName);
	strcpy(gNodeName, kDefaultNodeName);
	gPriority = kDefaultPriority;
	
	gVerify = true;
	gCheckFault = true;
	
	
	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Test routines for AddHead
  *
  ******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeAddVerifyStr	"Before iteration (%i)/\tlist/ $%x,\tnode/ $%x"
#define		kAfterAddVerifyStr	"After iteration (%i)."

Boolean		VerifyAddHead ( Boolean beforeCall, long aIndx, List *aList, Node *aNode )
/* Functionality verification for AddHead */
{
	Boolean succeeded;
	Node *aFirstNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeAddVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterAddVerifyStr, aIndx);
		
		aFirstNode = FirstNode( aList );
		succeeded = (aFirstNode == aNode);
		if (!succeeded)
			LogComment("!! Not the first node in list");
			
		return ( succeeded );
	}
	
}

#define		CLEANUP		CleanupList(aList, aNode)

long	CallAddHead (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Node *aNode = NULL;
	List *aList = NULL;
	
	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyAddHead ( kBeforeCall, minInnerIterations, aList, aNode );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		AddHead( aList, aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyAddHead ( kAfterCall, indx, aList, aNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPAddHead


#define	TEST_ONE_ADDHEAD(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneAddHead(expVal, list, node); \
											


long OneAddHead (  Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
  	gCurrentIndex++;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyAddHead ( kBeforeCall, gCurrentIndex, aList, aNode );

	AddHead( aList, aNode);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyAddHead ( kAfterCall, gCurrentIndex, aList, aNode );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPAddHead;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPAddHead;
	
	return 0;
}

Boolean ExerciseAddHead( void )
{
	char	*callName;

	/* Ancillary variables */
	Node *aNode = NULL;
	List *aList = NULL;
	Node *a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "AddHead";
	STATUS(callName);
	
	/***** Empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			FreeList(aList);
			return FALSE;
		}
	}
	TEST_ONE_ADDHEAD(kExpectSuccess, aList, aNode);

	/***** Add a second node *****/
	a2ndNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			CleanupList(aList, aNode);
			return FALSE;
		}
	}
	a2ndNode->n_Name = gNodeName;
	TEST_ONE_ADDHEAD(kExpectSuccess, aList, a2ndNode);
	
	/***** Add the first node again *****/
	TEST_ONE_ADDHEAD(kExpectSuccess, aList, aNode);
	/* THIS IS APPARENTLY OK (NO EFFECT) */
	
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 low memory
*/
}

void TestAddHead( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseAddHead();
	else
	{
		callName = "AddHead";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallAddHead(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for AddTail
  *
  ******************************/

Boolean		VerifyAddTail ( Boolean beforeCall, long aIndx, List *aList, Node *aNode )
/* Functionality verification for AddTail */
{
	Boolean succeeded;
	Node *aLastNode;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeAddVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterAddVerifyStr, aIndx);
		
		aLastNode = LastNode( aList );
		succeeded = (aLastNode == aNode);
		if (!succeeded)
			LogComment("!! Not the last node in the list");
			
		return ( succeeded );
	}
	
}

long	CallAddTail (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Node *aNode = NULL;
	List *aList = NULL;
	
	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyAddTail ( kBeforeCall, minInnerIterations, aList, aNode );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		AddTail( aList, aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyAddTail ( kAfterCall, indx, aList, aNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}


#define	TEST_ONE_ADDTAIL(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneAddTail(expVal, list, node); \
											

long OneAddTail (  Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyAddTail ( kBeforeCall, gCurrentIndex, aList, aNode );

	AddTail( aList, aNode);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyAddTail ( kAfterCall, gCurrentIndex, aList, aNode );
		
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUP;
	
	return 0;
}

Boolean ExerciseAddTail( void )
{
	char	*callName;

	/* Ancillary variables */
	Node *aNode = NULL;
	List *aList = NULL;
	Node *a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "AddTail";
	STATUS(callName);
	
	/***** Empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			FreeList(aList);
			return FALSE;
		}
	}
	TEST_ONE_ADDTAIL(kExpectSuccess, aList, aNode);

	/***** Add a second node *****/
	a2ndNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			CleanupList(aList, aNode);
			return FALSE;
		}
	}
	a2ndNode->n_Name = gNodeName;
	TEST_ONE_ADDTAIL(kExpectSuccess, aList, a2ndNode);
	
	/***** Add the first node again *****/
	TEST_ONE_ADDTAIL(kExpectSuccess, aList, aNode);
	/* THIS IS APPARENTLY OK (NO EFFECT) */
	
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 low memory
*/
}

void TestAddTail( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseAddTail();
	else
	{
		callName = "AddTail";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallAddTail(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/* NOT UPGRADED BECAUSE CALL NOT IMPLEMENTED */
/* CDE, 9/22/93: Upgraded for parameter-testing before I noticed the above comment. */

#if USE_INSERTNODEFROMHEAD
/******************************
  *
  * Test routines for InsertNodeFromHead
  *
  ******************************/

Boolean		VerifyInsertNodeFromHead ( Boolean beforeCall, long aIndx, ListP aList, NodeP aNode )
/* Functionality verification for InsertNodeFromHead */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeAddVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterAddVerifyStr, aIndx);
		if (IsNode(aList, aNode)) return VerifyList(aList, aNode); else return false;
	}
	
}

long	CallInsertNodeFromHead (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyInsertNodeFromHead ( kBeforeCall, minInnerIterations, aList, aNode );

	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		InsertNodeFromHead( aList, aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyInsertNodeFromHead ( kAfterCall, indx, aList, aNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPInsertNodeFromHead


#define	TEST_ONE_INSERTNODEFROMHEAD(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneInsertNodeFromHead(expVal, list, node); \
											

long OneInsertNodeFromHead ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyInsertNodeFromHead ( kBeforeCall, gCurrentIndex, aList, aNode );

	InsertNodeFromHead( aList, aNode);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyInsertNodeFromHead ( kAfterCall, gCurrentIndex, aList, aNode );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPInsertNodeFromHead;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPInsertNodeFromHead;
	
	return 0;
}

Boolean ExerciseInsertNodeFromHead( void )
{
	char	*callName;

	/* Ancillary variables */
	Node *aNode = NULL;
	List *aList = NULL;
	Node *a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "InsertNodeFromHead";
	STATUS(callName);
	
	/***** Empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			FreeList(aList);
			return FALSE;
		}
	}
	TEST_ONE_INSERTNODEFROMHEAD(kExpectSuccess, aList, aNode);

	/***** Add a second node *****/
	a2ndNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			CleanupList(aList, aNode);
			return FALSE;
		}
	}
	a2ndNode->n_Name = gNodeName;
	TEST_ONE_INSERTNODEFROMHEAD(kExpectSuccess, aList, a2ndNode);
	
	/***** Add the first node again *****/
	TEST_ONE_INSERTNODEFROMHEAD(kExpectSuccess, aList, aNode);
	/* THIS IS APPARENTLY OK (NO EFFECT) */
	
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 low memory
*/
}

void TestInsertNodeFromHead( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseInsertNodeFromHead();
	else
	{
		callName = "InsertNodeFromHead";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			theTime = CallInsertNodeFromHead(indx, gMinInnerIterations, gMaxInnerIterations);
			accumulator += theTime;
			if (theTime == kBreakTime) break;
		}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}
#endif

/******************************
  *
  * Test routines for InsertNodeFromTail
  *
  ******************************/

Boolean		VerifyInsertNodeFromTail ( Boolean beforeCall, long aIndx, ListP aList, NodeP aNode )
/* Functionality verification for InsertNodeFromTail */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeAddVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterAddVerifyStr, aIndx);
		if (IsNode(aList, aNode)) return VerifyList(aList, aNode); else return false;
	}
	
}

long	CallInsertNodeFromTail (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	
	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyInsertNodeFromTail ( kBeforeCall, minInnerIterations, aList, aNode );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		InsertNodeFromTail( aList, aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyInsertNodeFromTail ( kAfterCall, indx, aList, aNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPInsertNodeFromTail


#define	TEST_ONE_INSERTNODEFROMTAIL(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneInsertNodeFromTail(expVal, list, node); \
											

long	OneInsertNodeFromTail ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyInsertNodeFromTail ( kBeforeCall, gCurrentIndex, aList, aNode );

	InsertNodeFromTail( aList, aNode);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyInsertNodeFromTail ( kAfterCall, gCurrentIndex, aList, aNode );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPInsertNodeFromTail;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPInsertNodeFromTail;
	
	return 0;
}

Boolean ExerciseInsertNodeFromTail( void )
{
	char	*callName;

	/* Ancillary variables */
	Node *aNode = NULL;
	List *aList = NULL;
	Node *a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "InsertNodeFromTail";
	STATUS(callName);
	
	/***** Empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			FreeList(aList);
			return FALSE;
		}
	}
	TEST_ONE_INSERTNODEFROMTAIL(kExpectSuccess, aList, aNode);

	/***** Add a second node *****/
	a2ndNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			CleanupList(aList, aNode);
			return FALSE;
		}
	}
	a2ndNode->n_Name = gNodeName;
	TEST_ONE_INSERTNODEFROMTAIL(kExpectSuccess, aList, a2ndNode);
	
	/***** Add the first node again *****/
	TEST_ONE_INSERTNODEFROMTAIL(kExpectSuccess, aList, aNode);
	/* THIS IS APPARENTLY OK (NO EFFECT) */
	
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 low memory
*/
}

void TestInsertNodeFromTail( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseInsertNodeFromTail();
	else
	{
		callName = "InsertNodeFromTail";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			theTime = CallInsertNodeFromTail(indx, gMinInnerIterations, gMaxInnerIterations);
			accumulator += theTime;
			if (theTime == kBreakTime) break;
		}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/* NOT UPGRADED BECAUSE CALL NOT IMPLEMENTED */
#if USE_SETNODEPRI
/******************************
  *
  * Test routines for SetNodePri
  *
  ******************************/

Boolean		VerifySetNodePri ( Boolean beforeCall, long aIndx, uint8 aResult, Node *aNode )
/* Functionality verification for SetNodePri */
{
	static uint8 expectedResult;
	uint8 aPriority;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kSetNodePriVerifyStr, "Before", aIndx);
		expectedResult = aResult;
		return true;
	}
	else
	{
		aPriority = aNode->n_Priority;
		LogDiagnostic( 12,kSetNodePriVerifyStr, "After", aIndx, expectedResult, aResult, gPriority, aPriority);
/*  UNIMPLEMENTED: need to check position in list */
		return ( ( expectedResult == aResult ) && ( gPriority == aPriority ) );
	}
	
}

#define		CLEANUPSetNodePri	CLEANUP;\
								FreeNode(a2ndNode)
								
long	CallSetNodePri (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError;
	
  	/* Variables for call parameters and return value */
	List *aList = NULL;
	Node *aNode = NULL;
	Node *a2ndNode = NULL;
	uint8 aPriority = gPriority;
	uint8 aResult;
	
	/* Preparation for call iterations */
	theError = FALSE;

	if ( !SetupList( &aList, &aNode) ) return kBreakTime;
	
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	a2ndNode->n_Name = gNodeName;
/*  Might want to use a different name instead of the first node's name */
	a2ndNode->n_Priority = kDefaultPriority;
	
	/* Add the 2nd node */
	AddTail( aList, a2ndNode);
	
	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifySetNodePri ( kBeforeCall, minInnerIterations, kDefaultPriority, a2ndNode );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = SetNodePri( a2ndNode, aPriority);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifySetNodePri ( kAfterCall, indx, aResult, a2ndNode );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUPSetNodePri;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUPSetNodePri;
				
	return accumulator;
}


#define	TEST_ONE_SETNODEPRI(expVal, pri, name)		sprintf( gTestName, "%s | pri= 0x%x; name= '%s'", gTestAreaName, pri, name);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneSetNodePri(expVal, pri, name); \
											

long OneSetNodePri (Boolean expectError, uint8 aPriority, char *aName )
/* Individual call for parameter-testing */
{
	Boolean theError=FALSE;
	
	/* Return value */
	uint8 aResult;

	/* Ancillary variables */
	List *aList = NULL;
	Node *aNode = NULL;
	Node *a2ndNode = NULL;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	if ( !SetupList( &aList, &aNode) ) return kBreakTime;
	
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
			}
		}
	}
	a2ndNode->n_Name = aName;
/*  Might want to use a different name instead of the first node's name */
	a2ndNode->n_Priority = kDefaultPriority;
	
	/* Add the 2nd node */
	AddTail( aList, a2ndNode);
	
	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifySetNodePri ( kBeforeCall, gCurrentIndex, kDefaultPriority, a2ndNode );

	aResult = SetNodePri( a2ndNode, aPriority);
		
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifySetNodePri ( kAfterCall, gCurrentIndex, aResult, a2ndNode );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPSetNodePri;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPSetNodePri;
	
	return 0;
}

Boolean ExerciseSetNodePri( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "SetNodePri";
	STATUS(callName);
	
	/* params: priority, name */
	TEST_ONE_SETNODEPRI(kExpectSuccess, gPriority, gNodeName);
/*
	...
 Invalid priority
 low memory
*/
}

void TestSetNodePri( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseSetNodePri();
	else
	{
		callName = "SetNodePri";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallSetNodePri(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


#endif


/******************************
  *
  * Test routines for UniversalInsertNode
  *
  ******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeUniversalInsertNodeVerifyStr	"Before iteration (%i)/\tlist/ $%x,\tnode/ $%x,\tf/ $%x"
#define		kAfterUniversalInsertNodeVerifyStr	"After iteration (%i)."

Boolean		VerifyUniversalInsertNode ( Boolean beforeCall, long aIndx, ListP aList, NodeP aNode, bool (*f)(Node *n, Node *m) )
/* Functionality verification for UniversalInsertNode */
{
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeUniversalInsertNodeVerifyStr, aIndx, aList, aNode);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterUniversalInsertNodeVerifyStr, aIndx);
		if (IsNode(aList, aNode)) return VerifyList(aList, aNode); else return false;
	}
	
}

long	CallUniversalInsertNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	
	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyUniversalInsertNode ( kBeforeCall, minInnerIterations, aList, aNode, CompareNodes );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		UniversalInsertNode( aList, aNode, CompareNodes);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyUniversalInsertNode ( kAfterCall, indx, aList, aNode, CompareNodes );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPUniversalInsertNode


#define	TEST_ONE_UNIVINSERTNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneUniversalInsertNode(expVal, list, node); \
											

long OneUniversalInsertNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyUniversalInsertNode ( kBeforeCall, gCurrentIndex, aList, aNode, CompareNodes );

	UniversalInsertNode( aList, aNode, CompareNodes);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyUniversalInsertNode ( kAfterCall, gCurrentIndex, aList, aNode, CompareNodes );
		
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPUniversalInsertNode;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPUniversalInsertNode;
	
	return 0;
}

Boolean ExerciseUniversalInsertNode( void )
{
	char	*callName;

	/* Ancillary variables */
	Node *aNode = NULL;
	List *aList = NULL;
	Node *a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "UniversalInsertNode";
	STATUS(callName);
	
	/***** Empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			FreeList(aList);
			return FALSE;
		}
	}
	TEST_ONE_UNIVINSERTNODE(kExpectSuccess, aList, aNode);

	/***** Add a second node *****/
	a2ndNode = GetValidNode();
	if (gCheckIncidentalErrors == kStop)
	{
		if (aNode == NULL)
		{
			CleanupList(aList, aNode);
			return FALSE;
		}
	}
	a2ndNode->n_Name = gNodeName;
	TEST_ONE_UNIVINSERTNODE(kExpectSuccess, aList, a2ndNode);
	
	/***** Add the first node again *****/
	TEST_ONE_UNIVINSERTNODE(kExpectSuccess, aList, aNode);
	/* THIS IS APPARENTLY OK (NO EFFECT) */
	
	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 low memory
*/
}

void TestUniversalInsertNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseUniversalInsertNode();
	else
	{
		callName = "UniversalInsertNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallUniversalInsertNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory()
/* HINTS:

	 Add a conditional statement for each call test, substituting the specific call constant
		and test routine name
*/
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kAddHead ) )
	{
		gTestAreaName = "AddHead ";		/* contains the name of the test    */
		TestAddHead(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kAddTail ) )
	{
		gTestAreaName = "AddTail";		/* contains the name of the test    */
		TestAddTail(gMinOuterIterations, gMaxOuterIterations);
	}

#if USE_INSERTNODEFROMHEAD
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kInsertNodeFromHead ) )
	{
		gTestAreaName = "InsertNodeFromHead";		/* contains the name of the test    */
		TestInsertNodeFromHead(gMinOuterIterations, gMaxOuterIterations);
	}
#endif

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kInsertNodeFromTail ) )
	{
		gTestAreaName = "InsertNodeFromTail";		/* contains the name of the test    */
		TestInsertNodeFromTail(gMinOuterIterations, gMaxOuterIterations);
	}

#if USE_SETNODEPRI
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSetNodePri ) )
	{
		gTestAreaName = "SetNodePri";		/* contains the name of the test    */
		TestSetNodePri(gMinOuterIterations, gMaxOuterIterations);
	}
#endif

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kUniversalInsertNode ) )
	{
		gTestAreaName = "UniversalInsertNode";		/* contains the name of the test    */
		TestUniversalInsertNode(gMinOuterIterations, gMaxOuterIterations);
	}
}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram()
{
	int32 theError;
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
	return TRUE;
}

int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
			
		}
	TermScaffold();
	}
	
 }












