
/*
 * @(#) LTraversalTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		LTraversalTests.c

	Contains:	List traversal call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/1/93		CDE		Derived from shell

	To Do:
*/


/***** Version string *****/
#define		kVersionStr		"V1.0"


/***** Call selection constants *****/
#define		kFindNamedNode			3
#define		kFirstNode				4
#define		kInitList				5
#define		kIsEmptyList			8
#define		kIsNode					9
#define		kIsNodeB				10
#define		kLastNode				11
#define		kNextNode				12
#define		kPrevNode				13


/***** Default log filename: *****/
#define		kDefaultLogFile			"LTraversalTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
/*#define		kIncrement1Units	"units" */
#define		kDefaultIncrement1	1


/***** Other test options *****/
#define		kDefaultListName			"TestList"
#define		kDefaultNodeName			"TestNode"
#define		kDefaultPriority			10


/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeTestNodeVerifyStr	"Before iteration (%i)/\tlist/ $%x,\texpected result/ $%x"
#define		kAfterTestNodeVerifyStr		"After iteration (%i)/\tresult/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr		"Error initializing for %s"
#define		kIncAllocMemErrorStr	"!! Unable to allocate memory"
#define		kIncAllocListErrorStr	"!! Unable to allocate list"
#define		kIncAllocNodeErrorStr	"!! Unable to allocate node"
#define		kIncAddNodeErrorStr		"!! Unable to add node to list"

#define FreeList(aList)		FreeMem( aList, sizeof(List) )

#define FreeNode(aNode)		FreeMem( aNode, sizeof(Node) )

/***** Include files *****/





/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "LTraversalTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "LTraversalTests.c";	/* contains the name of the program */
char	* gTestName= "LTraversalTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */






/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
/* HINTS:

	 Add one global for each additional tool option
*/
char	gListName[64];
char	gNodeName[64];
uint8	gPriority;


/***** Utilities *****/
/* HINTS:

	 Anything you write that looks like it might be a generally useful call belongs here.
	 These will gradually migrate to our utilities libraries.
*/

char *printable(char *aName)
{
	if (aName == NULL) return "<NULL>";
	if (aName == " ") return "<space>";
	if (strlen(aName)) return aName;
	return "<empty>";
}


List *GetAList()
/* Allocate and initialize a list */
{
	List	*aList = NULL;
	int32	aSize = sizeof(List);
	uint32	aMemType = MEMTYPE_NORMAL;
	char	*aName = gListName;

	aList = (List *) AllocMem( aSize, aMemType );
	memset(aList, 0, aSize);
	InitList (aList, aName);
	return aList;
}

Node *GetANode()
/* Allocate and initialize a list */
{
	uint32	aMemType = MEMTYPE_NORMAL;
	Node	*aNode = NULL;
	int32	aSize = sizeof(Node);

	aNode = (Node *) AllocMem( aSize, aMemType );
	memset(aNode, 0, aSize);
	return (Node *)aNode;
}

void CleanupList(ListP aList, NodeP aNode)
{
	FreeList(aList);
	FreeNode(aNode);
}

Boolean SetupList(ListP *pList, NodeP *pNode)
/*
	Allocate a list and add a node to its tail.
	Return TRUE if successful.
*/
{
	ListP	aList;
	NodeP	aNode;
	
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = gNodeName;
	aNode->n_Priority = kDefaultPriority;
	
	/* Add the node */
	AddTail( aList, aNode);
	
	*pList= aList;
	*pNode= aNode;
	
	return TRUE;
}
	
void InspectNode( Node *aNode )
{
	LogDiagnostic( 12,"Node:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"n_Next: $%x", aNode->n_Next);
	LogDiagnostic( 12,"n_Prev: $%x", aNode->n_Prev);
	LogDiagnostic( 12,"n_SubsysType: %i", aNode->n_SubsysType);
	LogDiagnostic( 12,"n_Type: %i", aNode->n_Type);
	LogDiagnostic( 12,"n_Priority: %i", aNode->n_Priority);
	LogDiagnostic( 12,"n_Flags: %i", aNode->n_Flags);
	LogDiagnostic( 12,"n_Size: %i", aNode->n_Size);
	LogDiagnostic( 12,"n_Name: %s", printable(aNode->n_Name));
	LogDiagnostic( 12,"}");
}

void InspectLink( Link *aLink )
{
	LogDiagnostic( 12,"Link:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"   flink: $%x", aLink->flink);
	LogDiagnostic( 12,"   blink: $%x", aLink->blink);
	LogDiagnostic( 12,"}");
}

void InspectListAnchor( ListAnchor *aListAnchor )
{
	LogDiagnostic( 12,"ListAnchor:");
	LogDiagnostic( 12,"{");
	LogDiagnostic( 12,"   head:");
	LogDiagnostic( 12,"      links: ");
	InspectLink(&aListAnchor->head.links);
	LogDiagnostic( 12,"      filler: $%x", aListAnchor->head.filler);
	LogDiagnostic( 12,"   tail:");
	LogDiagnostic( 12,"      filler: $%x", aListAnchor->tail.filler);
	LogDiagnostic( 12,"      links: ");
	InspectLink(&aListAnchor->tail.links);
	LogDiagnostic( 12,"}");
}

void InspectList( List *aList )
{
	LogDiagnostic( 12,"List: $%x:", aList);
	LogDiagnostic( 12,"{");
	
#if 0
/* removed for M2 compatibility */
	LogDiagnostic( 12,"   l:");
	InspectNode(&aList->l);
#endif	
	
	LogDiagnostic( 12,"   ListAnchor:");
	InspectListAnchor(&aList->ListAnchor);
	LogDiagnostic( 12,"}");
}

/**** (End Utilities) ****/


void	ShowHelp()
/* Show Help for this tool */
{
		kprintf ( "#################################################################");
		kprintf ( "List Traversal Tests %s Help", kVersionStr );
		kprintf ( "Written by Charlie Eckhaus for The 3DO Company" );
		kprintf (    "#################################################################");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-out << no parameter>>\tDo not use a log file.");
		LogDiagnostic( 80, "     <<filename>>     \tUse specified log file.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80,"-icmin <<value>>\tIterate calls starting at <<value>>.");
		LogDiagnostic( 80,"-icmax <<value>>\tEnd call iterations at <<value>>.");
		LogDiagnostic( 80,"-itmin <<value>>\tIterate tests starting at <<value>>.");
		LogDiagnostic( 80,"-itmax <<value>>\tEnd test iterations at <<value>>.");
		LogDiagnostic( 80, "");

		kprintf ( "-inc1 <<value>>\tUse <<value>> as increment."); 
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-name <<value>>\tUse <<value>> as list name.");
		LogDiagnostic( 80, "");
		
		LogDiagnostic( 80, "-error none\tDon't check for errors.");
		LogDiagnostic( 80, "       cont\tWhen an error occurs, report it and continue.");
		LogDiagnostic( 80, "       none\tWhen an error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-ierror none\tDon't check for incidental errors.");
		LogDiagnostic( 80, "        cont\tWhen an incidental error occurs, report it and continue.");
		LogDiagnostic( 80, "        none\tWhen an incidental error occurs, report it and stop the tool.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80, "-time off\tDon't output timing information.");
		LogDiagnostic( 80, "      on \tOutput timing information.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-func off\tDon't verify functionality.");
		LogDiagnostic( 80,"      on \tVerify functionality.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-fault off\tMake known assumptions.");
		LogDiagnostic( 80,"       on \tSkip known assumptions.");
		LogDiagnostic( 80, "");

		LogDiagnostic( 80,"-sel <<no parameter>>\tRun the entire test suite.");
		LogDiagnostic( 80,"     FindNamedNode         \tTest FindNamedNode.");
		LogDiagnostic( 80,"     FirstNode             \tTest FirstNode.");
		LogDiagnostic( 80,"     InitList              \tTest InitList.");
		LogDiagnostic( 80,"     IsEmptyList           \tTest IsEmptyList.");
		LogDiagnostic( 80,"     IsNode                \tTest IsNode.");
		LogDiagnostic( 80,"     IsNodeB               \tTest IsNodeB.");
		LogDiagnostic( 80,"     LastNode              \tTest LastNode.");
		LogDiagnostic( 80,"     NextNode              \tTest NextNode.");
		LogDiagnostic( 80,"     PrevNode              \tTest PrevNode.");
		LogDiagnostic( 80, "");
		
		kprintf ( "#################################################################");
}

void ShowOptions()
/* Show the option settings for this session */
/* HINTS:

	 Incorporate your tool name, your name, additional options and the category's calls
*/
{
		kprintf ( "#################################################################");
		kprintf ( "List Traversal Tests %s - Session options", kVersionStr);
		kprintf ( "Written by Charlie Eckhaus for The 3DO Company" );
		kprintf (   "#################################################################");
		
		if (gUseLogFile) LogDiagnostic( 80, "-out: Use log file: %s.",gLogFile); else LogDiagnostic( 80,"Do not use a log file.");
		
		kprintf ( "Iterate each test from %i",gMinOuterIterations);
		kprintf ( " to %i,", gMaxOuterIterations);
		kprintf ( " in increments of %i.",gIncrement1); 
		kprintf ( "Within each test, iterate the call from %i to %i.",gMinInnerIterations,gMaxInnerIterations);
		
		kprintf ( "-name: Use %s as list name.", gListName);
		kprintf ( "-nodename: Use %s as node name.", gNodeName);
		kprintf ( "-priority: Use %i as new node priority.", gPriority);
		
		LogDiagnostic( 80, "-error: ");
		switch (gCheckErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80, "-ierror: ");
		switch (gCheckIncidentalErrors)
			{
			case kNone:
				LogDiagnostic( 80,"Don't check for incidental errors.");
				break;
				
			case kCont:
				LogDiagnostic( 80,"When an incidental error occurs, report it and continue.");
				break;
				
			case kStop:
				LogDiagnostic( 80,"Stop the tool on the first incidental error to occur.");
				break;
			
			} /* end switch */

		LogDiagnostic( 80,"-time: ");
		if (gOutputTiming) LogDiagnostic( 80,"Output timing information."); else LogDiagnostic( 80,"Don't output timing information.");

		LogDiagnostic( 80,"-func: ");
		if (gVerify) LogDiagnostic( 80,"Verify functionality."); else LogDiagnostic( 80,"Don't verify functionality.");

		LogDiagnostic( 80,"-fault: ");
		if (gCheckFault) LogDiagnostic( 80,"Skip known assumptions."); else LogDiagnostic( 80,"Make known assumptions.");

		LogDiagnostic( 80,"-sel: ");
		switch (gCallSelector)
			{
			case kFullTest:
				LogDiagnostic( 80,"Run the entire test suite.");
				break;
				
			case kFindNamedNode:
				LogDiagnostic( 80,"Test FindNamedNode.");
				break;
				
			case kFirstNode:
				LogDiagnostic( 80,"Test FirstNode.");
				break;
				
			case kInitList:
				LogDiagnostic( 80,"Test InitList.");
				break;
				
			case kIsEmptyList:
				LogDiagnostic( 80,"Test IsEmptyList.");
				break;
				
			case kIsNode:
				LogDiagnostic( 80,"Test IsNode.");
				break;
				
			case kIsNodeB:
				LogDiagnostic( 80,"Test IsNodeB.");
				break;
				
			case kLastNode:
				LogDiagnostic( 80,"Test LastNode.");
				break;
				
			case kNextNode:
				LogDiagnostic( 80,"Test NextNode.");
				break;
				
			case kPrevNode:
				LogDiagnostic( 80,"Test PrevNode.");
				break;
				
			} /* end switch */
		
		kprintf ( "#################################################################");
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	strcpy(gListName, kDefaultListName);
	strcpy(gNodeName, kDefaultNodeName);
	gPriority = kDefaultPriority;
	
	gVerify = true;
	gCheckFault = true;

	return kRunTest;
}	/* end GetParams */



/******************************
  *
  * Test routines for FindNamedNode
  *
  ******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeFindNamedNodeVerifyStr	"Before iteration (%i)/\tlist/ $%x,\tname/ %s"
#define		kAfterFindNamedNodeVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyFindNamedNode ( Boolean beforeCall, long aIndx, Node *aResult, List *aList, char *aName )
/* Functionality verification for FindNamedNode */
{
	Boolean succeeded;
	char *actualName;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeFindNamedNodeVerifyStr, aIndx, aList, printable(aName));
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterFindNamedNodeVerifyStr, aIndx, aResult);
		if ( aResult == NULL )
			return false;
			
		actualName = aResult->n_Name;
		succeeded = ( strcmp(aName, actualName) == 0 );
		if (!succeeded)
			LogComment("!! Actual name: %s", printable(actualName));
			
		return ( succeeded );
	}
	
}

#define		CLEANUP		CleanupList(aList, aNode)

long CallFindNamedNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Node *aNode = NULL;
	List *aList = NULL;
	Node *aResult = NULL;
	char *aName = gNodeName;
	
	/***** Preparation for call iterations *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	aNode->n_Name = aName;
	
	/* Add the node */
	AddTail( aList, aNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFindNamedNode ( kBeforeCall, minInnerIterations, aResult, aList, aName );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = FindNamedNode( aList, aName);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyFindNamedNode ( kAfterCall, indx, aResult, aList, aName );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPFindNamedNode


#define	TEST_ONE_FINDNAMEDNODE(expVal, list, name)	if (name != NULL) sprintf( gTestName, "%s |  name = '%s'", gTestAreaName, name);	\
											else sprintf( gTestName, "%s | name = NULL", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneFindNamedNode(expVal, list, name); \
											

long OneFindNamedNode (Boolean expectError, List *aList, char *aName )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Node *aResult = NULL;

	/* Ancillary variables */
	/* (none) */

  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFindNamedNode ( kBeforeCall, gCurrentIndex, aResult, aList, aName );

	aResult = FindNamedNode( aList, aName);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyFindNamedNode ( kAfterCall, gCurrentIndex, aResult, aList, aName );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPFindNamedNode;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPFindNamedNode;
	
	return 0;
}

#define kNodeName "TestNode"
#define k2ndNodeName "TestNode2"

Boolean ExerciseFindNamedNode( void )
{
	char	*callName;
	List *aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "FindNamedNode";
	STATUS(callName);
	
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	/***** Empty list *****/
	LogDiagnostic( 12,"-- Empty list --");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, " ");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "W");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "w");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "5");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "This is quite an unreasonably lengthy name");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, kNodeName);

	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = kNodeName;
	
	/* Add the node */
	AddTail( aList, aNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	
	/***** Node in list *****/
	gCurrentIndex = 0;
	LogDiagnostic( 12,"-- Node %s in list --", kNodeName);
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, " ");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "W");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "w");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "5");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "This is quite an unreasonably lengthy name");
	TEST_ONE_FINDNAMEDNODE(kExpectSuccess, aList, kNodeName);

	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	a2ndNode->n_Name = k2ndNodeName;

	/* Add the 2nd node */
	AddTail( aList, a2ndNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				FreeNode(a2ndNode);
				return FALSE;
			}
		}
	}
	
	/***** 2nd Node in list *****/
	gCurrentIndex = 0;
	LogDiagnostic( 12,"-- 2nd Node (%s) in list --", k2ndNodeName);
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, " ");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "W");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "w");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "5");
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, "This is quite an unreasonably lengthy name");
	TEST_ONE_FINDNAMEDNODE(kExpectSuccess, aList, kNodeName);
	TEST_ONE_FINDNAMEDNODE(kExpectSuccess, aList, k2ndNodeName);

	/***** Change 2nd node name to same as first *****/
	a2ndNode->n_Name = kNodeName;
	gCurrentIndex = 0;
	LogDiagnostic( 12,"-- After changing 2nd node name to %s --", kNodeName);
	TEST_ONE_FINDNAMEDNODE(kExpectSuccess, aList, kNodeName);
	TEST_ONE_FINDNAMEDNODE(kExpectError, aList, k2ndNodeName);
	

	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestFindNamedNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseFindNamedNode();
	else
	{
		callName = "FindNamedNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallFindNamedNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for FirstNode
  *
  ******************************/

Boolean		VerifyFirstNode ( Boolean beforeCall, long aIndx, NodeP aResult, ListP aList )
/* Functionality verification for FirstNode */
{
	static NodeP expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeTestNodeVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterTestNodeVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

long CallFirstNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
/*
	CDE: This test is a little different, in that:
	
	 the first call needs to be made just after the list has been initialized, so it is outside the iteration loop,
		but still counts as an iteration.
		
	 the subsequent calls are made after a single node has been added to the list
	
*/
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP aResult = NULL;

	/***** Preparation for call iterations *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* FirstNode is a macro defined thusly: */
	aNode = (NodeP) (aList->ListAnchor.head.links.flink);
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFirstNode ( kBeforeCall, minInnerIterations, aNode, aList );

	startTime = 0; 
	
	aResult = FirstNode( aList);

	endTime = 0;
	accumulator += endTime - startTime;

	if (gVerify) theError = !VerifyFirstNode ( kAfterCall, minInnerIterations, aResult, aList );

	/* Error-checking */
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			LogComment(kCallErrorStr, indx);
			if (gCheckErrors == kStop) return kBreakTime;
		}
	}
	
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Add the node */
	AddTail( aList, aNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	
	minInnerIterations++;
	if ( minInnerIterations > maxInnerIterations )
		return accumulator;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFirstNode ( kBeforeCall, minInnerIterations, aNode, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = FirstNode( aList);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyFirstNode ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPFirstNode


long OneFirstNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	NodeP aResult = NULL;

	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFirstNode ( kBeforeCall, gCurrentIndex, aNode, aList );

	aResult = FirstNode( aList);

	if (gVerify) theError = !VerifyFirstNode ( kAfterCall, gCurrentIndex, aResult, aList );

	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPFirstNode;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPFirstNode;
	
	return 0;
}

#define	TEST_ONE_FIRSTNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneFirstNode(expVal, list, node); \
											
Boolean ExerciseFirstNode( void )
{
	char	*callName;
	ListP aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "FirstNode";
	STATUS(callName);
	
	/***** With empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = kNodeName;
	
	LogDiagnostic( 12,"-- Before adding node --");
	TEST_ONE_FIRSTNODE(kExpectError, aList, aNode);	/* before adding the node */
	
	/***** With one node *****/
	AddTail( aList, aNode);
	LogDiagnostic( 12,"-- After adding node --");
	TEST_ONE_FIRSTNODE(kExpectSuccess, aList, aNode);
	
	/***** With two nodes *****/
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	a2ndNode->n_Name = k2ndNodeName;
	
	/* Add the node */
	AddTail( aList, a2ndNode);
	LogDiagnostic( 12,"-- After adding 2nd node --");
	TEST_ONE_FIRSTNODE(kExpectSuccess, aList, aNode);

	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestFirstNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseFirstNode();
	else
	{
		callName = "FirstNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallFirstNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for InitList
  *
  ******************************/

#define		kBeforeInitListVerifyStr	"Before iteration (%i)/ list/ $%x, name/ %s"
#define		kAfterInitListVerifyStr		"After iteration (%i)."

Boolean		VerifyInitList ( Boolean beforeCall, long aIndx, List *aList, char *aName )
/* Functionality verification for InitList */
{
	
	Boolean succeeded = true;
	char *actualName;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeInitListVerifyStr, aIndx, aList, printable(aName));
		return true;
	}
	else
	{
#if 0
/* removed for M2 cause lists are no longer named! */

		LogDiagnostic( 12,kAfterInitListVerifyStr, aIndx);
		actualName = aList->l.n_Name;

		if ( aName != NULL )
			succeeded = ( strcmp(aName, actualName) == 0 );
		else
			succeeded = false;
		if (!succeeded)
			LogComment("!! Actual name = %s", printable(actualName));

		/* DEBUG */
		/* InspectList(aList); */
#endif
		
		return ( succeeded );
	}
	
}

long CallInitList (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	List	*aList = NULL;
	char	*aName = gListName;
	int32	aSize = sizeof(List);
	uint32	aMemType = MEMTYPE_NORMAL;
	
	/***** Preparation for call iterations *****/
	
	/* Allocate and clear the list */
	aList = (List *) AllocMem( aSize, aMemType );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL )
		{
			LogComment(kIncAllocMemErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	memset(aList, 0, aSize);
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyInitList ( kBeforeCall, minInnerIterations, aList, aName );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		InitList (aList, aName);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyInitList ( kAfterCall, indx, aList, aName );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			theError = FALSE;
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}



long OneInitList ( Boolean expectError, char *aName, uint32 aMemType )
/* Individual call for parameter-testing */
{
 	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	List	*aList = NULL;
	int32	aSize = sizeof(List);
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Allocate and clear the list */
	aList = (List *) AllocMem( aSize, aMemType );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL )
		{
			LogComment(kIncAllocMemErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	memset(aList, 0, aSize);
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyInitList ( kBeforeCall, gCurrentIndex, aList, aName );

	InitList (aList, aName);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyInitList ( kAfterCall, gCurrentIndex, aList, aName );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	FreeList(aList);

	return 0;
}


#define	TEST_ONE_EXERCISE_NAME(expVal, name, type) sprintf( gTestName, "%s | ", gTestAreaName);	\
											if (type == MEMTYPE_NORMAL) strcat(gTestName, "MEMTYPE_NORMAL");	\
											if (name != NULL) \
											{	\
												strcat(gTestName, " name = '");	\
												strcat(gTestName, name);		\
												strcat(gTestName, "' ");		\
											}	\
											else strcat(gTestName, " name = NULL");	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneInitList(expVal, name, type); \
											



#define EXERCISE_NAME(type)		TEST_ONE_EXERCISE_NAME(kExpectSuccess, NULL, type);\
								TEST_ONE_EXERCISE_NAME(kExpectSuccess, "", type);\
								TEST_ONE_EXERCISE_NAME(kExpectSuccess, " ", type);\
								TEST_ONE_EXERCISE_NAME(kExpectSuccess, "W", type);\
								TEST_ONE_EXERCISE_NAME(kExpectSuccess, "w", type);\
								TEST_ONE_EXERCISE_NAME(kExpectSuccess, "5", type);\
								TEST_ONE_EXERCISE_NAME(kExpectSuccess, "This is quite an unreasonably lengthy name", type)
	
void ExerciseInitList( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "InitList";
	STATUS(callName);
	
	/* params: name, memtype */
	LogDiagnostic( 12,"-- For MEMTYPE_NORMAL, varying name --");
	EXERCISE_NAME(MEMTYPE_NORMAL);
	
/*
	...
 Invalid type
 low memory
*/
}

void TestInitList( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseInitList();
	else
	{
		callName = "InitList";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
			
			LogDiagnostic( 80," before CallInitList indx = %d", indx);
			
				theTime = CallInitList(indx, gMinInnerIterations, gMaxInnerIterations);
			LogDiagnostic( 80," before CallInitList indx = %d", indx);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for IsEmptyList
  *
  ******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeIsEmptyListVerifyStr	"Before iteration (%i)/\tlist/ $%x,\texpected result/ $%x"
#define		kAfterIsEmptyListVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean		VerifyIsEmptyList ( Boolean beforeCall, long aIndx, Boolean aResult, List *aList )
/* Functionality verification for IsEmptyList */
{
	static Boolean expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeIsEmptyListVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterIsEmptyListVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

long CallIsEmptyList (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
/*
	CDE: This test is a little different, in that:
	
	 the first call needs to be made just after the list has been initialized, so it is outside the iteration loop,
		but still counts as an iteration.
		
	 the subsequent calls are made after a single node has been added to the list
	
*/
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Node *aNode = NULL;
	List *aList = NULL;
	Boolean aResult = true;

	/***** Preparation for call iterations *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsEmptyList ( kBeforeCall, minInnerIterations, TRUE, aList );

	startTime = 0; 
	
	aResult = IsEmptyList( aList);

	endTime = 0;
	accumulator += endTime - startTime;

	if (gVerify) theError = !VerifyIsEmptyList ( kAfterCall, minInnerIterations, aResult, aList );

	/* Error-checking */
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			LogComment(kCallErrorStr, indx);
			if (gCheckErrors == kStop) return kBreakTime;
		}
	}
	
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Add the node */
	AddTail( aList, aNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	
	minInnerIterations++;
	if ( minInnerIterations > maxInnerIterations )
		return accumulator;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsEmptyList ( kBeforeCall, minInnerIterations, FALSE, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = IsEmptyList( aList);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyIsEmptyList ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPIsEmptyList


#define	TEST_ONE_ISEMPTYLIST(expVal, list)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneIsEmptyList(expVal, list); \
											


long OneIsEmptyList ( Boolean expectError, ListP aList )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Boolean aResult = true;

	/* Ancillary variables */
	/* (none) */

  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsEmptyList ( kBeforeCall, gCurrentIndex, TRUE, aList );

	aResult = IsEmptyList( aList);

	if (gVerify) theError = !VerifyIsEmptyList ( kAfterCall, gCurrentIndex, aResult, aList );

	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPIsEmptyList;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPIsEmptyList;
	
	return 0;
}

Boolean ExerciseIsEmptyList( void )
{
	char	*callName;
	ListP	aList;
	NodeP	aNode;
	
	gCurrentIndex = 0;
	
	callName = "IsEmptyList";
	STATUS(callName);
	
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	/* params: (none) */
	LogDiagnostic( 12,"-- Before adding node --");
	TEST_ONE_ISEMPTYLIST(kExpectSuccess, aList);
	
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	
	/* Add the node */
	AddTail( aList, aNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	
	LogDiagnostic( 12,"-- After adding node --");
	TEST_ONE_ISEMPTYLIST(kExpectError, aList);
	
	CleanupList(aList, aNode);
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestIsEmptyList( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseIsEmptyList();
	else
	{
		callName = "IsEmptyList";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallIsEmptyList(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for IsNode
  *
  ******************************/

Boolean		VerifyIsNode ( Boolean beforeCall, long aIndx, Boolean aResult, List *aList )
/* Functionality verification for IsNode */
{
	static Boolean expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeTestNodeVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterTestNodeVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

long CallIsNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Node *aNode = NULL;
	List *aList = NULL;
	Boolean aResult = true;
	
	/***** Preparation for call iterations *****/

	if ( !SetupList( &aList, &aNode) ) return kBreakTime;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsNode ( kBeforeCall, minInnerIterations, TRUE, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = IsNode( aList, aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyIsNode ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define	TEST_ONE_ONEISNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneIsNode(expVal, list, node); \
											

long OneIsNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
 	Boolean theError = FALSE;
	
	/* Return value */
	Boolean aResult = true;
	
	/* Ancillary variables */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsNode ( kBeforeCall, gCurrentIndex, TRUE, aList );

	aResult = IsNode( aList, aNode);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyIsNode ( kAfterCall, gCurrentIndex, aResult, aList );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	return 0;
}

Boolean ExerciseIsNode( void )
{
	char	*callName;
	Node *aNode = NULL;
	List *aList = NULL;

	gCurrentIndex = 0;
	
	callName = "IsNode";
	STATUS(callName);
	
	if ( !SetupList( &aList, &aNode) ) return FALSE;
	
	/* params: (none: UNIMPLEMENTED) */
	LogDiagnostic( 12,"-- Tail anchor --");
	TEST_ONE_ONEISNODE(kExpectError, aList, (NodeP) &((aList)->ListAnchor.head.links.blink));

	LogDiagnostic( 12,"-- Real node --");
	TEST_ONE_ONEISNODE(kExpectSuccess, aList, aNode);

	LogDiagnostic( 12,"-- Head anchor (should use IsNodeB) --");
	TEST_ONE_ONEISNODE(kExpectSuccess, aList, (NodeP) &((aList)->ListAnchor.head.links.flink));
	/* CAVEAT? */
	
	CleanupList(aList, aNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestIsNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseIsNode();
	else
	{
		callName = "IsNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallIsNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for IsNodeB
  *
  ******************************/

Boolean		VerifyIsNodeB ( Boolean beforeCall, long aIndx, Boolean aResult, List *aList )
/* Functionality verification for IsNodeB */
{
	static Boolean expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeTestNodeVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterTestNodeVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

long CallIsNodeB (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Node *aNode = NULL;
	List *aList = NULL;
	Boolean aResult = true;
	
	/* Preparation for call iterations */
	if ( !SetupList( &aList, &aNode) ) return kBreakTime;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsNodeB ( kBeforeCall, minInnerIterations, TRUE, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = IsNodeB( aList, aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyIsNodeB ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}


#define	TEST_ONE_ONEISNODEB(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneIsNodeB(expVal, list, node); \
											

long OneIsNodeB ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	Boolean aResult = true;
	
	/* Ancillary variables */
	/* (none) */

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyIsNodeB ( kBeforeCall, gCurrentIndex, TRUE, aList );

	aResult = IsNodeB( aList, aNode);
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyIsNodeB ( kAfterCall, gCurrentIndex, aResult, aList );
	
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	return 0;
}

Boolean ExerciseIsNodeB( void )
{
	char	*callName;
	Node *aNode = NULL;
	List *aList = NULL;

	gCurrentIndex = 0;
	
	callName = "IsNodeB";
	STATUS(callName);
	
	if ( !SetupList( &aList, &aNode) ) return FALSE;

	/* params: (none: UNIMPLEMENTED)*/
	LogDiagnostic( 12,"-- Head anchor --");
	TEST_ONE_ONEISNODEB(kExpectError, aList, (NodeP) &((aList)->ListAnchor.head.links.flink));

	LogDiagnostic( 12,"-- Real node --");
	TEST_ONE_ONEISNODEB(kExpectSuccess, aList, aNode);

	LogDiagnostic( 12,"-- Tail anchor (should use IsNode) --");
	TEST_ONE_ONEISNODEB(kExpectSuccess, aList, (NodeP) &((aList)->ListAnchor.head.links.blink));
	/* CAVEAT? */
	
	CleanupList(aList, aNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestIsNodeB( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseIsNodeB();
	else
	{
		callName = "IsNodeB";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallIsNodeB(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for LastNode
  *
  ******************************/

Boolean		VerifyLastNode ( Boolean beforeCall, long aIndx, NodeP aResult, ListP aList )
/* Functionality verification for LastNode */
{
	static NodeP expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeTestNodeVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterTestNodeVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

long CallLastNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
/*
	CDE: This test is a little different, in that:
	
	 the first call needs to be made just after the list has been initialized, so it is outside the iteration loop,
		but still counts as an iteration.
		
	 the subsequent calls are made after a single node has been added to the list
	
*/
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	NodeP aNode = NULL;
	ListP aList = NULL;
	NodeP aResult = NULL;

	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* LastNode is a macro defined thusly: */
	aNode = (NodeP) (aList->ListAnchor.tail.links.blink);
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyLastNode ( kBeforeCall, minInnerIterations, aNode, aList );

	startTime = 0; 
	
	aResult = LastNode( aList);

	endTime = 0;
	accumulator += endTime - startTime;

	if (gVerify) theError = !VerifyLastNode ( kAfterCall, minInnerIterations, aResult, aList );

	/* Error-checking */
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			LogComment(kCallErrorStr, indx);
			if (gCheckErrors == kStop) return kBreakTime;
		}
	}
	
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return kBreakTime;
			}
		}
	}
	
	/* Add the node */
	AddTail( aList, aNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	
	minInnerIterations++;
	if ( minInnerIterations > maxInnerIterations )
		return accumulator;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyLastNode ( kBeforeCall, minInnerIterations, aNode, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = LastNode( aList);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyLastNode ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Clean up */
	CLEANUP;
				
	return accumulator;
}

#define		CLEANUPLastNode


#define	TEST_ONE_LASTNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneLastNode(expVal, list, node); \
											

long OneLastNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	NodeP aResult = NULL;

 	/* Ancillary variables */
	/* (none) */

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* (none) */

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyLastNode ( kBeforeCall, gCurrentIndex, aNode, aList );

	aResult = LastNode( aList);

	if (gVerify) theError = !VerifyLastNode ( kAfterCall, gCurrentIndex, aResult, aList );

	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUPLastNode;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	CLEANUPLastNode;
	
	return 0;
}

Boolean ExerciseLastNode( void )
{
	char	*callName;
	ListP aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "LastNode";
	STATUS(callName);
	
	/***** With empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = kNodeName;
	
	LogDiagnostic( 12,"-- Before adding node --");
	TEST_ONE_LASTNODE(kExpectError, aList, aNode);	/* before adding the node */
	
	/***** With one node *****/
	AddTail( aList, aNode);
	LogDiagnostic( 12,"-- After adding node --");
	TEST_ONE_LASTNODE(kExpectSuccess, aList, aNode);
	
	/***** With two nodes *****/
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	a2ndNode->n_Name = k2ndNodeName;
	
	/* Add the node */
	AddTail( aList, a2ndNode);
	LogDiagnostic( 12,"-- After adding 2nd node --");
	TEST_ONE_LASTNODE(kExpectSuccess, aList, a2ndNode);

	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestLastNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseLastNode();
	else
	{
		callName = "LastNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallLastNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for NextNode
  *
  ******************************/

Boolean		VerifyNextNode ( Boolean beforeCall, long aIndx, NodeP aResult, ListP aList )
/* Functionality verification for NextNode */
{
	static NodeP expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeTestNodeVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterTestNodeVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

#define  CLEANUPNextNode	CLEANUP;\
							FreeNode(a2ndNode)
							
long CallNextNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
/*
	CDE: This test is a little different, in that:
	
	 the first call needs to be made just after the list has been initialized, so it is outside the iteration loop,
		but still counts as an iteration.
		
	 the subsequent calls are made after a single node has been added to the list
	
*/
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	ListP aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	NodeP aResult = NULL;

	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* NextNode is a macro defined thusly: */
	aNode = FirstNode(aList);
	aResult = aNode->n_Next;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyNextNode ( kBeforeCall, minInnerIterations, aResult, aList );

	startTime = 0; 
	
	aResult = NextNode( aNode );

	endTime = 0;
	accumulator += endTime - startTime;

	if (gVerify) theError = !VerifyNextNode ( kAfterCall, minInnerIterations, aResult, aList );

	/* Error-checking */
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			LogComment(kCallErrorStr, indx);
			if (gCheckErrors == kStop) return kBreakTime;
		}
	}
	
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	
	/* Add the node */
	AddTail( aList, a2ndNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUPNextNode;
				return kBreakTime;
			}
		}
	}
	
	minInnerIterations++;
	if ( minInnerIterations > maxInnerIterations )
		return accumulator;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyNextNode ( kBeforeCall, minInnerIterations, a2ndNode, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = NextNode( aNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyNextNode ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUPNextNode;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Cleanup */
	CLEANUPNextNode;
				
	return accumulator;
}


#define	TEST_ONE_NEXTNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneNextNode(expVal, list, node); \
											

long OneNextNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
/*
	CDE: This test is a little different, in that:
	
	 the first call needs to be made just after the list has been initialized, so it is outside the iteration loop,
		but still counts as an iteration.
		
	 the subsequent calls are made after a single node has been added to the list
	
*/
{
	Boolean theError = FALSE;
	
	/* Return value */
	NodeP aResult = NULL;

	/* Ancillary variables */
	/* (none)	 */
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* NextNode is a macro defined thusly: */
	aResult = aNode->n_Next;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyNextNode ( kBeforeCall, gCurrentIndex, aResult, aList );

	aResult = NextNode( aNode );

	if (gVerify) theError = !VerifyNextNode ( kAfterCall, gCurrentIndex, aResult, aList );

	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	return 0;
}

Boolean ExerciseNextNode( void )
{
	char	*callName;
	ListP aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "NextNode";
	STATUS(callName);
	
	/***** With empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = kNodeName;
	
	LogDiagnostic( 12,"-- Before adding node --");
	TEST_ONE_NEXTNODE(kExpectSuccess, aList, aNode);
	
	/***** With one node *****/
	AddTail( aList, aNode);
	LogDiagnostic( 12,"-- After adding node --");
	TEST_ONE_NEXTNODE(kExpectSuccess, aList, aNode);
	
	/***** With two nodes *****/
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	a2ndNode->n_Name = k2ndNodeName;
	
	/* Add the node */
	AddTail( aList, a2ndNode);
	LogDiagnostic( 12,"-- After adding 2nd node --");
	TEST_ONE_NEXTNODE(kExpectSuccess, aList, aNode);

	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestNextNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseNextNode();
	else
	{
		callName = "NextNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallNextNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Test routines for PrevNode
  *
  ******************************/

Boolean		VerifyPrevNode ( Boolean beforeCall, long aIndx, NodeP aResult, ListP aList )
/* Functionality verification for PrevNode */
{
	static NodeP expectedResult;
	
	if (beforeCall == kBeforeCall)
	{		
		expectedResult = aResult;
		LogDiagnostic( 12,kBeforeTestNodeVerifyStr, aIndx, aList, expectedResult);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterTestNodeVerifyStr, aIndx, aResult);
		return ( aResult == expectedResult );
	}
	
}

#define		CLEANUPPrevNode		CLEANUPNextNode

long CallPrevNode (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
/*
	CDE: This test is a little different, in that:
	
	 the first call needs to be made just after the list has been initialized, so it is outside the iteration loop,
		but still counts as an iteration.
		
	 the subsequent calls are made after a single node has been added to the list
	
*/
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	ListP aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	NodeP aResult = NULL;

	/* Preparation for call iterations */
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* PrevNode is a macro defined thusly: */
	aNode = LastNode(aList);
	aResult = aNode->n_Prev;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyPrevNode ( kBeforeCall, minInnerIterations, aResult, aList );

	startTime = 0; 
	
	aResult = PrevNode( aNode );

	endTime = 0;
	accumulator += endTime - startTime;

	if (gVerify) theError = !VerifyPrevNode ( kAfterCall, minInnerIterations, aResult, aList );

	/* Error-checking */
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			LogComment(kCallErrorStr, indx);
			if (gCheckErrors == kStop) return kBreakTime;
		}
	}
	
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
	}
	
	/* Add the node */
	AddTail( aList, a2ndNode);
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAddNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUPPrevNode;
				return kBreakTime;
			}
		}
	}
	
	minInnerIterations++;
	if ( minInnerIterations > maxInnerIterations )
		return accumulator;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyPrevNode ( kBeforeCall, minInnerIterations, aNode, aList );

	/* Call iterations */
	for ( indx = minInnerIterations; indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = PrevNode( a2ndNode);
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyPrevNode ( kAfterCall, indx, aResult, aList );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUPPrevNode;
					return kBreakTime;
				}
			}
		}
		
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	/* Cleanup */
	CLEANUPPrevNode;
				
	return accumulator;
}


#define	TEST_ONE_PREVNODE(expVal, list, node)		sprintf( gTestName, "%s", gTestAreaName);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OnePrevNode(expVal, list, node); \
											

long OnePrevNode ( Boolean expectError, ListP aList, NodeP aNode )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	NodeP aResult = NULL;

	/* Ancillary variables */
	/* (none)	 */

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* PrevNode is a macro defined thusly: */
	aResult = aNode->n_Prev;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyPrevNode ( kBeforeCall, gCurrentIndex, aResult, aList );

	aResult = PrevNode( aNode );

	if (gVerify) theError = !VerifyPrevNode ( kAfterCall, gCurrentIndex, aResult, aList );

	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}

	return 0;
}

Boolean ExercisePrevNode( void )
{
	char	*callName;
	ListP aList = NULL;
	NodeP aNode = NULL;
	NodeP a2ndNode = NULL;
	
	gCurrentIndex = 0;
	
	callName = "PrevNode";
	STATUS(callName);
	
	/***** With empty list *****/
	aList = GetAList();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aList == NULL)
		{
			LogComment(kIncAllocListErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	aNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (aNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				FreeList(aList);
				return FALSE;
			}
		}
	}
	aNode->n_Name = kNodeName;
	
	LogDiagnostic( 12,"-- Before adding node --");
	TEST_ONE_PREVNODE(kExpectSuccess, aList, aNode);	/* before adding the node */
	
	/***** With one node *****/
	AddTail( aList, aNode);
	LogDiagnostic( 12,"-- After adding node --");
	TEST_ONE_PREVNODE(kExpectSuccess, aList, aNode);
	
	/***** With two nodes *****/
	a2ndNode = GetANode();
	if (gCheckIncidentalErrors != kNone)
	{
		if (a2ndNode == NULL)
		{
			LogComment(kIncAllocNodeErrorStr);
			if (gCheckIncidentalErrors == kStop)
			{
				CleanupList(aList, aNode);
				return FALSE;
			}
		}
	}
	a2ndNode->n_Name = k2ndNodeName;
	
	/* Add the node */
	AddTail( aList, a2ndNode);
	LogDiagnostic( 12,"-- After adding 2nd node --");
	TEST_ONE_PREVNODE(kExpectSuccess, aList, aNode);

	CleanupList(aList, aNode);
	FreeNode(a2ndNode);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestPrevNode( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExercisePrevNode();
	else
	{
		callName = "PrevNode";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallPrevNode(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory()
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kInitList ) )
	{
		gTestAreaName = "InitList";		/* contains the name of the test    */
		TestInitList(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kFindNamedNode ) )
	{
		gTestAreaName = "FindNamedNode";		/* contains the name of the test    */
		TestFindNamedNode(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kFirstNode ) )
	{
		gTestAreaName = "FirstNode";		/* contains the name of the test    */
		TestFirstNode(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kIsEmptyList ) )
	{
		gTestAreaName = "IsEmptyList";		/* contains the name of the test    */
		TestIsEmptyList(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kIsNode ) )
	{
		gTestAreaName = "IsNode";		/* contains the name of the test    */
		TestIsNode(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kIsNodeB ) )
	{
		gTestAreaName = "IsNodeB";		/* contains the name of the test    */
		TestIsNodeB(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kLastNode ) )
	{
		gTestAreaName = "LastNode";		/* contains the name of the test    */
		TestLastNode(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kNextNode ) )
	{
		gTestAreaName = "NextNode";		/* contains the name of the test    */
		TestNextNode(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kPrevNode ) )
	{
		gTestAreaName = "PrevNode";		/* contains the name of the test    */
		TestPrevNode(gMinOuterIterations, gMaxOuterIterations);
	}
	
}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram()
{
	int32 theError;
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);
	
	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
			
		}
	TermScaffold();
	}
	
 }


