
/*
 * @(#) LumberJackUtils.c 96/10/21 1.2
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <kernel/bitarray.h>


#include <kernel/types.h>
#include <kernel/item.h>
#include <kernel/debug.h>
#include <kernel/list.h>
#include <kernel/device.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
 /* #include "math.h" */
 
#include <kernel/lumberjack.h>

#include "LumberJackUtils.h"

#include <tests/3do_scaf.h>


/* ---------------------- */
/*       FORWARDS         */
/* ---------------------- */
	

/* ---------------------- */
/*        GLOBALS         */
/* ---------------------- */

/* --------------------------------------------------- */

/* --------------------------------------------------- */

/* --------------------------------------------------- */
void	TestCreateLumberjack( void )
{
	Err				err;
	


	BeginTest(BAT, "CreateLumberjack");
	err = CreateLumberjack( NULL );
	if (err >= 0)
	{
		LogResult(PASSED, "CreateLumberjack returns no error");
		BeginTest(BAT, "CreateLumberjack");
		err = DeleteLumberjack();
		if (err < 0)
			LogError(err );
	}
	else
	{
		LogError(err );
		LogResult(FAILED, "CreateLumberjack returns error #%x", err);
	}
		
}

/* --------------------------------------------------- */
void	TestDeleteLumberjack( void )
{
	Err				err;
	


	BeginTest(BAT, "DeleteLumberjack");
	err = CreateLumberjack( NULL );
	if (err >= 0)
	{
		err = DeleteLumberjack();
		if (err < 0)
		{
			LogError(err );
			LogResult(FAILED, "DeleteLumberjack returns no error");
		}
		else
			LogResult(PASSED, "DeleteLumberjack returns no error");
	}
	else
	{
		LogError(err );
		LogResult(UNRESOLVED, "CreateLumberjack returns error #%x", err);
	}
		
}


/* --------------------------------------------------- */
void	TestControlLumberjack( uint32 eventsLogged)
{
	Err				err;


	BeginTest(BAT, "ControlLumberjack( %d )", eventsLogged);
	err = CreateLumberjack( NULL );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "CreateLumberjack returns error #%x", err);
		return;
	}
	
	err = ControlLumberjack( eventsLogged );
	if (err < 0)
	{
		LogError(err );
		LogResult(FAILED, "ControlLumberjack returns error #%x", err);
	}
	else
	{
		LogResult(PASSED, "ControlLumberjack returns no error");
	}
	
	err = DeleteLumberjack();
	if (err < 0)
		LogError(err );
}

/* --------------------------------------------------- */
void	TestObtainLumberjackBuffer( uint32 eventsLogged, TestFunc tf, char *tfName)
{
	Err				err;
	LumberjackBuffer	*logBuff;
	
	BeginTest(BAT, "ObtainLumberjackBuffer Evts = %d; funcName = \"%s\")", eventsLogged, tfName);
	

	err = CreateLumberjack( NULL );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "CreateLumberjack returns error #%x", err);
		return;
	}
	
	err = ControlLumberjack( eventsLogged );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "ControlLumberjack returns error #%x", err);
	}
	else
	{
		tf();
		err = ControlLumberjack( 0 );		/* stop logging events to buffer */
		if (err < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ControlLumberjack(0) returns error #%x - Failed to stop logging", err);
		}
		logBuff = ObtainLumberjackBuffer();
		if (logBuff < 0)
		{
			LogError(err );
			LogResult(FAILED, "ObtainLumberjackBuffer returns error #%x", logBuff);
		}
		else
			LogResult(PASSED, "ObtainLumberjackBuffer returns no error");
			
	}
	
	err = DeleteLumberjack();
	if (err < 0)
		LogError(err );
}


/* --------------------------------------------------- */
void	TestReleaseLumberjackBuffer( uint32 eventsLogged, TestFunc tf, char *tfName)
{
	Err				err;
	
	LumberjackBuffer	*logBuff;
	
	
	BeginTest(BAT, "ReleaseLumberjackBuffer Evts = %d; funcName = \"%s\")", eventsLogged, tfName);
	
	err = CreateLumberjack( NULL );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "CreateLumberjack returns error #%x", err);
		return;
	}
	
	err = ControlLumberjack( eventsLogged );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "ControlLumberjack returns error #%x", err);
	}
	else
	{
		tf();
		err = ControlLumberjack( 0 );		/* stop logging events to buffer */
		if (err < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ControlLumberjack(0) returns error #%x - Failed to stop logging", err);
		}
		logBuff = ObtainLumberjackBuffer();
		if (logBuff < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ObtainLumberjackBuffer returns error #%x", logBuff);
		}
		else
		{
			ReleaseLumberjackBuffer( logBuff );
			LogResult(PASSED, "ReleaseLumberjackBuffer didn't crash");
		}
	}
	
	err = DeleteLumberjack();
	if (err < 0)
		LogError(err );
}

/* --------------------------------------------------- */
void	TestDumpLumberjackBuffer( uint32 eventsLogged, TestFunc tf, char *tfName, char *banner)
{
	Err				err;
	LumberjackBuffer	*logBuff;
	
	BeginTest(BAT, "DumpLumberjackBuffer Evts = %d; funcName = \"%s\")", eventsLogged, tfName);
	

	err = CreateLumberjack( NULL );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "CreateLumberjack returns error #%x", err);
		return;
	}
	
	err = ControlLumberjack( eventsLogged );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "ControlLumberjack returns error #%x", err);
	}
	else
	{
		tf();
		err = ControlLumberjack( 0 );		/* stop logging events to buffer */
		if (err < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ControlLumberjack(0) returns error #%x - Failed to stop logging", err);
		}
		logBuff = ObtainLumberjackBuffer();
		if (logBuff < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ObtainLumberjackBuffer returns error #%x", logBuff);
		}
		else
		{
			DumpLumberjackBuffer( banner, logBuff );
			LogResult(PASSED, "DumpLumberjackBuffer didn't crash");
		}
	}
	
	err = DeleteLumberjack();
	if (err < 0)
		LogError(err );
}


/* --------------------------------------------------- */
void	TestLogEvent( uint32 eventsLogged, TestFunc tf, char *tfName, char *banner)
{
	Err				err;
	LumberjackBuffer	*logBuff;
	
	BeginTest(BAT, "LogEvent Evts = %d; funcName = \"%s\")", eventsLogged, tfName);

	err = CreateLumberjack( NULL );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "CreateLumberjack returns error #%x", err);
		return;
	}
	
	err = ControlLumberjack( eventsLogged );
	if (err < 0)
	{
		LogError(err );
		LogResult(UNRESOLVED, "ControlLumberjack returns error #%x", err);
	}
	else
	{
		tf();
		err = ControlLumberjack( 0 );		/* stop logging events to buffer */
		if (err < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ControlLumberjack(0) returns error #%x - Failed to stop logging", err);
		}
		logBuff = ObtainLumberjackBuffer();
		if (logBuff < 0)
		{
			LogError(err );
			LogResult(UNRESOLVED, "ObtainLumberjackBuffer returns error #%x", logBuff);
		}
		else
		{
			DumpLumberjackBuffer( banner, logBuff );
			LogResult(PASSED, "DumpLumberjackBuffer didn't crash");
		}
	}
	
	err = DeleteLumberjack();
	if (err < 0)
		LogError(err );
}

