
/*
 * @(#) MessageTests.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		MessageTests.c

	Contains:	Message call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/21/93	CDE		Derived from shell version 1.6

	To Do:
*/

#define		USE_4B1			1

/***** Tool description for Intro, Exit, Help and Session Options displays *****/
#define		kToolDescription		"Message Call Tests"

/***** Version string *****/
#define		kVersionStr		"V1.0"

/***** Programmer credit for Help and Session Options displays *****/
#define		kProgrammerName		"Charlie Eckhaus"

/***** Call selection constants *****/
/* HINTS:
		
	 Create one constant for each call, e.g., kCallA corresponds to CallA().
		- Values needn't be sequential but must be  kFullTest (see baseUtils.h)
*/
#define		kGetMsg				1
#define		kGetThisMsg			2
#define		kSendMsg			3
#define		kSendSmallMsg		4
#define		kReplyMsg			5
#define		kReplySmallMsg		6
#define		kWaitPort			7
#if USE_4B1
#define		kCreateMsg			8
#define		kDeleteMsg			9
#define		kCreateMsgPort		10
#define		kDeleteMsgPort		11
#define		kCreateSmallMsg		12
#define		kCreateUniqueMsgPort	13
#endif


/***** Default log filename: *****/
#define		kDefaultLogFile			"MessageTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
#define		kIncrement1Units	"units"
#define		kDefaultIncrement1	1


/***** Other test options *****/
#define		kDefaultMsgPortName	"TestMsgPort"
#define		kDefaultMsgName		"TestMsg"

/***** Include files *****/

#include "threadUtils.h"
#if 0
#include "testGetMsg.h"
#include "testGetThisMsg.h"
#include "testSendMsg.h"
#include "testSendSmallMsg.h"
#include "testReplyMsg.h"
#include "testReplySmallMsg.h"
#include "testWaitPort.h"
#endif
#if 0
#if USE_4B1
#include "testCreateMsg.h"
#include "testDeleteMsg.h"
#include "testCreateMsgPort.h"
#include "testCreateUniqueMsgPort.h"
#include "testDeleteMsgPort.h"
#include "testCreateSmallMsg.h"
#endif
#endif


/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "MessageTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "MessageTests.c";	/* contains the name of the program */
char	* gTestName= "MessageTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */

/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/

/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* Parameter-checking */
Boolean gInvalidArg;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;
long gCheckIncidentalErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */
char	gPortName[64];
char	gMsgName[64];
uint8	gPriority;


/***** Utilities *****/

/**** (End Utilities) ****/



/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory()
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kGetMsg ) )
	{
		gTestAreaName = "GetMsg";		/* contains the name of the test    */
		TestGetMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kGetThisMsg ) )
	{
		gTestAreaName = "GetThisMsg";		/* contains the name of the test    */
		TestGetThisMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSendMsg ) )
	{
		gTestAreaName = "SendMsg";		/* contains the name of the test    */
		TestSendMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kSendSmallMsg ) )
	{
		gTestAreaName = "SendSmallMsg";		/* contains the name of the test    */
		TestSendSmallMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kReplyMsg ) )
	{
		gTestAreaName = "ReplyMsg";		/* contains the name of the test    */
		TestReplyMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kReplySmallMsg ) )
	{
		gTestAreaName = "ReplySmallMsg";		/* contains the name of the test    */
		TestReplySmallMsg(gMinOuterIterations, gMaxOuterIterations);
	}

#if USE_4B1
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateMsg ) )
	{
		gTestAreaName = "CreateMsg";		/* contains the name of the test    */
		TestCreateMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kDeleteMsg ) )
	{
		gTestAreaName = "DeleteMsg";		/* contains the name of the test    */
		TestDeleteMsg(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateUniqueMsgPort ) )
	{
		gTestAreaName = "CreateUniqueMsgPort";		/* contains the name of the test    */
		TestCreateUniqueMsgPort(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateMsgPort ) )
	{
		gTestAreaName = "CreateMsgPort";		/* contains the name of the test    */
		TestCreateMsgPort(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kDeleteMsgPort ) )
	{
		gTestAreaName = "DeleteMsgPort";		/* contains the name of the test    */
		TestDeleteMsgPort(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kCreateSmallMsg ) )
	{
		gTestAreaName = "CreateSmallMsg";		/* contains the name of the test    */
		TestCreateSmallMsg(gMinOuterIterations, gMaxOuterIterations);
	}
	
#endif

}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	
	int			paramIndx = 0;
	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gCheckIncidentalErrors = kDefaultCheckIncidentalErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */
	strcpy(gPortName, kDefaultMsgPortName);
	strcpy(gMsgName, kDefaultMsgName);
	gPriority = CURRENT_TASK_PRIORITY;
	
	gVerify = true;
	gCheckFault = true;


	
	return kRunTest;
}	/* end GetParams */





Boolean	InitProgram()
{
	int32 theError;
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);

	gTestName = (char *)malloc(1024);

	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{
			TestCategory(); 
		}
	}
	TermScaffold();
	
 }






