
/*
 * @(#) TestCreateMsg.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCreateMsg.c

	Contains:	Test routine for CreateMsg().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/28/93	CDE		Derived from TestCreateIOReq

	To Do:
*/


#include "TestCreateMsg.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"
#define		kSysErrStr		"!! %s"

/* Incidental error reporting format: (call name) */
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCreateMsgVerifyStr	"Before iteration (%i)/\tname/ %s,\tpri/ %i,\treplyPort/ $%x"
#define		kAfterCreateMsgVerifyStr	"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr "!! actual %s/ %s"
#define		kNumMismatchStr "!! actual %s/ %i"

Boolean		VerifyCreateMsg ( Boolean beforeCall, long aIndx, Item aResult, char *name, uint8 pri, Item rp )
/* Functionality verification for CreateMsg */
{
	ItemNodeP aItemNode;
	Boolean mismatch = FALSE;
	Boolean anyMismatch = FALSE;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeCreateMsgVerifyStr, aIndx, name, pri, rp);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCreateMsgVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;

		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
		anyMismatch = ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0 ) );
		if (mismatch) LogComment(kStrMismatchStr, "name", aItemNode->n_Name);
		
		anyMismatch |= ( mismatch = ( pri != aItemNode->n_Priority ) );
		if (mismatch) LogComment(kNumMismatchStr, "priority", aItemNode->n_Priority);
		
/* UNIMPLEMENTED: WANT TO CHECK REPLYPORT */
		return ( !anyMismatch );
	}
	
}

#define		CLEANUP		KillItem( aReplyPort )

long CallCreateMsg (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	char	*aPortName = gPortName;
	char	*aMsgName = gMsgName;
	uint8	aPriority = gPriority;
	Item	aReplyPort = (Item) 0;
	
	/***** Functionality verification (pre-call) *****/
	aReplyPort = CreateMessagePort( aPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aReplyPort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	
	if (gVerify) VerifyCreateMsg ( kBeforeCall, minInnerIterations, aResult, aMsgName, aPriority, aReplyPort );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = CreateMsg( aMsgName, aPriority, aReplyPort  );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCreateMsg ( kAfterCall, indx, aResult, aMsgName, aPriority, aReplyPort );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	CLEANUP;
	
	return accumulator;
}

long OneCreateMsg ( Boolean expectError, Item aReplyPort, char *aMsgName, uint8 aPriority )
/* Individual call for parameter-testing */
{
	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	
  	gCurrentIndex++;

	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyCreateMsg ( kBeforeCall, gCurrentIndex, aResult, aMsgName, aPriority, aReplyPort );

	aResult = CreateMsg( aMsgName, aPriority, aReplyPort  );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCreateMsg ( kAfterCall, gCurrentIndex, aResult, aMsgName, aPriority, aReplyPort );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	DeleteMsg(aResult);
	
	return 0;
}


#define	TEST_ONE_CREATEMSG(expVal, port, name, pri)	sprintf( gTestName, "%s | name= '%s'; port= 0x%x; pri= %d", gTestAreaName, name, port, pri);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCreateMsg(expVal, port, name, pri); \
														

#define EXERCISE_NAME(priority,expect)	gCurrentIndex = 0;\
										LogDiagnostic( 12,"-- For priority %i, varying name --", priority);\
										TEST_ONE_CREATEMSG(expect, aReplyPort, "", priority);\
										TEST_ONE_CREATEMSG(expect, aReplyPort, " ", priority);\
										TEST_ONE_CREATEMSG(expect, aReplyPort, "W", priority);\
										TEST_ONE_CREATEMSG(expect, aReplyPort, "w", priority);\
										TEST_ONE_CREATEMSG(expect, aReplyPort, "5", priority);\
										TEST_ONE_CREATEMSG(expect, aReplyPort, "This is quite an unreasonably lengthy name", priority)

Boolean ExerciseCreateMsg( void )
{
	char	*callName;
	Item	aReplyPort = (Item) 0;

	gCurrentIndex = 0;
	
	callName = "CreateMsg";
	STATUS(callName);
	
	aReplyPort = CreateMessagePort( gPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aReplyPort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	
	/* params: replyPort, msgname, priority */
	EXERCISE_NAME(0, kExpectSuccess);
	EXERCISE_NAME(TASK_MIN_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(CURRENT_TASK_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(TASK_MAX_PRIORITY, kExpectSuccess);
	EXERCISE_NAME(255, kExpectSuccess);
	
	KillItem(aReplyPort);
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestCreateMsg( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCreateMsg();
	else
	{
		callName = "CreateMsg";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCreateMsg(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


