
/*
 * @(#) TestCreateMsgPort.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestCreateMsgPort.c

	Contains:	Test routine for CreateMsgPort().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/28/93	CDE		Derived from TestCreateMsg

	To Do:
*/


#include "TestCreateMsgPort.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncLookupErrorStr	"!! Unable to lookup item $%x"
#define		kIncAllocSignalErrorStr	"!! Unable to allocate signal"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/***** Test routines *****/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeCreateMsgPortVerifyStr	"Before iteration (%i)/\tname/ %s,\tpri/ %i,\tsignal/ $%x"
#define		kAfterCreateMsgPortVerifyStr	"After iteration (%i)/\tresult/ $%x"
#define		kStrMismatchStr "!! actual %s/ %s"
#define		kNumMismatchStr "!! actual %s/ %i"

Boolean		VerifyCreateMsgPort ( Boolean beforeCall, long aIndx, Item aResult, char *name, uint8 pri, uint32 signal )
/* Functionality verification for CreateMsgPort */
{
	ItemNodeP aItemNode;
	Boolean mismatch = FALSE;
	Boolean anyMismatch = FALSE;
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeCreateMsgPortVerifyStr, aIndx, name, pri, signal);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterCreateMsgPortVerifyStr, aIndx, aResult);
		if (aResult < 0)
			return false;

		aItemNode = (ItemNodeP) LookupItem( aResult );
		/* Incidental error-checking */
		if (gCheckIncidentalErrors != kNone)
		{
			if (aItemNode == NULL)
			{
				LogComment(kIncLookupErrorStr, aResult);
				if (gCheckIncidentalErrors == kStop) return false;
			}
		}
		
		anyMismatch = ( mismatch = ( strcmp( name, aItemNode->n_Name) != 0) );
		if (mismatch) LogComment(kStrMismatchStr, "name", aItemNode->n_Name);
		
		anyMismatch |= ( mismatch = ( pri != aItemNode->n_Priority ) );
		if (mismatch) LogComment(kNumMismatchStr, "priority", aItemNode->n_Priority);
		
/* UNIMPLEMENTED: WANT TO CHECK SIGNAL */
		return ( !anyMismatch );
	}
	
}

#define		CLEANUP		FreeSignal( aSignal )

long CallCreateMsgPort (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError= FALSE;
	
  	/* Variables for call parameters and return value */
	Item	aResult = (Item) 0;
	char	*aPortName = gPortName;
	uint8	aPriority = gPriority;
	int32	aAllocSignalResult;
	uint32	aSignal;
	
	/***** Functionality verification (pre-call) *****/
	aAllocSignalResult = AllocSignal( 0 );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aAllocSignalResult <= 0)
		{
			LogComment(kIncAllocSignalErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	aSignal = (uint32) aAllocSignalResult;
	
	if (gVerify) VerifyCreateMsgPort ( kBeforeCall, minInnerIterations, aResult, aPortName, aPriority, aSignal );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = CreateMsgPort( aPortName, aPriority, aSignal );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyCreateMsgPort ( kAfterCall, indx, aResult, aPortName, aPriority, aSignal );

		if (aResult >= 0)
			DeleteMsgPort( aResult );
 		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	CLEANUP;
	
	return accumulator;
}

long OneCreateMsgPort ( Boolean expectError, char *aPortName, uint8 aPriority )
/* Individual call for parameter-testing */
{
 	Boolean theError= FALSE;
	
	/* Return value */
	Item	aResult = (Item) 0;

	/* Ancillary variables */
	int32	aAllocSignalResult;
	uint32	aSignal;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aAllocSignalResult = AllocSignal( 0 );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aAllocSignalResult <= 0)
		{
			LogComment(kIncAllocSignalErrorStr);
			if (gCheckIncidentalErrors == kStop) return false;
		}
	}
	aSignal = (uint32) aAllocSignalResult;
	
	/***** Functionality verification (pre-call) *****/
	if (gVerify) VerifyCreateMsgPort ( kBeforeCall, gCurrentIndex, aResult, aPortName, aPriority, aSignal );

	aResult = CreateMsgPort( aPortName, aPriority, aSignal );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyCreateMsgPort ( kAfterCall, gCurrentIndex, aResult, aPortName, aPriority, aSignal );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_CREATEMSGPORT(expVal, name, pri)	sprintf( gTestName, "%s | name= '%s'; pri= %d", gTestAreaName, name, pri);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneCreateMsgPort(expVal, name, pri); \
														


#define EXERCISE_NAME(priority,expect)	gCurrentIndex = 0;\
										LogDiagnostic( 12,"-- For priority %i, varying name --", priority);\
										TEST_ONE_CREATEMSGPORT(expect, "", priority);\
										TEST_ONE_CREATEMSGPORT(expect, " ", priority);\
										TEST_ONE_CREATEMSGPORT(expect, "W", priority);\
										TEST_ONE_CREATEMSGPORT(expect, "w", priority);\
										TEST_ONE_CREATEMSGPORT(expect, "5", priority);\
										TEST_ONE_CREATEMSGPORT(expect, "This is quite an unreasonably lengthy name", priority)

void ExerciseCreateMsgPort( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "CreateMsgPort";
	STATUS(callName);
	
	/* params: portname, priority */
	EXERCISE_NAME(0,kExpectSuccess);
	EXERCISE_NAME(TASK_MIN_PRIORITY,kExpectSuccess);
	EXERCISE_NAME(CURRENT_TASK_PRIORITY,kExpectSuccess);
	EXERCISE_NAME(TASK_MAX_PRIORITY,kExpectSuccess);
	EXERCISE_NAME(255,kExpectSuccess);
/*
	...
 Invalid type
 low memory
*/
}

void TestCreateMsgPort( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseCreateMsgPort();
	else
	{
		callName = "CreateMsgPort";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallCreateMsgPort(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


