
/*
 * @(#) TestGetMsg.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestGetMsg.c

	Contains:	Test routines for GetMsg().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/21/93	CDE		Derived from TestShell.c V1.6

	To Do:
*/


#include "TestGetMsg.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern char gPortName[];

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeGetMsgVerifyStr		"Before iteration (%i)/ port/ $%x"
#define		kAfterGetMsgVerifyStr		"After iteration (%i)/ result/ $%x"

Boolean		VerifyGetMsg ( Boolean beforeCall, long aIndx, Item aResult, Item aMessagePort )
/* Functionality verification for GetMsg */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeGetMsgVerifyStr, aIndx, aMessagePort);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterGetMsgVerifyStr, aIndx, aResult);
		return ( aResult != 0 );
	}
	
}

#define		CLEANUP		KillItem(aMessagePort);\
						KillItem(aMessage)
	
long	CallGetMsg (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Item aResult = 0;
	Item aMessagePort = 0;
	char *aName = gPortName;
	Item aMessage = 0;
	void *aDataPtr = NULL;
	int32 aDataSize = 0;
	
	/* Preparation for call iterations */
	if ( !SetupMessage( &aMessagePort, &aMessage, aName, aDataPtr, aDataSize ) ) return kBreakTime;	

	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifyGetMsg ( kBeforeCall, minInnerIterations, aResult, aMessagePort );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = GetMsg( aMessagePort );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyGetMsg ( kAfterCall, indx, aResult, aMessagePort );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop)
				{
					CLEANUP;
					return kBreakTime;
				}
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	CLEANUP;
	
	return accumulator;
}

long OneGetMsg ( Boolean expectError, Item aMessagePort )
/* Individual call for parameter-testing */
{
 	Boolean theError = FALSE;
	
	
	/* Return value */
	Item aResult = 0;


	/* Ancillary variables */
	/* (none) */

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifyGetMsg ( kBeforeCall, gCurrentIndex, aResult, aMessagePort );

	aResult = GetMsg( aMessagePort );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyGetMsg ( kAfterCall, gCurrentIndex, aResult, aMessagePort );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

#define	TEST_ONE_GETMSG(expVal, msg )		sprintf( gTestName, "%s |  msg= 0x%x", gTestAreaName, msg);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OneGetMsg(expVal, msg); \
											


Boolean ExerciseGetMsg( void )
{
	char	*callName;
	Item	aMessagePort = 0;
	Item	aMessage = 0;
	char	message[64] = "This is the test message";
	void 	*aDataPtr = (void *) message;
	int32	aDataSize = 64;
	int32	aResult;
	
	gCurrentIndex = 0;
	
	callName = "GetMsg";
	STATUS(callName);
	
	
	aMessagePort = CreateMessagePort( gPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessagePort <= 0)
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	
	TEST_ONE_GETMSG(kExpectError, aMessagePort);

	aMessage = CreateMessage(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessage <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop)
			{
				KillItem(aMessagePort);
				return FALSE;
			}
		}
	}
	
	aResult = SendMsg(aMessagePort, aMessage, aDataPtr, aDataSize);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aResult != 0)
		{
			LogComment(kIncSendMsgErrorStr, aResult, aMessagePort, aMessage, aDataPtr, aDataSize);
			if (gCheckIncidentalErrors == kStop)
			{
				CLEANUP;
				return FALSE;
			}
		}
	}
	
	TEST_ONE_GETMSG(kExpectSuccess, aMessagePort);
	
	CLEANUP;
	
	return TRUE;
/*
	...
 Invalid type
 low memory
*/
}

void TestGetMsg( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
		
	if (gCheckFault)
	{
		ExerciseGetMsg();
	}
	else
	{
		callName = "GetMsg";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallGetMsg(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


