
/*
 * @(#) TestSendMsg.c 96/10/21 1.2
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestSendMsg.c

	Contains:	Test routines for SendMsg().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/21/93	CDE		Derived from shell V1.6

	To Do:
*/

#include "TestSendMsg.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern char gPortName[];

/***** Test routines *****/
/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeSendMsgVerifyStr	"Before iteration (%i)/\tmessagePort/ $%x,\tmessage/ $%x,\tdataPtr/ $%x,\tdataSize/ $%x"
#define		kAfterSendMsgVerifyStr	"After iteration (%i)/\tresult/ $%x"

Boolean		VerifySendMsg ( Boolean beforeCall, long aIndx, int32 aResult, Item aMessagePort, Item aMessage, void *aDataPtr, int32 aDataSize )
/* Functionality verification for SendMsg */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeSendMsgVerifyStr, aIndx, aMessagePort, aMessage, aDataPtr, aDataSize);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterSendMsgVerifyStr, aIndx, aResult);
		return ( aResult == 0 );
	}
	
}

#define		CLEANUP

long	CallSendMsg (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	int32 aResult = 0;
	Item aMessagePort = 0;
	char *aName = gPortName;
	Item aMessage = 0;
	void *aDataPtr = NULL;
	int32 aDataSize = 0;
	
	/* Preparation for call iterations */
	aMessagePort = CreateMessagePort( aName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if ( aMessagePort <= 0 )
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	aMessage = CreateMessage(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessage <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifySendMsg ( kBeforeCall, minInnerIterations, aResult, aMessagePort, aMessage, aDataPtr, aDataSize );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = SendMsg( aMessagePort, aMessage, aDataPtr, aDataSize );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifySendMsg ( kAfterCall, indx, aResult, aMessagePort, aMessage, aDataPtr, aDataSize );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneSendMsg ( Boolean expectError, Item aMessagePort, void *aDataPtr, int32 aDataSize )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	int32 aResult = 0;
	
	/* Ancillary variables */
	Item aMessage = 0;
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aMessage = CreateMessage(aMessagePort);
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (aMessage <= 0)
		{
			LogComment(kIncCreateMsgErrorStr, aMessage, aMessagePort);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifySendMsg ( kBeforeCall, gCurrentIndex, aResult, aMessagePort, aMessage, aDataPtr, aDataSize );

	aResult = SendMsg( aMessagePort, aMessage, aDataPtr, aDataSize );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifySendMsg ( kAfterCall, gCurrentIndex, aResult, aMessagePort, aMessage, aDataPtr, aDataSize );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

#define	TEST_ONE_SENDMSG(expVal, port, val1, val2)	sprintf( gTestName, "%s | port= 0x%x; val1= 0x%x; val2= 0x%x", gTestAreaName, port, val1, val2);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneSendMsg(expVal, port, val1, val2); \
														


Boolean ExerciseSendMsg( void )
{
	char	*callName;
	Item aMessagePort = 0;
	char message[64] = "This is the test message";

	gCurrentIndex = 0;
	
	callName = "SendMsg";
	STATUS(callName);
	
	aMessagePort = CreateMessagePort( gPortName );
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if ( aMessagePort <= 0 )
		{
			LogComment(kIncCreateMsgPortErrorStr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}

	/* params: messagePort, dataptr, datasize */
	TEST_ONE_SENDMSG(kExpectSuccess, aMessagePort, (void *) message, 0);
	TEST_ONE_SENDMSG(kExpectSuccess, aMessagePort, (void *) message, 1);
	TEST_ONE_SENDMSG(kExpectSuccess, aMessagePort, (void *) message, strlen(message));
	TEST_ONE_SENDMSG(kExpectSuccess, aMessagePort, (void *) message, 64);
	
	KillItem(aMessagePort);
	
	return TRUE;
/*
	...
 Small message
 Pass-by-reference message
 Buffered message
 With/without another message port  at varying degrees of activity*/
}

void TestSendMsg( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseSendMsg();
	else
	{
		callName = "SendMsg";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallSendMsg(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


