
/*
 * @(#) MiscTests.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	<kernel/random.h>
#include	"BaseUtils.h"

/*
	File:		MiscTests.c

	Contains:	Miscellaneous call tests, Kernel folio

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <2>	 6/25/93	CDE		Derived from shell
		 <1>	 6/22/93	CDE		Version 1.0 of TestShell.c

	To Do:
*/

#define		USE_4B1			1
#define		USE_ffs			0

/***** Version string *****/
#define		kVersionStr		"V1.0"


/***** Call selection constants *****/
#define		kPrintfSysErr				1
#define		kGetSysErr					2
#define		kYield						3
#if USE_ffs
#define		kFfs						5
#endif
#if USE_4B1
#define		kReadHardwareRandomNumber	4
#endif

/***** Default log filename: *****/
#define		kDefaultLogFile			"MiscTests.out"


/***** Increment (the amount by which the outer iteration index is changed) *****/
#define		kIncrement1Units	"n/a"
#define		kDefaultIncrement1	1


/***** Other test options *****/


/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t$%i\t$%i\t%i"
#define		kHexOuterTimingStr	"\t$%x\t$%x\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr		"Error on iteration %i"
#define		kUnexpectedErrorStr	"UNEXPECTED error on iteration %i"
#define		kUnexpectedSuccessStr "UNEXPECTED success on iteration %i"



/***** Include files *****/

#undef	PrintfSysErr




/************************************EXTRA TEST ENV GLOBALS**************************/
char	* gTestAreaName = "MiscTests";
Boolean	gInvalidArg;
Boolean gTestHasExpectedResult = TRUE;

/************************************TEST ENV GLOBALS**************************/
int32	gFlag;
Boolean gContinue;
char	* gProgName = "MiscTests.c";	/* contains the name of the program */
char	* gTestName= "MiscTests";	/* contains the name of the test    */
int32	gTestLevel = BREADTH;			/* contains what level of testing occurs in this test */
int32	gTestType = 0;			/* interactive or automated */


/***********************************************************************************\
*
*	Globals
*
\***********************************************************************************/



/* Should we display status messages to the terminal? */
Boolean gShowStatus;	/* forced TRUE for now */

/* The Item number corresponding to this task */
Item gThisTask;

/* Invalid command line argument */
Boolean		gInvalidArg;

/* log file */
char	gLogFile[64];
Boolean gUseLogFile;
FILE	*gTheFile;

/* iterations */
long gMinInnerIterations;
long gMaxInnerIterations;
long gMinOuterIterations;
long gMaxOuterIterations;

/* error-checking */
long gCheckErrors;

/* functionality verification*/
Boolean gVerify;

/* timing output */
Boolean gOutputTiming;

/* fault checking*/
Boolean gCheckFault;

/* Call selection */
long gCallSelector;

/* Increments */
long gIncrement1;

/* Index for parameter-testing */
long gCurrentIndex;

/* Other test options */


/***** Utilities *****/

#define TestHexToInt32(aString)	LogDiagnostic( 80,"HexToInt32(%s)/ %i", aString, HexToInt32(aString))

/**** (End Utilities) ****/


void	ShowHelp(void);
void	ShowHelp()
/* Show Help for this tool */
{
}

void ShowOptions(void);
void ShowOptions()
/* Show the option settings for this session */
{
}

bool	GetParams(int theArgCount, char** theArgList)
/* Get the tool options from the command line arguments */
{
	TOUCH(theArgCount);
	TOUCH(theArgList);

	gShowStatus = TRUE;
	
/* log file */
	strcpy(gLogFile,kDefaultLogFile);
	gUseLogFile = FALSE;

/* parameter-checking */
	gInvalidArg = FALSE;

/* iterations */
	gMinInnerIterations = kDefaultMinInnerIterations;
	gMaxInnerIterations = kDefaultMaxInnerIterations;
	gMinOuterIterations = kDefaultMinOuterIterations;
	gMaxOuterIterations = kDefaultMaxOuterIterations;

/* other standard options */
	gCheckErrors = kDefaultCheckErrors;
	gVerify = kDefaultVerify;
	gOutputTiming = kDefaultOutputTiming;
	gCheckFault = kDefaultCheckFault;
	gCallSelector = kDefaultCallSelector;

/* increments */
	gIncrement1 = kDefaultIncrement1;

/* other parameters */

	gVerify = true;
	gCheckFault = true;
	
	return kRunTest;
}	/* end GetParams */



/* No cleanup required for these tests */
#define		CLEANUP

/******************************
*
* Test routines for PrintfSysErr
*
******************************/

#define kBeforePrintfSysErrVerifyStr "Before iteration %i/\terr/ $%x"
#define kAfterPrintfSysErrVerifyStr "After iteration %i."

Boolean	VerifyPrintfSysErr ( Boolean beforeCall, long aIndx, Item aErr )
/* Functionality verification for PrintfSysErr */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforePrintfSysErrVerifyStr, aIndx, aErr);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterPrintfSysErrVerifyStr, aIndx);
		return true;
	}
	
}

long CallPrintfSysErr (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError;
	
  	/* Variables for call parameters and return value */
	Item aErr = currentOuterIteration;
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyPrintfSysErr ( kBeforeCall, minInnerIterations, aErr );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		PrintfSysErr ( aErr );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyPrintfSysErr ( kAfterCall, indx, aErr );
		
#if 0
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			theError = FALSE;
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
#endif
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OnePrintfSysErr ( Boolean expectError, Item aErr )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyPrintfSysErr ( kBeforeCall, gCurrentIndex, aErr );

	PrintfSysErr ( aErr );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyPrintfSysErr ( kAfterCall, gCurrentIndex, aErr );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_PRINTFSYSERR(expVal, val)	sprintf( gTestName, "%s | error no.= 0x%x", gTestAreaName, val);	\
											if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
											BeginTest( BAT, gTestName );	\
											OnePrintfSysErr(expVal, val);
											


void ExercisePrintfSysErr( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "PrintfSysErr";
	STATUS(callName);
	
	/* params: error */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0xFFFFFFFF);	/* all bits on */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0x80000000);	/* Just the sign bit */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0xd57b9001);	/* THIS IS AN ERROR RETURNED BY SendMsg */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0xd57b9010);	/* THIS IS THE ERROR RETURNED BY SetItemOwner */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0xd57b9012);	/* THIS IS AN ERROR RETURNED BY SendMsg */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0xd57b900c);	/* THIS IS AN ERROR RETURNED BY CheckIO */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0xd57b900d);	/* THIS IS AN ERROR RETURNED BY CheckIO */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0x0);			/* no bits on */
	TEST_ONE_PRINTFSYSERR(kExpectSuccess, 0x7FFFFFFF);	/* all bits on except sign bit */
/*
Eventually want to exercise each sub-field:
#define ERR_ERRSIZE	(8)
#define ERR_CLASSIZE	(1)
#define ERR_ENVSIZE	(2)
#define ERR_SEVERESIZE	(2)
#define ERR_IDSIZE	(12)
#define ERR_OBJSIZE	(6)
	...
 Invalid type
 low memory
*/
}

void TestPrintfSysErr( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExercisePrintfSysErr();
	else
	{
		callName = "PrintfSysErr";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
			
		if (minOuterIterations > maxOuterIterations)
		{
			for ( indx = minOuterIterations; indx >= maxOuterIterations; indx-= gIncrement1)
			{
				theTime = CallPrintfSysErr(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		}
		else
			for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallPrintfSysErr(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
	
		if (gOutputTiming)
			LogDiagnostic( 12,kHexOuterTimingStr, minOuterIterations, maxOuterIterations, accumulator);
	}
}


/******************************
*
* Test routines for GetSysErr
*
******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeGetSysErrVerifyStr	"Before iteration %i/\t,\t/buffsize/ %i,\terr/ $%x"
#define		kAfterGetSysErrVerifyStr	"After iteration %i/\tresult/ %i,\t,buff/ %s"

Boolean	VerifyGetSysErr ( Boolean beforeCall, long aIndx, int32 aResult, char *aStr, int32 aSize, Item aErr)
/* Functionality verification for GetSysErr */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeGetSysErrVerifyStr, aIndx, aSize, aErr);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterGetSysErrVerifyStr, aIndx, aResult, aStr);
		return true;
	}
	
}

long	CallGetSysErr (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
  	/* Variables for call parameters and return value */
	Item aErr = ( Item ) currentOuterIteration;
	char *aStr[256];
	int32 aResult = 0;
	
	aStr[0] = "\0";
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyGetSysErr( kBeforeCall, minInnerIterations, aResult, (char *) aStr, 256, aErr );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = GetSysErr ( (char *) aStr, 256, aErr );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyGetSysErr( kAfterCall, indx, aResult, (char *) aStr, 256, aErr );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneGetSysErr ( Boolean expectError, Item aErr, int32 aSize )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	int32 aResult = 0;
	
	/* Ancillary variables */
	char *aStr[256];

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	aStr[0] = "\0";
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyGetSysErr( kBeforeCall, gCurrentIndex, aResult, (char *) aStr, aSize, aErr );

	aResult = GetSysErr ( (char *) aStr, aSize, aErr );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyGetSysErr( kAfterCall, gCurrentIndex, aResult, (char *) aStr, aSize, aErr );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_GETSYSERR(expVal, val, size)	sprintf( gTestName, "%s | error no.= 0x%x; size= %d", gTestAreaName, val, size);	\
												if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
												BeginTest( BAT, gTestName );	\
												OneGetSysErr(expVal, val, size);
												


void Exercise_Err( int32 aSize )
{
	TEST_ONE_GETSYSERR(kExpectSuccess, 0xFFFFFFFF, aSize);	/* all bits on */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0x80000000, aSize);	/* Just the sign bit */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0xd57b9001, aSize);	/* THIS IS AN ERROR RETURNED BY SendMsg */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0xd57b9010, aSize);	/* THIS IS THE ERROR RETURNED BY SetItemOwner */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0xd57b9012, aSize);	/* THIS IS AN ERROR RETURNED BY SendMsg */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0xd57b900c, aSize);	/* THIS IS AN ERROR RETURNED BY CheckIO */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0xd57b900d, aSize);	/* THIS IS AN ERROR RETURNED BY CheckIO */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0x0, aSize);			/* no bits on */
	TEST_ONE_GETSYSERR(kExpectSuccess, 0x7FFFFFFF, aSize);	/* all bits on except sign bit */
}

void ExerciseGetSysErr( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "GetSysErr";
	STATUS(callName);
	
	/* params: error */
	Exercise_Err(0);
	Exercise_Err(1);
	Exercise_Err(32);
	Exercise_Err(64);
	Exercise_Err(128);
	Exercise_Err(256);
/*
Eventually, want to exercise in same manner as PrintfSysError
...
 low memory
*/
}

void TestGetSysErr( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseGetSysErr();
	else
	{
		callName = "GetSysErr";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		if (minOuterIterations > maxOuterIterations)
		{
			for ( indx = minOuterIterations; indx >= maxOuterIterations; indx-= gIncrement1)
			{
				theTime = CallGetSysErr(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		}
		else
			for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallGetSysErr(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
	
		if (gOutputTiming)
			LogDiagnostic( 12,kHexOuterTimingStr, minOuterIterations, maxOuterIterations, accumulator);
	}
}


/******************************
*
* Test routines for Yield
*
******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeYieldVerifyStr	"Before iteration (%i)..."
#define		kAfterYieldVerifyStr	"After iteration (%i)."

Boolean	VerifyYield ( Boolean beforeCall, long aIndx )
/* Functionality verification for Yield */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeYieldVerifyStr, aIndx);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterYieldVerifyStr, aIndx);
		return true;
	}
	
}

long	CallYield (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
#if 0
	Boolean theError = FALSE;
#endif
	
  	/* Variables for call parameters and return value */
	
	TOUCH(currentOuterIteration);

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyYield ( kBeforeCall, minInnerIterations );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		Yield();
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) VerifyYield ( kAfterCall, indx );
		
#if 0
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			theError = FALSE;
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
#endif
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneYield ( Boolean expectError )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyYield ( kBeforeCall, gCurrentIndex );

	Yield();
	
	/* Functionality verification (post-call) */
	if (gVerify) VerifyYield ( kAfterCall, gCurrentIndex );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}


#define	TEST_ONE_YIELD(expVal)			sprintf( gTestName, "%s",gTestAreaName);	\
										if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
										BeginTest( BAT, gTestName );	\
										OneYield(expVal);
										

void ExerciseYield( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "Yield";
	STATUS(callName);
	
	/* params: (none: UNIMPLEMENTED) */
	TEST_ONE_YIELD(kExpectSuccess);
/*
	...
 Invalid type
 low memory
*/
}

void TestYield( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseYield();
	else
	{
		callName = "Yield";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			theTime = CallYield(indx, gMinInnerIterations, gMaxInnerIterations);
			accumulator += theTime;
			if (theTime == kBreakTime) break;
		}
	
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr, minOuterIterations, maxOuterIterations, accumulator);
	}
}

#if USE_ffs

/******************************
*
* Test routines for ffs
*
******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeFfsVerifyStr	"Before iteration (%i)..."
#define		kAfterFfsVerifyStr	"After iteration (%i)."

Boolean	VerifyFfs ( Boolean beforeCall, long aIndx )
/* Functionality verification for ffs */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeFfsVerifyStr, aIndx);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterFfsVerifyStr, aIndx);
		return true;
	}
	
}




long	CallFfs (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	int		n;
	long	result;
	uint32	flags;
	
	
		
  	/* Variables for call parameters and return value */
	
	/* Functionality verification (pre-call) */
/*	if (gVerify) VerifyFfs ( kBeforeCall, minInnerIterations ); */

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
	
		flags = 0L;
		result = ffs( flags );
		if (result != 0)
			{
				LogResult( FAILED, "ffs fails result = 0x%x", result );

			}
			else
			{
				LogResult( PASSED, "");
			}
		flags = 1L;
	
		for ( n = 1; n <= 32 ; ++n){
			startTime = 0; 
			sprintf( gTestName, "%s | val= 0x%x", gTestAreaName, flags);
			BeginTest( BAT, gTestName);
			result = ffs( flags );
			if (result != n) 
			{
				LogResult( FAILED, "ffs fails result = 0x%x", result );

			}
			else
			{
				LogResult( PASSED, "");
			}
				
		
			endTime = 0;
			accumulator += endTime - startTime;
			
			flags = flags << 1;
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneFfs ( Boolean expectError )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	/* (none) */
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFfs ( kBeforeCall, gCurrentIndex );

/*	Ffs(); */
	
	/* Functionality verification (post-call) */
	if (gVerify) VerifyFfs ( kAfterCall, gCurrentIndex );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

void ExerciseFfs( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "ffs";
	STATUS(callName);
	
	/* params: (none: UNIMPLEMENTED) */
	CallFfs(0,0,0);
/*
	...
 Invalid type
 low memory
*/
}

void TestFfs( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;


	if (gCheckFault)
		ExerciseFfs();
	else
	{
		callName = "ffs";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			theTime = CallFfs(indx, gMinInnerIterations, gMaxInnerIterations);
			accumulator += theTime;
			if (theTime == kBreakTime) break;
		}
	
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr, minOuterIterations, maxOuterIterations, accumulator);
	}
}

#endif
/* end of ffs stuff */

#if USE_4B1
/******************************
*
* Test routines for ReadHardwareRandomNumber
*
******************************/

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeReadHardwareRandomNumberVerifyStr	"Before iteration (%i)..."
#define		kAfterReadHardwareRandomNumberVerifyStr		"After iteration (%i)/\tresult/ %i"

Boolean	VerifyReadHardwareRandomNumber ( Boolean beforeCall, long aIndx, uint32 aResult )
/* Functionality verification for ReadHardwareRandomNumber */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeReadHardwareRandomNumberVerifyStr, aIndx);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterReadHardwareRandomNumberVerifyStr, aIndx, aResult);
		return true;
	}
	
}

long	CallReadHardwareRandomNumber (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
#if 0
	Boolean theError = FALSE;
#endif
	uint32	aResult = 0;
	
  	/* Variables for call parameters and return value */
	
	TOUCH(currentOuterIteration);

	/* Functionality verification (pre-call) */
	if (gVerify) VerifyReadHardwareRandomNumber ( kBeforeCall, minInnerIterations, aResult );

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = ReadHardwareRandomNumber();
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) VerifyReadHardwareRandomNumber ( kAfterCall, indx, aResult );
		
#if 0
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			theError = FALSE;
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
#endif
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneReadHardwareRandomNumber ( Boolean expectError )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;

	/* Return value */
	uint32	aResult = 0;
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyReadHardwareRandomNumber ( kBeforeCall, gCurrentIndex, aResult );

	aResult = ReadHardwareRandomNumber();
	
	/* Functionality verification (post-call) */
	if (gVerify) VerifyReadHardwareRandomNumber ( kAfterCall, gCurrentIndex, aResult );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_READHDWRRN(expVal)		sprintf( gTestName, "%s",gTestAreaName);	\
										if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
										BeginTest( BAT, gTestName );	\
										OneReadHardwareRandomNumber(expVal);
										

void ExerciseReadHardwareRandomNumber( void )
{
	char	*callName;

	gCurrentIndex = 0;
	
	callName = "ReadHardwareRandomNumber";
	STATUS(callName);
	
	/* params: (none: UNIMPLEMENTED) */
	TEST_ONE_READHDWRRN(kExpectSuccess);
/*
	...
 low memory
*/
}

void TestReadHardwareRandomNumber( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseReadHardwareRandomNumber();
	else
	{
		callName = "ReadHardwareRandomNumber";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			theTime = CallReadHardwareRandomNumber(indx, gMinInnerIterations, gMaxInnerIterations);
			accumulator += theTime;
			if (theTime == kBreakTime) break;
		}
	
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr, minOuterIterations, maxOuterIterations, accumulator);
	}
}
#endif

/******************************
  *
  * Category test
  *
  ******************************/
 
void TestCategory(void);
void TestCategory()
{
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kPrintfSysErr ) )
	{
		gTestAreaName = "PrintfSysErr";		/* contains the name of the test    */
		TestPrintfSysErr(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kGetSysErr ) )
	{
		gTestAreaName = "GetSysErr";		/* contains the name of the test    */
		TestGetSysErr(gMinOuterIterations, gMaxOuterIterations);
	}

	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kYield ) )
	{
		gTestAreaName = "Yield";		/* contains the name of the test    */
		TestYield(gMinOuterIterations, gMaxOuterIterations);
	}

#if USE_ffs
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kFfs ) )
	{
		gTestAreaName = "Ffs";		/* contains the name of the test    */
		TestFfs(gMinOuterIterations, gMaxOuterIterations);
	}
#endif

#if USE_4B1
	if ( ( gCallSelector == kFullTest ) || ( gCallSelector == kReadHardwareRandomNumber ) )
	{
		gTestAreaName = "ReadHardwareRandomNumber";		/* contains the name of the test    */
		TestReadHardwareRandomNumber(gMinOuterIterations, gMaxOuterIterations);
	}
#endif
}


/***********************************************************************************\
*
*	Just Do It!
*
\***********************************************************************************/


Boolean	InitProgram(void);
Boolean	InitProgram()
{
#if 0
	int32 theError;
#endif
	
	gThisTask = CURRENTTASKITEM;

#if 0
	if ( OpenAudioFolio() )
	{
		LogDiagnostic( 80,"Can't open the audio folio!");
		return FALSE;
	}
#endif	
#if 0
	theError = 0;
	if ( theError )
	{
		PrintfSysErr( theError );
		return FALSE;
	}
#endif
	
	return TRUE;
}


int main (int argc, char **argv)
{
	InitScaffold( KERNEL, argc, argv);

	gTestName = (char *)malloc( sizeof(char)*1024 );
	if ( InitProgram() )
	{
		if (GetParams(argc,argv) == kRunTest)
		{

			TestCategory(); 
			
		}
	TermScaffold();
	}
	
	return(0);
 }

