
/*
 * @(#) RawFileUtils.c 96/10/22 1.3
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>



#include <kernel/debug.h>
#include <kernel/device.h>
#include <string.h>
#include <kernel/types.h>
#include <kernel/item.h>
#include <kernel/list.h>
#include <kernel/mem.h>
#include <stdio.h>
#include <stdlib.h>
 /* #include "math.h" */
 #include <file/fileio.h>

#include <tests/3do_scaf.h>



/* ---------------------- */
/*       CONSTANTS        */
/* ---------------------- */
	TagArg tag1[] =
	{
		FILEATTRS_TAG_BLOCKSIZE,  (void *)0x00000010,
		TAG_END,  0
	};



/* ---------------------- */
/*       FORWARDS         */
/* ---------------------- */
void	MakeRFAttrTagStr( char *s, const TagArg	*tags );
							
/* ---------------------- */
/*        GLOBALS         */
/* ---------------------- */


/* ---------------------- */
/*       UTILITIES        */
/* ---------------------- */
#define		MakeRFModeString( s, m )	\
			if ( m == FILEOPEN_READ )				s = "FILEOPEN_READ";	\
			else if ( m == FILEOPEN_WRITE )			s = "FILEOPEN_WRITE";	\
			else if ( m == FILEOPEN_WRITE_NEW )		s = "FILEOPEN_WRITE_NEW";	\
			else if ( m == FILEOPEN_READWRITE )		s = "FILEOPEN_READWRITE";	\
			else if ( m == FILEOPEN_READWRITE_NEW )	s = "FILEOPEN_READWRITE_NEW";	\
			else  s = "FILEOPEN_UNKNOWN_MODE";

/* *********************************** */
/*   OpenRawFile Tests */
/* *********************************** */

/* --------------------------------------------------- */
void	TestOneOpenRawFile(	const char		*fPath,
							FileOpenModes	mode,
							Boolean			expErr)
{
	RawFile	*curFile;
	Err		result;
	char	*modeStr;
	char	testName[1024];

	MakeRFModeString( modeStr, mode );

	result = OpenRawFile( &curFile, fPath, mode );
	
	if ( fPath == NULL ) fPath = "_NULL_PTR_";
	
	sprintf( testName, "OpenRawFile( \"%s\", %s )", fPath, modeStr );
	if (expErr) strcat( testName, " - expErr");
	BeginTest( BAT, testName );
	
	if( (result < 0) && expErr )
		LogResult(PASSED, "OpenRawFile( file, \"%s\", %s )  returns EXPECTED Error", fPath, modeStr );
	else if( (result > 0) && expErr )
	{
		LogResult(FAILED, "OpenRawFile( file, \"%s\", %s )  UNEXPECTED No-Error", fPath, modeStr );
		result = CloseRawFile( curFile );
		if (result < 0)
			LogError(result );
	}
	else if( result < 0 )
	{
		LogError(result );
		LogResult(FAILED, "OpenRawFile( file, \"%s\", %s )  UNEXPECTED Error", fPath, modeStr );
	}
	else
	{
		LogResult(PASSED, "OpenRawFile( file, \"%s\", %s )  EXPECTED No-Error", fPath, modeStr );
		result = CloseRawFile( curFile );
		if (result < 0)
		{
			LogError(result );
			LogComment("Error in CloseRawFile - error = %d", result);
		}
	}
}


/* --------------------------------------------------- */
void TestOpenRawFile( Boolean	ql )
{
	TestOneOpenRawFile( "/remote/foo", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/Try_a_sorta_medium_length_name", FILEOPEN_READWRITE_NEW, FALSE );

	TestOneOpenRawFile( NULL, FILEOPEN_READWRITE_NEW, TRUE );

	TestOneOpenRawFile( "", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/  name with  spaces ", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/ ", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/funny«ƭϪ", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/An_altogether_much_too_long_name_for_this_poor_little_machine_to_handle", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/1234567890", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/12345678901234567890", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/123456789012345678901234567890", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/1234567890123456789012345678901", FILEOPEN_READWRITE_NEW, FALSE );
	TestOneOpenRawFile( "/remote/12345678901234567890123456789012", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/123456789012345678901234567890123", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/1234567890123456789012345678901234567890", FILEOPEN_READWRITE_NEW, TRUE );
	TestOneOpenRawFile( "/remote/12345678901234567890123456789012345678901234567890", FILEOPEN_READWRITE_NEW, TRUE );

	if (ql) return;	
	
}

/* *********************************** */
/*   SetRawFileAttrs Tests */
/* *********************************** */

void	MakeRFAttrTagStr( char *s, const TagArg	*tags )	

{
	char	tStr[64] = "\0";
	int32	n;
	
	s[0] = '\0';
	
	for (n = 0; tags[n].ta_Tag != TAG_END; n++)
	{
		if (tags[n].ta_Tag == FILEATTRS_TAG_FILETYPE)
			sprintf( tStr, " FileType = 0x%x ;", (uint32)(tags[n].ta_Arg) );
		else if (tags[n].ta_Tag == FILEATTRS_TAG_VERSION)
			sprintf( tStr, " Version = 0x%x ;", (uint8)(tags[n].ta_Arg) );
		else if (tags[n].ta_Tag == FILEATTRS_TAG_REVISION)
			sprintf( tStr, " Revision = 0x%x ;", (uint8)(tags[n].ta_Arg) );
		else if (tags[n].ta_Tag == FILEATTRS_TAG_BLOCKSIZE)
			sprintf( tStr, " BlockSize = 0x%x ;", (uint32)(tags[n].ta_Arg) );
			
		strcat( s, tStr );
	}
}
	

/* --------------------------------------------------- */
void	TestOneSetRawFileAttrs(	const char		*fPath,			/* file path and name */
								const TagArg	*tags,			/* attributes to set */
								uint32 			expFileType,	/* these are the values we */
								uint32 			expBlockSize,	/* expect to find after the */
								uint8			expVersion,		/* new attributes are set */
								uint8			expRevision,   
								Boolean			emptyFile,		/* if true use empty file */
								Boolean			expErr)
{
	RawFile		*curFile;
	Err			result;
	Err			err;
	char		modeStr[512];
	FileInfo	fInfo;
	char		testName[1024];
	
	char	*buff = "Now is the time for all good men to come to the aide of their country";
	int32	buffSize = 32;

	MakeRFAttrTagStr( modeStr, tags );
	
	if (emptyFile)
		strcat( modeStr, " -EMPTY");
	else
		strcat( modeStr, " -NOT EMPTY");

	err = OpenRawFile( &curFile, fPath, FILEOPEN_READWRITE_NEW );

	if ( fPath == NULL ) fPath = "_NULL_PTR_";
	
	sprintf( testName, "SetRawFileAttrs( \"%s\", %s )", fPath, modeStr );
	if (emptyFile) strcat( testName, " - emptyFile");
	if (expErr) strcat( testName, " - expErr");
	BeginTest( BAT, testName );

	if( err < 0 )
	{
		LogError(err );
		LogResult(UNRESOLVED, "OpenRawFile( file, \"%s\", FILEOPEN_READWRITE_NEW )  Can't create file", fPath );
	}
	
	if (!emptyFile)
	{
		err = WriteRawFile( curFile, buff, buffSize);
		if( err < 0 )
		{
			LogError(err );
			LogResult(UNRESOLVED, "WriteRawFile( file, \"%s\", %d )  error", buff, buffSize );
		}
	}
	
	result = SetRawFileAttrs( curFile, tags);
	
	if( (result < 0) && expErr )
		LogResult(PASSED, "SetRawFileAttrs \"%s\", %s  returns EXPECTED Error", fPath, modeStr );
	else if( (result > 0) && expErr )
	{
		LogResult(FAILED, "SetRawFileAttrs \"%s\", %s  UNEXPECTED No-Error", fPath, modeStr );
		result = CloseRawFile( curFile );
		if (result < 0)
			LogError(result );
	}
	else if( result < 0 )
	{
		LogError(result );
		LogResult(FAILED, "SetRawFileAttrs( file, \"%s\", %s )  UNEXPECTED Error", fPath, modeStr );
	}
	else
	{
		err = GetRawFileInfo( curFile, &fInfo, sizeof( FileInfo ) );
		if (err < 0)
		{
			LogError(err );
			LogComment("ERROR #0x%x from GetRawFileInfo - file attributes unconfirmed");
			LogResult(UNRESOLVED, "SetRawFileAttrs( file, \"%s\", %s )  GetRawFileInfo Error", fPath, modeStr );
		}
	    else if (fInfo.fi_FileType != expFileType)
		{
			LogComment("ERROR -Actual FileType 0x%x  --  Expected FileType 0x%x", fInfo.fi_FileType, expFileType);
			LogResult(FAILED, "SetRawFileAttrs( file, \"%s\", %s )  UNEXPECTED Error", fPath, modeStr );
		}
	    else if (fInfo.fi_BlockSize != expBlockSize)
		{
			LogComment("ERROR -Actual BlockSize 0x%x  --  Expected BlockSize 0x%x", fInfo.fi_BlockSize, expBlockSize);
			LogResult(FAILED, "SetRawFileAttrs( file, \"%s\", %s )  UNEXPECTED Error", fPath, modeStr );
		}
	    else if (fInfo.fi_Version != expVersion)
		{
			LogComment("ERROR -Actual Version 0x%x  --  Expected Version 0x%x",fInfo.fi_Version, expVersion);
			LogResult(FAILED, "SetRawFileAttrs( file, \"%s\", %s )  UNEXPECTED Error", fPath, modeStr );
		}
	    else if (fInfo.fi_Revision != expRevision)
		{
			LogComment("ERROR -Actual Revision 0x%x  --  Expected Revision 0x%x", fInfo.fi_Revision, expRevision);
			LogResult(FAILED, "SetRawFileAttrs( file, \"%s\", %s )  UNEXPECTED Error", fPath, modeStr );
		}
		else
			LogResult(PASSED, "SetRawFileAttrs( file, \"%s\", %s )  EXPECTED No-Error", fPath, modeStr );
			
		result = CloseRawFile( curFile );
		if (result < 0)
		{
			LogError(result );
			LogComment("Error in CloseRawFile - error = %d", result);
		}
	}
}


/* --------------------------------------------------- */
void TestSetRawFileAttrs( Boolean	ql )
{
												/* type				blockSize			Vers		Rev 		  mt    expErr*/
	
	TestOneSetRawFileAttrs(  "/remote/foo", tag1, (uint32)0x00000000,	(uint32)0x00000010,	(uint8)0x00, (uint8)0x00, TRUE, FALSE);

	if (ql) return;	
	
}


/* --------------------------------------------------- */
void TestOneReadRawFile (	char			*fPath,
							FileOpenModes	mode)
						
{
	
	RawFile	*curFile;
	
	int32	rdsBuff = 0;
	int32	lastRdsBuff;
	int32	actualCHK;
	int32	expectedCHK;
	int32	readCount;
	Err	err;
	char	*modeStr;

	MakeRFModeString( modeStr, mode );
	
	err = OpenRawFile( &curFile, fPath, mode );

	if ( fPath == NULL ) fPath = "_NULL_PTR_";
	
	BeginTest( BAT, "ReadRawFile( \"%s\", %s )", fPath, modeStr );


	if( err < 0 )
	{
		LogError(err );
		LogResult(UNRESOLVED, "OpenRawFile( file, \"%s\", %s )  Can't open file", fPath, modeStr );
	}
	else
	{
		rdsBuff = 0;
		lastRdsBuff = 0;
		actualCHK = 0;
#if 0
		expectedCHK = 0;
		readCount = 0;
#endif
		do {
			actualCHK += lastRdsBuff;
			lastRdsBuff = rdsBuff;				/* increment checksum */
			readCount =  ReadRawFile( curFile, (char *)(&rdsBuff), sizeof( rdsBuff) );
		} while ( readCount > 0 );
		expectedCHK = lastRdsBuff;				/* make sure to get last one */
		
		if (readCount < 0) {
			LogError(readCount );
			LogResult(FAILED, "ReadRawFile Fails  \"%s\", %s  Can't read file", fPath, modeStr );
		}
		
		/* output error message if checksums disagree */
		if ( actualCHK != expectedCHK) {
			LogComment(" - COMPUTED checksum  =  %x    - EXPECTED checksum  =  %x \n", actualCHK, expectedCHK);
			LogResult(FAILED, "ReadRawFile  \"%s\", %s  - CHECKSUMS DISAGREE", fPath, modeStr );
		}
		else {
			LogResult(PASSED, "ReadRawFile  \"%s\", %s  - checksums match", fPath, modeStr );
		}
		
		err = CloseRawFile( curFile );
		if (err < 0)
			LogError(err );
	}
			

}

/* --------------------------------------------------- */
void TestReadRawFile( Boolean	ql )
{
	
	
	BeginTest(BAT|FT|BT, "ReadRawFile");
	TestOneReadRawFile( "/remote/CHK2", FILEOPEN_READ);
	TestOneReadRawFile( "/remote/CHK3", FILEOPEN_READ);
	TestOneReadRawFile( "/remote/CHK4", FILEOPEN_READ);

	if (ql) return;	
	
}

