
/*
 * @(#) TestFreeSignal.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestFreeSignal.c

	Contains:	Test routine for FreeSignal().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/19/93	CDE		Derived from shell version 1.6

	To Do:
*/


#include "TestFreeSignal.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kBeforeFreeSignalVerifyStr		"Before iteration (%i)/\tsignal/ $%x"
#define		kAfterFreeSignalVerifyStr		"After iteration (%i)/\tresult/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern uint32 gSignalMask;

extern Boolean AllocateAllSignals( int32 *allocatedSignals );
/***** Test routines *****/


Boolean		VerifyFreeSignal ( Boolean beforeCall, long aIndx, int32 aResult, uint32 signalMask )
/* Functionality verification for FreeSignal */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeFreeSignalVerifyStr, aIndx, signalMask);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterFreeSignalVerifyStr, aIndx, aResult);
		return (aResult == 0);
	}
	
}

#define		CLEANUP

#define		kIncAllocSignalErrorStr	"!! Unable to allocate signal/ $%x"
long CallFreeSignal (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
	int32 	aResult = 0;

  	/* Variables for call parameters and return value */
	int32	aSignal;
	uint32 aSignalMask = gSignalMask;
	
	TOUCH(currentOuterIteration);

	/* Preparation for call iterations */
	
	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignal);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) theError = !VerifyFreeSignal ( kBeforeCall, minInnerIterations, aResult, aSignalMask );
	
	/* Replace with returned signal */
	aSignalMask = aSignal;

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = FreeSignal( aSignalMask );
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyFreeSignal ( kAfterCall, indx, aResult, aSignalMask );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}

long OneFreeSignal ( Boolean expectError, uint32 aSignalMask )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	int32 	aResult = 0;
	
	/* Ancillary variables */
	/* (none) */
	
  	gCurrentIndex++;
	
	/***** Call preparation *****/
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyFreeSignal ( kBeforeCall, gCurrentIndex, aResult, aSignalMask );
	
	aResult = FreeSignal( aSignalMask );
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyFreeSignal ( kAfterCall, gCurrentIndex, aResult, aSignalMask );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				CLEANUP;
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	CLEANUP;
	
	return 0;
}

#define	TEST_ONE_FREESIGNAL(expVal, sigMask)	sprintf( gTestName, "%s | task= 0x%x; SignalMask = 0x%x", gTestAreaName, sigMask);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneFreeSignal(expVal,sigMask); \
														

Boolean ExerciseFreeSignal( void )
{
	char	*callName;
	long	saveCheckErrors;
	int32	allocatedSignals = 0;
	uint32	aSignalBit;

	gCurrentIndex = 0;
	
	callName = "FreeSignal";
	STATUS(callName);
	
	LogDiagnostic( 12,"-- No bits set: --");
	TEST_ONE_FREESIGNAL(kExpectSuccess, 0);
	LogDiagnostic( 12,"-- All bits set: --");
	TEST_ONE_FREESIGNAL(kExpectError, 0xFFFFFFFF);

	/* should fail to allocate system signals */
	LogDiagnostic( 12,"-- System signals: --");
#if 0
	TEST_ONE_FREESIGNAL(kExpectError, SIGF_MEMLOW);
	TEST_ONE_FREESIGNAL(kExpectError, SIGF_MEMGONE);
#endif
	TEST_ONE_FREESIGNAL(kExpectError, SIGF_ABORT);
	TEST_ONE_FREESIGNAL(kExpectError, SIGF_IODONE);
	TEST_ONE_FREESIGNAL(kExpectError, SIGF_DEADTASK);

	saveCheckErrors = gCheckErrors;
	gCheckErrors = kStop;

	/* allocate all available signals and free collectively */
	LogDiagnostic( 12,"-- All available signals (collectively): --");
	AllocateAllSignals(&allocatedSignals);
	TEST_ONE_FREESIGNAL(kExpectSuccess, allocatedSignals);
	
	/* allocate all available signals and free individually */
	LogDiagnostic( 12,"-- All available signals (individually): --");
	AllocateAllSignals(&allocatedSignals);
	aSignalBit = 0x80000000;
	while (aSignalBit >= 0x100)
	{
		if (aSignalBit & allocatedSignals)
			TEST_ONE_FREESIGNAL(kExpectSuccess, aSignalBit);
		aSignalBit >>= 1;
	}

	gCheckErrors = saveCheckErrors;
	
	return TRUE;
		
/*
	...
 low memory
*/
}

void TestFreeSignal( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseFreeSignal();
	else
	{
		callName = "FreeSignal";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallFreeSignal(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}


