
/*
 * @(#) TestGetCurrentSignals.c 96/10/22 1.3
 */

#include	<tests/3do_scaf.h>	/* added by for 3do_scaf compatibility */
#include	<audio/audio.h>
#include	"BaseUtils.h"

/*
	File:		TestGetCurrentSignals.c

	Contains:	Test routine for GetCurrentSignals().

	Written by:	Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 7/27/93	CDE		Derived from TestFreeSignal

	To Do:
*/


#include "TestGetCurrentSignals.h"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s/"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr					"%s (%i)"
#define		kBeforeGetCurrentSignalsVerifyStr		"Before iteration (%i)..."
#define		kAfterGetCurrentSignalsVerifyStr		"After iteration (%i)/\tresult/ $%x"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s"
#define		kIncAllocSignalErrorStr	"!! Unable to allocate signal $%x, result/ $%x"
#define		kIncSendSignalErrorStr	"!! Unable to send signal $%x, result $%x"

#define		kSendingSignalStr		"Sending signal $%x..."

extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern Boolean gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Other test options */
extern uint32 gSignalMask;

/***** Test routines *****/


Boolean		VerifyGetCurrentSignals ( Boolean beforeCall, long aIndx, uint32 aResult )
/* Functionality verification for GetCurrentSignals */
{
	
	if (beforeCall == kBeforeCall)
	{		
		LogDiagnostic( 12,kBeforeGetCurrentSignalsVerifyStr, aIndx);
		return true;
	}
	else
	{
		LogDiagnostic( 12,kAfterGetCurrentSignalsVerifyStr, aIndx, aResult);
		return true;
	}
	
}

#define		CLEANUP

long CallGetCurrentSignals ( long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime, endTime, indx, accumulator = 0;
	Boolean theError = FALSE;
	
	uint32 	aResult = 0;

  	/* Variables for call parameters and return value */
	int32	aSignal;
	uint32 aSignalMask = gSignalMask;
	
	TOUCH(currentOuterIteration);

	/* Preparation for call iterations */
	
	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal < 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignalMask, aSignal);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyGetCurrentSignals ( kBeforeCall, minInnerIterations, aResult );
	
#if 0
	/* Replace with returned signal */
	aSignalMask = aSignal;
#endif

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		startTime = 0; 
		
		aResult = GetCurrentSignals();
		
		endTime = 0;
		accumulator += endTime - startTime;

		/* Functionality verification (post-call) */
		if (gVerify) theError = !VerifyGetCurrentSignals ( kAfterCall, indx, aResult );
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				LogComment(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}

	if (gOutputTiming)
		LogDiagnostic( 12,kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
	
	return accumulator;
}


long OneGetCurrentSignals ( Boolean expectError )
/* Individual call for parameter-testing */
{
	Boolean theError = FALSE;
	
	/* Return value */
	uint32 	aResult = 0;

  	gCurrentIndex++;
	
	/***** Call preparation *****/
	
	/* Functionality verification (pre-call) */
	if (gVerify) VerifyGetCurrentSignals ( kBeforeCall, gCurrentIndex, aResult );
	
	aResult = GetCurrentSignals();
	
	/* Functionality verification (post-call) */
	if (gVerify) theError = !VerifyGetCurrentSignals ( kAfterCall, gCurrentIndex, aResult );
	
	/* Error-checking */
	gTestHasExpectedResult = TRUE;
	if (gCheckErrors != kNone)
	{
		if (theError)
		{
			if (theError == expectError)
				LogDiagnostic( 12,kCallErrorStr, gCurrentIndex);
			else
			{
				LogDiagnostic( 12,kUnexpectedErrorStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedErrorStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
			if (gCheckErrors == kStop)
			{
				return kBreakTime;
			}
		}
		else
			if (expectError)
			{
				LogDiagnostic( 12,kUnexpectedSuccessStr, gCurrentIndex);
				LogResult( FAILED, kUnexpectedSuccessStr, gCurrentIndex );
				gTestHasExpectedResult = FALSE;
			}
	}
	if (gTestHasExpectedResult)
	{
		LogResult( PASSED, "expected results iteration = %d", gCurrentIndex);
	}
	
	return 0;
}

Boolean DoAllocSignal( uint32 aSignalMask )
{
	int32 aSignal;
	
	aSignal = AllocSignal( aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aSignal <= 0)
		{
			LogComment(kIncAllocSignalErrorStr, aSignalMask, aSignal);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	
	return TRUE;
}

Boolean DoSendSignal( uint32 aSignalMask )
{
	Err aErr;
	
	LogDiagnostic( 12,kSendingSignalStr, aSignalMask);
	aErr = SendSignal( gThisTask, aSignalMask );
	if (gCheckIncidentalErrors != kNone)
	{
		if (aErr < 0)
		{
			LogComment(kIncSendSignalErrorStr, aSignalMask, aErr);
			if (gCheckIncidentalErrors == kStop) return FALSE;
		}
	}
	
	return TRUE;
}
#define	TEST_ONE_GETCURSIGNAL(expVal)					sprintf( gTestName, "%s ", gTestAreaName);	\
														if (expVal == kExpectError) strcat(gTestName, kExpectErrorStr);	\
														BeginTest( BAT, gTestName );	\
														OneGetCurrentSignals(expVal);
														


void ExerciseGetCurrentSignals( void )
{
	char	*callName;
	int32	aFreeResult;
	uint32 aSignalBit;

	gCurrentIndex = 0;
	
	callName = "GetCurrentSignals";
	STATUS(callName);
	
	/* params: (none) */
	LogDiagnostic( 12,"-- Original received signals --");
	TEST_ONE_GETCURSIGNAL(kExpectSuccess);		/* original received signals */
	
	LogDiagnostic( 12,"-- Allocating system signals --");
#if 0
	DoAllocSignal(SIGF_MEMLOW);					/* All of these should fail to allocate a new signal */
	DoAllocSignal(SIGF_MEMGONE);
#endif
	DoAllocSignal(SIGF_ABORT);
	DoAllocSignal(SIGF_IODONE);
	DoAllocSignal(SIGF_DEADTASK);

	LogDiagnostic( 12,"-- Allocating all available signals --");
	while (DoAllocSignal(0));					/* allocate all available signals */
		
	LogDiagnostic( 12,"-- Sending all signals --");
	aSignalBit = 0x4000000;
	while ( aSignalBit != 0x0 )
	{
		DoSendSignal(aSignalBit);
		TEST_ONE_GETCURSIGNAL(kExpectSuccess);
		aSignalBit >>= 1;
	}

	/* Cleanup */
	aFreeResult = FreeSignal(0x7FFFFF00);
	if ( aFreeResult )	/* should free everything */
		LogComment("FreeSignal (all signals) failed: 0x%x", aFreeResult);
		
	LogDiagnostic( 12,"-- After freeing all signals --");
	TEST_ONE_GETCURSIGNAL(kExpectSuccess);
/*
	...
 Invalid type
 low memory
*/
}

void TestGetCurrentSignals( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime, indx, accumulator = 0;
	char	*callName;
		
	if (gCheckFault)
		ExerciseGetCurrentSignals();
	else
	{
		callName = "GetCurrentSignals";
		STATUS(callName);
		if (gOutputTiming)
			LogDiagnostic( 12,kTimingHeaderStr,callName);
	
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
			{
				theTime = CallGetCurrentSignals(indx, gMinInnerIterations, gMaxInnerIterations);
				accumulator += theTime;
				if (theTime == kBreakTime) break;
			}
		
		if (gOutputTiming)
			LogDiagnostic( 12,kOuterTimingStr,minOuterIterations,maxOuterIterations, accumulator);
	}
}

