
/*
 * @(#) TaskUtils.c 96/10/22 1.3
 */

#include <loader/loader3do.h>
#include <kernel/task.h>
#include <stdlib.h>



#include <kernel/debug.h>
#include <tests/3do_scaf.h>


char	*testArgs1[] = {"TM_SumArgs",
					"1",		"2",		"4",		"8",		"16",
					"32",		"64",		"128",		"256",		"512",
					"1024",		"2048",		"4096",		"8192",		"16384",
					"32768",	"65536",	"131072",	"262144",	"524288",
					"1048576",	"2097152",	"4194304",	"8388608",	"16777216",
					"33554432",	"67108864",	"134217728","268435456","536870912",
					"1073741824",		"2147483648"
					};
					
/* --------------------------------------------------- */
/* a stupid little function that returns the sum of the argc arguments passed into it */
int32 SumFunc1 ( uint32 argc, char **argv )
{
	int32	n;
	int32	sum = 0;
	
	
	for( n = 1; n < argc; n++)
		sum += atoi(argv[n]);
	
	return(sum);
}

/* --------------------------------------------------- */
/* a stupid little function that just loops */
void LoopFunc1 ( uint32 argc, char **argv )
{
	TOUCH(argc);
	TOUCH(argv);

	while( 1 );
}

/* ====================================================================	*/
/*					CloseModule		Tests								*/
/* ====================================================================	*/

/* --------------------------------------------------- */
void	TestOneCloseModule(	char			*path,
							OpenModuleTypes	type,
							Boolean			expErr)

{
	Item	result;
	Err		err;
	char	testName[1024];
	
	
	result = OpenModule( path, type, NULL);	
	
	if( path == NULL ) 
		path = "_NULL_PTR_";

	sprintf( testName, "CloseModule path \"%s\", type %d", path, type );
	if (expErr) strcat( testName, " - expErr");
	BeginTest( BAT, testName );
	
	if( result < 0)
	{
		LogError(result );
		LogResult(UNRESOLVED, "Unable to open module" );
		return;
	}

	err = CloseModule(result);
	if (err < 0 )
	{
		LogError(err );
		LogResult(FAILED, "CloseModule path \"%s\", type %d)  return UNEXPECTED error #%x", path, type, err );
	}
	else
		LogResult(PASSED, "CloseModule path \"%s\", type %d)  appears to work OK", path, type );
}


/* --------------------------------------------------- */
void	TestCloseModule( Boolean	ql )
{
	TestOneCloseModule( "/remote/TM_SumArgs", OPENMODULE_FOR_TASK, false );
	TestOneCloseModule( "/remote/TM_SumArgs", OPENMODULE_FOR_THREAD, false );
	
	if (ql) return;
}


/* ====================================================================	*/
/*					OpenModule		Tests								*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneOpenModule(	char			*path,
							OpenModuleTypes	type,
							Boolean			expErr)

{
	Item	result;
	Err		err;
	char	testName[1024];	
	
	result = OpenModule( path, type, NULL);	
	
	if( path == NULL ) 
		path = "_NULL_PTR_";
	
	sprintf( testName, "OpenModule(\"%s\", %d, NULL)", path, type );
	if (expErr) strcat( testName, " - expErr");
	BeginTest( BAT, testName );

	if( (result < 0) && (!expErr) )
	{
		LogError(result );
		LogResult(FAILED, "OpenModule(\"%s\", %d, NULL)  returns UNEXPECTED error #%x", path, type, result);
	}
	else if( (result < 0) && (expErr) )
		LogResult(PASSED, "OpenModule(\"%s\", %d, NULL)  returns expected error #%x", path, type, result);
	else if( expErr)
	{
		LogResult(FAILED, "OpenModule(\"%s\", %d, NULL)  should have returned an error but didn't",path, type);
		if ((err = CloseModule(result)) < 0)
			LogError(err );
	}		
	else 
	{
		LogResult(PASSED, "OpenModule(\"%s\", %d, NULL)  appears to have completed OK", path, type);
		if ((err = CloseModule(result)) < 0)
			LogError(err );
	}

}

/* --------------------------------------------------- */
void	TestOpenModule( Boolean	ql )
{
	TestOneOpenModule( "/remote/TM_SumArgs", OPENMODULE_FOR_TASK, false );
	TestOneOpenModule( "/remote/TM_SumArgs", OPENMODULE_FOR_THREAD, false );
	
	TestOneOpenModule( "", OPENMODULE_FOR_TASK, true );
	TestOneOpenModule( "", OPENMODULE_FOR_THREAD, true );

#if 0
/* commented out for CR # 5243 */
	TestOneOpenModule( NULL, OPENMODULE_FOR_TASK, true );
	TestOneOpenModule( NULL, OPENMODULE_FOR_THREAD, true );
#endif

	if (ql) return;
}

/* ====================================================================	*/
/*					ExecuteModule		Tests							*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneExecuteModule(	char			*path,
								int32			argc,
								char 			**argv,
								OpenModuleTypes	type,
								int32			expRtn)

{
	int32	result;
	Item	mItem;
	Err		err;
	char	testName[1024];
	
	
	mItem = OpenModule( path, type, NULL);
	if( path == NULL ) path = "_NULL_PTR_";
	
	sprintf( testName, "ExecuteModule  path \"%s\", argc= %d, expRtn= %d", path, argc, expRtn );
	BeginTest( BAT, testName );

	if (mItem < 0)
	{
		LogError(mItem );
		LogResult(UNRESOLVED, "OpenModule(\"%s\", %d, NULL)  returns UNEXPECTED error #%x", path, type, mItem);
		return;
	}
	

	result = ExecuteModule( mItem, argc, argv );
	
	if (result == BADITEM)
		LogComment("ExecuteModule retuns BADITEM error");
	

	if( result == expRtn )
		LogResult(PASSED, "ExecuteModule  path \"%s\", argc= %d,  returns  %d", path, argc, result);
	else 
		LogResult(FAILED, "ExecuteModule  path \"%s\", argc= %d, (returned  %d - exp %d)", path, argc, result, expRtn);
		
	if (mItem >= 0)										/* close the module */
		if ((err = CloseModule(mItem)) < 0)
		{
			LogError(err );
			LogComment("CloseModule return error #%x", err);
		}
}

/* --------------------------------------------------- */
void	TestExecuteModule( Boolean	ql )
{
	uint32			n;
		
	for (n =1; n < 4; n++)
	{
		TestOneExecuteModule( "/remote/TM_SumArgs", n, &testArgs1[0], OPENMODULE_FOR_TASK, ( (0x1<<(n-1)) -1 ) );
		TestOneExecuteModule( "/remote/TM_SumArgs", n, &testArgs1[0], OPENMODULE_FOR_THREAD, ( (0x1<<(n-1)) -1 ) );
	}
	
	if (ql) return;

	for (n =4; n < 33; n++)
	{
		TestOneExecuteModule( "/remote/TM_SumArgs", n, &testArgs1[0], OPENMODULE_FOR_TASK, ( (0x1<<(n-1)) -1 ) );
		TestOneExecuteModule( "/remote/TM_SumArgs", n, &testArgs1[0], OPENMODULE_FOR_THREAD, ( (0x1<<(n-1)) -1 ) );
	}
}



/* ====================================================================	*/
/*					CreateTask		Tests								*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneCreateTask(		char			*path,
								int32			argc,
								char 			**argv,
								char			*taskName,
								char			*cmdStr,
								OpenModuleTypes	type,
								int32			expRtn)

{
	Item	tItem;
	Item	mItem;
	Item	msgPort;
	Item	msgItem;
	Err		err;
	TagArg	tag[5];
	int32	tIndx = 0;
	int32	result;
	
	char	testName[1024];
	
	mItem = OpenModule( path, type, NULL);
	if( path == NULL ) path = "_NULL_PTR_";
	if( taskName == NULL ) taskName = "_NULL_PTR_";
	if( cmdStr == NULL ) cmdStr = "_NULL_PTR_";
	
	sprintf( testName, "CreateTask  path \"%s\", argc= %d, taskName \"%s\", cmdStr \"%s\", type= %d expRtn= %d ",
									path, argc, taskName, cmdStr, type, expRtn );
	if( strcmp(taskName, "_NULL_PTR_") == 0 ) taskName = NULL;
	BeginTest( BAT, testName );

	if (mItem < 0)
	{
		LogError(mItem );
		LogResult(UNRESOLVED, "OpenModule(\"%s\", %d, NULL)  returns UNEXPECTED error #%x", path, type, mItem);
		return;
	}
	
	msgPort = CreateMsgPort("CreateTask_MSG_PORT", 0, 0); 
	if (msgPort < 0)
	{
		LogError(msgPort );
		LogResult(UNRESOLVED, "CreateMsgPort returns UNEXPECTED error #%x", msgPort);
		return;
	}
	if( strcmp(cmdStr, "_NULL_PTR_") == 0 ) cmdStr = NULL;
	if( strcmp(path, "_NULL_PTR_") == 0 ) path = NULL;

	
	if (argc != 0)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_ARGC;
		tag[tIndx++].ta_Arg = (void *) argc;
	}
	if (argv != NULL)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_ARGP;
		tag[tIndx++].ta_Arg = (void *) argv;
	}
	if (cmdStr != NULL)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_CMDSTR;
		tag[tIndx++].ta_Arg = (void *) cmdStr;
	}

	tag[tIndx].ta_Tag = CREATETASK_TAG_MSGFROMCHILD;		/* we want our children to stay in touch */
	tag[tIndx++].ta_Arg = (void *) msgPort;

	tag[tIndx].ta_Tag = TAG_END;
	


	tItem = CreateTask( mItem, taskName, &tag[0] );

	if (tItem < 0 )
	{
		LogError(tItem);
		LogResult(FAILED, "CreateTask   path \"%s\" - reurned error #%x", path, tItem);
	}
	else
	{
	
		msgItem = WaitPort( msgPort, 0);				/* Wait for the task to die and kernel to send us a message */
			
		result = MESSAGE(msgItem)->msg_Result;			/* check all the message fields for proper info */
	
		if( MESSAGE(msgItem)->msg_Val1 != tItem )
			LogResult(FAILED, "CreateTask  path \"%s\", argc= %d,  msgVal1: exp %x, act %x", path, argc, MESSAGE(msgItem)->msg_Val1, tItem);
	
		else if( MESSAGE(msgItem)->msg_Val2 != tItem )
			LogResult(FAILED, "CreateTask  path \"%s\", argc= %d,  msgVal2: exp %x, act %x", path, argc, MESSAGE(msgItem)->msg_Val2, tItem);
	
		else if( result == expRtn )
			LogResult(PASSED, "CreateTask  path \"%s\", argc= %d,  returns  %d", path, argc, result);
	
		else 
			LogResult(FAILED, "CreateTask  path \"%s\", argc= %d, (returned  %d - exp %d)", path, argc, result, expRtn);
	
	
		if ((err = DeleteMsg( msgItem )) < 0)					/* lets be nice and delete the message */
		{
			LogError(err);
			LogComment("Error #%x in DeleteMsg", err);
		}
	}
	
	if ((err = DeleteMsgPort( msgPort )) < 0)					/* let's be super nice and delete the message port */
	{
		LogError(err);
		LogComment("Error #%x in DeleteMsgPort", err);
	}

	if (mItem >= 0)										/* close the module */
		if ((err = CloseModule(mItem)) < 0)
		{
			LogError(err );
			LogComment("Error #%x in CloseModule", err);
		}

	
}


/* --------------------------------------------------- */
void	TestCreateTask( Boolean	ql )
{
	uint32			n;
	

	TestOneCreateTask( "/remote/TM_SumArgs", 0, NULL, "TM_SumArgs", "TM_SumArgs 1 2 4 8 16", OPENMODULE_FOR_TASK, 31);

	for (n = 1; n < 4; n++)
	{
		TestOneCreateTask( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_TASK, ( (0x1<<(n-1)) -1 ) );
		TestOneCreateTask( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_THREAD, ( (0x1<<(n-1)) -1 ) );
	}

	
	if (ql) return;

	for (n = 4; n < 33; n++)
	{
		TestOneCreateTask( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_TASK, ( (0x1<<(n-1)) -1 ) );
		TestOneCreateTask( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_THREAD, ( (0x1<<(n-1)) -1 ) );
	}
}


/* ====================================================================	*/
/*					CreateModule	Tests								*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneCreateModuleThread(	char			*path,
									int32			argc,
									char 			**argv,
									char			*taskName,
									char			*cmdStr,
									OpenModuleTypes	type,
									int32			expRtn)

{
	Item	tItem;
	Item	mItem;
	Item	msgPort;
	Item	msgItem;
	Err		err;
	TagArg	tag[5];
	int32	tIndx = 0;
	int32	result;
	char	testName[1024];
	
	mItem = OpenModule( path, type, NULL);
	if( path == NULL ) path = "_NULL_PTR_";
	if( cmdStr == NULL ) cmdStr = "_NULL_PTR_";
	if( taskName == NULL ) taskName = "_NULL_PTR_";
	
	sprintf( testName, "CreateModuleThread  path \"%s\", argc= %d, taskName \"%s\", cmdStr \"%s\", type= %d expRtn= %d ",
									path, argc, taskName, cmdStr, type, expRtn );
	BeginTest( BAT, testName );
	if( strcmp(taskName, "_NULL_PTR_") == 0 ) taskName = NULL;

	if (mItem < 0)
	{
		LogError(mItem );
		LogResult(UNRESOLVED, "OpenModule(\"%s\", %d, NULL)  returns UNEXPECTED error #%x", path, type, mItem);
		return;
	}
	
	msgPort = CreateMsgPort("CreateTask_MSG_PORT", 0, 0); 
	if (msgPort < 0)
	{
		LogError(msgPort );
		LogResult(UNRESOLVED, "CreateMsgPort returns UNEXPECTED error #%x", msgPort);
		return;
	}

	
	if (argc != 0)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_ARGC;
		tag[tIndx++].ta_Arg = (void *) argc;
	}
	if (argv != NULL)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_ARGP;
		tag[tIndx++].ta_Arg = (void *) argv;
	}
	if (cmdStr != NULL)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_CMDSTR;
		tag[tIndx++].ta_Arg = (void *) cmdStr;
	}

	tag[tIndx].ta_Tag = CREATETASK_TAG_MSGFROMCHILD;		/* we want our children to stay in touch */
	tag[tIndx++].ta_Arg = (void *) msgPort;

	tag[tIndx].ta_Tag = TAG_END;
	


	tItem = CreateModuleThread( mItem, taskName, &tag[0] );
	
	if (mItem >= 0)										/* close the module */
		if ((err = CloseModule(mItem)) < 0)
		{
			LogError(err );
			LogComment("Error #%x in CloseModule", err);
		}

	if (tItem < 0 )
	{
		LogError(tItem);
		LogResult(FAILED, "CreateModuleThread   path \"%s\" - reurned error #%x", path, tItem);
	}
	else
	{
	
		msgItem = WaitPort( msgPort, 0);				/* Wait for the task to die and kernel to send us a message */
			
		result = MESSAGE(msgItem)->msg_Result;			/* check all the message fields for proper info */
	
		if( MESSAGE(msgItem)->msg_Val1 != tItem )
			LogResult(FAILED, "CreateModuleThread  path \"%s\", argc= %d,  msgVal1: exp %x, act %x", path, argc, MESSAGE(msgItem)->msg_Val1, tItem);
	
		else if( MESSAGE(msgItem)->msg_Val2 != tItem )
			LogResult(FAILED, "CreateModuleThread  path \"%s\", argc= %d,  msgVal2: exp %x, act %x", path, argc, MESSAGE(msgItem)->msg_Val2, tItem);
	
		else if( result == expRtn )
			LogResult(PASSED, "CreateModuleThread  path \"%s\", argc= %d,  returns  %d", path, argc, result);
	
		else 
			LogResult(FAILED, "CreateModuleThread  path \"%s\", argc= %d, (returned  %d - exp %d)", path, argc, result, expRtn);
	
	
		if ((err = DeleteMsg( msgItem )) < 0)					/* lets be nice and delete the message */
		{
			LogError(err);
			LogComment("Error #%x in DeleteMsg", err);
		}
	}
	
	if ((err = DeleteMsgPort( msgPort )) < 0)					/* let's be super nice and delete the message port */
	{
		LogError(err);
		LogComment("Error #%x in DeleteMsgPort", err);
	}
}


/* --------------------------------------------------- */
void	TestCreateModuleThread( Boolean	ql )
{
	uint32			n;
	
	for (n = 1; n < 4; n++)
	{
		TestOneCreateModuleThread( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_TASK, ( (0x1<<(n-1)) -1 ) );
		TestOneCreateModuleThread( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_THREAD, ( (0x1<<(n-1)) -1 ) );
	}

	
	if (ql) return;

	for (n = 4; n < 33; n++)
	{
		TestOneCreateModuleThread( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_TASK, ( (0x1<<(n-1)) -1 ) );
		TestOneCreateModuleThread( "/remote/TM_SumArgs", n, &testArgs1[0],"TM_SumArgs", NULL, OPENMODULE_FOR_THREAD, ( (0x1<<(n-1)) -1 ) );
	}
}

/* ====================================================================	*/
/*					CreateThread		Tests							*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneCreateThread(	void			(*code)(),
								int32			argc,
								char 			**argv,
								uint8			pri,
								int32			stackSize,
								char			*taskName,
								int32			expRtn)

{
	Item	tItem;
	Item	msgPort;
	Item	msgItem;
	Err		err;
	TagArg	tag[5];
	int32	tIndx = 0;
	int32	result;
	char	testName[1024];
	
	
	
	if( taskName == NULL ) taskName = "_NULL_PTR_";

	sprintf( testName, "CreateThread  argc= %d, taskName \"%s\", pri \"%s\", StackSize= %d expRtn= %d ",
									argc, taskName, pri, stackSize, expRtn );
	BeginTest( BAT, testName );
	
	if( strcmp(taskName, "_NULL_PTR_") == 0 ) taskName = NULL;

	msgPort = CreateMsgPort("CreateThread_MSG_PORT", 0, 0); 
	if (msgPort < 0)
	{
		LogError(msgPort );
		LogResult(UNRESOLVED, "CreateMsgPort returns UNEXPECTED error #%x", msgPort);
		return;
	}

	
	if (argc != 0)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_ARGC;
		tag[tIndx++].ta_Arg = (void *) argc;
	}
	if (argv != NULL)
	{
		tag[tIndx].ta_Tag = CREATETASK_TAG_ARGP;
		tag[tIndx++].ta_Arg = (void *) argv;
	}

	tag[tIndx].ta_Tag = CREATETASK_TAG_MSGFROMCHILD;		/* we want our children to stay in touch */
	tag[tIndx++].ta_Arg = (void *) msgPort;

	tag[tIndx].ta_Tag = TAG_END;
	


	tItem = CreateThread( code, taskName, pri, stackSize, &tag[0] );
	if( taskName == NULL ) taskName = "_NULL_PTR_";
	

	if (tItem < 0 )
	{
		LogError(tItem);
		LogResult(FAILED, "CreateThread   taskName \"%s\" - reurned error #%x", taskName, tItem);
	}
	else
	{
	
		msgItem = WaitPort( msgPort, 0);				/* Wait for the task to die and kernel to send us a message */
			
		result = MESSAGE(msgItem)->msg_Result;			/* check all the message fields for proper info */
	
		if( MESSAGE(msgItem)->msg_Val1 != tItem )
			LogResult(FAILED, "CreateThread  taskName \"%s\", argc= %d,  msgVal1: exp %x, act %x", taskName, argc, MESSAGE(msgItem)->msg_Val1, tItem);
	
		else if( MESSAGE(msgItem)->msg_Val2 != tItem )
			LogResult(FAILED, "CreateThread  taskName \"%s\", argc= %d,  msgVal2: exp %x, act %x", taskName, argc, MESSAGE(msgItem)->msg_Val2, tItem);
	
		else if( result == expRtn )
			LogResult(PASSED, "CreateThread  taskName \"%s\", argc= %d,  returns  %d", taskName, argc, result);
	
		else 
			LogResult(FAILED, "CreateThread  taskName \"%s\", argc= %d, (returned  %d - exp %d)", taskName, argc, result, expRtn);
	
	
		if ((err = DeleteMsg( msgItem )) < 0)					/* lets be nice and delete the message */
		{
			LogError(err);
			LogComment("Error #%x in DeleteMsg", err);
		}
	}
	
	if ((err = DeleteMsgPort( msgPort )) < 0)					/* let's be super nice and delete the message port */
	{
		LogError(err);
		LogComment("Error #%x in DeleteMsgPort", err);
	}
}


/* --------------------------------------------------- */
void	TestCreateThread( Boolean	ql )
{
	uint32			n;
	
	for (n = 1; n < 4; n++)
		TestOneCreateThread( (void *)SumFunc1, n, &testArgs1[0], 0, 4096, "TM_SumArgs", ( (0x1<<(n-1)) -1 ) );

	
	if (ql) return;

	for (n = 4; n < 33; n++)
		TestOneCreateThread( (void *)SumFunc1, n, &testArgs1[0], 0, 4096, "TM_SumArgs", ( (0x1<<(n-1)) -1 ) );

}



/* ====================================================================	*/
/*					DeleteTask		Tests								*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneDeleteTask(		char			*path,
								char			*taskName,
								OpenModuleTypes	type)

{
	Item	tItem;
	Item	mItem;
	Err		err;
	char	testName[1024];
	

	mItem = OpenModule( path, type, NULL);
	if( path == NULL ) path = "_NULL_PTR_";
	if( taskName == NULL ) taskName = "_NULL_PTR_";
	
	sprintf( testName, "DeleteTask  path \"%s\", taskName \"%s\"", path, taskName);
	if( strcmp(taskName, "_NULL_PTR_") == 0 ) taskName = NULL;
	BeginTest( BAT, testName );

	if (mItem < 0)
	{
		LogError(mItem );
		LogResult(UNRESOLVED, "OpenModule(\"%s\", %d, NULL)  returns UNEXPECTED error #%x", path, type, mItem);
		return;
	}


	tItem = CreateTask( mItem, taskName, NULL );
	
	if (mItem >= 0)										/* close the module */
		if ((err = CloseModule(mItem)) < 0)
		{
			LogError(err );
			LogComment("Error #%x in CloseModule", err);
		}

	if (tItem < 0 )
	{
		LogError(tItem);
		LogResult(UNRESOLVED, "CreateTask   path \"%s\" - reurned error #%x", path, tItem);
	}
	else
	{
		err = DeleteTask( tItem );
		
		if (err < 0)
		{
			LogError(err);
			LogResult(FAILED, "DeleteTask   path \"%s\"  taskName \"%s\" - reurned error #%x", path,taskName, err);
		}
		else if (FindTask( taskName ) < 0)
			LogResult(PASSED, "DeleteTask   path \"%s\"  taskName \"%s\" - appears to complete OK", path,taskName );
		
		else 
			LogResult(FAILED, "DeleteTask   path \"%s\"  taskName \"%s\" - task appears to still be running", path,taskName );
		
	}
	
	
}


/* --------------------------------------------------- */
void	TestDeleteTask( Boolean	ql )
{
/*
 * Hack to get rid of warning (Do the same thing no matter what
 */
	if (ql)
	{
	    TestOneDeleteTask( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_TASK );
	    TestOneDeleteTask( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_THREAD );
	}
	else
	{
	    TestOneDeleteTask( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_TASK );
	    TestOneDeleteTask( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_THREAD );
	}
}



/* ====================================================================	*/
/*					DeleteModuleThread	Tests							*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneDeleteModuleThread(		char			*path,
										char			*taskName,
										OpenModuleTypes	type)

{
	Item	tItem;
	Item	mItem;
	Err		err;
	char	testName[1024];
	
	
	mItem = OpenModule( path, type, NULL);
	if( path == NULL ) path = "_NULL_PTR_";
	if( taskName == NULL ) taskName = "_NULL_PTR_";
	
	
	sprintf( testName, "DeleteModuleThread  path \"%s\", taskName \"%s\", type= %d ", path, taskName, type );
	if( strcmp(taskName, "_NULL_PTR_") == 0 ) taskName = NULL;
	BeginTest(BAT, testName );
	
	if (mItem < 0)
	{
		LogError(mItem );
		LogResult(UNRESOLVED, "OpenModule(\"%s\", %d, NULL)  returns UNEXPECTED error #%x", path, type, mItem);
		return;
	}


	tItem = CreateModuleThread( mItem, taskName, NULL );
	
	if (mItem >= 0)										/* close the module */
		if ((err = CloseModule(mItem)) < 0)
		{
			LogError(err );
			LogComment("Error #%x in CloseModule", err);
		}

	if (tItem < 0 )
	{
		LogError(tItem);
		LogResult(UNRESOLVED, "CreateModuleThread   path \"%s\" - reurned error #%x", path, tItem);
	}
	else
	{
		err = DeleteModuleThread( tItem );
		
		if (err < 0)
		{
			LogError(err);
			LogResult(FAILED, "DeleteModuleThread   path \"%s\"  taskName \"%s\" - reurned error #%x", path,taskName, err);
		}
		else if (FindTask( taskName ) < 0)
			LogResult(PASSED, "DeleteModuleThread   path \"%s\"  taskName \"%s\" - appears to complete OK", path,taskName );
		
		else 
			LogResult(FAILED, "DeleteModuleThread   path \"%s\"  taskName \"%s\" - task appears to still be running", path,taskName );
		
	}
	
	
}


/* --------------------------------------------------- */
void	TestDeleteModuleThread( Boolean	ql )
{
/*
 * Hack to get rid of warning (Do the same thing no matter what
 */
	if (ql)
	{
	    TestOneDeleteModuleThread( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_TASK );
	    TestOneDeleteModuleThread( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_THREAD );
	}
	else
	{
	    TestOneDeleteModuleThread( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_TASK );
	    TestOneDeleteModuleThread( "/remote/TM_Loop", "TM_Loop", OPENMODULE_FOR_THREAD );
	}
}


/* ====================================================================	*/
/*					DeleteThread	Tests								*/
/* ====================================================================	*/
/* --------------------------------------------------- */
void	TestOneDeleteThread(	void			(*code)(),
								char			*taskName)

{
	Item	tItem;
	Err		err;
	char	testName[1024];
	

	tItem = CreateThread( code, taskName, 0, 4096, NULL );
	
	
	if( taskName == NULL ) taskName = "_NULL_PTR_";
	sprintf( testName, "DeleteThread   taskName \"%s\"", taskName);
	BeginTest(BAT, testName );
	
	if (tItem < 0 )
	{
		LogError(tItem);
		LogResult(UNRESOLVED, "CreateModuleThread   taskName \"%s\" - reurned error #%x", taskName, tItem);
	}
	else
	{
		err = DeleteThread( tItem );
		
		if (err < 0)
		{
			LogError(err);
			LogResult(FAILED, "DeleteThread   taskName \"%s\" - reurned error #%x",taskName, err);
		}
		else if (FindTask( taskName ) < 0)
			LogResult(PASSED, "DeleteThread   taskName \"%s\" - appears to complete OK",taskName );
		
		else 
			LogResult(FAILED, "DeleteThread   taskName \"%s\" - task appears to still be running",taskName );
		
	}
	
	
}


/* --------------------------------------------------- */
void	TestDeleteThread( Boolean	ql )
{
/*
 * Hack to get rid of warning (Do the same thing no matter what
 */
	if (ql)
	    TestOneDeleteThread( (void *)LoopFunc1,"TM_Loop");
	else
	    TestOneDeleteThread( (void *)LoopFunc1,"TM_Loop");
}

