
/*
 * @(#) UserException.c 96/10/21 1.2
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <kernel/mem.h>
#include <kernel/item.h>
#include <kernel/types.h>
#include <kernel/kernelnodes.h>
#include <kernel/nodes.h>
#include <kernel/task.h>
#include <tests/3do_scaf.h>
Item port;
Item port2;



static void HandlerFunction(void)
 {
  Err err;
  Item msg;
  Message *msgPtr;
  UserExceptionContext *uec;
  Task *task;
  Item msgitem;
  FPRegBlock *fprb;
  int32 i;
  fprb = AllocMem(sizeof(FPRegBlock),MEMTYPE_NORMAL);
  
  task = CURRENTTASK;
  port = CreateMsgPort("MainPort",0,0);
	if (port < 0)
	{
		printf("CreateMsgPort returned error:  rc=%d\n",port);
		exit( -1);
	}
  printf("HandlerFunction is about to call RegisterUserExceptionHandler\n");
  err = RegisterUserExceptionHandler(task->t.n_Owner,port);
  if (err < 0)
  {
  	printf("RegisterUserExceptionHandler returned error: rc=%d\n",err);
	exit(-1);
  }
  
  /*send a message to the main task that we can trap errors now*/
  msgitem = CreateMsg(NULL,50,0);
  err = SendMsg(port2,msgitem,NULL,0);
  if (err < 0)
  {
  	printf("SendMsg returned error: rc=%d\n",err);
	exit(-1);
  }
  
  
  printf("HandlerFunction now calling WaitPort()\n");
  msg = WaitPort(port,0);
  if (msg < 0)
  {
   	printf("WaitPort() call failed - rc=%d\n",msg);
	exit( -1);
  }
  printf("Message received - Handler function now running\n");
  msgPtr = MESSAGE(msg);
  uec = msgPtr->msg_DataPtr;
  printf("Task   %s\n", TASK(uec->uec_Task)->t.n_Name);
  printf("Trigger 0x%x\n",uec->uec_Trigger);
  printf("%d\n",uec->uec_FPRegBlock->fprb_FPRs[0]);
  
  fprb->fprb_FPSCR = 0; 
  fprb->fprb_SP = uec->uec_FPRegBlock->fprb_SP;
  fprb->fprb_LT = uec->uec_FPRegBlock->fprb_LT;
  for (i = 0; i < 32; i++)
  {
    
  	fprb->fprb_FPRs[i] = 0;
  }
  
  printf("created fprb structure to pass to CUE\n");
  err = CompleteUserException(uec->uec_Task,NULL,fprb);
  if (err < 0)
  {
  	printf("CompleteUserException failed! rc=%d\n",err);
	exit( -1);
  }
  printf("Got past CompleteUserException\n");
  exit(0);
  }


  
int main (int32 argc, char *argv[])
  {
	Err err;
	Item thread;
	float ex;
	float32 zero = 0.0;
	Item msg2;
	
	
	TOUCH(argc);
	TOUCH(argv);

	/*Setup Message Port for user exceptions*/
	port2 = CreateMsgPort("SyncPort",0,0);
	if (port2 < 0)
	{
		printf("CreateMsgPort(Port2) returned error: rc=%d\n",port2);
		exit(-1);
	}
	
	/*Create Thread to handle user exceptions*/
	thread = CreateThreadVA(HandlerFunction,"HandlerFunction",0,2048,TAG_END);
	if (thread < 0)
	{
		printf("CreateThreadVA returned error: rc=%d\n",thread);
		exit( -1);
	}
	
	
	/*wait for subtask to synchronize */
	msg2 = WaitPort(port2,0);
	
	
	DeleteMsgPort(port2);
	DeleteMsg(msg2);
	
	
	/* set divide by zero as a user-handled exception*/
	err = ControlUserExceptions(USEREXC_FP_ZERODIVIDE,TRUE);
	if (err < 0)
	{
		printf("ControlUserExceptions returned error: rc=%d\n",err);
		exit( -1);
	}
	printf("Main Task now about to divide by zero\n");
	ex = 25.0 / zero;
	printf("25.0/0.0 = %f\n", ex);
	printf("Control Returned to Main Task\n");
	
	

	
	exit( 0);
}

