##########################################################################
#   File:       lib3do_scaf.a.make
#
#	Contains:	make file for building Scaffold lib for M2/PowerPC
#
#	Copyright  1995 The 3DO Company
# 	All rights reserved. This material constitutes confidential and proprietary 
#	information of the 3DO Company and shall not be used by any Person or for any 
#	purpose except as expressly authorized in writing by the 3DO Company.
#
#
#	History:
#	3/14/96	elh		converted to work with 3do_scaf
#	5/17/95	elh		conglomeration of Anup and Jim's make files with the earlier scaffold
#					make file for use with the diab MPW compiler
#
#
##########################################################################

#####################################
#		Symbol definitions
#####################################

ObjectDir		= :Objects:
Library			= 3do_scaf
LibraryFileName = lib{Library}.a
MakeFileName	= lib{Library}.a.make

#####################################
#		Define diab tools
#####################################
LIBRARIAN		= dar
CC				= dcc

#####################################
#		LibOptions
#####################################
	# Note: These LibOptions ASSUME the library file is being created from scratch.
	# To update an existing library file, use something like "-rcu".
LibOptions		= -qc


#####################################
# variables for compiler tools
#####################################
CCopts		=	-g								# 
				-DMEMDEBUG						# 
				-I{3doincludes} 				# include path.  Be careful of the new heirarchical includes! 
				-I{3doincludes}audio:			# 	the final layout and heirarchy is not yet final 
				-I{3doincludes}file:			#	the compiler may or may not deal with this in the future 
				-I{3doincludes}graphics:		#	we just don't know yet! 
				-I{3doincludes}hardware:		# 
				-I{3doincludes}international:	# 
				-I{3doincludes}k9:				# 
				-I{3doincludes}kernel:			# 
				-I{3doincludes}lib3DO:			# 
				-I{3doincludes}misc:			# 
				-c								# MANDATORY		only run the c compiler, don't exec the linker 
				-WDDTARGET=PPC602				# MANDATORY		Tells the compiler you're building for the 602 (which is 603 safe) 
				-Xstring-align=1				# RECOMMENDED	word aligns strings 
				-Xdouble-warning -Xuse-float	# RECOMMENDED	produces single precision instructions/constants 
				-Ximport						# RECOMMENDED forces headers to be included only once 
				-Xstrict-ansi					# RECOMMENDED unless you are building code to be released, then its MANDATORY ;-) 
				-Xunsigned-char					# ? forces string constants to be unsigned char? 
				-XO -Xunroll=1 -Xtest-at-bottom	# ? Optimizations on, unroll loops (but not too aggressively), test loop conditional at bottom 
				-Xextend-args=0					# ? 
				-Xforce-prototypes				# RECOMMENDED 
				-Xinline=5						# ? 5 levels of inline/macro expansion 
				-Xno-libc-inlining -Xno-recognize-lib	# MANDATORY(?) Don't use any compiler built in functions/inlines 
				-Xno-bss=2						# MANDATORY(?) 
				-Xlint=0x10						# RECOMMENDED lovely commentary on your code quality 
				-Xtrace-table=0					# ? Don't generate traceback tables.  This is like macsbug names for you macheads	
				-Xsmall-data=0					# don't generate small references 
				-Xsmall-const=0					#	<as above> 
				-Xuse-.init=0					# supress .init and .fini sections

#####################################
#		Object files
#####################################
OBJECTS =	"{ObjectDir}{Library}.c.o"	
			"{ObjectDir}scaf_MemUtils.c.o"	
			"{ObjectDir}scaf_MemDebug.c.o"	
			"{ObjectDir}scaf_MemRation.c.o"



All					{Library}



#####################################
#	Target build rules
#####################################
{LibraryFileName}			"{MakeFileName}" {OBJECTS}
	echo "   Linking {LibraryFileName}"
	set WorkingDisk ""	# if (`Exists -d "{RAMDISKDIR}"`); set WorkingDisk "{RAMDISKDIR}"; end
	delete -i "{WorkingDisk}{LibraryFileName}"
	{LIBRARIAN}	{LibOptions} "{WorkingDisk}{LibraryFileName}" {OBJECTS}
	if "{WorkingDisk}" != ""; Move -y "{WorkingDisk}{LibraryFileName}" {LibraryFileName}; end
	duplicate -y {LibraryFileName} {3dolibs}default	
	duplicate -y 3do_scaf.h {3doincludes}
	duplicate -y scaf_MemUtils.h {3doincludes}

# files in the objects sub-dir are dependent upon sources in the current dir
{ObjectDir}  :


# .c.o files are dependent only upon the .c source (This is not very complete but good enough for now)
.c.o			.c
	{CC} {CCopts} {depDir}{default}.c -o {targDir}{default}.c.o
