/* @(#) scaf_MemUtils.c 96/07/11 1.1 */

#include <kernel/debug.h>
#include <kernel/mem.h>
#include <kernel/types.h>
#include <stdlib.h>
#include <string.h>

#define	ErrReport		kprintf
#define	ErrNumReport	PrintfSysErr
#define	Report			kprintf

static	uint32	gBeforeSysAlloc = 0; 	/* stores allocated system memory at init time */
static	uint32	gBeforeTaskAlloc = 0; 	/* stores allocated task memory at init time */

static	Boolean	gScafMemCheckInited = FALSE;
static	Boolean	gScafMemCheckVerbose;

/* 	--------------------------------------	*/
/* prints out memory information */
void ReportMemInfo( void )
{
	MemInfo mi;
	int32	pSize;		/* page size */
	
	ScavengeMem();											/* return unused memory to system pool */
	
	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_NORMAL );
	Report("---------- Memory Information ------------\n");
	Report("current page size in bytes:....... %d\n", GetPageSize( MEMTYPE_NORMAL ));	/* page size */
	Report("pages allocated by current task:.. %d\n", mi.minfo_TaskAllocatedPages);		/* pages allocated by current task */
	Report("bytes allocated by current task:.. %d\n", mi.minfo_TaskAllocatedBytes);		/* bytes allocated by current task */
	
	Report("unallocated pages:................ %d\n", mi.minfo_FreePages);				/* unallocated pages               */
	Report("largest extent of free pages:..... %d\n", mi.minfo_LargestFreePageSpan);	/* largest extent of free pages    */
	Report("pages allocated by supervisor:.... %d\n", mi.minfo_SystemAllocatedPages);	/* pages allocated by supervisor   */
	Report("bytes allocated by supervisor:.... %l\n", mi.minfo_SystemAllocatedBytes);	/* bytes allocated by supervisor   */
	Report("pages allocated by other tasks:... %d\n", mi.minfo_OtherAllocatedPages);	/* pages allocated by other tasks  */
	Report("------------------------------------------\n");
}

/* 	--------------------------------------	*/
/* returns the amount of memory currently allocated by the task */
uint32	GetAllocTaskMem( void )
{

	MemInfo mi;
	uint32	pSize;		/* page size */
	
	ScavengeMem();											/* return unused memory to system pool */
	
	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_NORMAL );
		
	return(	mi.minfo_TaskAllocatedBytes ); 			/* allocated task mem */
}
/* 	--------------------------------------	*/
/* returns the amount of memory currently allocated by the system */
uint32	GetAllocSysMem( void )
{

	MemInfo mi;
	uint32	pSize;		/* page size */
	
	ScavengeMem();											/* return unused memory to system pool */

	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_NORMAL );
		
	return(	mi.minfo_SystemAllocatedBytes ); 			/* allocatedSys mem */
}

/* 	--------------------------------------	*/
/* returns the amount of memory not currently in use and available for allocation */
uint32	GetFreeMem( void )
{

	MemInfo mi;
	uint32	pSize;		/* page size */
	
	ScavengeMem();											/* return unused memory to system pool */
	
	pSize = GetPageSize( MEMTYPE_NORMAL );
	if (pSize < 0)
	{
		ErrNumReport( pSize );
		ErrReport( "Error #%d when attempting to get page size in GetFreeMem\n", pSize);
		return (0);
	}

	GetMemInfo( &mi, sizeof(MemInfo), MEMTYPE_NORMAL );
		
	return(	(mi.minfo_TaskAllocatedPages * pSize) - mi.minfo_TaskAllocatedBytes +			/* free task mem */
			(mi.minfo_SystemAllocatedPages * pSize) - mi.minfo_SystemAllocatedBytes +		/* free sys mem */
			(mi.minfo_FreePages * pSize) );													/* unallocated mem */
}

/* 	--------------------------------------	*/

void InitMemCheck( Boolean verbose )
{
	gScafMemCheckInited = TRUE;
	gScafMemCheckVerbose = verbose;
	
	gBeforeSysAlloc = GetAllocSysMem();
	gBeforeTaskAlloc = GetAllocTaskMem();
	if (verbose)
	{
		Report("** Available Task Memory at Init   = %d\n", gBeforeTaskAlloc);
		Report("** Available System Memory at Init = %d\n", gBeforeSysAlloc);
		ReportMemInfo();
	}
}
	
/* 	--------------------------------------	*/
/* check for memory leaks since init */
void TermMemCheck( void )
{
	uint32	afterSysAlloc;
	uint32	afterTaskAlloc;
	
	if (!gScafMemCheckInited) return; 		/* don't do anything if not initialized */
	gScafMemCheckInited = FALSE;
	
	afterTaskAlloc = GetAllocTaskMem();
	
	if (gBeforeTaskAlloc > afterTaskAlloc)
		Report( "** TASK MEMORY APPEARED - %d bytes\n", (gBeforeTaskAlloc-afterTaskAlloc) );
	else if (gBeforeTaskAlloc < afterTaskAlloc)
		Report( "** TASK MEMORY DISAPPEARED - %d bytes\n", (afterTaskAlloc-gBeforeTaskAlloc) );
	else
		Report( "** NO TASK MEMORY LOST **\n" );
	
	afterSysAlloc = GetAllocSysMem();
	
	if (gBeforeSysAlloc > afterSysAlloc)
		Report( "** SYSTEM MEMORY APPEARED - %d bytes\n", (gBeforeSysAlloc-afterSysAlloc) );
	else if (gBeforeSysAlloc < afterSysAlloc)
		Report( "** SYSTEM MEMORY DISAPPEARED - %d bytes\n", (afterSysAlloc-gBeforeSysAlloc) );
	else
		Report( "** NO SYSTEM MEMORY LOST **\n" );
	
	if (gScafMemCheckVerbose)
		ReportMemInfo();
}
	
