/* @(#) DSBranchSuite.c 96/07/24 1.1 */
/*
	File:		DSBranchSuite.c

	Contains:	data streaming branching tests

	Written by:	amasri

	Copyright:	 1996 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <3>	  5/8/96	amasri		add scaffold return value
		 <1>	 4/15/96	am		first checked in

	To Do:
*/

/* Copyright (C) 1995, an unpublished work by The 3DO Company.*/
/* All rights reserved. This material contains confidential*/
/* information that is the property of The 3DO Company. Any*/
/* unauthorized duplication, disclosure or use is prohibited.*/
/* */
/* @(#) DSAPISuite.c 95/11/13 1.0 */

#include <stdio.h>
#include <string.h>
#include <kernel/debug.h>
#include <kernel/operror.h>
#include <kernel/mem.h>
#include <graphics/view.h>
#include <audio/audio.h>			/* OpenAudioFolio */


#include "joypad.h"
#include "testvideostream.h"
#include "tests.h"
#include <tests/3do_scaf.h>

char* sTestName;

/******************************/
/* Library and system globals */
/******************************/

static ScreenInfo screenInfo;

#define DEFAULT_BITMAP_DEPTH		24

#define DEFAULT_STREAM_FILENAME		"video1.stream"

#define	MAX_BUFFER_SIZE			(24 * 1024L)

/*****************************************************************************
 * Compile switch implementations
 *****************************************************************************/

#if TEST_TRACE

	#define TRACE_LEVEL		3

	/* Declare the actual trace buffer.  Let it be global in case other 
	 * modules in the subscriber want to write into it. */
	TraceBuffer		TestTraceBuffer;
	TraceBufferPtr	TestBufPtr 	= &TestTraceBuffer;

	/* Allow for multiple levels of tracing */
	#if (TRACE_LEVEL >= 1)
		#define		ADD_TRACE_L1(bufPtr, event, chan, value, ptr)	\
						AddTrace(bufPtr, event, chan, value, ptr)
	#else
		#define		ADD_TRACE_L1(bufPtr, event, chan, value, ptr)
	#endif
	
	#if (TRACE_LEVEL >= 2)
		#define		ADD_TRACE_L2(bufPtr, event, chan, value, ptr)	\
						AddTrace(bufPtr, event, chan, value, ptr)
	#else
		#define		ADD_TRACE_L2(bufPtr, event, chan, value, ptr)
	#endif
	
	#if (TRACE_LEVEL >= 3)
		#define		ADD_TRACE_L3(bufPtr, event, chan, value, ptr)	\
						AddTrace(bufPtr, event, chan, value, ptr)
	#else
		#define		ADD_TRACE_L3(bufPtr, event, chan, value, ptr)
	#endif


#else	/* Trace is off */
	#define		ADD_TRACE_L1(bufPtr, event, chan, value, ptr)
	#define		ADD_TRACE_L2(bufPtr, event, chan, value, ptr)	
	#define		ADD_TRACE_L3(bufPtr, event, chan, value, ptr)
#endif

/*********************************************************************************************
 * Perform one-time graphics initializations:
 * Open the Graphics Folio, create two Bitmap Items (for double-buffering), allocate their
 * bitmap buffers (with appropriate alignment and filled with zeros), create a View Item, and
 * add the View Item to the system View List.
 *
 * [TBD] We're using MEMTYPE_STARTPAGE here to get an alignment workable to the FMV decoder
 * hardware. Note that, as long as we DON'T ask for MEMTYPE_TRACKSIZE, any allocation will
 * happen to be adequately aligned with the current hardware. We need a better solution requiring
 * participation of the FMV driver and the OS memory allocator.
 *********************************************************************************************/
static Err InitGraphics(uint32 bitDepth)
	{
	int32		i;
	Err			err;
	Bitmap		*bm;

	if ( (err = OpenGraphicsFolio()) < 0 )
		return err;

	for ( i = 0; i < cntFrameBuffers; ++i )
		{
		screenInfo.bitmapItem[i] = err = CreateItemVA(
			MKNODEID(NST_GRAPHICS, GFX_BITMAP_NODE),
			BMTAG_WIDTH,		NTSC_FRAME_WIDTH,
			BMTAG_HEIGHT,		NTSC_FRAME_HEIGHT,
			BMTAG_TYPE,			(bitDepth == 16) ? BMTYPE_16 : BMTYPE_32,
			BMTAG_DISPLAYABLE,	TRUE,
			BMTAG_MPEGABLE,		TRUE,
		/*	BMTAG_BUMPDIMS,		TRUE,	*/	/* [TBD] This asks the Graphics
		 * Folio to bump the width and height to satisfy all hardware
		 * requirements. Before we can ask for BUMPDIMS, we have to use the
		 * (bumped) bm_Width and bm_Height fields rather than using the
		 * requested width and height anywhere else in the code. */
			TAG_END);
		FAIL_NEG("Test InitGraphics: Create a Bitmap Item", err);
		screenInfo.bitmap[i] = bm = LookupItem(screenInfo.bitmapItem[i]);

		screenInfo.allocatedBuffer[i] = AllocMemMasked(
			bm->bm_BufferSize,
			bm->bm_BufMemType | MEMTYPE_FILL,
			bm->bm_BufMemCareBits,
			bm->bm_BufMemStateBits);
		err = ER_NoMem;
		FAIL_NIL("Test.InitGraphics: Alloc a bitmap buffer",
			screenInfo.allocatedBuffer[i]);

		err = ModifyGraphicsItemVA(screenInfo.bitmapItem[i],
			BMTAG_BUFFER,	screenInfo.allocatedBuffer[i],
			TAG_END);
		FAIL_NEG("Test.InitGraphics: Install a bitmap buffer", err);
		}

	screenInfo.renderSignal = AllocSignal(0);
	err = ( screenInfo.renderSignal == 0 ) ? kDSNoSignalErr : 0;
	FAIL_NEG("Test: Alloc render signal", err);

	screenInfo.viewItem = err = CreateItemVA(MKNODEID(NST_GRAPHICS, GFX_VIEW_NODE),
				VIEWTAG_PIXELWIDTH,		NTSC_FRAME_WIDTH,
				VIEWTAG_BITMAP,			screenInfo.bitmapItem[1],
				VIEWTAG_RENDERSIGNAL,	screenInfo.renderSignal,
				TAG_END);
	FAIL_NEG("Test.InitGraphics: Create a View Item", err);

	err = AddViewToViewList(screenInfo.viewItem, 0);
	FAIL_NEG("Test.InitGraphics: AddViewToViewList", err);

	WaitSignal(screenInfo.renderSignal);	/* discard the first renderSignal */

	return 0;

FAILED:
	return err;		/* [TBD] cleanup? */
	}


/*********************************************************************************************/
static Boolean StartUp(uint32 bitDepth)
{
if ( OpenAudioFolio() < 0 )
	return false;

if ( InitGraphics(bitDepth) < 0 )
	return false;

return true;
}

/***************************************************
 *
 *	Usage
 *
 */
static void Usage(void){
	APRNT(("StreamTester [-16 | -24] [-?] [<stream file>]\n"
		"  -16 for 16 bits/pixel. -24 for 24 bits/pixel. Default is %d.\n"
		"  -? for usage info.\n"
		"  Default stream file is \"" DEFAULT_STREAM_FILENAME "\".\n"
		"\n"));
}


/*********************************************************************************************
 * Play a video stream, given a display pixel size and a filename.
 *********************************************************************************************/
static void StreamTester(uint32 bitDepth, char *filename){
	int32			status;
	JoyPadState 	jpState;

	/* Initialize the library code */
	if ( !StartUp(bitDepth) )
		{
		fputs("StreamTester StartUp() initialization failed!\n",stdout);
		return;
		}

	/* Run the tests */
	status = TestVideoStream(&screenInfo, bitDepth, filename,
							(PlayVideoUserFn)myUserFunction, &jpState);
	
	FAIL_NEG("TestVideoStream", status);

	if ( status == 0 )
		fputs("StreamTester end of stream reached\n",stdout);
	else if ( status == 1 )
		fputs("StreamTester stream stopped by control pad\n",stdout);


FAILED:		
	
	/* Free resources allocated by GetJoyPad.*/

	status = KillJoypad();
	CHECK_NEG("KillJoypad",status);


	/* [TBD] Free resources allocated by InitGraphics, in particular, the memory allocated.
	 * Probably have to delete the Items first. */
}


/******************************************************************************
|||	A DataStream example program that plays synchronized video and audio.
|||
|||	  Format -preformatted
|||
|||	    StreamTester [-16 | -24] [-?] [<stream file>]
|||	        -16 for 16 bits/pixel. -24 for 24 bits/pixel. Default is 24.
|||	        -? for usage info.
|||	        The default stream file is "video1.stream".
|||
|||
|||	  Description
|||
|||	    StreamTester performs BATs for Data Streamer.
|||
|||	  Location
|||
|||
 ******************************************************************************/
void main(int32 argc, char **argv){
	char			*filename = DEFAULT_STREAM_FILENAME;
	uint32			bitDepth = DEFAULT_BITMAP_DEPTH;
	int32			i;
		
	argc = InitScaffold(DATA_STREAM, argc, argv);
	DeferLog(MAX_BUFFER_SIZE);

	/* Parse command line args */
	for ( i = 1; i < argc; ++i )
	{
		if ( argv[i] == NULL )
			break;
		if (!strcmp(argv[i], "-16"))
			{
			bitDepth = 16;
			break;
			}
		if (!strcmp(argv[i], "-24")  || !strcmp(argv[i], "-32"))
			{
			bitDepth = 24;
			break;
			}
		if ( argv[i][0] == '-')
			{
			Usage();
			return;
			}
		else if ( argv[i][0] != '\0' )
			{
			filename = argv[i];
			}
	}
	StreamTester(bitDepth, filename);
#if TEST_TRACE
	sTestName = "DumpTraceCompletionStats_1";
	BeginTest(FT, sTestName);
	status = DumpTraceCompletionStats(TestBufPtr, kTraceOpDoConnect, kTraceOpDoDisconnect,"TracePairLogFile");
	if (status < 0){
		LogResult(FAILED, "Test %s returns error %d", sTestName, (int)status);
		}
	else{
		LogResult(PASSED,"Test %s passes",sTestName);
	}
	sTestName = "DumpRawTraceBuffer_1";
	BeginTest(FT, sTestName);
	status = DumpRawTraceBuffer(TestBufPtr,"TraceLogFile");
	if (status < 0){
		LogResult(FAILED, "Test %s fails with error %d", sTestName, (int)status);
		}
	else{
		LogResult(PASSED,"Test %s passes",sTestName);
	}
#endif
TermScaffold();
}
