#####################################
##
##	Copyright (C) 1995, an unpublished work by The 3DO Company.
##	All rights reserved. This material contains confidential
##	information that is the property of The 3DO Company. Any
##	unauthorized duplication, disclosure or use is prohibited.
##	@(#) DSBranchSuite.make 96/08/06 1.2
##
#####################################
#
#   File:       DSBranchSuite.make
#
#	Contains:	make file for building DSBranchSuite, which performs funtional
tests for data streaming)
#
#	To regenerate the .c.o -> .h file dependencies, get write access to this
#	make file and execute the following MPW code:
#	make Depends -f DSBranchSuite.make   "{worksheet}" > temp.makeout;
temp.makeout  "{worksheet}"; delete -i temp.makeout
#
#####################################

#####################################
#		Symbol definitions
#####################################
App				=	DSBranchSuite
DebugFlag		=	1
ObjectDir		=	:Objects:
AppDir          =   "Omicron:DS_BAT_1.0:DSBranchSuite:"
VectorDir =  {3DOVectors}{M2LibRelease}:
MoveDir			= 	"{3DORemote}Examples:Streaming:{App}:"
CC				=	dcc
LINK			=	link3do

MakeFileName	=	"{App}.make"


#####################################
#	Default compiler options for Diab Data
#####################################
CBaseOptions =	-I"{3doincludes}" 				# include path.  Be careful of 
#				-I{3doincludes}audio:			# the new heirarchical 
#				-I{3doincludes}file:			# includes! 
#				-I{3doincludes}graphics:		# 
#				-I{3doincludes}hardware:		# 
#				-I{3doincludes}international:	# 
#				-I{3doincludes}streaming:		# 
				-I"{3doincludes}kernel:"		# 
#				-I{3doincludes}lib3DO:			# 
#				-I{3doincludes}misc:			# 
				-c								# MANDATORY		only run the c
compiler, don't exec the linker 
				-WDDTARGET=PPC602				# MANDATORY		Tells the compiler
you're building for the 602 (which is 603 safe) 
				-Xstring-align=1				# RECOMMENDED	word aligns
strings 
				-Xdouble-avoid -Xuse-float		# RECOMMENDED	produces single
precision instructions/constants 
				-Ximport						# RECOMMENDED forces headers to be
included only once 
				-Xstrict-ansi					# RECOMMENDED unless you are
building code to be released, then its MANDATORY ;-) 
				-Xunsigned-char					# ? forces string constants to be
unsigned char? 
				#-XO -Xunroll=1 -Xtest-at-bottom	# ? Optimizations on, unroll
loops (but not too aggressively), test loop conditional at bottom 
				-Xextend-args=0					# ? 
				-Xforce-prototypes				# RECOMMENDED 
				-Xinline=5						# ? 5 levels of inline/macro
expansion 
				-xno-double -Xno-libc-inlining -Xno-recognize-lib	# MANDATORY(?)
Don't use any compiler built in functions/inlines 
				-Xno-bss=2						# MANDATORY(?) 
				-Xlint							# RECOMMENDED lovely commentary on
your code quality 
				-Xtrace-table=0					# ? Don't generate traceback
tables.  This is like macsbug names for you macheads 
				-Xsmall-data=0					# 
				-DMEMDEBUG						# 
				-Xsmall-const=0					# 

# ? 5 levels of inline/macro expansion
CInLineOption		= -Xinline=5

CSymOption			= -g -DDEBUG

LOptions			= -D -r -g

Link3DOOptions		= -Htime=now -Hsubsys=1 -Htype=5 -Hstack=4096 -Hname={App}

#####################################
# Trace switches
#
# Each XXX_TRACE_XXX switch causes one module of the program to compile-in
# code that logs high-level events to a trace buffer. It's very helpful for
# debugging. If you turn any of these switches on, you must turn on
# -DTEST_TRACE, which allocates the trace buffer.
#
#####################################

#DSBranchSuiteTraceSwitches  = -DTEST_TRACE=1  
#CTraceOptions 		= {DSBranchSuiteTraceSwitches}

#####################################
#		Object files
# LIBS and OBJECTS are files to link in.
# LIBDEPENDS are library files worth having "make" dependencies on.
# OBJECTDEPENDS are files to create "make" dependency rules for.
#####################################
LIBDEPENDS		=	"{3DOLibs}{M2LibRelease}:libsubscriber.a"	
					"{3DOLibs}{M2LibRelease}:libds.a"			
					"{3DOLibs}{M2LibRelease}:libEZFlixDecoder.a" 	
					"{3DOLibs}{M2LibRelease}:libdsutils.a"		
					"{3DOLibs}{M2LibRelease}:lib3do_scaf.a"		
					"{3DOLibs}{M2LibRelease}:libeventbroker.a"	
					"{3DOLibs}{M2LibRelease}:libmusic.a"		
					"{3DOLibs}{M2LibRelease}:libc.a"			
					
LIBS			=	-L"{3dolibs}{M2LibRelease}"	# lib search path	
					-lsubscriber									
					-lds											
					-ldsutils										
					-l3do_scaf										
					-lezflixdecoder									
					-leventbroker									
					-lmusic											
					-lc
					
DLLS			= 	"{VectorDir}"audio						
					"{VectorDir}"graphics					
					"{VectorDir}"kernel 						
					"{VectorDir}"filesystem
					
OBJECTS			=	"{ObjectDir}{App}.c.o"					
					"{ObjectDir}TestVideoStream.c.o"		
					"{ObjectDir}Tests.c.o"					
					"{ObjectDir}JoyPad.c.o"
				
OBJECTDEPENDS	=	"{App}.c.depends"				
					"TestVideoStream.c.depends"		
					"tests.c.depends"				
					"JoyPad.c.depends"

#####################################
#	Default build rules
#####################################
All					{App}

{ObjectDir}			:

.c.o				.c "{MakeFileName}"
    if !`exists "{ObjectDir}"`
        newfolder "{ObjectDir}"
    end
	if ( {DebugFlag} )
		echo "	compiling {Default}.c with {CSymOption}"
		{CC} {DepDir}{Default}.c -o {TargDir}{Default}.c.o {CSymOption}
{CBaseOptions} 
	else
		echo "	compiling {Default}.c"
		{CC} {DepDir}{Default}.c -o {TargDir}{Default}.c.o  {CBaseOptions}
	end

.c.depends			.c
	set BaseDir "`directory`"
	{CC} {CBaseOptions} {DepDir}{Default}.c -E -H > Dev:Null  "c.includefiles"
	# Extract the non-3DOIncludes lines and generate Make lines, using ':' instead
of absolute pathname
	search -i -q -r -ns "{3DOIncludes}" "c.includefiles" 
		| StreamEdit -e "1,$ Replace /(?)1/ '"{ObjectDir}""{Default}.c.o"	
'1; Replace /{BaseDir}/ ':'" 
		>> "{MakeFileName}"
	# Extract the {3DOIncludes}streaming: lines and generate Make lines, using
"{3DOIncludes}streaming:" instead of absolute pathnames
	search -i -q -ns "{3DOIncludes}streaming:" "c.includefiles" 
		| StreamEdit -e "1,$ Replace /(?)1/ '"{ObjectDir}""{Default}.c.o"	
'1; Replace /{3DOIncludes}streaming:/ '"{3DOIncludes}"streaming:'" 
		>> "{MakeFileName}"
	delete -i "c.includefiles"


#####################################
#	Dependency re-building rules
#	The .c.depends rule asks the compiler to generate source file dependencies,
then
#	removes the first line (.c.o dependency on .c), substitutes a symbolic
reference
#	to "{ObjectDir}", puts in a tab before the s, and appends the result to this
make
#	file. The following rules setup and sequence the work.
#
#	HOW TO USE IT: Get write access to this make file then make "depends".
#	This will replace the include file dependencies lines at the end of this
makefile.
#####################################
Depends						DeleteOldDependencies {ObjectDepends} SaveNewMakefile

DeleteOldDependencies	
	target "{MakeFileName}"
	Find  
	Find /#tInclude file dependencies (Dont change this line or put anything
after this section.)/ 
	Find /[#]/  
	Replace Ƥ: "n" 

SaveNewMakefile			
	Save "{MakeFileName}"

############################################
# This generates the .spt file for debugging
############################################
CreateScriptFile		=	
	 "setsourcedir Omicron:DataStreamingAPISuite:"n
	 "setsourcedir {3DOFolder}Streaming:{3dorelease}:Libs:DataStream:"n
	 "setsourcedir {3DOFolder}Streaming:{3dorelease}:Libs:DSUtils:"n
	 "setsourcedir
{3DOFolder}Streaming:{3dorelease}:Libs:Subscribers:SAudioSubscriber:"n
	 "setsourcedir
{3DOFolder}Streaming:{3dorelease}:Libs:Subscribers:SubscriberUtilities:"n
	 "setsourcedir
{3DOFolder}Streaming:{3dorelease}:Libs:Subscribers:MPEGVideoSubscriber:"n	

#####################################
#	Target build rules
#####################################
{App}			"{AppDir}"{App}.make {OBJECTS} {LIBDEPENDS}
	echo '   Linking ' "{AppDir}{App}"
	{LINK} {Link3DOOptions} -o {App}		
		{OBJECTS}							
		{DLLS}							
		{LIBS}								
		#{LOptions} > {App}.map				
		{LOptions}
	SetFile {App} -c '3DOD' -t 'PROJ'
	if !`Exists "{MoveDir}"`
		NewFolder "{MoveDir}"
	end
	move -y {App} "{MoveDir}"
	if !`Exists "{MoveDir}{App}.spt"`
		echo {CreateScriptFile} >> "{MoveDir}{App}.spt"
	End
	echo " Built {App}"


#####################################
#	Include file dependencies (Dont change this line or put anything after this
section.)
#####################################

"{ObjectDir}"DSBranchSuite.c.o		:joypad.h
"{ObjectDir}"DSBranchSuite.c.o		:testvideostream.h
"{ObjectDir}"DSBranchSuite.c.o		:tests.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:datastream.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:msgutils.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:mempool.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:dsstreamdefs.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:dserror.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:saudiosubscriber.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:mpegvideosubscriber.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:threadhelper.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:mpegbufferqueues.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:ezflixsubscriber.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:preparestream.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:datastreamlib.h
"{ObjectDir}"DSBranchSuite.c.o	
"{3DOIncludes}"streaming:subscribertraceutils.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:satemplatedefs.h
"{ObjectDir}"DSBranchSuite.c.o		"{3DOIncludes}"streaming:sacontrolmsgs.h
"{ObjectDir}"TestVideoStream.c.o		:testvideostream.h
"{ObjectDir}"TestVideoStream.c.o		:joypad.h
"{ObjectDir}"TestVideoStream.c.o		:tests.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:datastreamlib.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:datastream.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:msgutils.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:mempool.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:dsstreamdefs.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:dserror.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:satemplatedefs.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:sacontrolmsgs.h
"{ObjectDir}"TestVideoStream.c.o	
"{3DOIncludes}"streaming:saudiosubscriber.h
"{ObjectDir}"TestVideoStream.c.o	
"{3DOIncludes}"streaming:mpegvideosubscriber.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:threadhelper.h
"{ObjectDir}"TestVideoStream.c.o	
"{3DOIncludes}"streaming:mpegbufferqueues.h
"{ObjectDir}"TestVideoStream.c.o	
"{3DOIncludes}"streaming:ezflixsubscriber.h
"{ObjectDir}"TestVideoStream.c.o		"{3DOIncludes}"streaming:preparestream.h
"{ObjectDir}"TestVideoStream.c.o	
"{3DOIncludes}"streaming:subscribertraceutils.h
"{ObjectDir}"tests.c.o		:testvideostream.h
"{ObjectDir}"tests.c.o		:joypad.h
"{ObjectDir}"tests.c.o		:tests.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:datastreamlib.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:datastream.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:msgutils.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:mempool.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:dsstreamdefs.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:dserror.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:satemplatedefs.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:sacontrolmsgs.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:saudiosubscriber.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:mpegvideosubscriber.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:threadhelper.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:mpegbufferqueues.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:ezflixsubscriber.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:preparestream.h
"{ObjectDir}"tests.c.o		"{3DOIncludes}"streaming:subscribertraceutils.h
"{ObjectDir}"JoyPad.c.o		:joypad.h
