/* @(#) tests.c 96/07/24 1.2 */
/*
	File:		tests.c

	Contains:	branch tests for dsbranchsuite

	Written by:	amasri

	Copyright:	 1996 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <3>	  6/6/96	am		remove compiler warnings
		 <2>	 4/24/96	am		give branch statistics for all passing tests

	To Do:
*/

#include <string.h>				/* memset */
#include <kernel/debug.h>
#include <kernel/mem.h>			/* for AllocMemTrack, FreeMemTrack   */
#include <audio/audio.h>
#include <graphics/view.h>		/* VIEWTAG_BITMAP */

#include <streaming/datastreamlib.h>
#include <streaming/dsstreamdefs.h>
#include <streaming/satemplatedefs.h>
#include <streaming/sacontrolmsgs.h>

#include "testvideostream.h"
#include <tests/3do_scaf.h>
#include "joypad.h"
#include "tests.h"

static char* sTestName;

/* MPEG_GOMARKER_FORW_TIME_SETUP calls DSGoMarker with GOMARKER_FORW_TIME option
 * while test stream is running. GOMARKER_FORW_TIME_TEST then compares the two 
 * time stamps to determine whether the jump was successful. */

Err MPEG_GOMARKER_FORW_TIME_SETUP(PlayerPtr ctx){
	Err status;
	uint32	startTime;
	uint32  branchTime;
	
	sTestName = "GOMARKER_FORW_TIME_1";
	BeginTest(FT, sTestName);
	startTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	branchTime = N_SECONDS * AUDIO_TICKS_PER_SECOND;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, branchTime,
							GOMARKER_FORW_TIME);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}

Err MPEG_GOMARKER_FORW_TIME_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	uint32	branchTime;
	int32	diffTime;
	int32	expectedDiff;

	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	startTime = ctx->startTime;
	diffTime = (branchTime - startTime);
	expectedDiff = N_MARKERS * MPEG_TICKS_PER_MARKER;
	if ((diffTime < expectedDiff - MPEG_TIME_TOLERANCE)|(diffTime > expectedDiff + MPEG_TIME_TOLERANCE)){
		LogResult(FAILED, " Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d", sTestName, (int)expectedDiff, (int)diffTime);
		status = 0;
	}
FAILED:
	return status;
}

/* MPEG_GOMARKER_BACK_TIME_SETUP calls DSGoMarker with GOMARKER_BACK_TIME  option
 * while test stream is running. GOMARKER_BACK_TIME_TEST then compares the 
 * actual pts (timestamp) with the expected value to determine whether the 
 * correct jump has been made */

Err MPEG_GOMARKER_BACK_TIME_SETUP(PlayerPtr ctx){
	Err status;
	uint32	startTime;
	uint32	branchTime;
	
	sTestName = "GOMARKER_BACK_TIME_1";
	BeginTest(FT, sTestName);
	startTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	branchTime = N_SECONDS * AUDIO_TICKS_PER_SECOND;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, branchTime,
							GOMARKER_BACK_TIME);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup of %s fails", sTestName);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}

Err MPEG_GOMARKER_BACK_TIME_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	branchTime;
	uint32	startTime;
	int32	diffTime;
	int32	expectedDiff;

	startTime = ctx->startTime;
	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	diffTime = (startTime - branchTime);
	expectedDiff = N_SECONDS * MPEG_TICKS_PER_SECOND;
	if ((diffTime < expectedDiff - MPEG_TIME_TOLERANCE)|(diffTime > expectedDiff + MPEG_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch time: %d. Actual: %d",sTestName,(int)expectedDiff,(int)branchTime);
		status = 0;
	}
FAILED:
	return status;
}

/* MPEG_GOMARKER_FORWARD_SETUP calls DSGoMarker with GOMARKER_FORWARD option set
 * to N_MARKERS. MPEG_GOMARKER_FORWARD_TEST then compares the actual PTS with the
 * anticipated time value for the branch.
 */

Err MPEG_GOMARKER_FORWARD_SETUP(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	
	sTestName = "GOMARKER_FORWARD_1";
	BeginTest(FT, sTestName);
	startTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, N_MARKERS,
							GOMARKER_FORWARD);
	if (status != 0){
		LogResult(SETUP_FAILED, "Test %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}

Err MPEG_GOMARKER_FORWARD_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	uint32  branchTime;
	int32   diffTime;
	int32	expectedDiff;
	
	startTime = ctx->startTime;
	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	diffTime = branchTime - startTime;
	expectedDiff = N_MARKERS * MPEG_TICKS_PER_MARKER;
	if ((diffTime < expectedDiff - MPEG_TIME_TOLERANCE)|(diffTime > expectedDiff + MPEG_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d",sTestName, (int)expectedDiff, (int)diffTime);
		status = 0;
	}
FAILED:
	return status;
}


/* MPEG_GOMARKER_BACKWARD_SETUP calls DSGoMarker with GOMARKER_BACKWARD option
 * while test stream is running. MPEG_GOMARKER_BACKWARD_TEST then compares the 
 * two time stamps to determine whether the jump was successful.
 */

Err MPEG_GOMARKER_BACKWARD_SETUP(PlayerPtr ctx){
	Err status;
	uint32	startTime;
	
	sTestName = "GOMARKER_BACKWARD_1";
	BeginTest(FT, sTestName);
	startTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, N_MARKERS,
							GOMARKER_BACKWARD);
	if (status < 0){
		LogResult(SETUP_FAILED, "Test %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}
Err MPEG_GOMARKER_BACKWARD_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	uint32	branchTime;
	int32	diffTime;
	int32	expectedDiff;
	
	startTime = ctx->startTime;
	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	diffTime = startTime - branchTime;
	expectedDiff = N_MARKERS * MPEG_TICKS_PER_MARKER;
	if  ((diffTime < expectedDiff - MPEG_TIME_TOLERANCE)|(diffTime > expectedDiff + MPEG_TIME_TOLERANCE)){
		LogResult(FAILED, "Failure: Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff );
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d",sTestName, (int)expectedDiff, (int)diffTime);
		status = 0;
	}
FAILED:
	return status;
}

/* MPEG_GOMARKER_ABSOLUTE_SETUP calls DSGoMarker with GOMARKER_ABSOLUTE option
 * while test stream is running. MPEG_GOMARKER_ABSOLUTE_TEST then compares the new pts number 
 * with the expected pts, to determine whether the jump is successful.
 */

Err MPEG_GOMARKER_ABSOLUTE_SETUP(PlayerPtr ctx){
	Err 	status;
	
	sTestName = "GOMARKER_ABSOLUTE_1";
	BeginTest(FT, sTestName);
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, MPEG_ABSOLUTE_BYTE,
							GOMARKER_ABSOLUTE);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup of %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
FAILED:
	return status;
}
Err MPEG_GOMARKER_ABSOLUTE_TEST(PlayerPtr ctx){
	Err status;
	int32	branchTime;
	int32	diffTime;
	
	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	diffTime = (MPEG_ABSOLUTE_BYTE_TIME - branchTime);
	if ((diffTime > MPEG_TIME_TOLERANCE)|(diffTime < - MPEG_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches to %d instead of to %d", sTestName, (int)branchTime, MPEG_ABSOLUTE_BYTE_TIME);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d",sTestName, (int)MPEG_ABSOLUTE_BYTE_TIME, (int)branchTime);
		status = 0;
	}
FAILED:
	return status;
}

/* MPEG_GOMARKER_ABS_MIDGOP_SETUP calls DSGoMarker with GOMARKER_ABSOLUTE option
 * and jumps to the middle of a GOP. MPEG_GOMARKER_ABS_MIDGOP_TEST then compares the new pts number 
 * with the expected pts, to determine whether the jump is successful.
 */

Err MPEG_GOMARKER_ABS_MIDGOP_SETUP(PlayerPtr ctx){
	Err 	status;
	
	sTestName = "GOMARKER_ABSOLUTE_2";
	BeginTest(FT, sTestName);
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, MPEG_ABS_MIDGOP_BYTE,
							GOMARKER_ABSOLUTE);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup of %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
FAILED:
	return status;
}
Err MPEG_GOMARKER_ABS_MIDGOP_TEST(PlayerPtr ctx){
	Err status;
	int32	branchTime;
	int32	diffTime;
	
	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	diffTime = (MPEG_ABS_MIDGOP_TIME - branchTime);
	if ((diffTime > MPEG_TIME_TOLERANCE)|(diffTime < - MPEG_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches to %d instead of to %d", sTestName, (int)branchTime, MPEG_ABSOLUTE_BYTE_TIME);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d",sTestName, (int)MPEG_ABS_MIDGOP_TIME, (int)branchTime);
		status = 0;
	}
FAILED:
	return status;
}

/* GOMARKER_ABS_TIME_SETUP calls DSGoMarker with GOMARKER_ABS_TIME option
 * while test stream is running. GOMARKER_ABS_TIME_TEST then compares the 
 * new time with the expected time to determine whether the jump is successful.
 */

Err MPEG_GOMARKER_ABS_TIME_SETUP(PlayerPtr ctx){
	Err status;
	
	sTestName = "GOMARKER_ABS_TIME_1";
	BeginTest(FT, sTestName);
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, MPEG_ABS_TIME_MARKER,
							GOMARKER_ABS_TIME);
	if (status < 0){
		LogResult(SETUP_FAILED, "Setup of %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
FAILED:
	return status;
}

Err MPEG_GOMARKER_ABS_TIME_TEST(PlayerPtr ctx){
	Err status;
	uint32	branchTime;
	int32	diffTime;
	uint32	actualTime;
	
	branchTime = ctx->lastMPEGBufferPtr->pts.FMVOpt_PTS;
	diffTime = MPEG_ABS_TIME_EXPECTED - branchTime;
	actualTime = MPEG_ABS_TIME_MARKER * MPEG_FACTOR;
	if ((diffTime > MPEG_TIME_TOLERANCE)|(diffTime < -MPEG_TIME_TOLERANCE )){
		LogResult(FAILED, "Test %s branches to %d instead of to %d", sTestName, (int)branchTime, (int)actualTime );
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected time: %d. Actual time: %d",sTestName,(int)actualTime,(int)branchTime);
		status = 0;
	}
FAILED:
	return status;
}
							
/* EZFX_GOMARKER_FORW_TIME_SETUP calls DSGoMarker with GOMARKER_FORW_TIME option
 * while test stream is running. GOMARKER_FORW_TIME_TEST then compares the two 
 * time stamps to determine whether the jump was successful. */

Err EZFX_GOMARKER_FORW_TIME_SETUP(PlayerPtr ctx){
	Err status;
	uint32	startTime;
	uint32  branchTime;
	
	sTestName = "GOMARKER_FORW_TIME_1";
	BeginTest(FT, sTestName);
	startTime = ctx->channelPtr->curFrameTime;
	branchTime = N_SECONDS * AUDIO_TICKS_PER_SECOND;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, branchTime,
							GOMARKER_FORW_TIME);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}

Err EZFX_GOMARKER_FORW_TIME_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	uint32	branchTime;
	int32	diffTime;
	int32	expectedDiff;

	branchTime = ctx->channelPtr->curFrameTime;
	startTime = ctx->startTime;
	diffTime = (branchTime - startTime);
	expectedDiff = N_MARKERS * AUDIO_TICKS_PER_MARKER;
	if ((diffTime < expectedDiff - AUDIO_TIME_TOLERANCE)|(diffTime > expectedDiff + AUDIO_TIME_TOLERANCE)){
		LogResult(FAILED, " Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d", sTestName, (int)expectedDiff, (int)diffTime);
		status = 0;
	}
FAILED:
	return status;
}

/* EZFX_GOMARKER_BACK_TIME_SETUP calls DSGoMarker with GOMARKER_BACK_TIME  option
 * while test stream is running. GOMARKER_BACK_TIME_TEST then compares the 
 * actual pts (timestamp) with the expected value to determine whether the 
 * correct jump has been made */

Err EZFX_GOMARKER_BACK_TIME_SETUP(PlayerPtr ctx){
	Err status;
	uint32	startTime;
	uint32	branchTime;
	
	sTestName = "GOMARKER_BACK_TIME_1";
	BeginTest(FT, sTestName);
	startTime =  ctx->channelPtr->curFrameTime;
	branchTime = N_SECONDS * AUDIO_TICKS_PER_SECOND;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, branchTime,
							GOMARKER_BACK_TIME);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup of %s fails", sTestName);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}

Err EZFX_GOMARKER_BACK_TIME_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	branchTime;
	uint32	startTime;
	int32	diffTime;
	int32	expectedDiff;

	startTime = ctx->startTime;
	branchTime = ctx->channelPtr->curFrameTime;
	diffTime = (startTime - branchTime);
	expectedDiff = N_SECONDS * AUDIO_TICKS_PER_SECOND;
	if ((diffTime < expectedDiff - AUDIO_TIME_TOLERANCE)|(diffTime > expectedDiff + AUDIO_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch time: %d. Actual: %d",sTestName,(int)expectedDiff,(int)branchTime);
		status = 0;
	}
FAILED:
	return status;
}

/* EZFX_GOMARKER_FORWARD_SETUP calls DSGoMarker with GOMARKER_FORWARD option set
 * to N_MARKERS. EZFX_GOMARKER_FORWARD_TEST then compares the actual PTS with the
 * anticipated time value for the branch.
 */

Err EZFX_GOMARKER_FORWARD_SETUP(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	
	sTestName = "GOMARKER_FORWARD_1";
	BeginTest(FT, sTestName);
	startTime = ctx->channelPtr->curFrameTime;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, N_MARKERS,
							GOMARKER_FORWARD);
	if (status != 0){
		LogResult(SETUP_FAILED, "Test %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}

Err EZFX_GOMARKER_FORWARD_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	uint32  branchTime;
	int32   diffTime;
	int32	expectedDiff;
	
	startTime = ctx->startTime;
	branchTime = ctx->channelPtr->curFrameTime;
	diffTime = branchTime - startTime;
	expectedDiff = N_MARKERS * AUDIO_TICKS_PER_MARKER;
	if ((diffTime < expectedDiff - AUDIO_TIME_TOLERANCE)|(diffTime > expectedDiff + AUDIO_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d",sTestName, (int)expectedDiff, (int)diffTime);
		status = 0;
	}
FAILED:
	return status;
}


/* EZFX_GOMARKER_BACKWARD_SETUP calls DSGoMarker with GOMARKER_BACKWARD option
 * while test stream is running. EZFX_GOMARKER_BACKWARD_TEST then compares the 
 * two time stamps to determine whether the jump was successful.
 */

Err EZFX_GOMARKER_BACKWARD_SETUP(PlayerPtr ctx){
	Err status;
	uint32	startTime;
	
	sTestName = "GOMARKER_BACKWARD_1";
	BeginTest(FT, sTestName);
	startTime = ctx->channelPtr->curFrameTime;
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, N_MARKERS,
							GOMARKER_BACKWARD);
	if (status < 0){
		LogResult(SETUP_FAILED, "Test %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
	ctx->startTime = startTime;
FAILED:
	return status;
}
Err EZFX_GOMARKER_BACKWARD_TEST(PlayerPtr ctx){
	Err 	status;
	uint32	startTime;
	uint32	branchTime;
	int32	diffTime;
	int32	expectedDiff;
	
	startTime = ctx->startTime;
	branchTime = ctx->channelPtr->curFrameTime;
	diffTime = startTime - branchTime;
	expectedDiff = N_MARKERS * AUDIO_TICKS_PER_MARKER;
	if  ((diffTime < expectedDiff - AUDIO_TIME_TOLERANCE)|(diffTime > expectedDiff + AUDIO_TIME_TOLERANCE)){
		LogResult(FAILED, "Failure: Test %s branches %d ticks instead of %d ticks", sTestName, (int)diffTime, (int)expectedDiff );
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected branch: %d. Actual branch: %d",sTestName, (int)expectedDiff, (int)diffTime);
		status = 0;
	}
FAILED:
	return status;
}

/* GOMARKER_ABSOLUTE_SETUP calls DSGoMarker with GOMARKER_ABSOLUTE option
 * while test stream is running. GOMARKER_ABSOLUTE_TEST then compares the new pts number 
 * with the expected pts, to determine whether the jump is successful.
 */

Err EZFX_GOMARKER_ABSOLUTE_SETUP(PlayerPtr ctx){
	Err 	status;
	
	sTestName = "GOMARKER_ABSOLUTE_1";
	BeginTest(FT, sTestName);
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, EZFX_ABSOLUTE_BYTE,
							GOMARKER_ABSOLUTE);
	if (status != 0){
		LogResult(SETUP_FAILED, "Setup of %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
FAILED:
	return status;
}
Err EZFX_GOMARKER_ABSOLUTE_TEST(PlayerPtr ctx){
	Err status;
	int32	branchTime;
	int32	diffTime;
	
	branchTime = ctx->channelPtr->curFrameTime;
	diffTime = (EZFX_ABSOLUTE_BYTE_TIME - branchTime);
	if ((diffTime > AUDIO_TIME_TOLERANCE)|(diffTime < -AUDIO_TIME_TOLERANCE)){
		LogResult(FAILED, "Test %s branches to %d instead of to %d", sTestName, (int)branchTime, EZFX_ABSOLUTE_BYTE_TIME);
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes",sTestName);
		status = 0;
	}
FAILED:
	return status;
}

/* GOMARKER_ABS_TIME_SETUP calls DSGoMarker with GOMARKER_ABS_TIME option
 * while test stream is running. GOMARKER_ABS_TIME_TEST then compares the 
 * new time with the expected time to determine whether the jump is successful.
 */

Err EZFX_GOMARKER_ABS_TIME_SETUP(PlayerPtr ctx){
	Err status;
	
	sTestName = "GOMARKER_ABS_TIME_1";
	BeginTest(FT, sTestName);
	status = DSGoMarker(ctx->messageItem, NULL, ctx->streamCBPtr, EZFX_ABS_TIME_MARKER,
							GOMARKER_ABS_TIME);
	if (status < 0){
		LogResult(SETUP_FAILED, "Setup of %s fails with error %d", sTestName, (int)status);
		goto FAILED;
	}
FAILED:
	return status;
}

Err EZFX_GOMARKER_ABS_TIME_TEST(PlayerPtr ctx){
	Err status;
	uint32	branchTime;
	int32	diffTime;
	uint32	actualTime;
	
	branchTime = ctx->channelPtr->curFrameTime;
	diffTime = EZFX_ABS_TIME_EXPECTED - branchTime;
	actualTime = EZFX_ABS_TIME_MARKER;
	if ((diffTime > AUDIO_TIME_TOLERANCE)|(diffTime < -AUDIO_TIME_TOLERANCE )){
		LogResult(FAILED, "Test %s branches to %d instead of to %d", sTestName, (int)branchTime, (int)actualTime );
		status = 0;
		goto FAILED;
	}
	else {
		LogResult(PASSED,"Test %s passes. Expected time: %d. Actual time: %d",sTestName,(int)actualTime,(int)branchTime);
		status = 0;
	}
FAILED:
	return status;
}
							
							

